// maAudioMac.c

/*
 * MoreAmp macOS carbon/classic audio 
 * Copyright (C) 2004-2008 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "maConfig.h" // included by all moreamp files

#ifdef AUDIO_MAC_SOUNDMGR

//#warning Using mac Sound Manager audio

// this is for OS 9
// it can be tested on OS X (with high cpu usage, and a long unpause delay)

#include <Sound.h>
#include <Multiprocessing.h>

#include <string.h>

#ifdef __cplusplus
extern "C"
{
#endif

#include "maPlayer.h"
#include "maUtils.h"
#include "maEqualizer.h"
#include "maAudioMac.h"

SndCallBackUPP l_psSndCallBackUPP = 0;
SndChannelPtr l_pSoundChannel;
ScheduledSoundHeader l_sndhdrA;
ScheduledSoundHeader l_sndhdrB;

SndCommand l_sndCmdA = {bufferCmd, 0, (long)&l_sndhdrA};
SndCommand l_sndCmdB = {bufferCmd, 0, (long)&l_sndhdrB};
SndCommand l_CallBackCmdA = {callBackCmd, 0, (long)&l_sndhdrA};
SndCommand l_CallBackCmdB = {callBackCmd, 0, (long)&l_sndhdrB};
	
pascal void psSndCallBackProc(SndChannelPtr channel, SndCommand* command);
int maPauseSound();
int maResumeSound();
int CreateMPTaskSoundStart();

signed long maTaskSoundStart(void *parameter)
{
	(void)parameter;
	char cstr[256];

	OSErr err = noErr;
	
	if(l_pSoundChannel)
	{
		SndCommand theCommand = {flushCmd, 0, 0};
		SndDoImmediate(l_pSoundChannel, &theCommand);

		theCommand.cmd = quietCmd;
		SndDoImmediate(l_pSoundChannel, &theCommand);

		if(g_sets.fileout[kOUTSOUND].mode) // close and re-open the sound channel
		{
			SndDisposeChannel(l_pSoundChannel, true);
			l_pSoundChannel = NULL;
		}
	}
		
	if(!l_pSoundChannel) // (re)open a sound channel :
	{
		if((err = SndNewChannel(&l_pSoundChannel, sampledSynth, initStereo, l_psSndCallBackUPP)) != noErr
																				|| !l_pSoundChannel)
		{
			sprintf(cstr, "maTaskSoundStart SndNewChannel err=%d l_pSoundChannel=x%lx", (int)err,
																	(unsigned long)l_pSoundChannel);
			maLog(cstr);
			err = !noErr;
		}
	}
	
	if(err == noErr)
	{
		maOutputSetVolume((kVOLMAX * volumevalL()) / VOLUMEMAX, (kVOLMAX * volumevalR()) / VOLUMEMAX);

		// wait for fifo and first two buffs loaded
		while(!g_fifoready || !g_bufready[0] || !g_bufready[1])
		{
			ThreadYield();
			if(g_cancel || g_state.stop)
				goto errorret;
		}

		// output first buf

		l_sndhdrA.u.cmpHeader.numFrames = g_nframes[0];
		l_sndhdrA.u.cmpHeader.samplePtr = g_sndbuf[0];
			
		if((err = SndDoCommand(l_pSoundChannel, &l_sndCmdA, FALSE)) != noErr)
		{
			sprintf(cstr, "maTaskSoundStart SndDoCommand l_sndCmdA err=%d l_pSoundChannel=x%lx",
														(int)err, (unsigned long)l_pSoundChannel);
			maLog(cstr);
		}
		else
		{
			if((err = SndDoCommand(l_pSoundChannel, &l_CallBackCmdA, FALSE)) != noErr)
			{
				sprintf(cstr, "maTaskSoundStart SndDoCommand l_CallBackCmdA err=%d l_pSoundChannel=x%lx",
															(int)err, (unsigned long)l_pSoundChannel);
				maLog(cstr);
			}
		}
					
		if(err == noErr) // this task ends here, the sound callback takes over
			return 0;
				
	} // if(err == noErr) on SndNewChannel
		
	// gets here if error or stopped before return above
errorret:
	if(l_pSoundChannel)
	{
		SndCommand theCommand = {flushCmd, 0, 0};
		SndDoImmediate(l_pSoundChannel, &theCommand);

		theCommand.cmd = quietCmd;
		SndDoImmediate(l_pSoundChannel, &theCommand);
	}
				
	g_state.stop |= STOP_SNDCHANNEL;
		
	return 0;
	
} // maTaskSoundStart


int CreateMPTaskSoundStart()
{
	OSStatus err;
	MPTaskID taskID;
	
	if((err = MPCreateTask(maTaskSoundStart,
		NULL,		// the parameter passed to the task
		65536,		// 0, // stack size
		NULL,		// g_taskQueueSound, // no notification queue
		NULL, NULL, // result parameters, unused
		0,			// no special task flags
		&taskID)) != 0)	
	{
		char cstr[256];
		sprintf(cstr, "CreateMPTaskSoundStart MPCreateTask error %d", (int)err);
		maLog(cstr);
	}

	return err;
	
} // CreateMPTaskSoundStart


int maOpenSndChannel()
{
	// install the snd cmd callback
	if(!l_psSndCallBackUPP)
		if((l_psSndCallBackUPP = NewSndCallBackUPP(psSndCallBackProc)) == NULL)
		{
			maLog("NewSndCallBackProc failed");
			return 1;
		}
	
	// init sound parameters
	l_sndhdrA.u.cmpHeader.numChannels	= MAOUTCHANNELS;
	l_sndhdrA.u.cmpHeader.encode		= cmpSH;
	l_sndhdrA.u.cmpHeader.numFrames		= 0;
#if defined(MA_CPU_I86)
	l_sndhdrA.u.cmpHeader.format		= k16BitLittleEndianFormat;
#else
	l_sndhdrA.u.cmpHeader.format		= k16BitBigEndianFormat;
#endif
	l_sndhdrA.u.cmpHeader.compressionID	= fixedCompression;
	l_sndhdrA.u.cmpHeader.sampleSize	= MAOUTBITS;
	l_sndhdrA.u.cmpHeader.sampleRate	= g_state.insamplerate << 16;
	l_sndhdrA.flags						= kScheduledSoundDoCallBack;
	l_sndhdrA.callBackParam1			= 0;
	l_sndhdrA.callBackParam2			= (long)&l_sndhdrA;
	l_sndhdrA.u.cmpHeader.samplePtr		= g_sndbuf[0]; // first buf
	l_sndhdrA.u.cmpHeader.numFrames		= 0;
		
	l_sndhdrB = l_sndhdrA;
	l_sndhdrB.callBackParam2			= (long)&l_sndhdrB;
	l_sndhdrB.u.cmpHeader.samplePtr		= g_sndbuf[1]; // second buf
	
	// create and run a task to start the sound
	if(CreateMPTaskSoundStart() != noErr)
		return 1;

	return 0;
	
} // maOpenSndChannel


void maCloseSndChannel(int wait)
{	
	SCStatus status;

	if(!l_pSoundChannel)
	{
		//maLog("maLog maCloseSndChannel error null l_pSoundChannel");
		return;
	}

	// wait 'til not busy
	if(wait)
	{
		g_stopB = 0;
		while(!SndChannelStatus(l_pSoundChannel, sizeof(status), &status) && status.scChannelBusy && !g_stopB)
		{
			ThreadYield();
			AppYield();
		}
	}
	
	g_waitaudiodone = 0;

	if(!g_sets.fileout[kOUTSOUND].mode) // don't close and re-open the sound channel
		return;
		
	SndDisposeChannel(l_pSoundChannel, true);
	l_pSoundChannel = NULL;
	
	if(l_psSndCallBackUPP)
	{
		DisposeSndCallBackUPP(l_psSndCallBackUPP);
		l_psSndCallBackUPP = 0;
	}
	
} // maCloseSndChannel


int maPause(int pause) // returns previous pause state
{
	int previous = g_pause;
		
	if(pause && previous == 0)
	{
		if(!maPauseSound())
			g_pause = 1;	
	}
	else if(previous == 1)
	{
		if(!maResumeSound())
			g_pause = 0;	
	}
		
	return previous;
		
} // maPause


void maOutputSetVolume(int volumeL, int volumeR)	
{
	SndCommand volumecmd;

	volumecmd.cmd = volumeCmd;
	volumecmd.param1 = 0;
	volumecmd.param2 = ((volumeR << 16) + volumeL); // 0-255

	if(l_pSoundChannel)
		SndDoImmediate(l_pSoundChannel, &volumecmd);
}


int maPauseSound()
{
	SndCommand pausecmd = {pauseCmd,  0, 0};
		
	if(!l_pSoundChannel)
		return 1;
		
		return SndDoImmediate(l_pSoundChannel, &pausecmd);
	
} // maPauseSound


int maResumeSound()
{
	SndCommand resumecmd = {resumeCmd, 0, 0};
		
	if(!l_pSoundChannel)
		return 1;
			
	return SndDoImmediate(l_pSoundChannel, &resumecmd);
		
} // maResumeSound


pascal void psSndCallBackProc(SndChannelPtr channel, SndCommand* command)
{
	(void)channel;
	int next;
	
	if((next = g_buftoplay + 1) >= NSNDBUFS)		// (g_buftoplay + 1) % NSNDBUFS;
		next = 0;
	
	if(command->param2 == (long)&l_sndhdrA)			// just finished playing A
	{
		if(g_bufready[next])							// next buffer is ready
		{
			g_bufready[g_buftoplay] = 0;					// release A's buffer
			g_buftoplay = next;								// play B with next buffer
			
			l_sndhdrB.u.cmpHeader.numFrames = g_nframes[next];			
			l_sndhdrB.u.cmpHeader.samplePtr = g_sndbuf[next];
			SndDoCommand(l_pSoundChannel, &l_sndCmdB, FALSE);
			SndDoCommand(l_pSoundChannel, &l_CallBackCmdB, FALSE);		
		}
		else											// next buffer is not ready, repeat	A	
		{
			if(g_state.stop)								// unless stopping
				return;
			
			memset(l_sndhdrA.u.cmpHeader.samplePtr, 0, SOUNDBUFSIZE); // silence instead of repeat
			SndDoCommand(l_pSoundChannel, &l_sndCmdA, FALSE);
			SndDoCommand(l_pSoundChannel, &l_CallBackCmdA, FALSE);		
		}			
	}
	else if(command->param2 == (long)&l_sndhdrB)	// just finished playing B
	{
		if(g_bufready[next])							// next buffer is ready
		{
			g_bufready[g_buftoplay] = 0;					// release B's buffer
			g_buftoplay = next;								// play A with next buffer
			
			l_sndhdrA.u.cmpHeader.numFrames = g_nframes[next];			
			l_sndhdrA.u.cmpHeader.samplePtr = g_sndbuf[next];			
			SndDoCommand(l_pSoundChannel, &l_sndCmdA, FALSE);
			SndDoCommand(l_pSoundChannel, &l_CallBackCmdA, FALSE);		
		}
		else											// next buffer is not ready, repeat	B	
		{
			if(g_state.stop)								// unless stopping
				return;
			
			memset(l_sndhdrB.u.cmpHeader.samplePtr, 0, SOUNDBUFSIZE); // silence instead of repeat
			SndDoCommand(l_pSoundChannel, &l_sndCmdB, FALSE);
			SndDoCommand(l_pSoundChannel, &l_CallBackCmdB, FALSE);			
		}
	}
	// else false callback

} // psSndCallBackProc

#ifdef __cplusplus
}
#endif

#endif

// eof
