// maAudioOSX.c

/*
 * MoreAmp Mac OS X native audio 
 * Copyright (C) 2004-2008 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "maConfig.h" // included by all moreamp files

#ifdef AUDIO_MAC_CORE

#include <CoreServices/CoreServices.h>
#include <AudioUnit/AudioUnit.h>

#ifdef __cplusplus
extern "C"
{
#endif

#include "maPlayer.h"
#include "maEqualizer.h"
#include "maUtils.h"
#include "maAudioOSX.h"

int maPauseSound();
int maResumeSound();

OSStatus macOSXrender( void *inRefCon, AudioUnitRenderActionFlags *ioActionFlags,
						const AudioTimeStamp *inTimeStamp, UInt32 inBusNumber,
						UInt32 inNumberFrames, AudioBufferList *ioData);

AudioUnit l_au = NULL;
AudioStreamBasicDescription l_streamFormat;

// extra fifo to transfer chunks of g_mafifo to AudioUnit's buffers
maFifo g_aufifo;
maFifo *g_paufifo = NULL;
char g_aubuf[AUBUFSIZE];

// frame buffer
#define FRAMEBUFSIZE (2 * 2 * 2048) // bytesperchannel * channels * room for 2048 frames
char l_frame[FRAMEBUFSIZE];  // s.b. a fct of inNumberFrames and ioData->mBuffers[0].mDataByteSize
unsigned long l_bytesin; // ioData->mBuffers[0].mDataByteSize + ioData->mBuffers[1].mDataByteSize;

int l_pause = 0;

int maOpenSndChannel()
{
	if(l_au)
	{
		if(!g_sets.fileout[kOUTSOUND].mode) // don't close and re-open the sound channel, unless required
		{
			if(l_streamFormat.mSampleRate == g_state.insamplerate)
				return 0;
			g_sets.fileout[kOUTSOUND].mode = 1;
			maCloseSndChannel(0);
			g_sets.fileout[kOUTSOUND].mode = 0;
		}
		else
			maCloseSndChannel(0);
	}
			
	ComponentDescription desc;
	Component comp;
	AURenderCallbackStruct callback;
	
	desc.componentType 		= kAudioUnitType_Output;
	desc.componentSubType  	= kAudioUnitSubType_DefaultOutput;
	desc.componentManufacturer = kAudioUnitManufacturer_Apple;
	desc.componentFlags		= 0;
	desc.componentFlagsMask	= 0;

	comp = FindNextComponent(0, &desc);
	if(comp == 0)
	{
		maLog("maOpenSndChannel FindNextComponent(kAudioUnitType_Output) failed");
		return 1;
	}

	if(OpenAComponent(comp, &l_au) != noErr)
	{
		maLog("maOpenSndChannel OpenAComponent(kAudioUnitType_Output) failed");
		l_au = NULL;
		return 1;
	}

	callback.inputProc = macOSXrender;
	callback.inputProcRefCon = 0;

	if(AudioUnitSetProperty(l_au, kAudioUnitProperty_SetRenderCallback,
							kAudioUnitScope_Input, 0, &callback, sizeof(callback)))
	{
		maLog("maOpenSndChannel AudioUnitSetProperty(kAudioUnitProperty_SetRenderCallback) failed");
		CloseComponent(l_au);
		l_au = NULL;
		return 1;
	}
		
	l_streamFormat.mSampleRate = g_state.insamplerate; //44100.0;
	l_streamFormat.mFormatID = kAudioFormatLinearPCM;
	l_streamFormat.mFormatFlags =   kLinearPCMFormatFlagIsSignedInteger |
#if defined(MA_CPU_I86)
									kAudioFormatFlagsNativeEndian |
#else
									kLinearPCMFormatFlagIsBigEndian |
#endif
									kLinearPCMFormatFlagIsPacked |
									kAudioFormatFlagIsNonInterleaved;
	l_streamFormat.mBytesPerPacket = 2;	
	l_streamFormat.mFramesPerPacket = 1;	
	l_streamFormat.mBytesPerFrame = 2;		
	l_streamFormat.mChannelsPerFrame = 2;	
	l_streamFormat.mBitsPerChannel = 16;	
	if(AudioUnitSetProperty(l_au,
							kAudioUnitProperty_StreamFormat,
							kAudioUnitScope_Input,
							0, // AudioUnitElement
							&l_streamFormat,
							sizeof(AudioStreamBasicDescription)))
	{
		maLog("maOpenSndChannel AudioUnitSetProperty(kAudioUnitProperty_StreamFormatfailed)");
		CloseComponent(l_au);
		l_au = NULL;
		return 1;
	}

	if(AudioUnitInitialize(l_au))
	{
		maLog("maOpenSndChannel AudioUnitInitialize failed");
		CloseComponent(l_au);
		l_au = NULL;
		return 1;
	}

	// init the extra audio fifo
	maFifoSetStorage(g_paufifo, g_aubuf);

	l_pause = 0;
	if(AudioOutputUnitStart(l_au) != 0)
	{
		maLog("maOpenSndChannel AudioOutputUnitStart failed");
		maCloseSndChannel(0);
		return 1;
	}
	
	return 0;
	
} // maOpenSndChannel


#define WAITSLICE 100 // ms
void maCloseSndChannel(int wait)
{
	int ms;
	int slice;
	
	if(!l_au)
	{
		//maLog("maCloseSndChannel error null audio l_au unit");
		return;
	}

	if(wait) // wait until audio has played (one buffer size's worth)
	{
		if(g_state.insamplerate > 0)
		{
			//       (1000 * FIFOSIZE / 2) / 44100 -> 313.4 ms
			//       (24000000) / 44100 -> 544.2 ms
			if((ms = (24000000) / g_state.insamplerate) > WAITSLICE)
				slice = WAITSLICE;
			else
				slice = ms;
		}
		else
		{
			ms = wait;
			slice = WAITSLICE;
		}
		g_stopB = 0;
		while(!g_stopB && ms > 0)
		{
			ThreadYield();
			AppYield();
			maSleep(slice);
			ms -= slice;
		}
	}
	
	g_waitaudiodone = 0;

	if(!g_sets.fileout[kOUTSOUND].mode) // don't close and re-open the sound channel
		return;
	
	if(!g_stopB)
		maSleep(200); // for intel mac aggregate device
	
	AudioOutputUnitStop(l_au);
	AudioUnitUninitialize(l_au);
	CloseComponent(l_au);
	l_au = NULL;		
	
} // maCloseSndChannel


int maPause(int pause) // returns previous pause state
{
	int previous = g_pause;
		
	if(pause && previous == 0)
	{
		if(!maPauseSound())
			g_pause = 1;	
	}
	else if(previous == 1)
	{
		if(!maResumeSound())
			g_pause = 0;	
	}
		
	return previous;
		
} // maPause


int l_osxvolumer;
int l_osxvolumel;
void maOutputSetVolume(int volumeL, int volumeR)
{
	// this sets up integer multipliers used in macOSXrender to reduce the volume
	l_osxvolumer = volumeR;
	l_osxvolumel = volumeL;
	
} // maOutputSetVolume


int maPauseSound()
{
	if(l_au)
	{
		/* 
		if(AudioOutputUnitStop(l_au) != 0)
		{
			maLog("maPauseSound AudioOutputUnitStop failed");
			return 1;
		}
		*/
		l_pause = 1;
	}

	return 0;

} // maPauseSound


int maResumeSound()
{
	if(l_au)
	{
		/* au is very slow to restart.. so just output silence while paused
		   at the expense of non-zero cpu time while paused
		if(AudioOutputUnitStart(l_au) != 0)
		{
			maLog("maResumeSound AudioOutputUnitStart failed");
			return 1;
		}
		*/
		l_pause = 0;
	}
	
	return 0;
	
} // maResumeSound


// dump chunks of aufifo to the AudioUnit
OSStatus macOSXrender( void *inRefCon, AudioUnitRenderActionFlags *ioActionFlags,
						const AudioTimeStamp *inTimeStamp, UInt32 inBusNumber,
						UInt32 inNumberFrames, AudioBufferList *ioData)
{
	short *p, *q, *r;
	unsigned long i;
	
	if(g_paufifo == NULL)
	{
		memset(ioData->mBuffers[0].mData, 0, ioData->mBuffers[0].mDataByteSize); // silence
		memset(ioData->mBuffers[1].mData, 0, ioData->mBuffers[1].mDataByteSize);
		maLog("macOSXrender null g_paufifo");
		g_state.stop |= STOP_SNDCHANNEL;
		return paramErr; // noErr
	}
	
	if(l_pause)
	{
		memset(ioData->mBuffers[0].mData, 0, ioData->mBuffers[0].mDataByteSize); // silence
		memset(ioData->mBuffers[1].mData, 0, ioData->mBuffers[1].mDataByteSize);
		return noErr;
	}
	
	l_bytesin = ioData->mBuffers[0].mDataByteSize + ioData->mBuffers[1].mDataByteSize; // bytes in fifo == bytes out to AudioUnit

	if(maFifoGetFilledSpace(g_paufifo) < l_bytesin)
	{
		memset(ioData->mBuffers[0].mData, 0, ioData->mBuffers[0].mDataByteSize); // silence
		memset(ioData->mBuffers[1].mData, 0, ioData->mBuffers[1].mDataByteSize);
# if(defined(__MAOSX__) && !defined(MAUSEMPSERVICES)) || defined(__MACUNIX__)
		maNamedSemaphorePost(g_aufifo.semaphore, MAFIFOSEMNAME);
#else
		maSemaphorePost(g_aufifo.semaphore);
#endif
		return noErr;
	}
	
	maFifoDraw(g_paufifo, l_frame, l_bytesin);
	
# if(defined(__MAOSX__) && !defined(MAUSEMPSERVICES)) || defined(__MACUNIX__)
		maNamedSemaphorePost(g_aufifo.semaphore, MAFIFOSEMNAME);
#else
		maSemaphorePost(g_aufifo.semaphore);
#endif

	// silence if quitting
	if(g_stopB)
	{
		memset(ioData->mBuffers[0].mData, 0, ioData->mBuffers[0].mDataByteSize); // silence
		memset(ioData->mBuffers[1].mData, 0, ioData->mBuffers[1].mDataByteSize);
		return noErr;
	}
			
	// uninterleave :
	p = (short *)(ioData->mBuffers[0].mData);
	q = (short *)(ioData->mBuffers[1].mData);
	r = (short *)&l_frame;

	// we do the volume control here because we don't understand CoreAudio
	if(l_osxvolumer == kVOLMAX && l_osxvolumel == kVOLMAX) // no change in volume
	{
		for(i = 0; i < l_bytesin >> 2; i++)
		{
			*p++ = *r++;
			*q++ = *r++;
		}
	}
	else if(l_osxvolumer == kVOLMAX) // reduce left volume only
	{
		for(i = 0; i < l_bytesin >> 2; i++)
		{
			*p++ = (l_osxvolumel * *r++) >> kVOLEXP;
			*q++ = *r++;
		}
	}
	else if(l_osxvolumel == kVOLMAX) // reduce right volume only
	{
		for(i = 0; i < l_bytesin >> 2; i++)
		{
			*p++ = *r++;
			*q++ = (l_osxvolumer * *r++) >> kVOLEXP;
		}
	}
	else // reduce both left and right volumes
	{
		for(i = 0; i < l_bytesin >> 2; i++)
		{
			*p++ = (l_osxvolumel * *r++) >> kVOLEXP;
			*q++ = (l_osxvolumer * *r++) >> kVOLEXP;
		}
	}

	return noErr;

} // macOSXrender


// this blocks until the rendercallback macOSXrender has made room in aufifo
int maOutputToSound(void* pcmblock, unsigned long frames, unsigned long len, int whichbuf)
{
	while(g_pause)
		ThreadYield();
	
	while(maFifoGetFreeSpace(g_paufifo) < len)
# if(defined(__MAOSX__) && !defined(MAUSEMPSERVICES)) || defined(__MACUNIX__)
		maNamedSemaphoreWait(g_aufifo.semaphore, MAFIFOSEMNAME);
#else
		maSemaphoreWait(g_aufifo.semaphore);
#endif

	maFifoStore(g_paufifo, (char *)pcmblock, len);
			
	return 0;

} // maOutputToSound

#ifdef __cplusplus
}
#endif

#endif

// eof
