// maAudioPA.c

/*
 * MoreAmp PortAudio interface
 * Copyright (C) 2004-2008 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "maConfig.h" // included by all moreamp files

#ifdef __MAWIN__
 #ifdef MAUNUSEDWIN
  #pragma warning( disable : 4100 ) // unreferenced formal parameter
 #endif
 #pragma warning( disable : 4206 ) // nonstandard extension used : translation unit is empty
 #pragma warning( disable : 4244 ) // conversion from 'double ' to 'int ', possible loss of data
#endif

#ifdef AUDIO_PORTAUDIO

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>
#include <string.h>
#include <math.h>

#if defined(__MAOS9__)
 //#warning Using PortAudio Mac OS9
 #include "maAudioMac.h"
#endif

#if defined(__MAOSX__)
 #include <CoreServices/CoreServices.h>
 #include <AudioUnit/AudioUnit.h>
 AudioDeviceID g_osxDevID = kAudioDeviceUnknown;
#endif

#include "maPlayer.h"
#include "maUtils.h"
#include "maEqualizer.h"

#if defined(__MAOSX__)
 #ifdef AUDIO_PORTAUDIO_SYSTEM
  //#warning Using PortAudio (system) Mac OS X
 #else
  //#warning Using PortAudio (libdir) Mac OS X
 #endif
 #include "maAudioMac.h"
#endif

#if defined(__MAWIN__)
 //#warning Using PortAudio Win32
 #include "maAudioWin.h"
#endif

#if defined(__MAUNIX__)
 //#warning Using PortAudio Unix
 #include "maAudioUnix.h"
#endif

#include "portaudio.h"

#ifdef __MAOS9__
 // Set portaudio.lib in the lib tree of the Files window of the CW projects
 // For Classic set SoundLib in the tree of the Files window of the CW Classic project
 // FIXME For Carbon need to resolve 'NewRoutineDescriptor'
#endif

int maOutputToSound(void* pcmblock, unsigned long frames, unsigned long len, int whichbuf);
int maPauseSound();
int maResumeSound();

int l_pavolumer;
int l_pavolumel;
int l_insamplerate;
int l_paInitCount = 0;

PaStream *l_PaStream = NULL;
#if !defined(NODECODEAUDIOIN)
 PaStream *l_PaStreamIn = NULL;
#endif

// extra fifo to transfer chunks of g_mafifo to AudioUnit's buffers
maFifo g_aufifo;
maFifo *g_paufifo = NULL;
char g_aubuf[AUBUFSIZE];

// frame buffer
#define PABYTESPERFRAME 4   // a frame here is bytesperchannel * channels
#define PAMAXFRAMES 512		// SOUNDBUFSIZE / PABYTESPERFRAME is 'too big'
char l_frame[PAMAXFRAMES * PABYTESPERFRAME];

static int maPArender(const void *inputBuffer, void *outputBuffer,
						unsigned long framesPerBuffer,
						const PaStreamCallbackTimeInfo* timeInfo,
						PaStreamCallbackFlags statusFlags,
						void *userData);

int maPaTerminate() // on windows calling Pa_Terminate blocks drag and drop until MoreAmp is restarted...
{					// so we only call it from app exit, not every time we close the sound channel
	PaError err;
	char cstr[256];

	if(l_paInitCount > 0)
	{
		if((err = Pa_Terminate()) != paNoError)
		{
			sprintf(cstr, "maPaTerminate failed, %s", Pa_GetErrorText(err));
			maLog(cstr);
			return 1;
		}
		l_paInitCount--;
	}
	return 0;

} // maPaTerminate


int maOpenSndChannel()
{
	void *pauserdata = NULL;
	PaStreamParameters outputParameters;
	PaError err;
	char cstr[256];

#if !defined(NODECODEAUDIOIN)
	if(l_PaStream)
	{
		if((g_state.infiletype == kMAMAA && l_PaStreamIn != l_PaStream) ||
		   (g_state.infiletype != kMAMAA && l_PaStreamIn != NULL))
		{
			g_sets.fileout[kOUTSOUND].mode = 1;
			maCloseSndChannel(0);
			g_sets.fileout[kOUTSOUND].mode = 0;
		}
	}
#endif
	if(l_PaStream)
	{
		if(!g_sets.fileout[kOUTSOUND].mode) // don't close and re-open the sound channel, unless required
		{
			if(l_insamplerate == g_state.insamplerate)
				return 0;
			g_sets.fileout[kOUTSOUND].mode = 1;
			maCloseSndChannel(0);
			g_sets.fileout[kOUTSOUND].mode = 0;
		}
		else
			maCloseSndChannel(0);
	}
			
	memset(l_frame, 0, PAMAXFRAMES * PABYTESPERFRAME);
	
	if(l_paInitCount <= 0)
	{
		if((err = Pa_Initialize()) != paNoError)
		{
			sprintf(cstr, "maOpenSndChannel Pa_Initialize failed, %s", Pa_GetErrorText(err));
			maLog(cstr);
			return 1;
		}
		l_paInitCount++;
	}

	outputParameters.device = Pa_GetDefaultOutputDevice();  // default output device
	outputParameters.channelCount = 2;						// stereo
	outputParameters.sampleFormat = paInt16;				// 16-bit int
	outputParameters.suggestedLatency =
				Pa_GetDeviceInfo(outputParameters.device)->defaultHighOutputLatency;
	outputParameters.hostApiSpecificStreamInfo = NULL;

#if !defined(NODECODEAUDIOIN)
	if(g_state.infiletype == kMAMAA)
	{
		PaStreamParameters inputParameters;
		inputParameters.device = Pa_GetDefaultInputDevice();	// default input device
		inputParameters.channelCount = 2;						// stereo
		inputParameters.sampleFormat = paInt16;					// 16-bit int
		inputParameters.suggestedLatency =
				Pa_GetDeviceInfo(inputParameters.device)->defaultHighInputLatency;
		inputParameters.hostApiSpecificStreamInfo = NULL;

		if((err = Pa_OpenStream(&l_PaStream, &inputParameters, &outputParameters, g_state.insamplerate,
			PAMAXFRAMES, paClipOff | paDitherOff, maPArender, pauserdata)) != paNoError)
		goto paopenerror;
		l_PaStreamIn = l_PaStream;
	}
	else
#endif
	if((err = Pa_OpenStream(&l_PaStream, NULL, &outputParameters, g_state.insamplerate,
			PAMAXFRAMES, paClipOff | paDitherOff, maPArender, pauserdata)) != paNoError)
		goto paopenerror;

	if((err = Pa_StartStream(l_PaStream)) != paNoError)
		goto paopenerror;

	l_insamplerate = g_state.insamplerate;
	
	return 0;
	
paopenerror:
	// maPaTerminate();
	l_PaStream = NULL;
#if !defined(NODECODEAUDIOIN)
	l_PaStreamIn = NULL;
#endif
	sprintf(cstr, "maOpenSndChannel failed, %s", Pa_GetErrorText(err));
	maLog(cstr);
	return 1;
	
} // maOpenSndChannel


void maCloseSndChannel(int wait)
{
	PaError err;
	char cstr[256];

	if(!l_PaStream)
	{
		//maLog("maCloseSndChannel error null l_PaStream");
		g_waitaudiodone = 0;
		return;
	}

	// wait 'til not busy
	if(wait)
	{
		g_waitaudiodone = 1;
		g_stopB = 0;
		while(g_waitaudiodone && !g_stopB)
		{
			ThreadYield();
			AppYield();
			maSleep(100);
		}
	}

	if(!g_sets.fileout[kOUTSOUND].mode) // don't close and re-open the sound channel
	{
		g_waitaudiodone = 0;
		return;
	}			

	if((err = Pa_AbortStream(l_PaStream)) != paNoError)
		goto pacloseerror;

	if((err = Pa_CloseStream(l_PaStream)) != paNoError)
		goto pacloseerror;

	// maPaTerminate();
	l_PaStream = NULL;
#if !defined(NODECODEAUDIOIN)
	l_PaStreamIn = NULL;
#endif
	g_waitaudiodone = 0;
	return;
	
pacloseerror:
	// maPaTerminate();
	l_PaStream = NULL;
#if !defined(NODECODEAUDIOIN)
	l_PaStreamIn = NULL;
#endif
	sprintf(cstr, "maCloseSndChannel failed, %s", Pa_GetErrorText(err));
	maLog(cstr);
	g_waitaudiodone = 0;

} // maCloseSndChannel


int maPause(int pause) // returns previous pause state
{
	int previous = g_pause;
		
	if(pause && previous == 0)
	{
		if(!maPauseSound())
			g_pause = 1;	
	}
	else if(previous == 1)
	{
		if(!maResumeSound())
			g_pause = 0;	
	}
		
	return previous;
		
} // maPause


void maOutputSetVolume(int volumeL, int volumeR)
{
	// this sets up integer multipliers used in ReduceVolume()
	l_pavolumer = volumeR;
	l_pavolumel = volumeL;

} // maOutputSetVolume


int maPauseSound()
{
	// do nothing here
	return 0;
} // maPauseSound


int maResumeSound()
{
	// do nothing here
	return 0;
	
} // maResumeSound


static int maPArender(const void *inputBuffer, void *outputBuffer,
						unsigned long framesPerBuffer,
						const PaStreamCallbackTimeInfo* timeInfo,
						PaStreamCallbackFlags statusFlags,
						void *userData)
{
#ifdef MAUNUSED
 (void)inputBuffer; (void)timeInfo; (void)statusFlags; (void)userData;
#endif

	unsigned long bytes;
	unsigned long bytesin;
	short *p, *r;
	unsigned long i;
	
	if(outputBuffer)
	{
		bytesin = PABYTESPERFRAME * framesPerBuffer; // bytes in fifo == bytes out to AudioUnit

		if(g_paufifo == NULL)
		{
			// memset(outputBuffer, 0, bytesin); // silence, if paContinue 
			maLog("maPArender null g_paufifo");
			return paAbort; // paContinue;
		}

		if((bytes = maFifoGetFilledSpace(g_paufifo)) < bytesin)
		{
			if(bytes > 0)
			{
				maFifoDraw(g_paufifo, l_frame, bytes);
#if(defined(__MAOSX__) && !defined(MAUSEMPSERVICES)) || defined(__MACUNIX__)
				maNamedSemaphorePost(g_aufifo.semaphore, MAFIFOSEMNAME);
#else
				maSemaphorePost(g_aufifo.semaphore);
#endif
				memset(l_frame + bytes, 0, bytesin - bytes); // pad with silence
			}
			else // if(bytes <= 0)
			{
				memset(outputBuffer, 0, bytesin); // silence
				g_waitaudiodone = 0;
				goto parendercontinue; // paComplete;
			}
		} // < bytesin
		else
		{
			maFifoDraw(g_paufifo, l_frame, bytesin);	
# if(defined(__MAOSX__) && !defined(MAUSEMPSERVICES)) || defined(__MACUNIX__)
			maNamedSemaphorePost(g_aufifo.semaphore, MAFIFOSEMNAME);
#else
			maSemaphorePost(g_aufifo.semaphore);
#endif
		}
	
		// silence if quitting
		if(g_stopB)
		{
			memset(outputBuffer, 0, bytesin);
			goto parendercontinue;
		}

		p = (short *)outputBuffer;
		r = (short *)&l_frame;
	
		// we do the volume control here
		if(l_pavolumer == kVOLMAX && l_pavolumel == kVOLMAX) // no change in volume
			memcpy((char *)p,(char *)r, bytesin);
		else if(l_pavolumer == kVOLMAX) // reduce left volume only
		{
			for(i = 0; i < framesPerBuffer; i++)
			{
				*p++ = (l_pavolumel * *r++) >> kVOLEXP;
				*p++ = *r++;
			}
		}
		else if(l_pavolumel == kVOLMAX) // reduce right volume only
		{
			for(i = 0; i < framesPerBuffer; i++)
			{
				*p++ = *r++;
				*p++ = (l_pavolumer * *r++) >> kVOLEXP;
			}
		}
		else // reduce both left and right volumes
		{
			for(i = 0; i < framesPerBuffer; i++)
			{
				*p++ = (l_pavolumel * *r++) >> kVOLEXP;
				*p++ = (l_pavolumer * *r++) >> kVOLEXP;
			}
		}
		
	} // if outputBuffer
	
parendercontinue:
#if !defined(NODECODEAUDIOIN)
	if(g_state.infiletype == kMAMAA && inputBuffer)
	{
		if(g_stopB)
			return paContinue;

		bytesin = PABYTESPERFRAME * framesPerBuffer;
	
		// Write to fifo
		if(maOutputWrite((char *)inputBuffer, bytesin, &g_state) != 0)
		{
			g_state.stop |= STOP_FLAG;
			return paAbort;
		}
	}
#endif
		
	return paContinue;
	
} // maPArender


// this blocks until the rendercallback has made room in aufifo
int maOutputToSound(void* pcmblock, unsigned long frames, unsigned long len, int whichbuf)
{
	while(g_pause)
		ThreadYield();
	
	while(maFifoGetFreeSpace(g_paufifo) < len)
	{
#if !defined(NODECODEAUDIOIN)
		if(g_state.infiletype == kMAMAA)
			return 0; // cannot block the audioin stream
#endif
		maSemaphoreWait(g_aufifo.semaphore); //maSemaphoreWaitTimeout(g_aufifo.semaphore, 5000);
	}
	maFifoStore(g_paufifo, (char *)pcmblock, len);
			
	return 0;

} // maOutputToSound

#ifdef __cplusplus
}
#endif

#endif

// eof
