// maCdaOS9.c

/*
 * MoreAmp cd reading for MacOS 9
 * Copyright (C) 2004-2008 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

// make classic shared library macdalib.lib for moreamp Carbon
// link with moreampClassic for moreampClassic

#include "maConfig.h" // included by all moreamp files

#ifndef MANOCDA

#ifdef __cplusplus
extern "C" {
#endif

#if !TARGET_API_MAC_CARBON

#include <Types.h>
#include <Files.h>
#include <Gestalt.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <StandardFile.h>
#include <Devices.h>

#include "maPlayer.h"
#define matLog(x)

#include "maCda.h"

#pragma export on
cdadrive g_cdadrive[CDAMAXNUMDRIVES + 1] = {0};
char *g_cdabuf = NULL;
int g_cdabufblocks = FIFOSIZE / CDABLOCKSIZE;
int g_cdaactive = 0; // 1-based index
int g_cdanumdrives = 0;
#pragma export off

int CdaBCDtoHex(char SrcByte);
char CdaHextoBCD(int SrcInt);
int Cdamsf_to_frames(int minutes, int seconds, int frames);
int Cdamsf_to_lba(int minutes, int seconds, int frames);


int CdaBCDtoHex(char SrcByte)
{
	int x, y;

	x = SrcByte % 16;
	y = SrcByte / 16;
	y = y * 10;
	return x + y;
}


char CdaHextoBCD(int SrcInt)
{
	int x, y;
	char z;

	z = 0;
	x = SrcInt % 256;
	y = x;
	if(y >= 10)
	{
		z = z + 16;
		do
		{
			y = y - 10;
            if(y >= 10)
				z = z + 16;
		} while(y >= 10);
	}
	
	return y + z;

} // CdaHextoBCD


int Cdamsf_to_frames(int minutes, int seconds, int frames)
{
  return minutes * 4500 + seconds * 75 + frames;
}


int Cdamsf_to_lba(int minutes, int seconds, int frames)
{
  if(Cdamsf_to_frames(minutes, seconds, frames) > 150)
	return Cdamsf_to_frames(minutes, seconds, frames) - 150;
   
  return 0;
}


#define PBCONTROL(x,y) PBControlSync(x)

#define SILENTFRAMES (150 - 118 - 1) // workaround
#pragma export on
int CdaGetTrackStartEnd(int drive)
{
	ParamBlockRec PB = {0};
	unsigned char tocdata[8] = {0};
	char cstr[256];
	OSErr err;
	
	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		sprintf(cstr, "CdaGetTrackStartEnd bad drive index=%d", drive);
		matLog(cstr);
		return 1;
	}
	
	// get MSF position of this track from cd TOC
	PB.cntrlParam.ioCRefNum = g_cdadrive[drive].driverRefNum;
	PB.cntrlParam.csCode = 100;
	PB.cntrlParam.csParam[0] = 3;
	*(char **)&PB.cntrlParam.csParam[1] = (char *)&tocdata;
	if(g_cdadrive[drive].track == g_cdadrive[drive].tracklast)
		PB.cntrlParam.csParam[3] = 4; // get this track
	else
		PB.cntrlParam.csParam[3] = 8; // get this and next track
	PB.cntrlParam.csParam[4] = CdaHextoBCD(g_cdadrive[drive].track) << 8;
	err = PBCONTROL(&PB, false);
	if(err != noErr)
	{
	 	sprintf(cstr, "CdaGetTrackStartEnd drive=%d track=%d PBControl 100-3 err=%d", drive, g_cdadrive[drive].track, err);
		matLog(cstr);
		return err;
	}

	// for last track get MSF position of leadout
	if(g_cdadrive[drive].track == g_cdadrive[drive].tracklast) 
	{			
		err = CdaGetEndOfDisk(drive, tocdata);
		if(err != noErr)
		{
	 		sprintf(cstr, "CdaGetTrackStartEnd CdaGetEndOfDisk drive=%d track=%d err=%d", drive, g_cdadrive[drive].track, err);
			matLog(cstr);
			return err;
		}
	}
	
	//														minutes							seconds					frames
	g_cdadrive[drive].lbafirst = -SILENTFRAMES + Cdamsf_to_lba(CdaBCDtoHex(tocdata[1]), CdaBCDtoHex(tocdata[2]), CdaBCDtoHex(tocdata[3]));
	if((int)g_cdadrive[drive].lbafirst < 1)
			g_cdadrive[drive].lbafirst = SILENTFRAMES;
	
	g_cdadrive[drive].lbalast = -SILENTFRAMES + Cdamsf_to_lba(CdaBCDtoHex(tocdata[4 + 1]), CdaBCDtoHex(tocdata[4 + 2]), CdaBCDtoHex(tocdata[4 + 3]));

	return noErr;
	
} // CdaGetTrackStartEnd


//	if((err = CdaReadTrack[drive].driverRefNum, g_cdadrive[drive].lba, 
//							g_cdabuf, g_cdabufblocks)) != noErr)
int CdaReadTrack(int drive, long *pbytesread)
{
	unsigned char *bytePoint;
	ParamBlockRec PB = {0};
	char cstr[256];
	OSErr err;
	
	*pbytesread = 0;
	
	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		sprintf(cstr, "CdaReadTrack bad drive index=%d", drive);
		matLog(cstr);
		return 1;
	}
	
	if(!g_cdabuf)
	{
		sprintf(cstr, "CdaReadTrack drive=%d NULL buf (lba=%d)", drive, g_cdadrive[drive].lba);
		matLog(cstr);
		return 1;
	}
	
	PB.fileParam.ioCompletion = nil;
	PB.cntrlParam.ioCRefNum = g_cdadrive[drive].driverRefNum;
	PB.cntrlParam.csCode = 115;
					
	// 0=Audio data 2352 bytes per audio block
	PB.cntrlParam.csParam[0] = 0;
	
	// buffer address												
	PB.cntrlParam.csParam[1] = (((unsigned long)g_cdabuf) >> 16)  & 0xffff;
	PB.cntrlParam.csParam[2] =  ((unsigned long)g_cdabuf) & 0xffff;

	// 32-bit Logical Block Address
	PB.cntrlParam.csParam[3] = 0;
	PB.cntrlParam.csParam[4] = (((unsigned long)g_cdadrive[drive].lba) >> 16)  & 0xffff;
	PB.cntrlParam.csParam[5] =  ((unsigned long)g_cdadrive[drive].lba) & 0xffff;
					
	// number of consecutive audio blocks to return
	bytePoint = (unsigned char *)&PB.cntrlParam.csParam[6];
	*bytePoint = g_cdabufblocks / 256;
	bytePoint++;
	*bytePoint = g_cdabufblocks % 256;

	err = PBCONTROL(&PB, false);

	if(err != noErr && err != paramErr)
	{
	 	sprintf(cstr, "CdaReadTrack drive=%d lba=%d lbasperread=%d err=%d",
	 						drive, g_cdadrive[drive].lba, g_cdabufblocks, err);
		matLog(cstr);
	}
	else
		*pbytesread = g_cdabufblocks * CDABLOCKSIZE;
		
	return err;
	
} // CdaReadTrack


int CdaGetCdInfo(int drive, char *StrtTrk, char *EndTrk)
{
	ParamBlockRec PB = {0};
	OSErr err;
	char cstr[256];

	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		sprintf(cstr, "CdaGetCdInfo bad drive index=%d", drive);
		matLog(cstr);
		return 1;
	}
	
	if(!StrtTrk)
	{
		sprintf(cstr, "CdaGetCdInfo drive=%d null StrtTrk", drive);
		matLog(cstr);
		return 1;
	}
	
	if(!EndTrk)
	{
		sprintf(cstr, "CdaGetCdInfo drive=%d null EndTrk", drive);
		matLog(cstr);
		return 1;
	}
	
	PB.fileParam.ioCompletion = nil;
	PB.cntrlParam.ioCRefNum = g_cdadrive[drive].driverRefNum;
	PB.cntrlParam.csCode = 100;
	PB.cntrlParam.csParam[0] = 1; // first and last track numbers
	err = PBCONTROL(&PB, false);
	if(err != noErr)
	{
		sprintf(cstr, "CdaGetCdInfo PBControl err=%d", err);
		matLog(cstr);
		return err;
	}
	
	*StrtTrk = ((unsigned short)PB.cntrlParam.csParam[0] >> 8) & 0xff;
	*EndTrk = PB.cntrlParam.csParam[0] & 0xff;

	return noErr;
	
} // CdaGetCdInfo


/*
int CdaGetCdFeatures(int drive, unsigned int *features)
{
	ParamBlockRec PB = {0};
	OSErr err;
	char cstr[256];

	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		sprintf(cstr, "CdaGetCdFeatures bad drive index=%d", drive);
		matLog(cstr);
		return 1;
	}
	
	PB.fileParam.ioCompletion = nil;
	PB.cntrlParam.ioCRefNum = g_cdadrive[drive].driverRefNum;
	PB.cntrlParam.csCode = 121;
	err = PBCONTROL(&PB, false);
	if(err != noErr)
	{
		sprintf(cstr, "CdaGetCdFeatures PBControl err=%d", err);
		matLog(cstr);
		return err;
	}

	if(features)
	{
		*features = PB.cntrlParam.csParam[0] && 0xff +
				   ((unsigned short)PB.cntrlParam.csParam[0] >> 8) && 0xff;
	}
			
	return noErr;
	
} // CdaGetCdFeatures


int CdaGetBlockSize(int drive, int *blocksize)
{
	ParamBlockRec PB = {0};
	OSErr err;
	char cstr[256];

	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		sprintf(cstr, "CdaGetBlockSize bad drive index=%d", drive);
		matLog(cstr);
		return 1;
	}
	
	if(!blocksize)
	{
		sprintf(cstr, "CdaGetBlockSize drive=%d null blocksize", drive);
		matLog(cstr);
		return 1;
	}
	
	PB.fileParam.ioCompletion = nil;
	PB.cntrlParam.ioCRefNum = g_cdadrive[drive].driverRefNum;
	PB.cntrlParam.csCode = 98;
	err = PBCONTROL(&PB, false);
	if(err != noErr)
	{
		sprintf(cstr, "CdaGetBlockSize PBControl err=%d", err);
		matLog(cstr);
		return err;
	}

	if(blocksize)
		*blocksize = PB.cntrlParam.csParam[0];
			
	return noErr;
	
} // CdaGetBlockSize
*/


int CdaSetBlockSize(int drive, int blocksize)
{
	ParamBlockRec PB = {0};
	OSErr err;
	char cstr[256];

	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		sprintf(cstr, "CdaSetBlockSize bad drive index=%d", drive);
		matLog(cstr);
		return 1;
	}
	
	PB.fileParam.ioCompletion = nil;
	PB.cntrlParam.ioCRefNum = g_cdadrive[drive].driverRefNum;
	PB.cntrlParam.csCode = 79;
	PB.cntrlParam.csParam[0] = blocksize;
	err = PBCONTROL(&PB, false);
	if(err != noErr)
	{
		sprintf(cstr, "CdaSetBlockSize %d err=%d", blocksize, err);
		matLog(cstr);
		return err;
	}
	
	return noErr;
	
} // CdaSetBlockSize


int CdaSetSpindleSpeed(int drive, int speed)
{
	ParamBlockRec PB = {0};
	OSErr err;
	char cstr[256];

	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		sprintf(cstr, "CdaSetSpindleSpeed bad drive index=%d", drive);
		matLog(cstr);
		return 1;
	}
	
	PB.fileParam.ioCompletion = nil;
	PB.cntrlParam.ioCRefNum = g_cdadrive[drive].driverRefNum;
	PB.cntrlParam.csCode = 114;
	PB.cntrlParam.csParam[0] = speed ? 0xff : 0; // 0 normal or 0xff for max
	err = PBCONTROL(&PB, false);
	if(err != noErr)
	{
		sprintf(cstr, "CdaSetSpindleSpeed PBControl err=%d", err);
		matLog(cstr);
		return err;
	}
	
	return noErr;
	
} // CdaSetSpindleSpeed


int CdaGetEndOfDisk(int drive, unsigned char *tocdata)
{
	ParamBlockRec PB = {0};
	OSErr err;
	char cstr[256];
	
	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		sprintf(cstr, "CdaGetEndOfDisk bad drive index=%d", drive);
		matLog(cstr);
		return 1;
	}
	
	if(!tocdata)
	{
		sprintf(cstr, "CdaGetBlockSize drive=%d null tocdata", drive);
		matLog(cstr);
		return 1;
	}
	
	PB.fileParam.ioCompletion = nil;
	PB.cntrlParam.ioCRefNum = g_cdadrive[drive].driverRefNum;
	PB.cntrlParam.csCode = 100;
	PB.cntrlParam.csParam[0] = 2;
	err = PBCONTROL(&PB, false);
	if(err == noErr)
	{
		tocdata[4 + 1] = ((unsigned short)PB.cntrlParam.csParam[0] >> 8) & 0xff;	// minutes
		tocdata[4 + 2] =  				  PB.cntrlParam.csParam[0]       & 0xff;	// seconds
		tocdata[4 + 3] = ((unsigned short)PB.cntrlParam.csParam[1] >> 8) & 0xff;	// frames
   }
	else
	{
    	sprintf(cstr, "CdaGetEndOfDisk PBControl err=%d", err);
    	matLog(cstr);
	}
	
	return err;
	
} // CdaGetEndOfDisk


int CdaCleanup(int drive)
{
	ParamBlockRec PB = {0};
	OSErr err;
	char cstr[256];

	/* this screws up Plextor drive on USB
	PB.fileParam.ioCompletion = nil;
	PB.cntrlParam.ioCRefNum = g_cdadrive[drive].driverRefNum;
	PB.cntrlParam.csCode = 1; // KillIO
	PBCONTROL(&PB, false);
	*/

	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		sprintf(cstr, "CdaCleanup bad drive index=%d", drive);
		matLog(cstr);
		return 1;
	}
	
	if(CdaSetBlockSize(drive, 512))
	{
		sprintf(cstr, "CdaCleanup CdaSetBlockSize 512 failed");
		matLog(cstr);
	}
	
	/*
	memset(&PB, 0, sizeof(PB));
	PB.fileParam.ioCompletion = nil;
	PB.cntrlParam.ioCRefNum = g_cdadrive[drive].driverRefNum;
	PB.cntrlParam.csCode = -1; // GoodBye
	err = PBCONTROL(&PB, false);
	if(err != noErr)
	{
		sprintf(cstr, "CdaCleanup PBControl err=%d", err);
		matLog(cstr);
		return err;
	}
	*/

	memset(&PB, 0, sizeof(PB));
	PB.fileParam.ioNamePtr = NULL; // specify volume by vrefnum
	PB.fileParam.ioVRefNum = g_cdadrive[drive].fsvolrefnum;
	err = PBFlushVolSync((ParamBlockRec *)&PB); // ignore errors from flush
	if(err != noErr)
	{
		sprintf(cstr, "CdaCleanup PBFlushVolSync err=%d", err);
		matLog(cstr);
	}
	
	return noErr;
	
} // CdaCleanup


int CdaEject(int drive)
{
	OSErr err;
	ParamBlockRec PB = {0};
	char cstr[256];

	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		sprintf(cstr, "CdaEject bad drive index=%d", drive);
		matLog(cstr);
		return 1;
	}
	
	err = CdaSetBlockSize(drive, 512);
	if(err != noErr)
	{
		sprintf(cstr, "CdaEject CdaSetBlockSize err=%d", err);
		matLog(cstr);
	}

	PB.fileParam.ioNamePtr = NULL; // specify volume by vrefnum
	PB.fileParam.ioVRefNum = g_cdadrive[drive].fsvolrefnum; // DrvNum for eject
	PBFlushVolSync((ParamBlockRec *)&PB); // ignore errors from flush

#if !TARGET_API_MAC_CARBON // eject is done by PBUnmountVol in Carbon
	PB.ioParam.ioNamePtr = NULL;
	PB.ioParam.ioVRefNum = g_cdadrive[drive].fsvolrefnum; // DrvNum for eject
	err = PBEject((ParamBlockRec *) &PB);
                
	if(err != noErr)
	{
		sprintf(cstr, "CdaEject vol=%d PBUnmountVol err=%d", g_cdadrive[drive].fsvolrefnum, err);
		matLog(cstr);
	}
#endif // !TARGET_API_MAC_CARBON

	PB.ioParam.ioNamePtr = NULL;
	PB.ioParam.ioVRefNum = g_cdadrive[drive].fsvolrefnum; // DrvNum for unmount
	err = PBUnmountVol((ParamBlockRec *) &PB);
                
	if(err != noErr)
	{
		sprintf(cstr, "CdaEject vol=%d PBUnmountVol err=%d", g_cdadrive[drive].fsvolrefnum, err);
		matLog(cstr);
	}

	return err;
	
} // CdaEject


/*
		$0000	Macintosh HFS or MFS
		$0100	ProDOS File System
		$0101	PowerTalk Mail Enclosures
		$4147	ISO 9660 File Access (through Foreign File Access)
		$4242	High Sierra File Access (through Foreign File Access)
		$464D	QuickTake File System (through Foreign File Access)
		$4953	Macintosh PC Exchange (MS-DOS)
		$4A48	Audio CD Access (through Foreign File Access)
		$4D4B	Apple Photo Access (through Foreign File Access)

/* Signatures used to differentiate between HFS and HFS Plus volumes * /
enum {
  kHFSSigWord                   = 0x4244, /* 'BD' in ASCII * /
  kHFSPlusSigWord               = 0x482B, /* 'H+' in ASCII * /
  kHFSPlusVersion               = 0x0004, /* will change as format changes (version 4 shipped with Mac OS 8.1) * /
  kHFSPlusMountVersion          = FOUR_CHAR_CODE('8.10') /* will change as implementations change ('8.10' in Mac OS 8.1) * /
};

on osX 10.3/classic 922 :
14:55:51 isaudio=0 volumeIndex=1 systemVersion=922 volumeInfo.signature=482b volumeInfo.filesystemID=0 y=4a48 'Macintosh HD'
14:55:51 isaudio=0 volumeIndex=2 systemVersion=922 volumeInfo.signature=482b volumeInfo.filesystemID=0 y=4a48 'OSX'
14:55:51 isaudio=0 volumeIndex=3 systemVersion=922 volumeInfo.signature=482b volumeInfo.filesystemID=0 y=4a48 'VirtualPC'
14:55:51 isaudio=0 volumeIndex=4 systemVersion=922 volumeInfo.signature=482b volumeInfo.filesystemID=0 y=4a48 'Macintosh HDBU'
14:55:51 isaudio=0 volumeIndex=5 systemVersion=922 volumeInfo.signature=482b volumeInfo.filesystemID=0 y=4a48 'empty'
14:55:51 isaudio=0 volumeIndex=6 systemVersion=922 volumeInfo.signature=4244 volumeInfo.filesystemID=0 y=4a48 'DEV1_250'
14:55:51 isaudio=1 volumeIndex=7 systemVersion=922 volumeInfo.signature=4244 volumeInfo.filesystemID=0 y=4a48 'Audio CD 1'
14:55:51 isaudio=1 volumeIndex=8 systemVersion=922 volumeInfo.signature=4244 volumeInfo.filesystemID=0 y=4a48 'Audio CD'

on os922 :
*/
#define kAudioCDFilesystemID ((UInt16)(('J' << 8) | 'H')) // $4A48

int CdaGetCDDrives() // updates g_cdadrive and returns number of cd drives containing audio media
{
	int numdrives = 0;
	int i;
	int isaudio;
	int y = kAudioCDFilesystemID;
	
	OSErr err;
	ItemCount volumeIndex;
	long systemVersion;
	HFSUniStr255 volumeName;
	FSVolumeInfo volumeInfo;
	FSVolumeRefNum fsvolrefnum = 0;
	FSVolumeRefNum *pfsvolrefnum = &fsvolrefnum;
	char cstr[256];
	char name[256];

	if(Gestalt(gestaltSystemVersion, &systemVersion) != noErr)
		systemVersion = 0;

	err = noErr;
	for(volumeIndex = 1; err == noErr || err != nsvErr; volumeIndex++)
	{
		memset((void *) &volumeInfo, 0, sizeof(volumeInfo));
		err = FSGetVolumeInfo(	kFSInvalidVolumeRefNum,
								volumeIndex,
								pfsvolrefnum,
								kFSVolInfoFSInfo | kFSVolInfoDriveInfo,
								&volumeInfo,
								&volumeName,
								NULL); 

		if(err == noErr)
		{
					for(i = 0; i < volumeName.length; i++)
						name[i] = volumeName.unicode[i];
					name[volumeName.length] = 0;
			/*			g_cdadrive[numdrives].name[i] = volumeName.unicode[i];
					g_cdadrive[numdrives].name[volumeName.length] = 0;
					
			if((systemVersion >= 0x00001000 && systemVersion < 0x00001010 &&
					volumeInfo.signature	== kAudioCDFilesystemID) ||
					volumeInfo.filesystemID == kAudioCDFilesystemID) // is an audio CD
			*/
			isaudio = 0;
			if(systemVersion >= 0x00001000 && systemVersion < 0x00001010 && volumeInfo.signature == y)
				isaudio = 1;
			else if(volumeInfo.filesystemID == y)
				isaudio = 1;
			/*
			else if(systemVersion == 0x0922 && volumeInfo.signature == 0x4244 && // kludge for when on osX 10.3/classic 922
							volumeInfo.filesystemID == 0 && strstr(name, "Audio"))
				isaudio = 1;

sprintf(cstr, "isaudio=%d volumeIndex=%d systemVersion=%x volumeInfo.signature=%x volumeInfo.filesystemID=%x y=%x '%s'",
	isaudio,  volumeIndex, systemVersion, volumeInfo.signature, volumeInfo.filesystemID, y, name);
matLog(cstr);
			*/
			
			if(isaudio)
			{
				numdrives++;
				if(numdrives <= CDAMAXNUMDRIVES)
				{
					g_cdadrive[numdrives].exists = 1;
					g_cdadrive[numdrives].isready = 1;
					g_cdadrive[numdrives].driverRefNum = volumeInfo.driverRefNum;
					g_cdadrive[numdrives].fsvolrefnum = fsvolrefnum;
					for(i = 0; i < volumeName.length; i++)
						g_cdadrive[numdrives].name[i] = volumeName.unicode[i];
					g_cdadrive[numdrives].name[volumeName.length] = 0;
				}
			}
		}
		else if(err != nsvErr)
		{
			sprintf(cstr, "CdaGetCDDrives FSGetVolumeInfo vol=%d err=%d", volumeIndex, err);
			matLog(cstr);
			return 0;
		}
			
	} // for volumeIndex
		
	return numdrives; // ok
		
} // CdaGetCDDrives


int CdaInitCDRom();
int CdaInitCDRom()
{
	int numready = 0;
	int whichready = 0;
	int i;
	char cstr[256];

	for(i = 1; i <= CDAMAXNUMDRIVES; i++)
	{
		g_cdadrive[i].exists = 0;
		g_cdadrive[i].isready = 0;
		g_cdadrive[i].isactive = 0;
		g_cdadrive[i].name[0] = 0;
	}
	g_cdaactive = 0;

	if((g_cdanumdrives = CdaGetCDDrives()) <= 0)
	{
		matLog("CdaInitCDRom no cdrom drive found");
		return 1;
	}

	if(g_cdanumdrives > CDAMAXNUMDRIVES)
	{
		sprintf(cstr, "CdaInitCDRom found %d cdrom drives, limited to first %d", 
													g_cdanumdrives, CDAMAXNUMDRIVES);
		matLog(cstr);
		g_cdanumdrives = CDAMAXNUMDRIVES;
	}

	for(i = 1; i <= g_cdanumdrives; i++)
	{
		if(g_cdadrive[i].exists)
			if(g_cdadrive[i].isready)
			{
				numready++;
				whichready = i;
			}
	}
		
	if(numready < 1) // || numready > 1)
	{
		//sprintf(cstr, "%d ready of %d cdroms", numready, g_cdanumdrives);
		//matLog(cstr);
		return 1;
	}

	whichready = 1;
	
	g_cdadrive[whichready].isactive = 1;
	g_cdaactive = whichready ;

	return 0;

} // CdaInitCDRom

#pragma export off

#else
 //#warning TARGET_API_MAC_CARBON
#endif // !TARGET_API_MAC_CARBON

#ifdef __cplusplus
}
#endif

#endif // MANOCDA

//eof