// maGUInone.cpp

/*
 * MoreAmp
 * Copyright (C) 2005-2008 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

// #define DEBUGGING_XCODE

#include "maConfig.h" // included by all moreamp files

#ifdef __MAWIN__
 #ifdef MAUNUSEDWIN
  #pragma warning( disable : 4100 ) // unreferenced formal parameter
 #endif
 #pragma warning( disable : 4706 ) // assignment within conditional expression
 #pragma warning( disable : 4127 ) // conditional expression is constant
#endif

// This file contains the code to run without a gui and stubs for gui functions.

#if defined(MAGUINONE)

// #define MATRACEGUINONE

#ifdef __MAOSX__
 #error __MAOSX__ is defined. NoGUI builds on OS X should be __MACUNIX__
 #error use `./configure --with-macunix --without-gui ... `
 // and __MAUNIX__ is defined whenever __MACUNIX__ is defined
#endif

#include <stdio.h>
#include <string.h>

#ifndef __MAWIN__
 #include <unistd.h>
#endif

#ifdef __MAOS9__
 #include <CarbonEvents.h>
 #include <Events.h>
#endif

#ifdef __MAUNIX__
 #include <signal.h>
#endif

#if defined(__MAWIN__) // have stdin
 #include <windows.h>
#endif

#if defined(__MAUNIX__) // have stdin
 #include <errno.h>
 #include <termios.h>
 #include <sys/time.h>
 #include <stdlib.h>
 #include <sys/stat.h>
#endif

#include "maGUI.h"
#include "maUtils.h"
#include "maPlayer.h"
#include "maSongList.h"

#ifdef __cplusplus
extern "C"
{
#endif

#if defined(__MAOS9__)
 EventLoopTimerRef l_timerrunloop;
#endif

#if defined(__MAUNIX__) || defined(__MAWIN__) 
 int l_timerrunloop = 0;
#endif

int l_timerplay = 0;
int l_timerplaysonglist = 0;
int l_appquitted = 0;
int l_prompting = 0;

#if defined MATRACEGUINONE
 #define maTrace maGuiLog
#endif

int maGetChar();
void maGuiLog(char *str);

void maGuiLog(char *str)
{
	printf("%s\n", str);
}


#if defined(__MAUNIX__)
 void onsigint(int sig);
 void onsigint(int sig)
 {
 #ifdef MAUNUSED
  (void)sig;
 #endif
	OnButtonQuit();
 }
#endif


#if defined(__MAOS9__)
 int l_skipfirst = 1;
 void OnTimerRunLoop(EventLoopTimerRef theTimer, void* userData); 
 void OnTimerRunLoop(EventLoopTimerRef theTimer, void* userData) 
 {
  #ifdef MAUNUSED
	(void)theTimer; (void)userData;
  #endif
  #ifdef MATRACEGUINONE
	maTrace("OnTimerRunLoop");
  #endif

	if(l_skipfirst) // startup problems
	{
		l_skipfirst = 0;
		return;
	}

	RunLoop(NULL);
 }
#endif


#if defined(__MAUNIX__)
 struct termios ttybu;
 int l_ttyinit = 0;
 void mattyexit(void);
 void mattyexit(void)
 {
	if(l_ttyinit)
	{
		tcsetattr(0, TCSANOW, &ttybu);
		l_ttyinit = 0;
	}
 }


 int mattyinit();
 int mattyinit()
 {
	struct termios tty;
	
	if(!isatty(0)) // piped input
		return 1;

	// if() // started in background e.g. 'moreampnogui &') // FIXME
	// {
	//		return 2; // else tcsetattr causes a program exit
	// }
	
	if(tcgetattr(0, &tty))
	{
		printf("mattyinit tcgetattr errno=%d\n", errno);
		return 3;
	}

	ttybu = tty;
	tty.c_iflag &= ~(IGNBRK | BRKINT | PARMRK | ISTRIP | INLCR | IGNCR | ICRNL | IXON);
	tty.c_oflag |= OPOST;
	tty.c_lflag &= ~(ECHO | ECHONL | ICANON | IEXTEN);
	tty.c_cflag &= ~(CSIZE | PARENB);
	tty.c_cflag |= CS8;
	tty.c_cc[VMIN] = 1;
	tty.c_cc[VTIME] = 0;
	
	if(tcsetattr(0, TCSANOW, &tty))
	{
		printf("mattyinit tcsetattr errno=%d\n", errno);
		return 4;
	}

	atexit(mattyexit);
	l_ttyinit = 1;
	
	return 0;
	
 } // mattyinit


 #if defined(DEBUGGING_XCODE)
  int maGetCharB() // when debugging from XCode stdio window
  {
	int i;
	char buf[16];
	
	printf("maGetCharB ");

	i = 0;
	
	if(fgets(buf, 14, stdin))
		i = 1;
	
	if(i > 0)
		if(buf[0] != MAWXK_RETURN)
			return buf[0];
	
	return 0;
	
  } // maGetCharB
 #endif


 int maGetChar() // async stdin read
 {
 #if defined(DEBUGGING_XCODE)
	return maGetCharB(); // when debugging from XCode stdio window
 #endif
	unsigned char c[16] = {0};
	struct timeval tv;
	fd_set rfds;
	int i;

	if(!l_ttyinit)
	{
		mattyinit();
		if(!l_ttyinit)
			return -1;
	}
	
	FD_ZERO(&rfds);
	FD_SET(0, &rfds);
	tv.tv_sec = 0;
	tv.tv_usec = 0;
	i = 0;
	
	while(select(1, &rfds, NULL, NULL, &tv) > 0)
	{
		tv.tv_sec = 0;
		tv.tv_usec = 0;
		if(read(0, &c[i++], 1) == 1)
		{
			if(i >= 16)
				break;
		}
	}

	if(i == 1)
	{
		if(c[0] == 13)
			return MAWXK_RETURN;
		else
			return c[0];
	}
			
	/* if(i == 3) // up/down arrow keys
	{
		if(c[0] == 0x1b) // Esc
		{
			if(c[1] == 0x5b) // [
			{
				if(c[2] == 0x41)
					return MAWXK_UP;
				else if(c[2] == 0x42) 
					return MAWXK_DOWN;
			}
		}
	} */
	
	return -1;

 } // maGetChar
#endif


#if defined(__MAWIN__) // have stdin
 int maGetChar()
 {
	HANDLE console;
	INPUT_RECORD input;
	DWORD count;

	console = GetStdHandle(STD_INPUT_HANDLE);

	do
	{
		if(GetNumberOfConsoleInputEvents(console, &count) == 0) // broken in wine?
		{
			printf("maGetChar GetNumberOfConsoleInputEvents error\n");
			return -1;
		}
		count = 0;
		GetNumberOfConsoleInputEvents(console, &count);
		
		if(count == 0)
			return 0;

		if(ReadConsoleInput(console, &input, 1, &count) == 0 || count != 1)
		{
			printf("maGetChar ReadConsoleInput error\n");
			return -1;
		}
	} while(input.EventType != KEY_EVENT || !input.Event.KeyEvent.bKeyDown ||
			input.Event.KeyEvent.uChar.AsciiChar == 0);

	return (int)input.Event.KeyEvent.uChar.AsciiChar;

 } // maGetChar
#endif


/*
// prompt stdin for a number > 0
int maPromptNumber(char *msg)
{
	int i;
	int val;
	char buf[MAPROMPTMAX + 2];
	
	printf("%s\n", msg);

	l_prompting = 1; // block calls to maGetChar from play loop
#if defined(MAKEYBOARD)
	g_maOnKeySkip = 1;
#endif


#if defined(__MAUNIX__)
	mattyexit();     // disable tty, reenable stdin
					 // then first call to maGetChar calls mattyinit	
#endif

	i = 0;
	
	if(fgets(buf, MAPROMPTMAX, stdin))
		i = 1;
	
	val = 0;
	if(i > 0 && !string2int(buf, &val))
	{
		l_prompting = 0;
#if defined(MAKEYBOARD)
		g_maOnKeySkip = 0;
#endif
		return val;
	}
	
	l_prompting = 0; // unblock calls to maGetChar from play loop
#if defined(MAKEYBOARD)
	g_maOnKeySkip = 0;
#endif

	return 0;
	
} // maPromptNumber
*/


int main(int argc, char *argv[]) // MAGUINONE
{
	g_argc = argc;
	g_argv = argv;
#if defined(__MAWIN__) // have stdin
	int c;
#endif

#if defined(MAINSTANCES)
	if(argc == 2 && !strcmp(argv[1], "maclearlocks")) // run this in case we have a stuck lock
	{
		InstanceClearAll();
		printf("cleared locks\n");
		return 0;
	}

	if(argc == 2 && !strcmp(argv[1], "malistlocks")) // run this to see locks
	{
		sprintf(g_logpath, "malistlocks.txt");
		InstanceListAll();
		printf("listed locks\n");
		return 0;
	}
#endif

#if defined(__MAUNIX__) // have stdin
 #if defined(MAKEYBOARD)
	int c;
 #endif
	// mattyinit();
#endif

#if defined(__MAOS9__)
	// runLoop timer
	EventLoopRef mainLoop;
	EventLoopTimerUPP timerUPP;
	mainLoop = GetMainEventLoop();
	timerUPP = 
 #if defined(__MAOS9__)
	NewEventLoopTimerUPP((pascal void (*)(OpaqueEventLoopTimerRef *, void *))OnTimerRunLoop);
	InstallEventLoopTimer(mainLoop, 2 * kEventDurationSecond,
		(0x7FFFFFFF * (double)60.0), // 0, max_Int32*60 ticks per second, non-zero makes it work
											timerUPP, NULL, &l_timerrunloop);
 #else
	NewEventLoopTimerUPP(OnTimerRunLoop);
	InstallEventLoopTimer(mainLoop, 2 * kEventDurationSecond, 0, timerUPP, NULL, &l_timerrunloop);
 #endif
	// for drop and quit
	InitAppleEvents(NULL);
#endif

#if defined(__MAUNIX__)
	struct sigaction sa;
	sa.sa_handler = onsigint;
	sigemptyset(&sa.sa_mask);
	sa.sa_flags = 0;
	sigaction(SIGINT, &sa, 0); // SIGKILL
#endif
	
	strcpy(&g_AboutText[strlen(g_AboutText) - 64], " No GUI"); 

	if(!OnInit_())		
		return 1; // failed

	// main loop
#if defined(__MAOS9__)
	RunApplicationEventLoop();
#endif

#if defined(__MAUNIX__) || defined(__MAWIN__)
	if(g_argc <= 1) // no args
	{
		if(l_timerplay)
		{
			l_timerplay = 0;
			OnButtonPlay();
		}
	}
	else if(g_argc > 1) // args
	{
		if(l_timerplay)
		{
			l_timerplay = 0;
			OnButtonPlay();
		}
		else if(l_timerplaysonglist)
		{
			l_timerplaysonglist = 0;
			PlaySonglistB(g_newsonglist);
		}	
	}

	while(1)
	{
		if(l_timerrunloop)
		{
 #if defined(MAKEYBOARD)
			if(!l_prompting && (c = maGetChar()) > 0)
				maOnKey(0, c);
			maSleep(l_timerrunloop);
 #endif
			if(l_timerplay)
			{
				l_timerplay = 0;
				OnButtonPlay();
			}
			l_timerrunloop = RunLoop(NULL);
		}
		else
		{
			if(l_appquitted)
				break;
 #if defined(MAKEYBOARD)
			if(!l_prompting && (c = maGetChar()) > 0)
				maOnKey(0, c);
 #endif
			if(l_timerplay)
			{
				l_timerplay = 0;
				OnButtonPlay();
			}
			maSleep(1000);
		}
	}
#endif // defined(__MAUNIX__) || defined(__MAWIN__)
	
#if defined(__MAOS9__)
 // do something to really quit
 g_sets.keyboard = 0;
 printf("Now Quit (File Menu) then Don't Save.\n");
#endif

	return 0;
	
} // main


void StaticTextSetText(int window, int statictext, char *text)
{
#ifdef MAUNUSED
 (void)window; (void)statictext;
#endif
#ifdef MATRACEGUINONE
	maTrace("StaticTextSetText");
#endif

	if(window == WINDOWPROMPTNUM)
	{
		printf("%s\n", text);
		fflush(stdout);
	}
}


int	AppWindowMake()
{
#ifdef MATRACEGUINONE
	maTrace("AppWindowMake");
#endif
	return 0;	
} // AppWindowMake


void guiAppYield() 
{
#ifdef MATRACEGUINONE
	maTrace("guiAppYield");
#endif
#if defined(__MAOS9__)	
	EventMask eventMask = highLevelEventMask;
	EventRecord event; 
	UInt32 ticks = 3; // 60 == 1 second
	if(WaitNextEvent(eventMask, &event, ticks, NULL))
		OnButtonQuit();
#endif
} // guiAppYield


void CheckBoxDisable(int window, int checkbox)
{
#ifdef MAUNUSED
 (void)window; (void)checkbox;
#endif
#ifdef MATRACEGUINONE
	maTrace("CheckBoxDisable");
#endif
}

void CheckBoxEnable(int window, int checkbox)
{
#ifdef MAUNUSED
 (void)window; (void)checkbox;
#endif
#ifdef MATRACEGUINONE
	maTrace("CheckBoxEnable");
#endif
}


int CheckBoxGetValue(int window, int checkbox)
{
#ifdef MAUNUSED
 (void)window; (void)checkbox;
#endif
#ifdef MATRACEGUINONE
	maTrace("CheckBoxGetValue");
#endif
return 0;
}


void CheckBoxHide(int window, int checkbox)
{
#ifdef MAUNUSED
 (void)window; (void)checkbox;
#endif
#ifdef MATRACEGUINONE
	maTrace("CheckBoxHide");
#endif
}


void CheckBoxSetDescriptor(int window, int checkbox, char *desc)
{
#ifdef MAUNUSED
 (void)window; (void)checkbox; (void)desc;
#endif
#ifdef MATRACEGUINONE
	maTrace("CheckBoxSetDescriptor");
#endif
}


void MenuSetLabel(int item, char *label)
{
#ifdef MAUNUSED
 (void)item; (void)label;
#endif
#ifdef MATRACEGUINONE
	maTrace("MenuSetLabel");
#endif
}


void MenuGetLabel(int item, char *label)
{
#ifdef MAUNUSED
 (void)item; (void)label;
#endif
#ifdef MATRACEGUINONE
	maTrace("MenuGetLabel");
#endif
}


void UpdateSubMenusRecent()
{
#ifdef MATRACEGUINONE
	maTrace("UpdateSubMenusRecent");
#endif
}


void CheckBoxSetValue(int window, int checkbox, int value)
{
#ifdef MAUNUSED
 (void)window; (void)checkbox; (void)value;
#endif
#ifdef MATRACEGUINONE
	maTrace("CheckBoxSetValue");
#endif
}


void CheckBoxShow(int window, int checkbox)
{
#ifdef MAUNUSED
 (void)window; (void)checkbox;
#endif
#ifdef MATRACEGUINONE
	maTrace("CheckBoxShow");
#endif
}


void DisplayGetSize(int *x, int *y)
{
#ifdef MAUNUSED
 (void)x; (void)y;
#endif
#ifdef MATRACEGUINONE
	maTrace("DisplayGetSize");
#endif
}


void DoTimerRunLoop(int delay)
{
#ifdef MATRACEGUINONE
	maTrace("DoTimerRunLoop");
#endif

#if defined(__MAOS9__)
	SetEventLoopTimerNextFireTime(l_timerrunloop, delay * kEventDurationMillisecond);
#endif

#if defined(__MAUNIX__)  
	maSleep(delay);
	l_timerrunloop = delay;
#endif

#if defined(__MAWIN__)  
	maSleep(delay);
	l_timerrunloop = delay;
#endif
}


void DoTimerPlay(int delay)
{
#ifdef MATRACEGUINONE
	maTrace("DoTimerPlay");
#endif
	maSleep(delay);
	l_timerplay = 1;
}


void DoTimerStop(int delay)
{
#ifdef MATRACEGUINONE
	maTrace("DoTimerStop");
#endif
	maSleep(delay);
	OnButtonStop();
}


void DoTimerPlaySonglist(int delay)
{
#ifdef MATRACEGUINONE
	maTrace("DoTimerPlaySonglist");
#endif
	maSleep(delay);
	l_timerplaysonglist = 1;
}


void EditTextSetText(int window, int edittext, char *text)
{
#ifdef MAUNUSED
 (void)window; (void)edittext; (void)text;
#endif
#ifdef MATRACEGUINONE
	maTrace("EditTextSetText");
#endif
}


void EditTextGetText(int window, int edittext, char *outBuffer, int inBufferLength, long *outTextLength)
{
#ifdef MAUNUSED
 (void)window; (void)edittext; (void)inBufferLength; (void)outTextLength;
#endif
#ifdef MATRACEGUINONE
	maTrace("EditTextGetText");
#endif

	if(window == WINDOWPROMPTNUM)
	{
		char buf[256];
	
#if defined(__MAUNIX__)
		mattyexit(); // disable tty, reenable stdin
					 // then first call to maGetChar calls mattyinit	
#endif

		if(fgets(buf, MAPROMPTMAX, stdin))
			strcpy(outBuffer, buf);
		else
			outBuffer[0] = 0;
	}
	
} // EditTextGetText


void EqualizerWindowMake()
{
#ifdef MATRACEGUINONE
	maTrace("EqualizerWindowMake");
#endif
} // EqualizerWindowMake


void Equalizer2WindowMake()
{
#ifdef MATRACEGUINONE
	maTrace("Equalizer2WindowMake");
#endif
} // EqualizerWindowMake


void EffectsWindowMake()
{
#ifdef MATRACEGUINONE
	maTrace("EffectsWindowMake");
#endif
} // EffectsWindowMake


#if !defined(MANOCDA) && defined(__MAUNIX__)
void AudioCDWindowMake()
{
#ifdef MATRACEGUINONE
	maTrace("AudioCDWindowMake");
#endif
} // AudioCDWindowMake
#endif


void AdvancedWindowMake()
{
#ifdef MATRACEGUINONE
	maTrace("AdvancedWindowMake");
#endif
} // AdvancedWindowMake


void FileOutWindowMake()
{
#ifdef MATRACEGUINONE
	maTrace("FileOutWindowMake");
#endif
} // FileOutWindowMake


void PromptNumWindowMake()
{
#ifdef MATRACEGUINONE
	maTrace("PromptNumWindowMake");
#endif

	g_w[WINDOWPROMPTNUM] = (void *)-1; // dummy non-null
	
} // PromptNumWindowMake


void ListAppend(int window, int list, char *text)
{
#ifdef MAUNUSED
 (void)window; (void)list; (void)text;
#endif
#ifdef MATRACEGUINONE
	maTrace("ListAppend");
#endif
	printf("%s\n", text);
	
} // ListAppend


void ListClear(int window, int list)
{
#ifdef MAUNUSED
 (void)window; (void)list;
#endif
#ifdef MATRACEGUINONE
	maTrace("ListClear");
#endif
}


int ListGetRowSelected(int window, int list)
{
#ifdef MAUNUSED
 (void)window; (void)list;
#endif
#ifdef MATRACEGUINONE
	maTrace("ListGetRowSelected");
#endif
	return -1; // none selected
}


void ListEnsureVisible(int window, int list, int row)
{
#ifdef MAUNUSED
 (void)window; (void)list; (void)row;
#endif
#ifdef MATRACEGUINONE
	maTrace("ListEnsureVisible");
#endif
}


void ListSelect(int window, int list, int row)
{
#ifdef MAUNUSED
 (void)window; (void)list; (void)row;
#endif
#ifdef MATRACEGUINONE
	maTrace("ListSelect");
#endif
}


void ListGetText(int window, int list, int row, char *text)
{
#ifdef MAUNUSED
 (void)window; (void)list; (void)row; (void)text;
#endif
#ifdef MATRACEGUINONE
	maTrace("ListGetText");
#endif
}


/* not used void ListSetText(int window, int list, int row, char *text)
{
#ifdef MAUNUSED
 (void)window; (void)list; (void)row; (void)text;
#endif
#ifdef MATRACEGUINONE
	maTrace("ListSetText");
#endif
}
*/


void ListSetHeader(int window, int list, int col, char *header)
{
#ifdef MAUNUSED
 (void)window; (void)list; (void)col; (void)header;
#endif
#ifdef MATRACEGUINONE
	maTrace("ListSetHeader");
#endif
}


void ListSetColumnWidth()
{
#ifdef MATRACEGUINONE
	maTrace("ListSetColumnWidth");
#endif
}


void ListWindowMake()
{
#ifdef MATRACEGUINONE
	maTrace("ListWindowMake");
#endif
} // ListWindowMake


int MenuIsChecked(int menu, int item)
{
#ifdef MAUNUSED
 (void)menu; (void)item;
#endif
#ifdef MATRACEGUINONE
	maTrace("MenuIsChecked");
#endif
	return 0;
}


void MenuEnable(int menu, int item)
{
#ifdef MAUNUSED
 (void)menu; (void)item;
#endif
#ifdef MATRACEGUINONE
	maTrace("MenuEnable");
#endif
}


void MenuDisable(int menu, int item)
{
#ifdef MAUNUSED
 (void)menu; (void)item;
#endif
#ifdef MATRACEGUINONE
	maTrace("MenuDisable");
#endif
}


void MenuSetChecked(int menu, int item, int value)
{
#ifdef MAUNUSED
 (void)menu; (void)item; (void)value;
#endif
#ifdef MATRACEGUINONE
	maTrace("MenuSetChecked");
#endif
}


void PopupButtonEnable(int window, int popup)
{
#ifdef MAUNUSED
 (void)window; (void)popup;
#endif
#ifdef MATRACEGUINONE
	maTrace("PopupButtonEnable");
#endif
}


void PopupButtonDisable(int window, int popup)
{
#ifdef MAUNUSED
 (void)window; (void)popup;
#endif
#ifdef MATRACEGUINONE
	maTrace("PopupButtonDisable");
#endif
}


void PopupButtonClear(int window, int popup)
{
#ifdef MAUNUSED
 (void)window; (void)popup;
#endif
#ifdef MATRACEGUINONE
	maTrace("PopupButtonClear");
#endif
}


void PopupButtonAppend(int window, int popup, char *text)
{
#ifdef MAUNUSED
 (void)window; (void)popup; (void)text;
#endif
#ifdef MATRACEGUINONE
	maTrace("PopupButtonAppend");
#endif
}


void PopupButtonSetValue(int window, int popup, int value)
{
#ifdef MAUNUSED
 (void)window; (void)popup; (void)value;
#endif
#ifdef MATRACEGUINONE
	maTrace("PopupButtonSetValue");
#endif
}


#if !defined(MANOCDA) && defined(__MAUNIX__)
void PopupButtonGetSelection(int window, int popup, int *item)
{
#ifdef MAUNUSED
 (void)window; (void)popup; (void)item;
#endif
#ifdef MATRACEGUINONE
	maTrace("PopupButtonGetSelection");
#endif
}


void PopupButtonSetSelection(int window, int popup, int item)
{
#ifdef MAUNUSED
 (void)window; (void)popup; (void)item;
#endif
#ifdef MATRACEGUINONE
	maTrace("PopupButtonSetSelection");
#endif
}
#endif


/* not used void SpinButtonEnable(int window, int button)
{
#ifdef MAUNUSED
 (void)window; (void)button;
#endif
#ifdef MATRACEGUINONE
	maTrace("SpinButtonEnable");
#endif
}


void SpinButtonDisable(int window, int button)
{
#ifdef MAUNUSED
 (void)window; (void)button;
#endif
#ifdef MATRACEGUINONE
	maTrace("SpinButtonDisable");
#endif
}
*/


void PushButtonPaintUp(int window, int button)
{
#ifdef MAUNUSED
 (void)window; (void)button;
#endif
#ifdef MATRACEGUINONE
	maTrace("PushButtonPaintUp");
#endif
}


void PushButtonPaintDown(int window, int button)
{
#ifdef MAUNUSED
 (void)window; (void)button;
#endif
#ifdef MATRACEGUINONE
	maTrace("PushButtonPaintDown");
#endif
}


void PushButtonEnable(int window, int button)
{
#ifdef MAUNUSED
 (void)window; (void)button;
#endif
#ifdef MATRACEGUINONE
	maTrace("PushButtonEnable");
#endif
}


void PushButtonDisable(int window, int button)
{
#ifdef MAUNUSED
 (void)window; (void)button;
#endif
#ifdef MATRACEGUINONE
	maTrace("PushButtonDisable");
#endif
}


void PushButtonSetDescriptor(int window, int button, char *desc)
{
#ifdef MAUNUSED
 (void)window; (void)button; (void)desc;
#endif
#ifdef MATRACEGUINONE
	maTrace("PushButtonSetDescriptor");
#endif
}


int HSliderGetValue(int window, int slider)
{
#ifdef MAUNUSED
 (void)window; (void)slider;
#endif
#ifdef MATRACEGUINONE
	maTrace("HSliderGetValue");
#endif
	return 0;
}


void HSliderSetValue(int window, int slider, int value)
{
#ifdef MAUNUSED
 (void)window; (void)slider; (void)value;
#endif
#ifdef MATRACEGUINONE
	maTrace("HSliderSetValue");
#endif
}


int SliderGetValue(int window, int slider)
{
#ifdef MAUNUSED
 (void)window; (void)slider;
#endif
#ifdef MATRACEGUINONE
	maTrace("SliderGetValue");
#endif
	return 0;
}


void SliderSetValue(int window, int slider, int value)
{
#ifdef MAUNUSED
 (void)window; (void)slider; (void)value;
#endif
#ifdef MATRACEGUINONE
	maTrace("SliderSetValue");
#endif
}


void SliderEnable(int window, int slider)
{
#ifdef MAUNUSED
 (void)window; (void)slider;
#endif
#ifdef MATRACEGUINONE
	maTrace("SliderEnable");
#endif
}


void SliderDisable(int window, int slider)
{
#ifdef MAUNUSED
 (void)window; (void)slider;
#endif
#ifdef MATRACEGUINONE
	maTrace("SliderDisable");
#endif
}


void StaticTextSetValue(int window, int statictext, int value)
{
#ifdef MAUNUSED
 (void)window; (void)statictext; (void)value;
#endif
#ifdef MATRACEGUINONE
	maTrace("StaticTextSetValue");
#endif
}


void WindowDoClose(int window)
{
#ifdef MAUNUSED
 (void)window;
#endif
#ifdef MATRACEGUINONE
	maTrace("WindowDoClose");
#endif

	if(window == WINDOWPROMPTNUM)
		OnWindowClosedPromptNum();
}


void WindowDoDestroy(int window)
{
#ifdef MAUNUSED
 (void)window;
#endif
#ifdef MATRACEGUINONE
	maTrace("WindowDoDestroy");
#endif
}


void WindowGetCoordinates(int window, int *xpos, int *ypos, int *xsize, int *ysize)
{
#ifdef MAUNUSED
 (void)window; (void)xpos; (void)ypos; (void)xsize; (void)ysize;
#endif
#ifdef MATRACEGUINONE
	maTrace("WindowGetCoordinates");
#endif
} // WindowGetCoordinates


void WindowItemHide(int window, int item)
{
#ifdef MAUNUSED
 (void)window; (void)item;
#endif
#ifdef MATRACEGUINONE
	maTrace("WindowItemHide");
#endif
} // WindowItemHide


void WindowItemShow(int window, int item)
{
#ifdef MAUNUSED
 (void)window; (void)item;
#endif
#ifdef MATRACEGUINONE
	maTrace("WindowItemShow");
#endif
} // WindowItemShow


int WindowIsShown(int window)
{
#ifdef MAUNUSED
 (void)window;
#endif
#ifdef MATRACEGUINONE
	maTrace("WindowIsShown");
#endif
	return 1;
}


void WindowSetSize(int window, int xsize, int ysize)
{
#ifdef MAUNUSED
 (void)window; (void)xsize; (void)ysize;
#endif
#ifdef MATRACEGUINONE
	maTrace("WindowSetSize");
#endif
}


void WindowMove(int window, int xpos, int ypos)
{
#ifdef MAUNUSED
 (void)window; (void)xpos; (void)ypos;
#endif
#ifdef MATRACEGUINONE
	maTrace("WindowMove");
#endif
}


void WindowShow(int window)
{
#ifdef MAUNUSED
 (void)window;
#endif
#ifdef MATRACEGUINONE
	maTrace("WindowShow");
#endif
}


void WindowSetTitle(int window, char *title)
{
#ifdef MAUNUSED
 (void)window;
#endif
#ifdef MATRACEGUINONE
	char cstr[256];
	sprintf(cstr, "maGUI WindowSetTitle %d '%s'", window, title);
	maTrace(cstr);
#endif
	printf("%s\n", title);
}


int maDirSelector(char *dirpath, char *prompt)
{
	char cstr[512];
#ifdef MATRACEGUINONE
	maTrace("maDirSelector");
#endif
	sprintf(cstr, "maGUI maDirSelector '%s' '%s'", prompt, dirpath);
	maGuiLog(cstr);
	return 1; // cancelled
}


int maFileDialog(char *selectedpath, int window, char *prompt, char *defdir, char *deffile, char *wildcard, int flags)
{
#ifdef MAUNUSED
 (void)selectedpath; (void)window; (void)deffile; (void)wildcard; (void)flags;
#endif
	char cstr[512];
#ifdef MATRACEGUINONE
	maTrace("maFileDialog");
#endif
	sprintf(cstr, "maGUI maFileDialog '%s' '%s'", prompt, defdir);
	maGuiLog(cstr);
	return 1; // cancelled
}


void maFileDialogMFree(maArray *parray)
{
#ifdef MAUNUSED
 (void)parray;
#endif
#ifdef MATRACEGUINONE
	maTrace("maFileDialogMFree");
#endif
}


int maFileDialogM(maArray *parray, int window, char *prompt, char *defdir, char *deffile, char *wildcard, int flags)
{
#ifdef MAUNUSED
 (void)parray; (void)window; (void)prompt; (void)defdir; (void)deffile; (void)wildcard; (void)flags;
#endif
#ifdef MATRACEGUINONE
	maTrace("maFileDialogM");
#endif
	return 1; // cancelled
	
} //maFileDialogM


int maMessageBox(char *s1, char *s2, int flags)
{
#ifdef MAUNUSED
 (void)s1; (void)s2; (void)flags;
#endif
	char cstr[512];
#ifdef MATRACEGUINONE
	sprintf(cstr, "maGUI maMessageBox '%s' '%s'", s1, s2);
	maTrace(cstr);
#endif
	sprintf(cstr, "maGUI maMessageBox '%s' '%s'", s1, s2);
	maGuiLog(cstr);
	return maNO;
} // maMessageBox


void maViewCatalog()
{
#ifdef MATRACEGUINONE
	maTrace("maViewCatalog");
#endif
} // maViewCatalog


void maViewLog()
{
#ifdef MATRACEGUINONE
	maTrace("maViewLog");
#endif
} // maViewLog


#if defined(MAKEYBOARD)
 void maEditKeyBindings()
 {
 #ifdef MATRACEGUINONE
	maTrace("maEditKeyBindings");
 #endif

	FILE *fp = NULL;
	char cstr[MAX_PATH + 256];

 	if(g_keybindingspath[0] && maFileExists(g_keybindingspath))
	{
		if((fp = mafopen(g_keybindingspath, "rb")) == NULL)
		{
			sprintf(cstr, "maEditKeyBindings mafopen failed on '%s'", g_keybindingspath);
			maLog(cstr);
			printf("%s\n", cstr);
		}
		else
		{
			while(fgets(cstr, 254, fp))
				if(strchr((const char *)cstr, '"'))
					printf(cstr);
			fclose(fp);
			fflush(stdout);
		}
	}

 } // maEditKeyBindings
#endif

void maViewReadme()
{
#ifdef MATRACEGUINONE
	maTrace("maViewReadme");
#endif
} // maViewReadme


void maViewAbout()
{
#ifdef MATRACEGUINONE
	maTrace("maViewAbout");
#endif
} // maViewAbout


void maGUIfree()
{
}


void maGUIQuit()
{
 #if defined(__MAOS9__)
	QuitApplicationEventLoop();
 #endif
	l_appquitted = 1;

} // maGUIQuit


int maModKeyDown()
{
	return 0;
}

#ifdef __cplusplus
}
#endif

#endif // MAGUINONE

// eof

