// maPlayCda.c

/*
 * MoreAmp
 * Copyright (C) 2004-2008 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "maConfig.h" // included by all moreamp files

#ifdef __MAWIN__
 #ifdef MAUNUSEDWIN
  #pragma warning( disable : 4100 ) // unreferenced formal parameter
 #endif
 #pragma warning( disable : 4127 ) // conditional expression is constant
 #pragma warning( disable : 4201 ) // nonstandard extension used : nameless struct/union
 #pragma warning( disable : 4244 ) // conversion from 'int ' to 'char ', possible loss of data
 #pragma warning( disable : 4706 ) // assignment within conditional expression
 #define __RPCASYNC_H__ // rpcasync.h(45) : warning C4115: '_RPC_ASYNC_STATE' : named type definition in parentheses
#endif

#ifndef MANOCDA

#ifdef __MAWIN__
 int CdaCloseRipper();
#endif

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#ifdef __MAWIN__
 #include <stdlib.h>
 #include <limits.h>
 #include <time.h>
 #define noErr 0
 #define paramErr -17
 int CdaOpenRipper();
#endif

#ifdef __MAOS9__
 #include "MacErrors.h"
 #include "Gestalt.h"
#endif

#ifdef __MAOSX__
 #include "Gestalt.h"
#endif

#ifdef __MAUNIX__
 #define noErr 0
 #define paramErr -17 // ?
 typedef int FSVolumeRefNum; 
#endif

#ifdef __MAWIN__
 typedef int FSVolumeRefNum;
 typedef int SInt16;
#endif

#include "maCda.h"
#include "maPlayCda.h"

#include "maPlayer.h"
#include "maUtils.h"
#include "maEqualizer.h"
#include "maMemFile.h"

#ifdef __MAWIN__
 #include "maAudioWin.h"
#endif

#ifdef __MAOS9__
 #include "maAudioMac.h"
#endif

#ifdef __MAOSX__
 #include "maAudioMac.h"
#endif

#ifdef __MAUNIX__
 #include "maAudioUnix.h"
#endif
extern char *g_cdabuf;

int BCDtoHex(char SrcByte);
char HextoBCD(int SrcInt);
void lba_to_msf(long lba, unsigned char *M, unsigned char *S, unsigned char *F);
int msf_to_frames(int minutes, int seconds, int frames);
int msf_to_lba(int minutes, int seconds, int frames);
void maPlayCda();
int CdaDecodeStream();
int CdaDoOpen(char *playpath);
int CdaOpenFile(char *Filename);
void CdaCloseFile(int drive);
int CdaGetAudioBlocks(int drive, void *pBlock, int *pdwBlockSize);


int BCDtoHex(char SrcByte)
{
	int x, y;

    x = SrcByte % 16;
    y = SrcByte / 16;
    y = y * 10;
    return x + y;
}


char HextoBCD(int SrcInt)
{
	int x, y;
	char z;

	z = 0;
	x = SrcInt % 256;
	y = x;
	if(y >= 10)
	{
        z = z + 16;
        do
		{
			y = y - 10;
            if(y >= 10)
				z = z + 16;
        } while(y >= 10);
	}
	
    return y + z;

} // HextoBCD


void lba_to_msf(long lba, unsigned char *M, unsigned char *S, unsigned char *F)
{
  if(lba >= -150)
  {
    *M = (lba + 150) / (60 * 75);
    lba -= (*M) * 60 * 75;
    *S = (lba + 150) / 75;
    lba -= (*S) * 75;
    *F = (lba + 150);
  }
  else
  {
    *M = (lba + 450150) / (60 * 75);
    lba -= (*M) * 60 * 75;
    *S = (lba + 450150) / 75;
    lba -= (*S) * 75;
    *F = (lba + 450150);
  }
  
} // lba_to_msf


int msf_to_frames(int minutes, int seconds, int frames)
{
  return minutes * 4500 + seconds * 75 + frames;
}


int msf_to_lba(int minutes, int seconds, int frames)
{
  if(msf_to_frames(minutes, seconds, frames) > 150)
    return msf_to_frames(minutes, seconds, frames) - 150;
   
  return 0;
}


// our implementations of win and osx cd readers return data at
// the beginning of a track that we don't want to hear. This may
// or may not be a bug, but for now just apply a temp fix

#if defined(__MAWIN__)
 // on win silence the first 8820 frames (about 200 ms)
 #define SILENTFRAMES 8820
#elif defined(__MAOSX__)
 // on osx silence first 128 frames (about 2.9 ms)
 #define SILENTFRAMES 128
#else
 #define SILENTFRAMES 0
#endif

#define SILENTBYTES (SILENTFRAMES * 4)

int g_silentbytes = 0;
int g_cdaseek = -1;
void cdaseek(int pos)
{
	if(g_cdaseek >= 0 || pos < 0)
		return;
		
	g_cdaseek = pos;
	
} // cdaseek


void maPlayCda()
{
	char cstr[256];

	if(!g_state.inpath)
	{
		maLog("maPlayCda error null filename");
		return;
	}
	
	// format of this decoder output
#if defined(WORDS_BIGENDIAN) && !defined(__MAOSX__) && !defined(__MAOS9__)
	g_state.inwordsbigendian = 1; // bigendian ppc, e.g. linux on mac, but not osx nor os9...
#else
	g_state.inwordsbigendian = 0;
#endif
	g_state.ininterleaved = 1;
	g_state.inchannels = 2;

	// open input
	if(CdaOpenFile(g_state.inpath))
	{
		sprintf(cstr, "maPlayCda CdaOpenFile failed on '%s'", g_state.inpath);
		maLog(cstr);
		return;
	}
	
 	g_state.insamplerate = MAOUTRATE;
	
	if(maOutputOpen() < 0)
	{
		CdaCloseFile(g_cdaactive);
		maLog("maPlayCda maOutputOpen failed");
		return;
	}

	if(!g_posendon)
		g_posend = g_state.duration;

	// cd loop
	while(1)
	{	
		ThreadYield();
		
		if(g_state.stop)
			break;
				
		if(g_cdaseek >= 0)
		{
			g_cdadrive[g_cdaactive].lba = LROUND(g_cdadrive[g_cdaactive].lbafirst +
								(((double)g_cdaseek * CDAFRAMESPERSECL) / MSPERSEC));
			g_cdaseek = -1;
		}

		if(CdaDecodeStream())
			break;
	
		// update position indicator
		g_state.position = LROUND((((double)g_cdadrive[g_cdaactive].lba -
						g_cdadrive[g_cdaactive].lbafirst) / CDAFRAMESPERSECL) * MSPERSEC);

	} // while 1

	// finish

	FlushFifo();

	CdaCloseFile(g_cdaactive);
	
	g_state.position = 0;
	g_cdaseek = -1;
	
} // maPlayCda


int CdaDecodeStream()
{
	int alldone;
	long pcmblocksize;
	int err;
	int drive;
	char cstr[256];

	if(!g_cdabuf)
	{
		maLog("CdaDecodeStream null g_pcdadrive or buf");
		return 1;
	}

	// Get block from CD
	alldone = 0;
	drive = g_cdaactive;

	if((err = CdaReadTrack(drive, &pcmblocksize)) != noErr)
	{	
		// workaround, ignore this error close to end of last track
		if(err == paramErr && g_cdadrive[drive].track == g_cdadrive[drive].tracklast &&
							   g_cdadrive[drive].lbalast - g_cdadrive[drive].lba < 200) 
		{
			pcmblocksize = 0;
			alldone = 1; // all done
		}
		else
		{
			sprintf(cstr,
				"CdaGetAudioBlocks CdaReadTrack err=%d track=%d tracklast=%d lba=%d numblocks=%d lbalast=%d",
				err, g_cdadrive[drive].track, g_cdadrive[drive].tracklast, g_cdadrive[drive].lba,
														g_cdabufblocks, g_cdadrive[drive].lbalast);
			maLog(cstr);
			pcmblocksize = 0;
			return 1; // err
		}
	}
		
	if(g_cdadrive[drive].lba == g_cdadrive[drive].lbafirst)
		g_silentbytes = SILENTBYTES;

	if(g_silentbytes > 0)
	{
		if(g_silentbytes <= pcmblocksize)
		{
			memset((unsigned char*)g_cdabuf, 0, g_silentbytes);
			g_silentbytes = 0;
		}
		else
		{
			memset((unsigned char*)g_cdabuf, 0, pcmblocksize);
			g_silentbytes -= pcmblocksize;
		}
	}
	
	if((g_cdadrive[drive].lba += g_cdabufblocks) >= g_cdadrive[drive].lbalast)
		alldone = 1;

	if(pcmblocksize > 0)
	{		
		// Write to fifo
		if(maOutputWrite((char *)g_cdabuf, pcmblocksize, &g_state) != 0)
		{
			sprintf(cstr, "CdaDecodeStream maOutputWrite failed : len=%ld g_state.stop=x%x g_fifoready=%d",
												pcmblocksize, g_state.stop, g_fifoready);
			maLog(cstr);
			return 1;
		}

	} //if(pcmblocksize > 0)

	if(!alldone)
		return 0;

	// song is finished
	if(g_reptsong)
	{
		g_cdadrive[g_cdaactive].lba = g_cdadrive[g_cdaactive].lbafirst;
		return 0;
	}
	
	return 1;

} // CdaDecodeStream


int CdaDoOpen(char *playpath)
{	
#ifndef __MAOSX__
	char trkfirstBCD, trklastBCD;
#endif
	char lplaypath[MAX_PATH];
	char cstr[256];
	
	if(!playpath || !playpath[0])
	{
		maLog("CdaDoOpen null playpath");
		return 1;
	}
	
#ifdef __MAWIN__
	if(CdaSetActiveDriveLetter((int)playpath[0]))
	{
		sprintf(cstr, "CdaDoOpen CdaSetActiveDriveLetter failed '%s'", playpath);
		maLog(cstr);
		g_cdaactive = 0;
		return 1;
	}
#endif

#if !defined(__MAWIN__)
	if(!(g_cdaactive = IsAudioCd(playpath)))
	{
		sprintf(cstr, "CdaDoOpen IsAudioCd '%s' not found", playpath);
		maLog(cstr);
		return 1;
	}
#endif
	
	// on imac internal cd system hangs after eject and reinsert cd if not reset to 512
	if(CdaSetBlockSize(g_cdaactive, CDABLOCKSIZE))
	{
		sprintf(cstr, "CdaDoOpen CdaSetBlockSize failed '%s'", playpath);
		maLog(cstr);
		goto cdaerrret;
	}
	
	if(CdaSetSpindleSpeed(g_cdaactive, 0)) // 0 for normal, else max
	{
		sprintf(cstr, "CdaDoOpen CdaSetSpindleSpeed failed '%s'", playpath);
		maLog(cstr);
		goto cdaerrret;
	}
	
#ifndef __MAOSX__
	if(CdaGetCdInfo(g_cdaactive, &trkfirstBCD, &trklastBCD))
	{
		sprintf(cstr, "CdaDoOpen CdaGetCdInfo failed '%s'", playpath);
		maLog(cstr);
		goto cdaerrret;
	}
	
	g_cdadrive[g_cdaactive].tracklast = BCDtoHex(trklastBCD);
#endif

	strcpy(g_state.inpath, playpath);
	
#ifdef __MAWIN__
	strcpy(lplaypath, playpath); // e.g. "D:/Track05.cda"
	lplaypath[strlen(playpath) - 4 ] = 0;
	g_cdadrive[g_cdaactive].track = atoi(&lplaypath[strlen(lplaypath) - 2 ]);
#endif

 #ifdef __MAOSX__
	strcpy(lplaypath, playpath); // e.g. "Audio CD/Track05.cda"
	lplaypath[strlen(playpath) - 4 ] = 0;
	g_cdadrive[g_cdaactive].track = atoi(&lplaypath[strlen(lplaypath) - 2 ]);
 #endif
 
 #if defined(__MAOS9__)
 	g_cdadrive[g_cdaactive].track = atoi(&playpath[strlen(playpath) - 2 ]);
 #endif

 #if defined(__MAUNIX__)
	strcpy(lplaypath, playpath); // e.g. "/dev/cdrom/Track05.cda"
	lplaypath[strlen(playpath) - 4 ] = 0;
	g_cdadrive[g_cdaactive].track = atoi(&lplaypath[strlen(lplaypath) - 2 ]);
 #endif

	if(g_cdadrive[g_cdaactive].track < 1 || g_cdadrive[g_cdaactive].track > 99) // never happens
	{
		sprintf(cstr, "CdaDoOpen bad track num %d '%s' '%s'",
					g_cdadrive[g_cdaactive].track, &lplaypath[5], playpath);
		maLog(cstr);
		goto cdaerrret;
	}
	
	// get starting lba for the track to play
	if(CdaGetTrackStartEnd(g_cdaactive))
	{
		sprintf(cstr, "CdaDoOpen bad start lba for track %d '%s'", g_cdadrive[g_cdaactive].track, playpath);
		maLog(cstr);
		goto cdaerrret;
	}
	
	g_cdadrive[g_cdaactive].lba = g_cdadrive[g_cdaactive].lbafirst;
		
	// length of track in blocks
	g_cdadrive[g_cdaactive].lbasintrack = 1 + g_cdadrive[g_cdaactive].lbalast - g_cdadrive[g_cdaactive].lbafirst;

	if(g_cdabuf)
		MAFREE(g_cdabuf);
	if((g_cdabuf = (char *)MAMALLOC(CDABLOCKSIZE * g_cdabufblocks)) == NULL)
	{
		sprintf(cstr, "CdaDoOpen MAMALLOC failed for %d bytes", CDABLOCKSIZE * g_cdabufblocks);
		maLog(cstr);
		goto cdaerrret;
	}

	return 0; // ok
	
cdaerrret:
		if(g_cdaactive)
			CdaCleanup(g_cdaactive);
		g_cdaactive = 0;
		return 1;
		
} // CdaDoOpen


int CdaOpenFile(char *filename)
{
	char cstr[256];
	
	if(CdaDoOpen(filename))
	{
		sprintf(cstr, "CdaOpenFile CdaDoOpen failed on '%s'", filename);
		maLog(cstr);
		return 1;
	}
	
	g_cdadrive[g_cdaactive].durationsecs = g_cdadrive[g_cdaactive].lbasintrack / CDAFRAMESPERSECL;
			
	g_state.position = 0;
	g_state.duration = g_cdadrive[g_cdaactive].durationsecs * 1000;

#ifdef __MAWIN__
	if(CdaOpenRipper())
	{
		g_cdaactive = 0;
		maLog("CdaOpenFile CdaOpenRipper failed");
		return 1;
	}
#endif

#ifdef __MAOSX__
	if(CdaPBOpenFile(g_cdaactive))
	{
		g_cdaactive = 0;
		sprintf(cstr, "CdaOpenFile CdaPBOpenFile failed on '%s'", filename);
		maLog(cstr);
		return 1;
	}
#endif

	return 0; // ok
	
} // CdaOpenFile


void CdaCloseFile(int drive)
{
#ifdef __MAWIN__
	CdaCloseRipper();
#endif

#ifdef __MAOSX__
	CdaPBCloseFile(drive);
#endif

	CdaCleanup(drive);
	if(g_cdabuf)
	{
		MAFREE(g_cdabuf);
		g_cdabuf = NULL;	
	}
	
} // CdaCloseFile


signed long ThreadPlayCda(void *parameter)
{
#ifdef MAUNUSED
 (void)parameter;
#endif

	g_waitaudiodone = 1;

	maPlayCda();
		
	g_state.stop |= STOP_PLAYDECODE;
	
	// wait for output owned by this thread to finish
	while(!g_stopB && g_waitaudiodone)
		maSleep(100);

	return 0;

} // ThreadPlayCda


 #if defined(__MACUNIX__)
 // stubs just for testing
  #include "../libs/cdparanoia/interface/cdda_interface.h"
  #include "../libs/cdparanoia/paranoia/cdda_paranoia.h"

int cdda_close(cdrom_drive *d)
{
	if(strerror_tr[0] == NULL) // use strerror_tr
		strerror_tr[0] = NULL; // to avoid a compiler warning
	return 0;
}

cdrom_drive *cdda_find_a_cdrom(int messagedest, char **message)
{
	return NULL;
}
				  
cdrom_drive *cdda_identify(const char *device, int messagedest, char **message)
{
	return NULL;
}
				  
int cdda_open(cdrom_drive *d)
{
	return 0;
}

long cdda_read(cdrom_drive *d, void *buffer, long beginsector, long sectors)
{
	return 0;
}


long cdda_track_firstsector(cdrom_drive *d,int track)
{
	return 0;
}

long cdda_track_lastsector(cdrom_drive *d,int track)
{
	return 0;
}

void paranoia_free(cdrom_paranoia *p)
{
}

cdrom_paranoia *paranoia_init(cdrom_drive *d)
{
	return NULL;
}

void paranoia_modeset(cdrom_paranoia *p,int mode)
{
}

int16_t *paranoia_read(cdrom_paranoia *p,void(*callback)(long,int))
{
	return NULL;
}

long paranoia_seek(cdrom_paranoia *p,long seek,int mode)
{
	return 0;
}

 #endif // __MACUNIX__


#ifdef __cplusplus
}
#endif

#else
 #pragma warning( disable : 4206 ) // nonstandard extension used : translation unit is empty
#endif // MANOCDA

// eof
