// maPlayFlac.c

/*
 * MoreAmp
 * Copyright (C) 2004-2008 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "maConfig.h" // included by all moreamp files

#ifdef __MAWIN__
 #ifdef MAUNUSEDWIN
  #pragma warning( disable : 4100 ) // unreferenced formal parameter
 #endif
 #pragma warning( disable : 4127 ) // conditional expression is constant
 #pragma warning( disable : 4201 ) // nonstandard extension used : nameless struct/union
 #pragma warning( disable : 4244 ) // conversion from 'double ' to 'int ', possible loss of data
 #pragma warning( disable : 4706 ) // assignment within conditional expression
 #define __RPCASYNC_H__ // rpcasync.h(45) : warning C4115: '_RPC_ASYNC_STATE' : named type definition in parentheses
#endif

#ifndef NODECODEFLAC

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "FLAC/assert.h"

#ifdef __MAWIN__
 #include <stdlib.h>
 #include <math.h>
 #include <limits.h>
 #include <time.h>
#endif

#if defined(__MAOSX__) || defined(__MAOS9__)
 #include <MacTypes.h>
#endif

#include "maDefs.h"
#include "maPlayer.h"
#include "maPlayFlac.h"
#include "maEqualizer.h"
#include "maMemFile.h"
#include "maPlayer.h"
#include "maUtils.h"

#ifdef __MAWIN__
 #include "maAudioWin.h"
#endif

#ifdef __MAOS9__
 #include "maAudioMac.h"
#endif

#ifdef __MAOSX__
 #include "maAudioMac.h"
#endif

#ifdef __MAUNIX__
 #include "maAudioUnix.h"
#endif

#include "FLAC/all.h"

#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT < 8
 #ifndef LEGACY_FLAC
  #define LEGACY_FLAC 1
 #endif
 #include "include/FLAC/file_decoder.h"
#else
 #undef LEGACY_FLAC
#endif

typedef enum
{
 FMT_U8, FMT_S8, FMT_U16_LE, FMT_U16_BE, FMT_U16_NE, FMT_S16_LE, FMT_S16_BE, FMT_S16_NE
}
AFormat;


typedef struct {
	FLAC__bool abort_flag;
	FLAC__bool eof;
	unsigned total_samples;
	unsigned bits_per_sample;
	unsigned channels;
	unsigned sample_rate;
	unsigned length_in_msec;
} file_info_struct;


typedef struct
{
	char *infilepath;
	int nomem; // file is not in mem, or don't use file in mem
#if defined(LEGACY_FLAC)
	FLAC__FileDecoder *decoder_;
#else
	FLAC__StreamDecoder *decoder_;
#endif
	FILE *infilefp;
	file_info_struct file_info_;
} FlacContext;


#ifdef min
 #undef min
#endif
#define min(x,y) ((x)<(y)?(x):(y))

#define SAMPLES_PER_WRITE 512
#define FLAC_PLUGIN__MAX_SUPPORTED_CHANNELS 2
#define BITSPERSAMPLE 16

void maPlayFlac();

#if defined(LEGACY_FLAC)
 static FLAC__bool safe_decoder_init_(const char *filename, FLAC__FileDecoder *decoder, FlacContext *pflac);
 static void safe_decoder_finish_(FLAC__FileDecoder *decoder);
 static void safe_decoder_delete_(FLAC__FileDecoder *decoder);
#else
 static FLAC__bool safe_decoder_init_(const char *filename, FLAC__StreamDecoder *decoder, FlacContext *pflac);
 static void safe_decoder_finish_(FLAC__StreamDecoder *decoder);
 static void safe_decoder_delete_(FLAC__StreamDecoder *decoder);
#endif

#if defined(LEGACY_FLAC)
 static FLAC__StreamDecoderWriteStatus write_callback_(const FLAC__FileDecoder *decoder, const FLAC__Frame *frame, const FLAC__int32 * const buffer[], void *client_data);
 static void metadata_callback_(const FLAC__FileDecoder *decoder, const FLAC__StreamMetadata *metadata, void *client_data);
 static void error_callback_(const FLAC__FileDecoder *decoder, FLAC__StreamDecoderErrorStatus status, void *client_data);
#else
 static FLAC__StreamDecoderWriteStatus write_callback_(const FLAC__StreamDecoder *decoder, const FLAC__Frame *frame, const FLAC__int32 * const buffer[], void *client_data);
 static void metadata_callback_(const FLAC__StreamDecoder *decoder, const FLAC__StreamMetadata *metadata, void *client_data);
 static void error_callback_(const FLAC__StreamDecoder *decoder, FLAC__StreamDecoderErrorStatus status, void *client_data);
#endif

unsigned FLAC__plugin_common__pack_pcm_signed_either_endian(FLAC__byte *data, FLAC__int32 *input, unsigned wide_samples, unsigned channels, unsigned source_bps, unsigned target_bps);
// unsigned long prng(unsigned long ulstate);

FlacContext l_FlacContext;
FlacContext *l_pflac;
unsigned int l_wide_samples_in_reservoir_ = 0;
FLAC__int32 l_reservoir_[FLAC__MAX_BLOCK_SIZE * 2/*for overflow*/ * FLAC_PLUGIN__MAX_SUPPORTED_CHANNELS];

void FlacDecodeStream(FlacContext *pflac);
int FlacOpenFile(FlacContext **ppflac);
void FlacCloseFile(FlacContext *pflac);

long l_flacseek = -1;

void flacseek(int posms)
{
	if(l_flacseek >= 0 || posms < 0 || !l_pflac)
		return;

	l_flacseek = posms;	

} // flacseek


void maPlayFlac() // (with code from flac 1.1.0 plugin_xmms plugin.c)
{
	unsigned int blocksize = 1;
	FLAC__uint64 endposframe;
	FLAC__uint64 sampleswritten = 0;
	FLAC__byte sample_buffer_[SAMPLES_PER_WRITE * FLAC_PLUGIN__MAX_SUPPORTED_CHANNELS * (BITSPERSAMPLE/8)];
	int pcmblocksize = 0;
	char cstr[256];
	
	if(!g_state.inpath)
	{
		maLog("maPlayFlac error null filename");
		return;
	}
	
	if(l_pflac)
	{
		maLog("maPlayFlac unclosed context");
		return;
	}
	
	// open
	l_pflac = &l_FlacContext;
	memset(l_pflac, 0, sizeof(FlacContext));
	l_pflac->nomem = g_state.nomem;

	
	// format of this decoder output
#ifdef WORDS_BIGENDIAN
	g_state.inwordsbigendian = 1;
#else
	g_state.inwordsbigendian = 0;
#endif
	g_state.ininterleaved = 1;

#if defined(__MAOSX__)
	if(l_pflac->nomem)
	{
		char tobsdname[MAX_PATH];
		strcpy(tobsdname, g_state.inpath);
		StrOSXtoStrBSD(tobsdname);
		l_pflac->infilepath = tobsdname;
	}
	else
#endif
		l_pflac->infilepath = g_state.inpath;
	
	g_noamem = l_pflac->nomem;

	if(FlacOpenFile(&l_pflac))
	{
		sprintf(cstr, "maPlayFlac FlacOpenFile failed on '%s'", g_state.inpath);
		maLog(cstr);
		FlacCloseFile(l_pflac);
		l_pflac = NULL;
		return;
	}
		
	g_state.insamplerate = l_pflac->file_info_.sample_rate;
	g_state.duration = l_pflac->file_info_.length_in_msec;
	g_state.position = 0;
	g_state.inchannels = l_pflac->file_info_.channels;

 	if(maOutputOpen() < 0)
	{
		FlacCloseFile(l_pflac);
		l_pflac = NULL;
		maLog("maPlayFlac maOutputOpen failed");
		return;
	}
	
	if(!g_posendon)
		g_posend = g_state.duration;
	
	endposframe = l_pflac->file_info_.channels * l_pflac->file_info_.total_samples;

	// flac decode loop (code from flac 1.1.0 plugin_xmms plugin.c)
	while(1)
	{	
		ThreadYield();
		
		if(g_state.stop)
			break;
				
		if(!l_pflac->file_info_.eof)
		{
			pcmblocksize = 0;
			while(l_wide_samples_in_reservoir_ < SAMPLES_PER_WRITE)
			{
				unsigned s;

				s = l_wide_samples_in_reservoir_;
#if defined(LEGACY_FLAC)
				if(FLAC__file_decoder_get_state(l_pflac->decoder_) == FLAC__FILE_DECODER_END_OF_FILE)
#else
				if(FLAC__stream_decoder_get_state(l_pflac->decoder_) == FLAC__STREAM_DECODER_END_OF_STREAM)
#endif
				{
					l_pflac->file_info_.eof = true;
					break;
				}
#if defined(LEGACY_FLAC)
				else if(!FLAC__file_decoder_process_single(l_pflac->decoder_))
				{
					// sprintf(cstr, "maPlayFlac FLAC__file_decoder_process_single failed samples=%d", l_wide_samples_in_reservoir_);
#else
				else if(!FLAC__stream_decoder_process_single(l_pflac->decoder_))
				{
					// sprintf(cstr, "maPlayFlac FLAC__stream_decoder_process_single failed samples=%d", l_wide_samples_in_reservoir_);
#endif
					// maLog(cstr);
					l_pflac->file_info_.eof = true;
					break;
				}
				blocksize = l_wide_samples_in_reservoir_ - s;
			}
			
			if(l_wide_samples_in_reservoir_ > 0)
			{
				const unsigned channels = l_pflac->file_info_.channels;
				const unsigned bits_per_sample = l_pflac->file_info_.bits_per_sample;
				const unsigned n = min(l_wide_samples_in_reservoir_, SAMPLES_PER_WRITE);
				const unsigned delta = n * channels;
				int bytes;
				unsigned i;

				pcmblocksize = bytes = (int)FLAC__plugin_common__pack_pcm_signed_either_endian(
						sample_buffer_,
						l_reservoir_,
						n,
						channels,
						bits_per_sample,
						min(bits_per_sample, BITSPERSAMPLE)); //flac_cfg.output.resolution.normal.dither_24_to_16 ? min(bits_per_sample, 16) : bits_per_sample);

				for(i = delta; i < l_wide_samples_in_reservoir_ * channels; i++)
					l_reservoir_[i-delta] = l_reservoir_[i];
				l_wide_samples_in_reservoir_ -= n;
			}
			else
			{
				pcmblocksize = 0;
				l_pflac->file_info_.eof = true;
			}

		} // if(!l_pflac->file_info_.eof)
		else
		{
			if(g_reptsong && l_pflac->file_info_.sample_rate != 22050
			/* && ( l_pflac->file_info_.sample_rate == 24000   // FLAC__file_decoder_seek_absolute fails at 22050.
							|| l_pflac->file_info_.sample_rate == 44100) */) // 8000, 24000, 44100 are ok
				l_flacseek = g_posbeg;
			else
				break;													
		}
			
		if(l_flacseek >= 0)
		{
			FLAC__uint64 target_sample = ((double)l_flacseek * (double)l_pflac->file_info_.total_samples) /
																	(double)l_pflac->file_info_.length_in_msec;
#if defined(LEGACY_FLAC)
			if(FLAC__file_decoder_seek_absolute(l_pflac->decoder_, target_sample))
#else
			if(FLAC__stream_decoder_seek_absolute(l_pflac->decoder_, target_sample))
#endif
			{
				l_pflac->file_info_.eof = false;
				l_wide_samples_in_reservoir_ = 0;
				pcmblocksize = 0;
				sampleswritten = target_sample * l_pflac->file_info_.channels;
				l_flacseek = -1;
			}
		}
			
		if(pcmblocksize > 0)
		{
			// Write to fifo
			if(maOutputWrite((char *)sample_buffer_, pcmblocksize, &g_state) != 0)
			{
				sprintf(cstr,
			"FlacDecodeStream maOutputWrite failed : len=%d g_state.stop=x%x g_fifoready=%d",
														pcmblocksize, g_state.stop, g_fifoready);
				maLog(cstr);
				break;
			}
		}

		// the current position in msec
		if((sampleswritten += (pcmblocksize >> 1)) >= endposframe)
			g_state.position = g_state.duration;
		else
			g_state.position = LROUND((sampleswritten * g_state.duration) / endposframe);

	} // while 1

	// finish

	FlushFifo();

	if(l_pflac)
	{
		FlacCloseFile(l_pflac);
		l_pflac = NULL;
	}
	
	g_state.position = 0;
	l_flacseek = -1;
	
} // maPlayFlac


int FlacOpenFile(FlacContext **ppflac)
{
	FlacContext *pflac;
	char cstr[256];

	if(ppflac == NULL || (pflac = *ppflac) == NULL)
	{
		maLog((char *)"FlacOpenFile null context pointer");
		return 1;
	}

#if defined(LEGACY_FLAC)
	if((pflac->decoder_ = FLAC__file_decoder_new()) == 0)
	{
		sprintf(cstr, "FlacOpenFile FLAC__file_decoder_new failed on '%s'", pflac->infilepath);
		maLog(cstr);
		return 1;
	}
#else
	if((pflac->decoder_ = FLAC__stream_decoder_new()) == 0)
	{
		sprintf(cstr, "FlacOpenFile FLAC__stream_decoder_new failed on '%s'", pflac->infilepath);
		maLog(cstr);
		return 1;
	}
#endif

	pflac->file_info_.abort_flag = false;
	pflac->file_info_.eof = false;

	if(!safe_decoder_init_(pflac->infilepath, pflac->decoder_, pflac))
	{
		sprintf(cstr, "FlacOpenFile safe_decoder_init_ failed on '%s'", pflac->infilepath);
		maLog(cstr);
		return 1;
	}

	return 0;

} // FlacOpenFile


void FlacCloseFile(FlacContext *pflac)
{
	if(!pflac)
	{
		maLog((char *)"FlacCloseFile null context pointer");
		return ;
	}

	safe_decoder_finish_(pflac->decoder_);

	// close pflac->decoder_
	if(pflac->decoder_)
	{
		safe_decoder_delete_(pflac->decoder_);
		pflac->decoder_ = 0;
	}
		
	// close file
	if(pflac->infilefp)
	{
		if(pflac->nomem)
			fclose(pflac->infilefp);
		else
			memfclose(pflac->infilefp);
					
		pflac->infilefp = NULL;
	}
	
} // FlacCloseFile


signed long ThreadPlayFlac(void *parameter)
{
#ifdef MAUNUSED
 (void)parameter;
#endif

	g_waitaudiodone = 1;

	maPlayFlac();
		
	g_state.stop |= STOP_PLAYDECODE;
	
	// wait for output owned by this thread to finish
	while(!g_stopB && g_waitaudiodone)
		maSleep(100);

	return 0;

} // ThreadPlayFlac


#if defined(LEGACY_FLAC)
 FLAC__bool safe_decoder_init_(const char *filename, FLAC__FileDecoder *decoder, FlacContext *pflac)
 {
	if(decoder == 0)
		return false;

	safe_decoder_finish_(decoder);

	FLAC__file_decoder_set_md5_checking(decoder, false);
	FLAC__file_decoder_set_filename(decoder, filename);
	FLAC__file_decoder_set_metadata_ignore_all(decoder);
	FLAC__file_decoder_set_metadata_respond(decoder, FLAC__METADATA_TYPE_STREAMINFO);
	FLAC__file_decoder_set_metadata_respond(decoder, FLAC__METADATA_TYPE_VORBIS_COMMENT);
	FLAC__file_decoder_set_write_callback(decoder, write_callback_);
	FLAC__file_decoder_set_metadata_callback(decoder, metadata_callback_);
	FLAC__file_decoder_set_error_callback(decoder, error_callback_);
	FLAC__file_decoder_set_client_data(decoder, &pflac->file_info_);
	if(FLAC__file_decoder_init(decoder) != FLAC__FILE_DECODER_OK)
		return false;

	if(!FLAC__file_decoder_process_until_end_of_metadata(decoder))
		return false;

	return true;
 }
#else
 FLAC__bool safe_decoder_init_(const char *filename, FLAC__StreamDecoder *decoder, FlacContext *pflac)
 {
	if(decoder == 0)
		return false;

	safe_decoder_finish_(decoder);

	FLAC__stream_decoder_set_md5_checking(decoder, false);
	FLAC__stream_decoder_set_metadata_ignore_all(decoder);
	FLAC__stream_decoder_set_metadata_respond(decoder, FLAC__METADATA_TYPE_STREAMINFO);
	FLAC__stream_decoder_set_metadata_respond(decoder, FLAC__METADATA_TYPE_VORBIS_COMMENT);
	if(FLAC__stream_decoder_init_file(decoder, filename, write_callback_, metadata_callback_,
			error_callback_, /*client_data= */ &pflac->file_info_) != FLAC__STREAM_DECODER_INIT_STATUS_OK)
		return false;

	if(!FLAC__stream_decoder_process_until_end_of_metadata(decoder))
		return false;

	return true;
 }
#endif

#if defined(LEGACY_FLAC)
 void safe_decoder_finish_(FLAC__FileDecoder *decoder)
 {
	if(decoder && FLAC__file_decoder_get_state(decoder) != FLAC__FILE_DECODER_UNINITIALIZED)
		FLAC__file_decoder_finish(decoder);
 }
#else
 void safe_decoder_finish_(FLAC__StreamDecoder *decoder)
 {
	if(decoder && FLAC__stream_decoder_get_state(decoder) != FLAC__STREAM_DECODER_UNINITIALIZED)
		(void)FLAC__stream_decoder_finish(decoder);
 }
#endif

#if defined(LEGACY_FLAC)
 void safe_decoder_delete_(FLAC__FileDecoder *decoder)
 {
	if(decoder) {
		safe_decoder_finish_(decoder);
		FLAC__file_decoder_delete(decoder);
	}
 }
#else
 void safe_decoder_delete_(FLAC__StreamDecoder *decoder)
 {
	if(decoder) {
		safe_decoder_finish_(decoder);
		FLAC__stream_decoder_delete(decoder);
	}
 }
#endif

#if defined(LEGACY_FLAC)
 FLAC__StreamDecoderWriteStatus write_callback_(const FLAC__FileDecoder *decoder, const FLAC__Frame *frame, const FLAC__int32 * const buffer[], void *client_data)
 {
	file_info_struct *file_info = (file_info_struct *)client_data;
	const unsigned channels = file_info->channels, wide_samples = frame->header.blocksize;
	unsigned wide_sample, offset_sample, channel;

	(void)decoder;

	if(file_info->abort_flag)
		return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;

	for(offset_sample = l_wide_samples_in_reservoir_ * channels, wide_sample = 0; wide_sample < wide_samples; wide_sample++)
		for(channel = 0; channel < channels; channel++, offset_sample++)
			l_reservoir_[offset_sample] = buffer[channel][wide_sample];

	l_wide_samples_in_reservoir_ += wide_samples;

	return FLAC__STREAM_DECODER_WRITE_STATUS_CONTINUE;
 }
#else
 FLAC__StreamDecoderWriteStatus write_callback_(const FLAC__StreamDecoder *decoder, const FLAC__Frame *frame, const FLAC__int32 * const buffer[], void *client_data)
 {
	file_info_struct *file_info = (file_info_struct *)client_data;
	const unsigned channels = file_info->channels, wide_samples = frame->header.blocksize;
	unsigned wide_sample, offset_sample, channel;

	(void)decoder;

	if(file_info->abort_flag)
		return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;

	for(offset_sample = l_wide_samples_in_reservoir_ * channels, wide_sample = 0; wide_sample < wide_samples; wide_sample++)
		for(channel = 0; channel < channels; channel++, offset_sample++)
			l_reservoir_[offset_sample] = buffer[channel][wide_sample];

	l_wide_samples_in_reservoir_ += wide_samples;

	/*
	if((sample_buffer_last_ + wide_samples) > (SAMPLE_BUFFER_SIZE / (channels * stream_data->sample_format_bytes_per_sample))) {
		memmove(sample_buffer_, sample_buffer_ + sample_buffer_first_ * channels * stream_data->sample_format_bytes_per_sample, (sample_buffer_last_ - sample_buffer_first_) * channels * stream_data->sample_format_bytes_per_sample);
		sample_buffer_last_ -= sample_buffer_first_;
		sample_buffer_first_ = 0;
	}
	sample_buffer_start = sample_buffer_ + sample_buffer_last_ * channels * stream_data->sample_format_bytes_per_sample;
	if(stream_data->has_replaygain && flac_cfg.output.replaygain.enable) {
		FLAC__replaygain_synthesis__apply_gain(
				sample_buffer_start,
				!is_big_endian_host_,
				stream_data->sample_format_bytes_per_sample == 1, / * unsigned_data_out * /
				buffer,
				wide_samples,
				channels,
				bits_per_sample,
				stream_data->sample_format_bytes_per_sample * 8,
				stream_data->replay_scale,
				flac_cfg.output.replaygain.hard_limit,
				flac_cfg.output.resolution.replaygain.dither,
				&stream_data->dither_context
		);
	}
	else if(is_big_endian_host_) {
		FLAC__plugin_common__pack_pcm_signed_big_endian(
			sample_buffer_start,
			buffer,
			wide_samples,
			channels,
			bits_per_sample,
			stream_data->sample_format_bytes_per_sample * 8
		);
	}
	else {
		FLAC__plugin_common__pack_pcm_signed_little_endian(
			sample_buffer_start,
			buffer,
			wide_samples,
			channels,
			bits_per_sample,
			stream_data->sample_format_bytes_per_sample * 8
		);
	}

	sample_buffer_last_ += wide_samples;
	*/
	
	return FLAC__STREAM_DECODER_WRITE_STATUS_CONTINUE;
 }
#endif


#if defined(LEGACY_FLAC)
void metadata_callback_(const FLAC__FileDecoder *decoder, const FLAC__StreamMetadata *metadata, void *client_data)
{
	file_info_struct *file_info = (file_info_struct *)client_data;
	( void )decoder;
	if(metadata->type == FLAC__METADATA_TYPE_STREAMINFO)
	{
		FLAC__ASSERT(metadata->data.stream_info.total_samples < 0x100000000); /* this plugin can only handle < 4 gigasamples */
		file_info->total_samples = (unsigned)(metadata->data.stream_info.total_samples & 0xffffffff);
		file_info->bits_per_sample = metadata->data.stream_info.bits_per_sample;
		file_info->channels = metadata->data.stream_info.channels;
		file_info->sample_rate = metadata->data.stream_info.sample_rate;

 /* #ifdef FLAC__DO_DITHER
		if(file_info->bits_per_sample == 8)
		{
			file_info->sample_format = FMT_S8;
		}
		else if(file_info->bits_per_sample == 16 || file_info->bits_per_sample == 24)
		{
			file_info->sample_format = FMT_S16_LE;
		}
		else
		{
			/w* @@@ need some error here like wa2: MessageBox(mod_.hMainWindow, "ERROR: plugin can only handle 8/16/24-bit samples\n", "ERROR: plugin can only handle 8/16/24-bit samples", 0); *w/
			file_info->abort_flag = true;
			return;
		}
 #else
		if(file_info->bits_per_sample == 8)
		{
			file_info->sample_format = FMT_S8;
		}
		else if(file_info->bits_per_sample == 16)
		{
			file_info->sample_format = FMT_S16_LE;
		}
		else
		{
			/w* @@@ need some error here like wa2: MessageBox(mod_.hMainWindow, "ERROR: plugin can only handle 8/16-bit samples\n", "ERROR: plugin can only handle 8/16-bit samples", 0); *w/
			file_info->abort_flag = true;
			return;
		}
 #endif
 */
		// file_info->length_in_msec = file_info->total_samples * 10 / (file_info->sample_rate / 100);
		file_info->length_in_msec = ((double)file_info->total_samples * (double)1000.0) / (double)file_info->sample_rate;
	}
	/*
	else if(metadata->type == FLAC__METADATA_TYPE_VORBIS_COMMENT)
	{
		double gain, peak;
		if(grabbag__replaygain_load_from_vorbiscomment(metadata, flac_cfg.output.replaygain.album_mode, &gain, &peak)) {
			file_info_.has_replaygain = true;
			file_info_.replay_scale = grabbag__replaygain_compute_scale_factor(peak, gain, (double)flac_cfg.output.replaygain.preamp, /u*prevent_clipping=*u/!flac_cfg.output.replaygain.hard_limit);
		}
	}
	*/
 }

 void error_callback_(const FLAC__FileDecoder *decoder, FLAC__StreamDecoderErrorStatus status, void *client_data)
 {
	file_info_struct *file_info = (file_info_struct *)client_data;
	( void )decoder;
	if(status != FLAC__STREAM_DECODER_ERROR_STATUS_LOST_SYNC)
		file_info->abort_flag = true;
 }
#else
 void metadata_callback_(const FLAC__StreamDecoder *decoder, const FLAC__StreamMetadata *metadata, void *client_data)
 {
	file_info_struct *file_info = (file_info_struct *)client_data;
	( void )decoder;
	if(metadata->type == FLAC__METADATA_TYPE_STREAMINFO)
	{
		FLAC__ASSERT(metadata->data.stream_info.total_samples < 0x100000000); /* this plugin can only handle < 4 gigasamples */
		file_info->total_samples = (unsigned)(metadata->data.stream_info.total_samples & 0xffffffff);
		file_info->bits_per_sample = metadata->data.stream_info.bits_per_sample;
		file_info->channels = metadata->data.stream_info.channels;
		file_info->sample_rate = metadata->data.stream_info.sample_rate;

 /* #ifdef FLAC__DO_DITHER
		if(file_info->bits_per_sample == 8)
		{
			file_info->sample_format = FMT_S8;
		}
		else if(file_info->bits_per_sample == 16 || file_info->bits_per_sample == 24)
		{
			file_info->sample_format = FMT_S16_LE;
		}
		else
		{
			/w* @@@ need some error here like wa2: MessageBox(mod_.hMainWindow, "ERROR: plugin can only handle 8/16/24-bit samples\n", "ERROR: plugin can only handle 8/16/24-bit samples", 0); *w/
			file_info->abort_flag = true;
			return;
		}
 #else
		if(file_info->bits_per_sample == 8)
		{
			file_info->sample_format = FMT_S8;
		}
		else if(file_info->bits_per_sample == 16)
		{
			file_info->sample_format = FMT_S16_LE;
		}
		else
		{
			/w* @@@ need some error here like wa2: MessageBox(mod_.hMainWindow, "ERROR: plugin can only handle 8/16-bit samples\n", "ERROR: plugin can only handle 8/16-bit samples", 0); *w/
			file_info->abort_flag = true;
			return;
		}
 #endif
 */
		// file_info->length_in_msec = file_info->total_samples * 10 / (file_info->sample_rate / 100);
		file_info->length_in_msec = ((double)file_info->total_samples * (double)1000.0) / (double)file_info->sample_rate;
	}
	/*
	stream_data_struct *stream_data = (stream_data_struct *)client_data;
	(void)decoder;
	if(metadata->type == FLAC__METADATA_TYPE_STREAMINFO) {
		stream_data->total_samples = metadata->data.stream_info.total_samples;
		stream_data->bits_per_sample = metadata->data.stream_info.bits_per_sample;
		stream_data->channels = metadata->data.stream_info.channels;
		stream_data->sample_rate = metadata->data.stream_info.sample_rate;
		{
			FLAC__uint64 l = (FLAC__uint64)((double)stream_data->total_samples / (double)stream_data->sample_rate * 1000.0 + 0.5);
			if (l > INT_MAX)
				l = INT_MAX;
			stream_data->length_in_msec = (int)l;
		}
	}
	else if(metadata->type == FLAC__METADATA_TYPE_VORBIS_COMMENT) {
		double reference, gain, peak;
		if(grabbag__replaygain_load_from_vorbiscomment(metadata, flac_cfg.output.replaygain.album_mode, / *strict=* /false, &reference, &gain, &peak)) {
			stream_data->has_replaygain = true;
			stream_data->replay_scale = grabbag__replaygain_compute_scale_factor(peak, gain, (double)flac_cfg.output.replaygain.preamp, / *prevent_clipping=* /!flac_cfg.output.replaygain.hard_limit);
		}
	}
	*/
 }

 void error_callback_(const FLAC__StreamDecoder *decoder, FLAC__StreamDecoderErrorStatus status, void *client_data)
 {
	file_info_struct *file_info = (file_info_struct *)client_data;
	( void )decoder;
	if(status != FLAC__STREAM_DECODER_ERROR_STATUS_LOST_SYNC)
		file_info->abort_flag = true;
	/*
	stream_data_struct *stream_data = (stream_data_struct *)client_data;
	(void)decoder;
	if(status != FLAC__STREAM_DECODER_ERROR_STATUS_LOST_SYNC)
		stream_data->abort_flag = true;
	*/
 }
#endif


/* dither routine derived from MAD winamp plugin */

typedef struct {
	FLAC__int32 error[3];
	FLAC__int32 random;
} dither_state;

static /*FLAC__INLINE*/ FLAC__int32 linear_dither(unsigned source_bps, unsigned target_bps, FLAC__int32 sample, dither_state *dither, const FLAC__int32 MIN, const FLAC__int32 MAX)
{
	unsigned scalebits;
	FLAC__int32 output, mask, random;

	FLAC__ASSERT(source_bps < 32);
	FLAC__ASSERT(target_bps <= BITSPERSAMPLE); // 24
	FLAC__ASSERT(target_bps <= source_bps);

	/* noise shape */
	sample += dither->error[0] - dither->error[1] + dither->error[2];

	dither->error[2] = dither->error[1];
	dither->error[1] = dither->error[0] / 2;

	/* bias */
	output = sample + (1L << (source_bps - target_bps - 1));

	scalebits = source_bps - target_bps;
	mask = (1L << scalebits) - 1;

	/* dither */
	// random = (FLAC__int32)prng(dither->random); // prng is in maPlayMp3.c
	random = (FLAC__int32)((dither->random * 0x0019660dL + 0x3c6ef35fL) & 0xffffffffL);
	output += (random & mask) - (dither->random & mask);

	dither->random = random;

	/* clip */
	if(output > MAX) {
		output = MAX;

		if(sample > MAX)
			sample = MAX;
	}
	else if(output < MIN) {
		output = MIN;

		if(sample < MIN)
			sample = MIN;
	}

	/* quantize */
	output &= ~mask;

	/* error feedback */
	dither->error[0] = sample - output;

	/* scale */
	return output >> scalebits;
}


unsigned FLAC__plugin_common__pack_pcm_signed_either_endian(FLAC__byte *data, FLAC__int32 *input, unsigned wide_samples, unsigned channels, unsigned source_bps, unsigned target_bps)
{
	static dither_state dither[FLAC_PLUGIN__MAX_SUPPORTED_CHANNELS];
	FLAC__byte * const start = data;
	FLAC__int32 sample;
	unsigned samples = wide_samples * channels;
	const unsigned bytes_per_sample = target_bps / 8;

	FLAC__ASSERT(FLAC_PLUGIN__MAX_SUPPORTED_CHANNELS == 2);
	FLAC__ASSERT(channels > 0 && channels <= FLAC_PLUGIN__MAX_SUPPORTED_CHANNELS);
	FLAC__ASSERT(source_bps < 32);
	FLAC__ASSERT(target_bps <= BITSPERSAMPLE); // 24
	FLAC__ASSERT(target_bps <= source_bps);
	FLAC__ASSERT((source_bps & 7) == 0);
	FLAC__ASSERT((target_bps & 7) == 0);

	if(source_bps != target_bps)
	{
		const FLAC__int32 MIN = -(1L << source_bps);
		const FLAC__int32 MAX = ~MIN; /*(1L << (source_bps-1)) - 1 */
		const unsigned dither_twiggle = channels - 1;
		unsigned dither_source = 0;

		while(samples--)
		{
			sample = linear_dither(source_bps, target_bps, *input++, &dither[dither_source], MIN, MAX);
			dither_source ^= dither_twiggle;

			switch(target_bps)
			{
				case 8:
					data[0] = sample ^ 0x80;
					break;
				case 24:
					data[2] = (FLAC__byte)(sample >> 16);
					/* fall through */
				case 16:
#ifdef WORDS_BIGENDIAN
					data[0] = (FLAC__byte)(sample >> 8); // big endian
					data[1] = (FLAC__byte)sample;
#else
					data[1] = (FLAC__byte)(sample >> 8);
					data[0] = (FLAC__byte)sample;
#endif
			}

			data += bytes_per_sample;
		}
	}
	else
	{
		while(samples--)
		{
			sample = *input++;

			switch(target_bps)
			{
				case 8:
					data[0] = sample ^ 0x80;
					break;
				case 24:
					data[2] = (FLAC__byte)(sample >> 16);
					/* fall through */
				case 16:
#ifdef WORDS_BIGENDIAN
					data[0] = (FLAC__byte)(sample >> 8); // big endian
					data[1] = (FLAC__byte)sample;
#else
					data[1] = (FLAC__byte)(sample >> 8);
					data[0] = (FLAC__byte)sample;
#endif
			}

			data += bytes_per_sample;
		}
	}

	return data - start;
}


 // code from metaflac in flac-1.1.2
#if !defined(__MAOS9__) // flac-1.1.2 is broken on OS9, using 1.1.0
 #include "FLAC/metadata.h"
 FLAC__bool do_major_operation_on_file(FILE *fp, const char *filename);
 FLAC__bool do_major_operation__list(FILE *fp, const char *filename, FLAC__Metadata_Chain *chain);
 void write_vc_field(char *fp, const char *filename, const FLAC__StreamMetadata_VorbisComment_Entry *entry, FLAC__bool raw, FILE *f);
#endif

int DumpInfoFlac(MASTATE *pstate, FILE *dumpfp)
{
#ifdef __MAOS9__ // flac-1.1.2 is broken on OS9, using 1.1.0
 (void)pstate; (void)dumpfp;
#else
	int ret;
	char cstr[256];
	
	g_nobmem = pstate->nomem;
	
 #if defined(__MAOSX__)
	if(pstate->nomem)
	{
		char tobsdname[MAX_PATH];
		strcpy(tobsdname, pstate->inpath);
		StrOSXtoStrBSD(tobsdname);
		ret = do_major_operation_on_file(dumpfp, tobsdname);
	}
	else
 #endif
		ret = do_major_operation_on_file(dumpfp, pstate->inpath);
	
	if(!ret)
	{
		DumpString(dumpfp, (char *)"flac comments:");
		sprintf(cstr, "DumpTagsFlac do_major_operation_on_file failed on '%s'", pstate->inpath);
		maLog(cstr);
		return 1;		
	}
#endif	

	return 0;
	
} // DumpInfoFlac


#if !defined(__MAOS9__) // flac-1.1.2 is broken on OS9, using 1.1.0
void write_vc_field(char *fp, const char *filename, const FLAC__StreamMetadata_VorbisComment_Entry *entry, FLAC__bool raw, FILE *f)
{
	char cstrx[512];
	
	if(0 != entry->entry)
	{
		if(!raw)
		{
			/*
			 * WATCHOUT: comments that contain an embedded null will
			 * be truncated by utf_decode().
			 */
			char *converted;

			if(0) //utf8_decode(entry->entry, &converted) >= 0)
			{
				strcat(fp, converted);
				MAFREE(converted);
			}
			else
			{
				strncpy(cstrx, (const char *)entry->entry, entry->length);
				cstrx[entry->length] = 0;
				strcat(fp, cstrx);
			}
		}
		else
		{
			strncpy(cstrx, (const char *)entry->entry, entry->length);
			cstrx[entry->length] = 0;
			strcat(fp, cstrx);
		}
	}

} // write_vc_field


void write_metadata(FILE *fp, const char *filename, FLAC__StreamMetadata *block, unsigned block_number, FLAC__bool raw, FLAC__bool hexdump_application)
{
	unsigned i, j;
	char tstr[32];
	char cstrx[512];

	if(block_number == 0) // do duration here
	{
		sprintf(cstrx, "Duration : %s", SecondsToStrHMS(tstr, (int)
			(block->data.stream_info.total_samples / block->data.stream_info.sample_rate)));
		DumpString(fp, cstrx);
	
		DumpString(fp, (char *)"flac comments:");	
	}
	
	sprintf(cstrx, "METADATA block #%u", block_number);
	DumpString(fp, cstrx);
	sprintf(cstrx, "  type: %u (%s)", (unsigned)block->type, block->type < FLAC__METADATA_TYPE_UNDEFINED? FLAC__MetadataTypeString[block->type] : "UNKNOWN");
	DumpString(fp, cstrx);
	sprintf(cstrx, "  is last: %s", block->is_last? "true":"false");
	DumpString(fp, cstrx);
	sprintf(cstrx, "  length: %u", block->length);
	DumpString(fp, cstrx);
	
	switch(block->type)
	{
		case FLAC__METADATA_TYPE_STREAMINFO:
			sprintf(cstrx, "  minumum blocksize: %u samples", block->data.stream_info.min_blocksize);
			DumpString(fp, cstrx);
			sprintf(cstrx, "  maximum blocksize: %u samples", block->data.stream_info.max_blocksize);
			DumpString(fp, cstrx);
			sprintf(cstrx, "  minimum framesize: %u bytes", block->data.stream_info.min_framesize);
			DumpString(fp, cstrx);
			sprintf(cstrx, "  maximum framesize: %u bytes", block->data.stream_info.max_framesize);
			DumpString(fp, cstrx);
			sprintf(cstrx, "  sample_rate: %u Hz", block->data.stream_info.sample_rate);
			DumpString(fp, cstrx);
			sprintf(cstrx, "  channels: %u", block->data.stream_info.channels);
			DumpString(fp, cstrx);
			sprintf(cstrx, "  bits-per-sample: %u", block->data.stream_info.bits_per_sample);
			DumpString(fp, cstrx);
#ifdef _MSC_VER
			sprintf(cstrx, "  total samples: %I64u", block->data.stream_info.total_samples);
#else
			sprintf(cstrx, "  total samples: %llu", block->data.stream_info.total_samples);
#endif
			DumpString(fp, cstrx);
			sprintf(cstrx, "  MD5 signature: ");
			for(i = 0; i < 16; i++)
				sprintf(cstrx, "%s%02x", cstrx, (unsigned)block->data.stream_info.md5sum[i]);
			DumpString(fp, cstrx);
		break;
		
		case FLAC__METADATA_TYPE_PADDING:
		break;
		
		case FLAC__METADATA_TYPE_APPLICATION:
			sprintf(cstrx, "  application ID: ");
			DumpString(fp, cstrx);
			for(i = 0; i < 4; i++)
				sprintf(cstrx, "%02x ", block->data.application.id[i]);
			DumpString(fp, cstrx);
			sprintf(cstrx, "  data contents:");
			DumpString(fp, cstrx);
			if(0 != block->data.application.data)
			{
				sprintf(cstrx, "  application data length : %u", block->length - FLAC__STREAM_METADATA_HEADER_LENGTH);
				DumpString(fp, cstrx);
			}
		break;
		
		case FLAC__METADATA_TYPE_SEEKTABLE:
			sprintf(cstrx, "  seek points: %u", block->data.seek_table.num_points);
			DumpString(fp, cstrx);
			for(i = 0; i < block->data.seek_table.num_points; i++)
			{
				if(block->data.seek_table.points[i].sample_number != FLAC__STREAM_METADATA_SEEKPOINT_PLACEHOLDER)
				{
#ifdef _MSC_VER
					sprintf(cstrx, "    point %u: sample_number=%I64u, stream_offset=%I64u, frame_samples=%u", i, block->data.seek_table.points[i].sample_number, block->data.seek_table.points[i].stream_offset, block->data.seek_table.points[i].frame_samples);
#else
					sprintf(cstrx, "    point %u: sample_number=%llu, stream_offset=%llu, frame_samples=%u", i, block->data.seek_table.points[i].sample_number, block->data.seek_table.points[i].stream_offset, block->data.seek_table.points[i].frame_samples);
#endif
					DumpString(fp, cstrx);
				}
				else
				{
					sprintf(cstrx, "    point %u: PLACEHOLDER", i);
					DumpString(fp, cstrx);
				}
			}
		break;
		
		case FLAC__METADATA_TYPE_VORBIS_COMMENT:
			sprintf(cstrx, "  vendor string: ");
			write_vc_field(cstrx, 0, &block->data.vorbis_comment.vendor_string, raw, stdout);
			DumpString(fp, cstrx);
			sprintf(cstrx, "  comments: %u", block->data.vorbis_comment.num_comments);
			DumpString(fp, cstrx);
			for(i = 0; i < block->data.vorbis_comment.num_comments; i++)
			{
				sprintf(cstrx, "    comment[%u]: ", i);
				write_vc_field(cstrx, 0, &block->data.vorbis_comment.comments[i], raw, stdout);
				DumpString(fp, cstrx);
			}
		break;
			
		case FLAC__METADATA_TYPE_CUESHEET:
			sprintf(cstrx, "  media catalog number: %s", block->data.cue_sheet.media_catalog_number);
			DumpString(fp, cstrx);
#ifdef _MSC_VER
			sprintf(cstrx, "  lead-in: %I64u", block->data.cue_sheet.lead_in);
#else
			sprintf(cstrx, "  lead-in: %llu", block->data.cue_sheet.lead_in);
#endif
			DumpString(fp, cstrx);
			sprintf(cstrx, "  is CD: %s", block->data.cue_sheet.is_cd? "true":"false");
			DumpString(fp, cstrx);
			sprintf(cstrx, "  number of tracks: %u", block->data.cue_sheet.num_tracks);
			for(i = 0; i < block->data.cue_sheet.num_tracks; i++)
			{
				const FLAC__StreamMetadata_CueSheet_Track *track = block->data.cue_sheet.tracks+i;
				const FLAC__bool is_last = (i == block->data.cue_sheet.num_tracks-1);
				const FLAC__bool is_leadout = is_last && track->num_indices == 0;
				sprintf(cstrx, "    track[%u]", i);
				DumpString(fp, cstrx);
#ifdef _MSC_VER
				sprintf(cstrx, "      offset: %I64u", track->offset);
#else
				sprintf(cstrx, "      offset: %llu", track->offset);
#endif
				DumpString(fp, cstrx);
				if(is_last)
				{
					sprintf(cstrx, "      number: %u (%s)", (unsigned)track->number, is_leadout? "LEAD-OUT" : "INVALID");
					DumpString(fp, cstrx);
				}
				else
				{
					sprintf(cstrx, "      number: %u", (unsigned)track->number);
					DumpString(fp, cstrx);
				}
				if(!is_leadout)
				{
					sprintf(cstrx, "      ISRC: %s", track->isrc);
					DumpString(fp, cstrx);
					sprintf(cstrx, "      type: %s", track->type == 1? "DATA" : "AUDIO");
					DumpString(fp, cstrx);
					sprintf(cstrx, "      pre-emphasis: %s", track->pre_emphasis? "true":"false");
					DumpString(fp, cstrx);
					sprintf(cstrx, "      number of index points: %u", track->num_indices);
					DumpString(fp, cstrx);
					for(j = 0; j < track->num_indices; j++)
					{
						const FLAC__StreamMetadata_CueSheet_Index *index = track->indices+j;
						sprintf(cstrx, "        index[%u]", j);
						DumpString(fp, cstrx);
#ifdef _MSC_VER
						sprintf(cstrx, "          offset: %I64u", index->offset);
#else
						sprintf(cstrx, "          offset: %llu", index->offset);
#endif
						DumpString(fp, cstrx);
						sprintf(cstrx, "          number: %u", (unsigned)index->number);
						DumpString(fp, cstrx);
					}
				}
			}
		break;
		
		default:
			sprintf(cstrx, "  data contents length = %u:", block->length);
			DumpString(fp, cstrx);
		break;
	}

} // write_metadata


FLAC__bool do_major_operation__list(FILE *fp, const char *filename, FLAC__Metadata_Chain *chain)
{
	FLAC__Metadata_Iterator *iterator = FLAC__metadata_iterator_new();
	FLAC__StreamMetadata *block;
	FLAC__bool ok = true;
	unsigned block_number;
	char cstr[256];

	if(0 == iterator)
	{
		maLog("Flac do_major_operation__list out of memory allocating iterator");
		return false;
	}

	FLAC__metadata_iterator_init(iterator, chain);

	block_number = 0;
	do
	{
		block = FLAC__metadata_iterator_get_block(iterator);
		ok &= (0 != block);
		if(!ok)
		{
			sprintf(cstr, "Flac do_major_operation__list couldn't get block from chain for '%s'", filename);
			maLog(cstr);
		}
		else
			write_metadata(fp, filename, block, block_number, 0, 0);
			
		block_number++;
	} while(ok && FLAC__metadata_iterator_next(iterator));

	FLAC__metadata_iterator_delete(iterator);

	return ok;
	
} // do_major_operation__list


FLAC__bool do_major_operation_on_file(FILE *fp, const char *filename)
{
	FLAC__bool ok;
	FLAC__Metadata_Chain *chain = FLAC__metadata_chain_new();
	char cstr[256];	

	if(0 == chain)
	{
		maLog("Flac do_major_operation_on_file out of memory allocating chain");
		return false;
	}
	
	if(!FLAC__metadata_chain_read(chain, filename))
	{
		sprintf(cstr, "Flac error reading metadata '%s'", filename);
		maLog(cstr);
		FLAC__metadata_chain_delete(chain);
		return false;
	}

	ok = do_major_operation__list(fp, 0, chain);

	FLAC__metadata_chain_delete(chain);

	return ok;

} // do_major_operation_on_file
#endif

#ifdef __cplusplus
}
#endif

#endif

// eof
