// maPlayOgg.c

/*
 * MoreAmp
 * Copyright (C) 2004-2008 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "maConfig.h" // included by all moreamp files

#ifdef __MAWIN__
 #ifdef MAUNUSEDWIN
  #pragma warning( disable : 4100 ) // unreferenced formal parameter
 #endif
 #pragma warning( disable : 4127 ) // conditional expression is constant
 #pragma warning( disable : 4201 ) // nonstandard extension used : nameless struct/union
 #pragma warning( disable : 4244 ) // conversion from 'double ' to 'int ', possible loss of data
 #pragma warning( disable : 4706 ) // assignment within conditional expression
 #define __RPCASYNC_H__ // rpcasync.h(45) : warning C4115: '_RPC_ASYNC_STATE' : named type definition in parentheses
#endif

#ifndef NODECODEOGG

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#ifdef __MAWIN__
 #include <wtypes.h>
 #include <limits.h>
 #include <time.h>
#endif

#include "ogg/ogg.h" // libs/libogg/include/ogg/os_types may need __MACOS__or __MACOSX__ or __MAWIN__ or __BEOS__ etc.
#include "vorbis/codec.h"
#include "vorbis/vorbisfile.h"

#include "maPlayer.h"
#include "maUtils.h"
#include "maPlayOgg.h"

#ifdef __MAWIN__
 #include "maAudioWin.h"
#endif

#ifdef __MAOS9__
 #include "maAudioMac.h"
#endif

#ifdef __MAOSX__
 #include "maAudioMac.h"
#endif

#ifdef __MAUNIX__
 #include "maAudioUnix.h"
#endif

typedef struct
{
	int m_iFileLength_Secs;
	int m_iBitRate_Kbs;
	int m_iFreq_Hz;
	int m_bStereo;
	int m_b16bit;
} OggFileInfo;

typedef struct
{
	FILE *m_hFile;
	OggFileInfo m_FileInfo;
	int oggInstance;
	OggVorbis_File vf;
	int current_section;
	MASTATE *pstate;
} OggContext;

void maPlayOgg();
int OggOpenFile(OggContext *pOggContext);
void OggDecodeStream();
void OggCloseFile(OggContext *pOggContext);
int OggGetPCMBlock(OggContext *pOggContext, void *pBlock, int *pdwBlockSize);

OggContext g_OggContext;
OggContext *g_pOggContext;

// from vorbisfile.c :
static int _memfseek64_wrap(FILE *f,ogg_int64_t off,int whence);
static int _memfseek64_wrap(FILE *f,ogg_int64_t off,int whence){
  if(f==NULL)return(-1);
  return memfseek(f,(int)off,whence);
}

int ov_memopen(FILE *f,OggVorbis_File *vf,char *initial,long ibytes);
int ov_memopen(FILE *f,OggVorbis_File *vf,char *initial,long ibytes){
  ov_callbacks callbacks = {
    (size_t (*)(void *, size_t, size_t, void *))  memfread,
    (int (*)(void *, ogg_int64_t, int))			  _memfseek64_wrap,
    (int (*)(void *))                             memfclose,
    (long (*)(void *))                            memftell
  };

  return ov_open_callbacks((void *)f, vf, initial, ibytes, callbacks);
}

int ov_memtest(FILE *f,OggVorbis_File *vf,char *initial,long ibytes);
int ov_memtest(FILE *f,OggVorbis_File *vf,char *initial,long ibytes){
  ov_callbacks callbacks = {
    (size_t (*)(void *, size_t, size_t, void *))  memfread,
    (int (*)(void *, ogg_int64_t, int))           _memfseek64_wrap,
    (int (*)(void *))                             memfclose,
    (long (*)(void *))                            memftell
  };

  return ov_test_callbacks((void *)f, vf, initial, ibytes, callbacks);
}


ogg_int64_t g_oggseek = -1;
void oggseek(int pos, int siz)
{
	if(!g_pOggContext || g_oggseek >= 0 || pos < 0 || siz <= 0)
		return;

	g_oggseek = (ogg_int64_t)(((float)pos / (float)siz) *
					(float)ov_pcm_total(&g_pOggContext->vf, -1));	

} // oggseek


void maPlayOgg()
{
	char cstr[256];

	if(!g_state.inpath)
	{
		maLog("maPlayOgg error null filename");
		return;
	}
	
	// format of this decoder output
#ifdef WORDS_BIGENDIAN
	g_state.inwordsbigendian = 1;
#else
	g_state.inwordsbigendian = 0;
#endif
	g_state.ininterleaved = 1;
	
	g_pOggContext = &g_OggContext;
	memset(g_pOggContext, 0, sizeof(OggContext));
	g_OggContext.pstate = &g_state;
	
	// open input
	if(OggOpenFile(g_pOggContext))
	{
		g_pOggContext = NULL;
		sprintf(cstr, "maPlayOgg OggOpenFile failed on '%s'", g_state.inpath);
		maLog(cstr);
		return;
	}
		
	g_state.position = 0;
	
 	if(maOutputOpen() < 0)
	{
		OggCloseFile(g_pOggContext);
		g_pOggContext = NULL;
		maLog( "maPlayOgg maOutputOpen failed");
		return;
	}

	if(!g_posendon)
		g_posend = g_state.duration;

	// ogg decode loop
	while(1)
	{	
		ThreadYield();
		
		if(g_state.stop)
			break;
				
		if(!g_pOggContext)
			break;

		if(g_oggseek >= 0)
		{
			ov_pcm_seek(&g_pOggContext->vf, g_oggseek);
			g_state.position = LROUND(ov_time_tell(&g_pOggContext->vf) * MSPERSEC);	
			g_oggseek = -1;
		}

		OggDecodeStream();
	
		if(g_pOggContext)
			g_state.position = LROUND(ov_time_tell(&g_pOggContext->vf) * MSPERSEC);

		if(!g_pOggContext) // OggDecodeStream sets to NULL at end of stream
			break;

	} // while 1

	// finish

	FlushFifo();

	if(g_pOggContext)
	{
		OggCloseFile(g_pOggContext);
		g_pOggContext = NULL;
	}
	
	g_state.position = 0;
	g_oggseek = -1;
	
} // maPlayOgg


#define CPC_OUTPUTBLOCKSIZE 0x1000 // Block

void OggDecodeStream()
{
	char pcmbuffer[CPC_OUTPUTBLOCKSIZE];
	int moredata;
	int pcmblocksize = CPC_OUTPUTBLOCKSIZE;
	char cstr[256];

	g_state.inchannels = g_pOggContext->m_FileInfo.m_bStereo == 0 ? 1 : 2;
	
	// Get block from Codec
	moredata = OggGetPCMBlock(g_pOggContext, pcmbuffer, &pcmblocksize);
		
	if(pcmblocksize > 0)
	{
		// Write to fifo
		if(maOutputWrite((char *)pcmbuffer, pcmblocksize, &g_state) != 0)
		{
			sprintf(cstr,
			"OggDecodeStream maOutputWrite failed : len=%d g_state.stop=x%x g_fifoready=%d",
												pcmblocksize, g_state.stop, g_fifoready);
			maLog(cstr);
			goto oggdecodereturn;
		}

	} //if(pcmblocksize > 0)

	if(moredata != 0)
		return;

	// song is finished
	if(g_reptsong)
	{
		g_oggseek = g_posbeg;
		return;
	}

oggdecodereturn:
	OggCloseFile(g_pOggContext);
	g_pOggContext = NULL;

} // OggDecodeStream


int OggOpenFile(OggContext *pOggContext)
{
	vorbis_info *pInfo = NULL;
	int ret;
	char cstr[256];
	
	// If a stream is open close it
	if(pOggContext->m_hFile != NULL)
	{
		ov_clear(&pOggContext->vf);
		pOggContext->m_hFile = NULL;
	}

	// Open the new stream
	if(pOggContext->pstate->nomem)
	{
		pOggContext->m_hFile = mafopen((char *)pOggContext->pstate->inpath, "rb");
		if(pOggContext->m_hFile == NULL)
		{
			sprintf(cstr, "OggOpenFile (mafopen) failed on '%s'", pOggContext->pstate->inpath);
			maLog(cstr);
			return 1;
		}
	}
	else
	{
		pOggContext->m_hFile = memfopen(pOggContext->pstate->inpath, "rb");
		if(pOggContext->m_hFile == NULL)
		{
			maLog("OggOpenFile (memfopen) failed");
			return 1;
		}
	}
	
	memset(&pOggContext->vf, 0, sizeof(pOggContext->vf));

	if(pOggContext->pstate->nomem)
		ret = ov_open(pOggContext->m_hFile, &pOggContext->vf, NULL, 0);
	else
		ret = ov_memopen(pOggContext->m_hFile, &pOggContext->vf, NULL, 0);

	if(ret < 0) 
	{
		maLog("OggOpenFile input does not appear to be an Ogg bitstream.");
		if(pOggContext->pstate->nomem)
			fclose(pOggContext->m_hFile);
		else
			memfclose(pOggContext->m_hFile);		
		pOggContext->m_hFile = NULL;
		return 1;
	}

	if(ov_streams(&pOggContext->vf) != 1)
	{
		ov_clear(&pOggContext->vf);
		if(pOggContext->pstate->nomem)
			fclose(pOggContext->m_hFile);
		else
			memfclose(pOggContext->m_hFile);		
		pOggContext->m_hFile = NULL;
		return 1;
	}

	pInfo = ov_info(&pOggContext->vf, -1);

	if(pInfo == NULL)
	{
		maLog("OggOpenFile unable to get ogg info.");
		ov_clear(&pOggContext->vf);
		if(pOggContext->pstate->nomem)
			fclose(pOggContext->m_hFile);
		else
			memfclose(pOggContext->m_hFile);		
		pOggContext->m_hFile = NULL;
		return 1;
	}

	if(pInfo->channels > 2)
	{
		ov_clear(&pOggContext->vf);
		if(pOggContext->pstate->nomem)
			fclose(pOggContext->m_hFile);
		else
			memfclose(pOggContext->m_hFile);		
		pOggContext->m_hFile = NULL;
		return 1;
	}

	// Load m_FileInfo
	pOggContext->m_FileInfo.m_b16bit = 1;
	pOggContext->m_FileInfo.m_iFileLength_Secs = (int)ov_time_total(&pOggContext->vf, -1);
	pOggContext->m_FileInfo.m_bStereo = (pInfo->channels == 2);
	pOggContext->pstate->insamplerate =
				pOggContext->m_FileInfo.m_iBitRate_Kbs = ov_bitrate(&pOggContext->vf, -1);
	pOggContext->m_FileInfo.m_iBitRate_Kbs /= 1000;
	pOggContext->pstate->insamplerate = pOggContext->m_FileInfo.m_iFreq_Hz = pInfo->rate;
	pOggContext->pstate->duration = pOggContext->m_FileInfo.m_iFileLength_Secs * 1000;

	return 0;
	
} // OggOpenFile


void OggCloseFile(OggContext *pOggContext)
{
	if(pOggContext)
	{
		ov_clear(&pOggContext->vf);
		if(pOggContext->m_hFile)
		 	pOggContext->m_hFile = NULL;
	}
	
} // OggCloseFile


int OggGetPCMBlock(OggContext *pOggContext, void *pBlock, int *pBlockSize)
{
	int ret;
	int bytes_to_read = *pBlockSize;

	while(bytes_to_read > 0) 
    { 
		ret = ov_read(&pOggContext->vf, (char *)pBlock + (*pBlockSize - bytes_to_read), bytes_to_read,

#ifdef WORDS_BIGENDIAN
			1, // big endian
#else
			0, // little endian
#endif
			2, 1, &pOggContext->current_section);

		if(ret == 0)
		{ 
			*pBlockSize = 0;
			return 0;
			break; 
		} else if(ret > 0) 
			bytes_to_read -= ret; 
	} 
	
	return 1; // more data to come
	
} // OggGetPCMBlock


signed long ThreadPlayOgg(void *parameter)
{
#ifdef MAUNUSED
 (void)parameter;
#endif

	g_waitaudiodone = 1;

	maPlayOgg();
		
	g_state.stop |= STOP_PLAYDECODE;
	
	// wait for output owned by this thread to finish
	while(!g_stopB && g_waitaudiodone)
		maSleep(100);

	return 0;

} // ThreadPlayOgg


int DumpInfoOgg(MASTATE *pstate, FILE *dumpfp)
{
	int intLength;
	char **ptr;
	OggContext lOggContext = {NULL};
	vorbis_info *vi;
	char tstr[32];
	char cstrx[512];
	
	lOggContext.pstate = pstate;
	
	if(OggOpenFile(&lOggContext))
	{
		sprintf(cstrx, "DumpInfoOgg OggOpenFile failed on '%s'", pstate->inpath);
		maLog(cstrx);
		return 1;		
	}
	
	intLength = LROUND(ov_time_total(&lOggContext.vf, -1));
		
	pstate->duration = intLength * MSPERSEC;
	sprintf(cstrx, "Duration : %s", SecondsToStrHMS(tstr, pstate->duration / MSPERSEC));
	DumpString(dumpfp, cstrx);

	ptr = ov_comment(&lOggContext.vf, -1)->user_comments;
	vi = ov_info(&lOggContext.vf, -1);

	sprintf(cstrx, "Channels=%d", vi->channels);
	DumpString(dumpfp, cstrx);
	
	sprintf(cstrx, "Samplerate=%ld Hz", vi->rate);
	DumpString(dumpfp, cstrx);
	
	if(vi->bitrate_upper > 0)
	{
		sprintf(cstrx, "Bitrate_upper=%ld bps", vi->bitrate_upper);
		DumpString(dumpfp, cstrx);
	}
	if(vi->bitrate_nominal > 0)
	{
		sprintf(cstrx, "Bitrate_nominal=%ld bps", vi->bitrate_nominal);
		DumpString(dumpfp, cstrx);
	}
	if(vi->bitrate_lower > 0)
	{
		sprintf(cstrx, "Bitrate_lower=%ld bps", vi->bitrate_lower);
		DumpString(dumpfp, cstrx);
	}
	if(vi->bitrate_window > 0)
	{
		sprintf(cstrx, "Bitrate_window=%ld bps", vi->bitrate_window);
		DumpString(dumpfp, cstrx);
	}

	sprintf(cstrx,"Decoded length=%ld samples", (long)ov_pcm_total(&lOggContext.vf, -1));
	DumpString(dumpfp, cstrx);

	DumpString(dumpfp, (char *)"ogg tags:");	

	while(*ptr)
	{
		sprintf(cstrx, "%s", *ptr);
		DumpString(dumpfp, cstrx);
		++ptr;
	}
		
	sprintf(cstrx,"Encoded by: %s", ov_comment(&lOggContext.vf, -1)->vendor);
	DumpString(dumpfp, cstrx);
	
	if(lOggContext.m_hFile != NULL)
		ov_clear(&lOggContext.vf);
	
	return 0;

} // DumpInfoOgg

#ifdef __cplusplus
}
#endif

#endif // NODECODEOGG

// eof
