// maPlayer.h

/*
 * MoreAmp
 * Copyright (C) 2004-2008 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef __MAPLAYER_H__
#define __MAPLAYER_H__

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>

#if defined(__MAOSX__) || defined(__MAOS9__)
 #include <MacTypes.h>
#endif

#include "maDefs.h"

# if(defined(__MAOSX__) && !defined(MAUSEMPSERVICES)) || defined(__MACUNIX__)
 // use named semaphore for mac bsd because wait_sem fails
 #define MAFIFOSEMNAME "mafifosemaphore"
#endif

typedef struct
{
	unsigned long size;
	unsigned long contains;
	unsigned long start;
	char *storage;
	int internal;
	void *semaphore;
} maFifo;

typedef struct
{
	int infiletype;
	FILE *fp;
	char inpath[MAX_PATH];			// songfile path
	int nomem;						// don't use song mem
	void *threadplay;				// thread playing inpath
	void *pthreadplayid;			// pointer to MPTaskID on Mac, to HANDLE on win, to pthread_t on unix 
	int threadplayprilo;
	int threadplayprihi;
	int size;						// songfile size in bytes
	int duration;					// total playing time in ms
	int bitrate;					// average bitrate in kbps
	int position;					// current playing position in ms
	double mpegattenuation;			// mp1 mp2 mp3 attenuation when limiter is on (gain 0.0 to 1.0)
	int seek;						// seek target in ms, or -1
	int stop;						// stop flag
	int insamplerate;				// songfile samplerate
	int inchannels;					// songfile number of channels
	int inwordsbigendian;			// songfile decoded data
	int ininterleaved;				// songfile decoded data
	void *fsf;						// aif SNDFILE *
	void *psf;						// aif SF_PRIVATE *
} MASTATE;

// settings
typedef struct
{
	int eqslider[EQBANDS31];
	int eqoffset;			// if adding members here
	char eqname[EQNAMELEN]; // keep eqname at end of struct
} EQUPRESET;

typedef struct
{
	/* don't use VBR
	int encoding;
	int minbitrate;
	int maxbitrate;
	int vbrquality;
	*/
	int type; // kMAMP3, kMAOGG, etc.
	int bitrate;
	int quality;
	int mode;
	int samplerate;
	char outdir[MAX_PATH];
} FILEOUT;								

typedef struct
{
	int autostart;					// start player when app is launched
	int spindown;					// spin down the hard disk
	int random;						// get songs from songlist randomly
	int reptplayed;					// allow to repeat a song when in random mode 
	int looploaded;					// loop on songs that have been loaded in mem or vdisk
	int looplist;					// loop on song list else stop when all have played 
	int highpriority;				// decoder task riority
	int effects; 					// effects active
	int limiter; 					// mad limiter
	int balance;					// free or joined left and right volume controls
	int monodiff;					// L-R in mono; reverse L,R in stereo
	int invertright;				// invert right channel
	int eqflags;					// equalizer flags
	int eqpreset;					// eq preset index
	int listwindow;					// show list window | which to show
	int effectswindow;				// show effects window
	int fileoutwindow;				// show file output window
	int volumesliderleft; 			// volume L value
	int volumesliderright; 			// volume R value
	int separationslider; 			// separation value (stereo) 
	int panslider;					// pan value (mono)
	int preampslider;   			// preamp value (mad)
	EQUPRESET eqpresets[EQNUMPRESETS];  // equalizer values
	int pitchslider;				// pitch effect value
	int pitchslider2;				// pitch effect value (fine)
	int temposlider;				// tempo effect value
	int speedslider;				// speed effect value
	int xpos[NUMWINCONF];   		// player window coordinates
	int ypos[NUMWINCONF];
	int xsize[NUMWINCONF];
	int ysize[NUMWINCONF];
	int eqxpos[NUMWINCONF]; 		// eq window coordinates
	int eqypos[NUMWINCONF];
	int eqxsize[NUMWINCONF];
	int eqysize[NUMWINCONF];
	int effectsxpos[NUMWINCONF];	// effects window coordinates
	int effectsypos[NUMWINCONF];
	int effectsxsize[NUMWINCONF];
	int effectsysize[NUMWINCONF];
	int listxpos[NUMWINCONF];		// list window coordinates
	int listypos[NUMWINCONF];
	int listxsize[NUMWINCONF];
	int listysize[NUMWINCONF];
	char vdiskname[MAX_PATH]; 		// virtual disk name
	int buffertype;					// type of buffering for songs (mem, vdisk, direct)
	int output;						// output to use from Output popup
	int buffersize;					// max mem or vdisk used, in Mbytes
	int titleinfo;					// put info in window's title bar
	int winconf;					// player window configuration
	int singleplay;					// stop afer every song
	FILEOUT fileout[kOUTMAX];		// file output parameters	
	int logflags;					// stop or limit logging
	int logmax;						// max log entries per session
	int logevents;					// max log entries per fast log period
	int logperiod;					// fast log period
	int multipleinst;				// multiple instances
	int keyboard;					// use keyboard
	int listcolxsize[NUMWINCONF][4];// h scrollable if wider than list window 
	int aboutxpos;					// about window coordinates
	int aboutypos;
	int aboutxsize;
	int aboutysize;
	int viewerxpos;					// viewer window coordinates
	int viewerypos;
	int viewerxsize;
	int viewerysize;
	int keybindingsxpos;			// keybindings window coordinates
	int keybindingsypos;
	int keybindingsxsize;
	int keybindingsysize;
	int fileoutxpos[NUMWINCONF];	// fileout window coordinates
	int fileoutypos[NUMWINCONF];
	int fileoutxsize[NUMWINCONF];
	int fileoutysize[NUMWINCONF];
	int advancedxpos;				// advanced window coordinates
	int advancedypos;
	int advancedxsize;
	int advancedysize;
	int mousewheel;					// use mousewheel
	int usem3u;						// use m3u files
	int addmode;					// add new files to existing madrop entries
	int eq2xpos[NUMWINCONF]; 		// eq2 window coordinates
	int eq2ypos[NUMWINCONF];
	int eq2xsize[NUMWINCONF];
	int eq2ysize[NUMWINCONF];
	EQUPRESET eq2presets[EQNUMPRESETS];  // equalizer 2 values
	int future1;					// for future use
	int future2;
	int future3;
	int future4;
	char settingsversion[VERSIONSTRINGSIZE]; // first version to use this settings format, keep this here (last)
} MASETS;

#define MOREAMP_SETTINGS_VERSION_STRING "0.1.22" // first moreamp version to use this settings format
#define MOREAMP_SETTINGS_VERSION_LEN 6

#if !defined(MANOCDA) && defined(__MAUNIX__)
 typedef struct
 {
	int cdparanoia;					// cdparanoia active
	int winx[NUMWINCONF];			// cdrom settings window coordinates
	int winy[NUMWINCONF];
	int winw[NUMWINCONF];
	int winh[NUMWINCONF];
	int item;						// last item selected to set
	char cdaname[CDAMAXNUMDRIVES + 1][32]; // cdrom device names, e.g. '/dev/cdrom'
 } MASETSCDROM;
 extern MASETSCDROM g_setscdrom;
#endif

#if defined(AUDIO_MAC_CORE) /* || defined(AUDIO_PORTAUDIO) */ || defined(AUDIO_WIN_WAVE)
 #define MACLOSESOUNDDEF 0
#else // AUDIO_MAC_SOUNDMGR
 #define MACLOSESOUNDDEF 1 // 0 untested
#endif

#if defined(MAMOUSEWHEEL)
 #if defined(MAGUIGTK)
  #define MAWHEELVOLUMEDELTA (VOLUMEMAX / 20) // 100)
 #else
  #define MAWHEELVOLUMEDELTA (VOLUMEMAX / 20)
 #endif
#endif

#if defined(MAKEYBOARD)
 #define MANUMKEYS 68
 #define MAKEYDELIM '\"'
 #define MAKEYVOLUMEDELTA (VOLUMEMAX / 20)
 typedef struct
 {
	int key;			// the key pressed
	char *label;		// label for internal use
	char *labeld;		// default label for user
	void (*function)(); // the function called
	int menuid;			// menu ids for displaying shortcuts
	int textid;			// text ids for displaying shortcuts
 } MAKEYBINDINGS;
 extern MAKEYBINDINGS g_keybindings[];
#endif
#define MAWXK_UP 317
#define MAWXK_DOWN 319
#define MAWXK_RETURN 13
#define MAWXK_RETURN_NUMPAD 3
#define MAPROMPTMAX 62
#define USEM3UDEF 0 // default off
#define ADDMODEDEF 0 // default off

extern int g_cancel;					// flag to return to stopped state
extern int g_stopB;						// flag set by stop button
extern int g_pause;
extern int g_loadednum;					// number of songs loaded on vdisk or in mem, 0 <= g_loadednum < SONGSLOADMAX
extern char g_hdsongpath[][MAX_PATH];   // full path and name of songs on the hd
extern MASTATE g_state;
extern MASETS g_sets;					// loaded from settings section of settings file
extern MASETS g_setsbu;					// copy of g_sets, updated when settings are loaded or saved
extern int g_k[kOUTMAX];				// indices into g_sets.fileout
extern int g_posend;
extern int g_posbegon;
extern int g_posendon;
extern int g_fifoready;
extern int g_posbeg;
extern int g_reptsong;
extern char g_songlistcda[];			// full path and name of the cda songs list file in songlists dir
extern char g_songlistdrop[];			// full path and name of the dropped songs list file in songlists dir
extern char g_songlisttemp[];			// full path and name of the temp songlist file in songlists dir
extern char g_prognametitle[];			// 'MoreAmp', 'MoreAmp2'..., or 'moreamp', 'moreamp2'
extern char *g_extstr[];
extern char g_songlistsdir[];			// full path and name of dir to contain songlists
extern char g_songlist[];				// full path and name of a songlist file in songlists dir
extern FILE *g_fpsonglist;
extern char g_settingsdir[];			// full path and name of dir to contain settings
extern char g_fileoutdir[];				// full path and name of dir to contain output song files
extern char g_settingspath[];			// full path and name of a settings file
extern char g_settingsdef[];			// full path and name of the default settings file
#if defined(MAKEYBOARD)
 extern char g_keybindingspath[];		// full path and name of the key bindings file
 extern char g_keybindingsfind[];
#endif
extern char g_loadedname[][MAX_PATH];	// names of songs loaded e.g. 'Blue Suede Shoes.mp3'
extern int g_loadedcurrent;				// 0-based index of song loaded currently playing
extern FILE *g_outputfilefp;
extern char g_outputfile[];
extern char *g_sndbuf[];
extern int g_argc;
extern char **g_argv;
extern char g_madir[];					// full path and name of 'moreamp' dir
extern char g_logpath[];
extern char *g_extstrout[];
extern char g_AboutText[];
extern char g_readmepath[];
extern char g_readmefind[];
extern char g_catalogfind[];
extern char g_logfind[];
extern int g_cataloguing;
extern int g_loadedmemsize;
#if defined(__MAOSX__) || defined(__MAOS9__)
 extern Handle g_memhandle;
#else
 extern char *g_memhandle;
#endif
extern int g_running;					// flag to stay in running loop
extern int g_noreload;					// flag to restart using songs already loaded
extern int g_effectschange;
extern unsigned char *g_fxbuffer;
extern int g_bufready[];
extern unsigned long g_nframes[];
#ifdef AUDIO_MAC_SOUNDMGR
 extern int g_buftoplay;
#endif
extern int g_listselect;
extern int g_songsnum;					// number of songs in the songlist
extern void *g_w[];
extern void *g_menubar;
#ifdef AUDIO_WIN_WAVE
 extern void *g_semaudiowin;
#endif
#if defined(MAINSTANCES)
 extern int g_thisinstance;
#endif
#if defined(MAKEYBOARD)
 extern int g_maOnKeySkip;
#endif
extern long g_promptnum;
extern int g_waitaudiodone;
extern int g_addmode;

void maFifoSetStorage(maFifo* pfifo, char *storage);
unsigned long maFifoGetFilledSpace(maFifo* pfifo);
unsigned long maFifoDraw(maFifo* pfifo, char* whereTo, unsigned long requested);
unsigned long maFifoStore(maFifo* pfifo, char* whereFrom, unsigned long requested);
unsigned long maFifoGetFreeSpace(maFifo* pfifo);
int maOutputOpen();
int maOutputWrite(char *buf, int len, MASTATE *pstate);
void FlushFifo();
void DumpString(FILE *fp, char *p);
int MadDoParamChange();
void DumpTagsID3(MASTATE *pstate, FILE *dumpfp);
int ChangeSongList(char *songlist);
int DeleteSongList(char *songlist);
int UpdateRecent(char *item, int del);
int GetRecent(char items[][OPENRECENTLENGTH]);
void SetAppWindowTitle(char *msg);
int PlaySonglist(char *songlist);
int PlaySonglistB(char *songlist);
int LoadDefaultSettings(MASETS *psets);
void OnMenuMakeList();
void OnMenuOpenList();
void OnMenuDeleteList();
void OnMenuSetVDisk();
#if defined(MAINSTANCES)
 void OnMenuMultipleInst(int checked);
#endif
#if defined(MAKEYBOARD)
 void OnMenuKeyboard(int checked);
#endif
#if defined(MAMOUSEWHEEL)
 void OnMenuMouseWheel(int checked);
#endif
void OnMenuPlayFile();
void OnMenuOpenRecent(int index);
void OnMenuSetSaved();
void OnMenuEqWindow();
void OnMenuEffectsWindow();
void OnMenuList();
void OnMenuFileOut();
void OnMenuSpindown(int checked);
void OnMenuRandom(int checked);
void OnMenuReptPlayed(int checked);
void OnMenuLoopLoaded(int checked);
void OnMenuLoopList(int checked);
void OnMenuAutoStart(int checked);
void OnMenuErrLogging(int checked);
void OnMenuSinglePlay(int checked);
void OnMenuHiPriority(int checked);
void OnMenuEqualizer(int checked);
void OnMenuEqExtra(int checked);
void OnMenuEffects(int checked);
void OnMenuMono(int checked);
void OnMenuInvertRight(int checked);
void OnMenuEq31Bands(int checked);
void OnMenuViewCatalog();
void OnMenuReadme();
void OnMenuViewLog();
#if defined(MAKEYBOARD)
 void OnMenuEditKeyBindings();
 void OnMenuBlankKeyBindings();
 void UpdateMenuKbLabels();
 void UpdateButtonLabels();
#endif
void OnMenuClearLog();
void OnMenuAltSize(int checked, int kbcommand);
void OnMenuFactorySize(int kbcommand);
void OnMenuSavedSize(int kbcommand);
void OnMenuDefaultSize(int kbcommand);

signed long RunLoop(void *parameter);
void UpdateMenus();
void UpdateDisplay();
#if defined(__MAOSX__) || defined(__MAOS9__)
 //void DoAEDiskEvent(void *inAppleEvent);
 void InitAppleEvents(void *app);
#endif
int OnInit_();
#if defined(MAINSTANCES)
 int ClearInitLock();
#endif
void OnCheckTitleInfo(int menu);
void OnCheckAltSize(int checked, int kbcommand);
void OnCheckRandom(int checked); 
void OnCheckReptPlayed(int checked);
void OnCheckLoopLoaded(int checked);
void OnCheckLoopList(int checked); 
void OnCheckSpindown(int checked);
void OnCheckHighPriority(int checked); 
void OnCheckSinglePlay(int checked);
void OnCheckEqWindow(int checked); 
void OnCheckListWindow(int checked); 
void OnCheckEqualizer(int checked);
void OnCheckEqualizerB(int checked);
void OnCheckEq31BandsB(int checked);
void OnCheckEqExtraB(int checked);
void OnCheckSeparateLR(int checked);
void OnCheckEffects(int checked);
void OnCheckEffectsB(int checked);
void OnCheckReptSong(int checked); 
void OnCheckBalance(int checked);
void OnCheckMonoLeft(int checked);
void OnCheckMonoRight(int checked);
void OnCheckMonoDiff(int checked);
void RefreshMonoDiff();
void OnCheckLimiter(int checked);
void OnCheckEqExtra(int checked);
void OnCheckEqZoom(int checked);
void OnCheckEqLink(int checked);
void OnCheckInvertRight(int checked);
void OnCheckEq31Bands(int checked);
void OnCheckPitch(int checked);
void OnCheckTempo(int checked);
void OnCheckSpeed(int checked);
void OnCheckQualSpeed(int checked);
void OnCheckQualFilter(int checked);
#if !defined(MANOCDA) && defined(__MAUNIX__)
 void OnCheckCdParanoia(int checked);
 int LoadSettingsCDRom();
#endif
#if defined(MAKEYBOARD)
 int LoadKeyBindings(int showerr);
#endif
int maOnKey(int window, int key);
void OnCheckBoxBeg(int checked);
void OnCheckBoxEnd(int checked);
void OnCheckBoxM4A(int checked);
void OnCheckBoxADTS(int checked);
void OnCheckCloseReopen(int checked);
void OnCheckUseM3u(int checked);
void OnButtonQuit();
void OnButtonStop();
void OnButtonPause();
void OnButtonPrevious();
void OnButtonPlay();
void OnButtonNext();
void OnMenuPlayCD();
void OnMenuMakeCatalog();
void OnMenuSave();
void OnMenuSpindownNow();
void OnMenuEjectCD();
void OnMenuAudioIn();
void OnMenuSaveListAs();
void OnButtonSong();
void OnButtonList();
void OnButtonInfo();
void OnMenuZero();
#if !defined(MANOCDA) && defined(__MAUNIX__)
 void OnButtonAudioCDSet();
 void OnButtonAudioCDClear();
 void OnButtonAudioCDSave();
#endif
void OnMenuAsDefaults();
void OnMenuSetFactory();
void OnMenuSetDefault();
void OnButtonEqPlus();
void OnButtonEqMinus();
void OnButtonEqCopy();
void OnButtonEqPaste();
void OnButtonEqRestore();
void OnButtonEqShiftL();
void OnButtonEqShiftR();
void OnButtonEqInvert();
void OnButtonEqMid();
void OnButtonEq2Copy();
void OnButtonEq2Paste();
void OnButtonEq2Restore();
void OnButtonEq2Mid();
void OnButtonLoaded();
void OnButtonFileOutDir();
void OnButtonPromptNum(int ok);
void OnEditTextPromptNum();
void OnSliderPosition(int value);
void OnSliderVolumeR(int value);
void OnSliderVolumeL(int value);
void OnSliderSeparation(int value);
void OnSliderPreamp(int value);
void OnSliderEq(int slider, int value, int setpos);
void OnSliderEq2(int slider, int value, int setpos);
void OnEditTextEq();
void OnSliderPitch(int value);
void OnSliderPitch2(int value);
void OnSliderTempo(int value);
void OnSliderSpeed(int value);
void OnPopupMem();
void OnPopupVDisk();
void OnPopupDirect();
void OnPopupOutput(int output);
void OnPopupBuffer(int buffer);
void OnPopupEqPreset(int preset);
void OnPopupBitRate(int bitrate);
void OnPopupQuality(int quality);
void OnPopupMode(int mode);
void OnPopupSampleRate(int samplerate);
#if !defined(MANOCDA) && defined(__MAUNIX__)
 void OnPopupAudioCD(int preset);
#endif
void OnLittleArrowBuffer(int value);
void OnLittleArrowVolumeL(int value);
void OnLittleArrowVolumeR(int value);
void OnLittleArrowSeparation(int value);
void OnLittleArrowPreamp(int value);
void OnLittleArrowBegin(int value);
void OnLittleArrowEnd(int value);
void OnLittleArrowEqOffset(int value);
void OnLittleArrowEq(int band, int value);
void OnLittleArrowEq2Offset(int value);
void OnLittleArrowEq2(int band, int value);
void OnLittleArrowPitch(int value);
void OnLittleArrowPitch2(int value);
void OnLittleArrowTempo(int value);
void OnLittleArrowSpeed(int value);
void OnWindowClosedEq();
void OnWindowClosedEq2();
void OnWindowClosedEffects();
#if !defined(MANOCDA) && defined(__MAUNIX__)
 void OnWindowClosedAudioCD();
#endif
void OnWindowClosedList();
void OnWindowClosedFileOut();
void OnWindowClosedPromptNum();
void OnWindowClosedAdvanced();
void OnWindowMoved(int window, int xdelta, int ydelta);
void OnListWindowSingleClick(int row);
void OnCheckShowIndex(int checked);
void OnCheckShowDir(int checked);
void OnCheckShowPath(int checked);
void OnCheckAddMode(int checked);
void OnListWindowDoubleClick();
#if !defined(MANOCDA) && defined(__MAUNIX__)
 void DoWindowAudioCD(int show);
#endif
 void DoWindowAdvanced(int show);
#if defined(MAINSTANCES)
 int InstanceClearAll();
 int InstanceListAll();
 int InstanceClearThis();
 #if defined(MAUSESHM)
  extern void *g_pmashm;
 #endif
#endif
void maLog(const char *str);
int AppWindowMakeA();
int SaveSettings();
void ChangeVolume(int value);
int volumevalL();
int volumevalR();
void OnKeyButtonPrevious();
void OnKeyButtonNext();
int maPromptNumber(char *msg);
void ResetAppTitle();

#ifdef __cplusplus
}
#endif

#endif 	// __MAPLAYER_H__

//eof
