// maUtils.cpp

/*
 * MoreAmp
 * Copyright (C) 2004-2008 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


#include "maConfig.h" // included by all moreamp files

#ifdef __MAWIN__
 #ifdef MAUNUSEDWIN
  #pragma warning( disable : 4100 ) // unreferenced formal parameter
 #endif
 #pragma warning( disable : 4127 ) // conditional expression is constant
 #pragma warning( disable : 4244 ) // conversion from 'x ' to 'y ', possible loss of data
 #pragma warning( disable : 4706 ) // assignment within conditional expression
 #pragma warning( disable : 4996 ) // The POSIX name for this item is deprecated.
#endif

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <stdlib.h>
#include <fcntl.h>
#include <time.h>
#include <signal.h>

#if defined(__MAOS9__)
 #include <unistd.h>
 #include <stat.h>
 #include <unix.h>
 #include <MacTypes.h>
 #include <Multiprocessing.h>
 #include <DriverServices.h>
 #include <LowMem.h>
 #define min(a,b) (a < b ? a : b)
 #include <MacErrors.h>
 #include <MacMemory.h>
 #include <Files.h>
 #include <TextUtils.h>
 #include <Aliases.h>
#endif

#ifdef __MAOSX__
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #if defined(MAINSTANCES) || (defined(__MAOSX__) && !defined(MAUSEMPSERVICES))
  #include <semaphore.h>
 #endif
 #include <MacTypes.h>
 #if defined(MAUSEMPSERVICES)
  #include <Multiprocessing.h>
 #endif
 #include <DriverServices.h>
 #define min(a,b) (a < b ? a : b)
 #include <dirent.h>
#endif

#ifdef __MAUNIX__
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <semaphore.h>
 #include <pthread.h>
 #include <sched.h>
 #define min(a,b) (a < b ? a : b)
 #include <dirent.h>
#endif

#ifdef __MAWIN__
 #include <io.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <process.h>
 #include <SHLOBJ.H>
 #include <SHLWAPI.H>
 #include <DIRECT.H>
 #if defined(MAINSTANCES)
  #include <WINUSER.H>
  #include <windows.h>
  #include <WINPERF.H>
 #endif
 #include <sys/timeb.h>
#endif

 // shared mem
#if defined(MAUSESHM)
 #if defined(__MAWIN__)
  typedef struct
  {
	char *addr;
	int size;
	int alone;
	HANDLE file;
	HANDLE map;
	LPVOID view;
  } MASHM;
 #endif
 #if defined(__MAUNIX__) || defined(__MAOSX__)
  #include <sys/ipc.h>
  #include <sys/shm.h> 
  typedef struct
  {
	char *addr;
	int size;
	int alone;
	key_t key;
	int shmid;
  } MASHM;
 #endif
#endif

#include "maDefs.h"
#include "maGUI.h"
#include "maPlayer.h"
#include "maCda.h"
#include "maUtils.h"

#ifdef __cplusplus
 extern "C"
 {
#endif

#if defined(MAINSTANCES)
 #ifdef __MAOS9__
	int l_pid = 0;
 #endif
 
 int magetpid()
 {
 #ifdef __MAWIN__
	return GetCurrentProcessId();
 #endif

 #ifdef __MAOS9__
	if(!l_pid)
		l_pid = (int)maGetLocalTimeMillis(); // get some system-wide unique int
	return l_pid;
 #endif

 #if defined(__MAOSX__) || defined(__MAUNIX__)
	return getpid();
 #endif

 } // magetpid


// return the number and a pid array of 'moreamp' processes running
// or if pid[0] whether or not 'moreamp' process pid[0] is running
int maNumProcessesWin(int *pid);
int maNumProcesses(int *pid)
{
 #if defined(__MAUNIX__) && !defined(__MACUNIX__)
	int ret;
	char cmd[128];
	char cstr[256];
	// we do this in the shell and we don't return an aray of pids
	if(pid && !*pid)
		return 0;
	if(pid && *pid)
		sprintf(cmd, "exit `ps xc -opid,command | fgrep %d | fgrep %s | fgrep -v fgrep | wc -l`", 
														*pid, MOREAMP_NAME);
	else
		sprintf(cmd, "exit `ps xc -opid,command | fgrep %s | fgrep -v fgrep | wc -l`",
															MOREAMP_NAME);
	if(((ret = system(cmd)) & 0xff) != 0)
	{
		sprintf(cstr, "maNumProcesses system ret=0x%x errno=%d", ret, errno);
		ShowErr(cstr);
		return 0;
	}
	return ret >> 8;
 #endif
 
 #if defined(__MAOSX__) || defined(__MACUNIX__)
	int ret;
	char cmd[128];
	char cstr[256];
	// we do this in the shell and we don't return an aray of pids
	if(pid && !*pid)
		return 0;
	if(pid && *pid)
		sprintf(cmd, "exit `ps xc -opid,command | fgrep %d | fgrep -i %s | fgrep -v fgrep | wc -l`", 
														*pid, MOREAMP_NAME);
	else
		sprintf(cmd, "exit `ps xc -opid,command | fgrep -i %s | fgrep -v fgrep | wc -l`",
															MOREAMP_NAME);
	if(((ret = system(cmd)) & 0xff) != 0)
	{
		sprintf(cstr, "maNumProcesses system ret=0x%x errno=%d", ret, errno);
		ShowErr(cstr);
		return 0;
	}
	return ret >> 8;
 #endif
 
 #if defined(__MAWIN__)
	// not implemented on win9x
	if(maIsWin9x())
		return (pid && *pid) ? 1 : 0;
  	return maNumProcessesWin(pid);
 #endif
 
 // not implemented on os9
 #if defined(__MAOS9__) 
  (void)pid;
	return (pid && *pid) ? 1 : 0;
 #endif
	
} // maNumProcesses
#endif // MAINSTANCES


void ShowErr(const char *msg)
{
	maMessageBox((char *)msg, (char *)"", maOK | maICON_EXCLAMATION);
		
} // ShowErr


// dbsliders (eq and preamp) range from -DBRANGE to +DBRANGE
double SliderDBFromPos(int value)
{
	if(value == ((DBSLIDERMAX - DBSLIDERMIN) / 2))
		return 0.0; // instead of the nearest non-zero db
		
	if(value >= DBSLIDERMAX)
		return DBRANGE;
		
	if(value <= DBSLIDERMIN)
		return -DBRANGE;

	return ((2. * DBRANGE * value) / (DBSLIDERMAX - DBSLIDERMIN)) - DBRANGE;

} // SliderDBFromPos


int SliderPosFromDB(double db)
{
	if(db <= -DBRANGE)
		return DBSLIDERMIN;
		
	if(db >= DBRANGE)
		return DBSLIDERMAX;

	return LROUND(((db + DBRANGE) * (DBSLIDERMAX - DBSLIDERMIN)) / (2. * DBRANGE));
	 
} // SliderPosFromDB


int SongIsInMemory(char *filepath)
{
	int i;
	
	if(!g_memhandle || g_loadedmemsize <= 0)
		return 0;
		
	if(g_sets.buffertype != BUFUSEMEM)
		return 0;
		
	for(i = 0; i < g_loadednum; i++)
	{
		if(!mastrcasecmp(filepath, g_hdsongpath[i]))
			return 1;
	}
		
	return 0;
	
} // SongIsInMemory


int SongListIsCda()
{
	if(!mastrcasecmp(g_songlistcda, g_songlist))
		return 1;
		
	return 0;
	
} // SongListIsCda


int SongFileExists(char *songfile)
{
		if(FileNameHasExtensionPlayable(songfile))
			return maFileExists(songfile);
#ifndef MANOCDA
		// maybe cda
		return IsAudioCd(songfile);
#else
		return 0;
#endif
	
} // SongFileExists 


int PathnameFromFilename(char *filename, char *pathname) // e.g. .../dir/name.ext -> .../dir
{
	int l;
	char *p;

	if(!filename || (l = strlen(filename)) == 0)
		return 1;

	strcpy(pathname, filename);
	p = pathname + l;
	while(l-- > 0)
	{
		p--;
		if(*p == FILESEP)
		{
			*p = 0;
			return 0;
		}
	}
	
	pathname[0] = 0;
	return 1;

} // PathnameFromFilename


int FilenameFromPathname(char *pathname, char *filename) // e.g. .../dir/name.ext -> name.ext
{
	int l;
	char *p;
	
	if(!pathname || (l = strlen(pathname)) == 0)
		return 1;

	p = pathname + l;
	while(l-- > 0)
	{
		p--;
		if(*p == FILESEP)
		{
			p++;
			strcpy(filename, p);
			return 0;
		}
	}
	
	// no file separator, already a filename
	strcpy(filename, pathname);
	
	return 0;

} // FilenameFromPathname


int FileNameRemoveExtension(char *filename)
{
	int l;
	char *p;
	
	if(!filename || (l = strlen(filename)) == 0)
		return 0;

	p = filename + l;
	while(l-- > 0)
	{
		p--;
		if(*p == '.')
		{
			*p = 0;
			return 1; // ext was removed
		}
	}
	
	return 1;

} // FileNameRemoveExtension


int FileNameHasExtension(char *fname, const char *ext)
{
	int i, l, lx;
	
	if(!fname || !ext || !fname[0] || !ext[0] || (lx = strlen(ext)) >= (l = strlen(fname)))
		return 0;
	
	for(i = 0, l -= lx; i < lx; i++)
	{
		if(tolower(fname[l++]) != tolower(ext[i]))
			return 0;
	}
	
	return 1;

} // FileNameHasExtension


int FileNameHasExtensionPlayable(char *fname)
{
	int i;

	for(i = kMAFIRSTPLAYBLE; i <= kMALASTPLAYBLE; i++) // not kMACDA
	{
#ifdef NODECODEMP3
		if(i == kMAMP1) continue;
		if(i == kMAMP2) continue;
		if(i == kMAMP3) continue;
#endif

#ifdef NODECODEOGG
		if(i == kMAOGG) continue;
#endif

#ifdef NODECODEFLAC
		if(i == kMAFLAC) continue;
		if(i == kMAFLA) continue;
#endif

#ifdef NODECODEAIF
		if(i == kMAAIF) continue;
		if(i == kMAAIFF) continue;
#endif

#ifdef NODECODEWAV
		if(i == kMAWAV) continue;
#endif

#ifdef NODECODEAAC
		if(i == kMAAAC) continue;
		if(i == kMAM2A) continue;
#endif

#ifdef NODECODEMP4
		if(i == kMAMP4) continue;
		if(i == kMAM4A) continue;
#endif

#ifdef NODECODEWMA
		if(i == kMAWMA) continue;
#endif

#if !defined(NODECODEAUDIOIN)
#else
		if(i == kMAMAA) continue;
#endif

		if(FileNameHasExtension(fname, g_extstr[i]))
			return 1;
	}
	return 0;
		
} // FileNameHasExtensionPlayable


int FileIsPlayable(char *filepath)
{
	const char *err = NULL;
	char cstr[256];
	
#if defined(__MAOSX__)	
	char lfilepath[MAX_PATH];
	strcpy(lfilepath, filepath);
	StrOSXtoStrBSD(lfilepath);
#else
	char *lfilepath = filepath;
#endif
	
	/*
	// skip if the file is a directory		
	if(maDirExists(lfilepath))
		goto noplay;
	
	// existance	
	if(!maFileExists(lfilepath))
	{
		err = "!maFileExists";
		goto noplay;
	}
	*/

	// check extension playable
	if(!FileNameHasExtensionPlayable(lfilepath) && !FileNameHasExtension(lfilepath, g_extstr[kMACDA]))
	{

#if defined(__MAOSX__) || defined(__MAOS9__) || defined(__MACUNIX__)
		// skip the file if it is named '.DS_Store' or 'Icon^M' or 'Icon' (mac invisible)
		if(!FilenameFromPathname(lfilepath, cstr))
			if(!strcmp(cstr, ".DS_Store") || !strcmp(cstr, "Icon\015") || !strcmp(cstr, "Icon"))
				goto noplay;
#endif

#ifndef MANOCDA
		if(!IsAudioCd(lfilepath))
		{
			err = "not a playable file or on an Audio CD";
			goto noplay;
		}
#else
			err = "not a playable file";
			goto noplay;
#endif
	}
			
	return 1; // playable
	
noplay:

	if(err)
	{
		sprintf(cstr, "FileIsPlayable %s '%s'", err, filepath);
		maLog(cstr);
	}
	
	return 0;

} // FileIsPlayable
	

int maCreateDir(char *dirname)
{  
	if(!dirname || !dirname[0])
	{
		maLog("maCreateDir null dirname");
		return 1;
	}
 
	if(mamkdir(dirname, 0))
	{
		char cstr[256];
		sprintf(cstr, "maCreateDir failed '%s'", dirname);
		maLog(cstr);
		return 1;
	}

#if defined(__MAOSX__) || defined(__MAUNIX__)
	machmod(dirname, ACCESSPERMS); // 0777
#endif

	return(0);

} // maCreateDir


int maFileExists(char *filepath)
{
    struct stat sb;

	if(!filepath || !filepath[0])
	{
		maLog("maFileExists null filepath");
		return 0;
	}
	
#if defined(__MAOSX__)
	char bsdname[MAX_PATH];
	strcpy(bsdname, filepath);
	StrOSXtoStrBSD(bsdname);
    return stat(bsdname, &sb) == 0 && (sb.st_mode & S_IFREG);
#else
    return stat(filepath, &sb) == 0 && (sb.st_mode & S_IFREG);
#endif
		
} // maFileExists


int maDirExists(char *dirpath)
{
    struct stat sb;

	if(!dirpath || !dirpath[0])
	{
		maLog("maDirExists null dirname");
		return 0;
	}
	
#if defined(__MAOSX__)
	char bsdname[MAX_PATH];
	strcpy(bsdname, dirpath);
	StrOSXtoStrBSD(bsdname);
    return stat(bsdname, &sb) == 0 && (sb.st_mode & S_IFDIR);
#else
    return stat(dirpath, &sb) == 0 && (sb.st_mode & S_IFDIR);
#endif
	
} // maDirExists


// maClearDir is only called from FillVirtualDisk, deletes files in dir, not in subdirs
#if defined(__MAOSX__) || defined(__MAUNIX__)
int maClearDir(char *dirpath)
{
	DIR *dir;
	struct dirent *dt;
	char file[MAX_PATH];
	char cstr[256];
	
	if(!dirpath || !dirpath[0])
	{
		maLog("maClearDir null dirpath");
		return 1;
	}
	
	//be safe
	if(!strstr(dirpath, MAVDISKTMP))
	{
		sprintf(cstr, "maClearDir '%s' not in '%s'", MAVDISKTMP, dirpath);
		maLog(cstr);
		return 1;
	}

#if defined(__MAOSX__) || defined(MACUNIX)
	char ldirname[MAX_PATH];
	strcpy(ldirname, dirpath);
	StrOSXtoStrBSD(ldirname);
#else
	char *ldirname = dirpath;
#endif
	
	if((dir = opendir(ldirname)) == NULL)
	{
		sprintf(cstr, "maClearDir opendir failed on '%s'", dirpath);
		maLog(cstr);
		return 0;
	}
	
	while((dt = readdir(dir)))
	{
 		if(strcmp(dt->d_name, ".") && strcmp(dt->d_name, ".."))
		{
			sprintf(file, "%s%c%s", ldirname, FILESEP, dt->d_name);
			if(unlink(file))
			{
				sprintf(cstr, "maClearDir maunlink errno=%d '%s'", errno, file);
				maLog(cstr);
			}		
		}
	}
	
	if(closedir(dir) == -1)
	{
		sprintf(cstr, "maClearDir closedir errno=%d on '%s'", errno, dirpath);
		maLog(cstr);
	}

	return 0;

} // maClearDir (__MAOSX__) || (__MAUNIX__)
#endif


#if defined(__MAWIN__)
int maClearDir(char *dirpath)
{
	HANDLE hFind;
	WIN32_FIND_DATA FindData;
	char dirwild[MAX_PATH];
	char file[MAX_PATH];
	char cstr[256];

	if(!dirpath || !dirpath[0])
	{
		maLog("maClearDir null dirpath");
		return 1;
	}
	
	// be safe
	if(!strstr(dirpath, MAVDISKTMP))
	{
		sprintf(cstr, "maClearDir '%s' not in '%s'", MAVDISKTMP, dirpath);
		maLog(cstr);
		return 1;
	}

	sprintf(dirwild, "%s%c*", dirpath, FILESEP);
	hFind = FindFirstFile(dirwild, &FindData);
	if(hFind == INVALID_HANDLE_VALUE)
		return 1;
	while(1)
	{
		if(strcmp(FindData.cFileName, ".") && strcmp(FindData.cFileName, ".."))
		{
			sprintf(file, "%s%c%s", dirpath, FILESEP, FindData.cFileName);
			if(unlink(file))
			{
				sprintf(cstr, "maClearDir unlink errno=%d '%s'", errno, file);
				maLog(cstr);
			}		
		}
		if(!FindNextFile(hFind, &FindData))
			break;
	}
	FindClose(hFind);

	return 0;

} // maClearDir __MAWIN__
#endif


#if defined(__MAOS9__)
void __ctopstring(char *cstr, Str255 pstr);
void __pstringtoc(Str255 pstr, char *cstr);
void __pstringtoc(Str255 pstr, char *cstr)
{
	pstr[1+pstr[0]] = 0;
	strcpy(cstr, (char *)&pstr[1]);
}
// from MoreFiles:
OSErr maGetCatInfoNoName(short vRefNum, long dirID, ConstStr255Param name, CInfoPBPtr pb);
OSErr maGetDirectoryID(short vRefNum, long dirID, ConstStr255Param name,
							   				long *theDirID, Boolean *isDirectory);
OSErr maDetermineVRefNum(ConstStr255Param pathname, short vRefNum, short *realVRefNum);
OSErr maGetVolumeInfoNoName(ConstStr255Param pathname, short vRefNum, HParmBlkPtr pb);

OSErr maGetVolumeInfoNoName(ConstStr255Param pathname, short vRefNum, HParmBlkPtr pb)
{
	Str255 tempPathname;
	OSErr error;
	
	if( pb != NULL )
	{
		pb->volumeParam.ioVRefNum = vRefNum;
		if( pathname == NULL )
		{
			pb->volumeParam.ioNamePtr = NULL;
			pb->volumeParam.ioVolIndex = 0;
		}
		else
		{
			BlockMoveData(pathname, tempPathname, pathname[0] + 1);
			pb->volumeParam.ioNamePtr = (StringPtr)tempPathname;
			pb->volumeParam.ioVolIndex = -1;
		}
		error = PBHGetVInfoSync(pb);
		pb->volumeParam.ioNamePtr = NULL;
	}
	else
		error = paramErr;
	return (error);
}


OSErr maDetermineVRefNum(ConstStr255Param pathname, short vRefNum, short *realVRefNum)
{
	HParamBlockRec pb;
	OSErr error;

	error = maGetVolumeInfoNoName(pathname,vRefNum, &pb);
	if( error == noErr )
		*realVRefNum = pb.volumeParam.ioVRefNum;

	return (error);
}

							   				
OSErr maGetCatInfoNoName(short vRefNum, long dirID, ConstStr255Param name, CInfoPBPtr pb)
{
	Str31 tempName;
	OSErr error;
	
	if( (name == NULL) || (name[0] == 0) )
	{
		tempName[0] = 0;
		pb->dirInfo.ioNamePtr = tempName;
		pb->dirInfo.ioFDirIndex = -1;
	}
	else
	{
		pb->dirInfo.ioNamePtr = (StringPtr)name;
		pb->dirInfo.ioFDirIndex = 0;
	}
	pb->dirInfo.ioVRefNum = vRefNum;
	pb->dirInfo.ioDrDirID = dirID;
	error = PBGetCatInfoSync(pb);
	pb->dirInfo.ioNamePtr = NULL;
	return (error);
}


OSErr maGetDirectoryID(short vRefNum, long dirID, ConstStr255Param name,
							  				long *theDirID, Boolean *isDirectory)
{
	CInfoPBRec pb;
	OSErr error;

	error = maGetCatInfoNoName(vRefNum, dirID, name, &pb);
	if( error == noErr )
	{
		*isDirectory = (pb.hFileInfo.ioFlAttrib & kioFlAttribDirMask) != 0;
		if( *isDirectory )
			*theDirID = pb.dirInfo.ioDrDirID;
		else
			*theDirID = pb.hFileInfo.ioFlParID;
	}
	
	return (error);
}

int maClearDir(char *dirpath)
{
	int i, n;
	char *p, *q;
	CInfoPBRec cPB;
	OSErr result;
	long theDirID;
	short theVRefNum;
	Boolean isDirectory;
	FSSpec fsspec;
	Str63 itemName;
	Str255 pstr;
	char item[MAX_PATH];
	char file[MAX_PATH];
	char cstr[256];

	if(!dirpath || !dirpath[0])
	{
		maLog("maClearDir null dirpath");
		return 1;
	}
	
	// be safe
	if(!strstr(dirpath, MAVDISKTMP))
	{
		sprintf(cstr, "maClearDir '%s' not in '%s'", MAVDISKTMP, dirpath);
		maLog(cstr);
		return 1;
	}
	
	// setup scan of dir
	sprintf(item, "%s%s", dirpath, ":");
	__ctopstring(item, pstr);
	result = FSMakeFSSpec(0, 0, pstr, &fsspec);
	if(result != noErr)
	{
		sprintf(cstr, "maClearDir FSMakeFSSpec error %d '%s'", result, item);
		maLog(cstr);
		return 1;
	}
	result = maGetDirectoryID(fsspec.vRefNum, fsspec.parID, fsspec.name, &theDirID, &isDirectory);
	if(result != noErr || isDirectory != true)
	{
		sprintf(cstr, "maClearDir maGetDirectoryID error %d '%s'", result, dirpath);
		maLog(cstr);
		return 1;
	}
	result = maDetermineVRefNum(fsspec.name, fsspec.vRefNum, &theVRefNum);
	if(result != noErr)
	{
		sprintf(cstr, "maClearDir DetermineVRefNum error %d '%s'", result, dirpath);
		maLog(cstr);
		return 1;
	}
	cPB.hFileInfo.ioNamePtr = (StringPtr)&itemName;
	cPB.hFileInfo.ioVRefNum = theVRefNum;

	// count files to delete
	n = 0;
	itemName[0] = 0;
	result = noErr;		
	short index = 1;
	do
	{
		cPB.dirInfo.ioFDirIndex = index;
		cPB.dirInfo.ioDrDirID = theDirID;
		result = PBGetCatInfoSync((CInfoPBPtr)&cPB);		
		if(result == noErr)
		{
			__pstringtoc(cPB.hFileInfo.ioNamePtr, file);
			if(strcmp(file, ".DS_Store") && strcmp(file, "Icon\015") && strcmp(file, "Icon"))
				n++;
		}
		++index;	
	} while(result == noErr);
	
	if(n <= 0) 
		return 0; // no files in dir
		
	// build an array of filenames to delete
	if((p = (char *)MAMALLOC(n * MAX_PATH)) == NULL)
	{
		sprintf(cstr, "maClearDir MAMALLOC %d failed", n * MAX_PATH);
		maLog(cstr);
		return 1;
	}			
	i = 0;
	q = p;
	itemName[0] = 0;
	result = noErr;		
	index = 1;
	do
	{
		cPB.dirInfo.ioFDirIndex = index;
		cPB.dirInfo.ioDrDirID = theDirID;
		result = PBGetCatInfoSync((CInfoPBPtr)&cPB);		
		if(result == noErr)
		{
			__pstringtoc(cPB.hFileInfo.ioNamePtr, file);
			if(strcmp(file, ".DS_Store") && strcmp(file, "Icon\015") && strcmp(file, "Icon"))
			{
				sprintf(q, "%s%c%s", dirpath, FILESEP, file);
				q += MAX_PATH;
				if(++i >= n)
					break;
			}
		}
		++index;	
	} while(result == noErr);
	
	// delete n files
	q = p;
	for(i = 0; i < n; i++)
	{
		if(unlink(q))
		{
			sprintf(cstr, "maClearDir unlink errno=%d '%s'", errno, q);
			maLog(cstr);
		}
		q += MAX_PATH;
	}

	MAFREE(p);
	
	return 0;

} // maClearDir __MAOS9__
#endif


int maFileCopy(char *frompath, char *topath, int concat)
{  
	int err = 0;
	FILE *fpfrom, *fpto;
	size_t bytesread;
	char buf[512];
	
	if(!frompath || !frompath[0])
	{
		maLog("maFileCopy null frompath");
		return 1;
	}
	
	if(!topath || !topath[0])
	{
		maLog("maFileCopy null topath");
		return 1;
	}
	
	if((fpfrom = mafopen(frompath, "rb")) == NULL)
	{
		sprintf(buf, "maFileCopy fopen errno=%d '%s'", errno, frompath);
		maLog(buf);
		return 1; // not dskFulErr
	}

	if((fpto = mafopen(topath, concat ? "a" : "wb")) == NULL)
	{
		sprintf(buf, "maFileCopy fopen errno=%d '%s'", errno, topath);
		maLog(buf);
		if(fclose(fpfrom) != 0)
		{
			sprintf(buf, "maFileCopy fclose errno=%d '%s'", errno, frompath);
			maLog(buf);
		}
		return 1;
	}

	if(concat)
	{
		if(fseek(fpto, 0, SEEK_END))
		{
			sprintf(buf, "maFileConcat fseek errno=%d '%s'", errno, topath);
			maLog(buf);
			if(fclose(fpfrom) != 0)
			{
				sprintf(buf, "maFileCopy fclose errno=%d '%s'", errno, frompath);
				maLog(buf);
			}
			if(fclose(fpto) != 0)
			{
				sprintf(buf, "maFileCopy fclose errno=%d '%s'", errno, topath);
				maLog(buf);
			}
			return 1;
		}
		/* if(concat == 1) // check that we concat text files with one newline ...
		} */
	}
	
	errno = 0;
	while((bytesread = fread(buf, 1, sizeof(buf), fpfrom)) > 0)
	{
		if(fwrite(buf, 1, bytesread, fpto) != bytesread)
		{
			sprintf(buf, "maFileCopy fwrite errno=%d '%s'", errno, topath);
			maLog(buf);
			err = 1;
			break;
		}
	}

	if(bytesread <= 0)
	{
		if(!feof(fpfrom))
		{
			sprintf(buf, "maFileCopy fread errno=%d '%s'", errno, frompath);
			maLog(buf);
			err = 2;
		}
	}
	
	if(fclose(fpfrom) != 0)
	{
		sprintf(buf, "maFileCopy fclose errno=%d '%s'", errno, frompath);
		maLog(buf);
		err = 3;
	}
	
	if(fclose(fpto) != 0)
	{
		sprintf(buf, "maFileCopy fclose errno=%d '%s'", errno, topath);
		maLog(buf);
		err = 4;
	}

#if defined(__MAOSX__) || defined(__MAUNIX__)
	if(machmod(topath, ACCESSPERMS) == -1) // 0777
	{
		sprintf(buf, "maFileCopy machmod errno=%d '%s'", errno, topath);
		maLog(buf);
		err = 5;
	}
#endif
	
	return err;
	
} // maFileCopy


int maFileConcat(char *frompath, char *topath)
{  
	int err = 0;
	FILE *fpfrom, *fpto;
	size_t bytesread;
	char buf[512];
	
	if(!frompath || !frompath[0])
	{
		maLog("maFileConcat null frompath");
		return 1;
	}
	
	if(!topath || !topath[0])
	{
		maLog("maFileConcat null topath");
		return 1;
	}
	
	if((fpfrom = mafopen(frompath, "rb")) == NULL)
	{
		sprintf(buf, "maFileConcat fopen errno=%d '%s'", errno, frompath);
		maLog(buf);
		return 1;
	}

	if((fpto = mafopen(topath, "wb")) == NULL)
	{
		sprintf(buf, "maFileConcat fopen errno=%d '%s'", errno, topath);
		maLog(buf);
		if(fclose(fpfrom) != 0)
		{
			sprintf(buf, "maFileConcat fclose errno=%d '%s'", errno, frompath);
			maLog(buf);
		}
		return 1;
	}

	if(fseek(fpto, 0, SEEK_END))
	{
		sprintf(buf, "maFileConcat fseek errno=%d '%s'", errno, topath);
		maLog(buf);
		return 1;
	}
	
	errno = 0;
	while((bytesread = fread(buf, 1, sizeof(buf), fpfrom)) > 0)
	{
		if(fwrite(buf, 1, bytesread, fpto) != bytesread)
		{
			sprintf(buf, "maFileConcat fwrite errno=%d '%s'", errno, topath);
			maLog(buf);
			err = 1;
			break;
		}
	}

	if(bytesread <= 0)
	{
		if(!feof(fpfrom))
		{
			sprintf(buf, "maFileConcat fread errno=%d '%s'", errno, frompath);
			maLog(buf);
			err = 2;
		}
	}
	
	if(fclose(fpfrom) != 0)
	{
		sprintf(buf, "maFileConcat fclose errno=%d '%s'", errno, frompath);
		maLog(buf);
		err = 3;
	}
	
	if(fclose(fpto) != 0)
	{
		sprintf(buf, "maFileConcat fclose errno=%d '%s'", errno, topath);
		maLog(buf);
		err = 4;
	}

#if defined(__MAOSX__) || defined(__MAUNIX__)
	if(machmod(topath, ACCESSPERMS) == -1) // 0777
	{
		sprintf(buf, "maFileConcat machmod errno=%d '%s'", errno, topath);
		maLog(buf);
		err = 5;
	}
#endif
	
	return err;
	
} // maFileConcat


// returns 1-based drive index if fname is a drive or a track
// and the drive is ready, else 0
#if defined(__MAUNIX__)
 int CdIsInited = 0;
#endif
int IsAudioCd(char *fname)
{
#ifdef MANOCDA
	return 0;
#else
	int i;
 #if !defined(__MAUNIX__)
	char *p;
	char name[MAX_PATH];
	char cdaname[MAX_PATH];
 #endif
	
	if(!fname || !fname[0])
	{
		maLog("IsAudioCd null fname");
		return 0;	
	}

 #if defined(__MAOSX__) // on osx ignore first '/'
	if(fname[0] == VOLSEP)
		strcpy(name, &fname[1]);
	else
		strcpy(name, fname);
	if((p = strchr(name, FILESEP)))
		*p = 0;
	for(i = 1; i <= g_cdanumdrives; i++)
	{
		strcpy(cdaname, g_cdadrive[i].name);
		if((p = strchr(cdaname, FILESEP)))
			*p = 0;
		if(!strcmp(name, cdaname))
			if(g_cdadrive[i].isready)
				return i;
	}
	return 0;
 #elif defined(__MAUNIX__)
	if(!CdIsInited)
	{
		CdIsInited = 1;
		if(g_cdanumdrives < 1)
		{
			LoadSettingsCDRom();
			CdaInitCDRom();
		}
	}
	for(i = 1; i <= g_cdanumdrives; i++)
	{
		if(strstr(fname, g_cdadrive[i].name) == fname)
			if(g_cdadrive[i].isready)
				return i;
	}
	return 0;
 #else // __MAWIN__, __MAOS9__
	strcpy(name, fname);
	if((p = strchr(name, FILESEP)))
		*p = 0;
	for(i = 1; i <= g_cdanumdrives; i++)
	{
		strcpy(cdaname, g_cdadrive[i].name);
		if((p = strchr(cdaname, FILESEP)))
			*p = 0;
		if(!strcmp(name, cdaname))
			if(g_cdadrive[i].isready)
				return i;
	}
	return 0;
 #endif
#endif
} // IsAudioCd


char *SecondsToStrHMS(char *strhms, int xs)
{
	int s, xm, m, h;

	if(strhms)
	{
		if(xs >= 0)
		{
			s = xs % SECSPERMIN;
			xm = (xs - s) / SECSPERMIN;
			m = xm % MINSPERHOUR;
			h = (xm - m) / MINSPERHOUR;
				
			if(h > 0)
				sprintf(strhms, "%d:%02d:%02d", h, m, s);
			else
				sprintf(strhms, "%d:%02d", m, s);
		}
		else
			sprintf(strhms, "?");
	}
	return strhms;
	
} // SecondsToStrHMS


char *MillisecondsToStrHMS(char *strhms, int ms)
{
	int d, xs, s, xm, m, h;
	
	if(strhms)
	{
		d = (ms % MSPERSEC) / MSPERDS;
		xs = ms / MSPERSEC;
		s = xs % SECSPERMIN;
		xm = (xs - s) / SECSPERMIN;
		m = xm % MINSPERHOUR;
		h = (xm - m) / MINSPERHOUR;
		sprintf(strhms, "%02d:%02d:%02d.%d", h, m, s, d);
	}	
	return strhms;
	
} // MillisecondsToStrHMS


int StrHMSToMilliSeconds(char *strhms)
{
	char cstr[32];

	if(strlen(strhms) != strlen(ZEROMSSTR)) // "00:00:00.0"
		return -1;
	
	if(!isdigit(strhms[ 0])) return -1; // hours
	if(!isdigit(strhms[ 1])) return -1;
	if(strhms[2] != ':') return -1;
	
	if(!isdigit(strhms[ 3])) return -1; // minutes
	if(!isdigit(strhms[ 4])) return -1;
	if(strhms[5] != ':') return -1;
	
	if(!isdigit(strhms[ 6])) return -1; // seconds
	if(!isdigit(strhms[ 7])) return -1;
	if(strhms[8] != '.') return -1;
	
	if(!isdigit(strhms[ 9])) return -1; // milliseconds
	
	strcpy(cstr, strhms);
	cstr[2] = cstr[5] = cstr[8] = 0;
	
	return atoi(cstr)     * MSPERHOUR + atoi(&cstr[3]) * MSPERMIN +
		   atoi(&cstr[6]) * MSPERSEC  + atoi(&cstr[9]) * MSPERDS;

} // StrHMSToMilliSeconds


FILE *mafopen(char *filename, const char *mode)
{
#if defined(__MAOSX__)  
	char bsdname[MAX_PATH];
	strcpy(bsdname, filename);
	StrOSXtoStrBSD(bsdname);
	return fopen(bsdname, mode);
#else
	return fopen(filename, mode);
#endif
}


#if defined(__MAOSX__) || defined(__MAUNIX__)
int machmod(char *filename, int mode)
{
 #if defined(__MAOSX__)  
	char bsdname[MAX_PATH];
	strcpy(bsdname, filename);
	StrOSXtoStrBSD(bsdname);
	return chmod(bsdname, mode);
 #else
	return chmod(filename, mode);
 #endif
}
#endif


int mamkdir(char *filename, int mode)
{
#if defined(__MAOSX__)  
	char bsdname[MAX_PATH];
	strcpy(bsdname, filename);
	StrOSXtoStrBSD(bsdname);
	return mkdir(bsdname, mode);
#else
 #if defined(__MAOS9__) || defined(__MAUNIX__)
	return mkdir(filename, mode);
 #else // __MAWIN__
	return !CreateDirectory(filename, 0);
 #endif
#endif
}


int maunlink(char *filename)
{
#if defined(__MAOSX__)  
	char bsdname[MAX_PATH];
	strcpy(bsdname, filename);
	StrOSXtoStrBSD(bsdname);
	return unlink(bsdname);
#else
	return unlink(filename);
#endif
}


char *mafgets(char *s, int n, FILE *fp)
{
	int i, j, k, c, d;
	char *p = s;
	char cstr[256];
	
	i = 0;
	errno = 0;
	while(!(c = 0) && i < n && (c = fgetc(fp)) != EOF)
	{
		if(errno)
		{
			sprintf(cstr, "mafgets 1 fgetc errno=%d", errno);
			maLog(cstr);
			return NULL;
		}
		
		if(c == '\r' || c == '\n') // eol
		{
			j = 0; // skip over next eol chars if any
			k = i;
			while(!(d = 0) && k++ < n && (d = fgetc(fp)) != EOF && (d == '\r' || d == '\n'))
				j++;
				
			if(errno)
			{
				sprintf(cstr, "mafgets 2 fgetc errno=%d", errno);
				maLog(cstr);
				return NULL;
			}
				
			if(d && d != EOF)
				ungetc(d, fp); //fseek(fp, -1, SEEK_CUR);
			
			break;
			
		} // eol
		
		*p = (char)c;
		p++;
		i++;
	}
	
	if(i == 0 && c == EOF)
		return NULL;
		
	*p = 0;
	
	return s;
	
} // mafgets


int mastrcasecmp(char *s1, char *s2)
{
	int i, l;

	if(!s1 || !s2)
		return 1;

	if((l = strlen(s1)) != (int)strlen(s2))
		return 1;

	for(i = 0; i < l; i++)
		if(tolower(s1[i]) != tolower(s2[i]))
			return 1;

	return 0;

} // mastrcasecmp


int mastrncasecmp(char *s1, char *s2, int n)
{
	int i, l1, l2;

	if(!s1 || !s2)
		return 1;

	if((l1 = strlen(s1)) != (l2 = strlen(s2)))
	{
		if(n > min(l1, l2))
			return 1;
		l1 = n;
	}
	else
		l1 = min(n, l2);

	for(i = 0; i < l1; i++)
	{
		if(tolower(s1[i]) != tolower(s2[i]))
			return 1;
	}

	return 0;

} // mastrncasecmp


char *mastrcasestr(char *s1, char *s2)
{
	unsigned char *i, *j;
	unsigned char *pos = (unsigned char *)s1;
	
mastrcasestrin:
	while(*pos != '\0')
	{
		if(tolower(*pos++) == tolower((unsigned char)*s2))
		{
			i = pos;
			j = (unsigned char *)s2 + 1;
			
			while(*j)
				if(tolower(*i++) != tolower(*j++))
					goto mastrcasestrin;
					
			return (char*)pos - 1;
		}
	}
 
	return NULL;
 
} // mastrcasestr


#if defined(__MAOSX__)
 #include <MacApplication.h> // for GetApplicationTextEncoding
TextEncoding g_appencoding = NULL;
int StrOSXtoSBSD(char *pfilepath, int direction);
int StrOSXtoSBSD(char *pfilepath, int direction)
{
	int err = 0;
	int done = 0;
	const char *p;
	char bdsfilepath[MAX_PATH];
	char lpath[MAX_PATH];
	char lfilepath[MAX_PATH];
	char *lpa, *lpb;
	TextEncoding encin, encout;
	
	if(!g_appencoding)
		g_appencoding = GetApplicationTextEncoding();
	
	if(direction)
	{
		encin = kCFStringEncodingUTF8;
		encout = g_appencoding;
	}
	else
	{
		encin = g_appencoding;
		encout = kCFStringEncodingUTF8;
	}
	
	strcpy(lpath, pfilepath);
	
	lpa = lpath;
	
	if(lpath[0] == '/')
	{
		lfilepath[0] = '/';
		lfilepath[1] = 0;
		lpa++;
	}
	else
		lfilepath[0] = 0;
	
	if((lpb = strchr(lpa, '/')) == NULL)
	{
		lpb = lpa + strlen(lpa);
		done = 1;
	}
	
	*lpb = 0;
	
	// do this one dir at a time
	while(1)
	{
		// string to cnvert is from lpa to *lpb=0
		
		CFStringRef cfstr = CFStringCreateWithCString(NULL, (const char *)lpa, encin);
		if(!cfstr)
			return 0;
			
		if((p = CFStringGetCStringPtr(cfstr, encout)))
			strcat(lfilepath, p);
		else
		{
			if(CFStringGetCString(cfstr, bdsfilepath, MAX_PATH, encout))
				strcat(lfilepath, bdsfilepath);
			else
			{
				err = 1;
				break;
			}
		}
		
		if(done)
			break;
			
		lpa = lpb + 1;
		
		if((lpb = strchr(lpa, '/')) == NULL)
		{
			lpb = lpa + strlen(lpa);
			done = 1;
		}

		strcat(lfilepath, "/");

		*lpb = 0;
				
	} // while 1
	
	// release cfstr ...
	
	if(err)
		return 0;
	
	strcpy(pfilepath, lfilepath);
	
	return 1; // was munged
	
} // StrOSXtoSBSD


int StrOSXtoStrBSD(char *pfilepath)
{
	StrOSXtoSBSD(pfilepath, 1);
	return StrOSXtoSBSD(pfilepath, 0);
	
} // StrOSXtoStrBSD


int StrBSDtoStrOSX(char *pfilepath)
{
	return StrOSXtoSBSD(pfilepath, 1);
	
} // StrBSDtoStrOSX


int maGetRootVolumeName(char *name)
{
	FSRef fsRef;
	FSSpec spec;
	OSStatus err = FSPathMakeRef((const unsigned char *)"/", &fsRef, NULL);
	if(err != noErr)
		return 1;
	err = FSGetCatalogInfo(&fsRef, kFSCatInfoNone, NULL, NULL, &spec, NULL);
	if(err != noErr)
		return 1;
	// deprecated p2cstrcpy(name, spec.name);
	int i = spec.name[0];
	name[i] = 0;
	while(i > 0)
	{
		name[i - 1] = spec.name[i];
		i--;
	}
	return 0;
} // maGetRootVolumeName
#else // !__MAOSX__
int StrOSXtoStrBSD(char *pfilepath)
{
 #ifdef MAUNUSED
 (void)pfilepath;
 #endif
	
	return 1;
} // StrOSXtoStrBSD


int StrBSDtoStrOSX(char *pfilepath)
{
 #ifdef MAUNUSED
 (void)pfilepath;
 #endif
	
	return 1;
} // StrBSDtoStrOSX
#endif // !__MAOSX__


#ifdef __MAWIN__
int maGetLongPathName(char *infname, char *outfname, int len)
{
	return ::GetLongPathName(infname, outfname, len);
}


int maWinResolveShortcut(char *shortcut, char *pathname)
{
	LPCSTR pszShortcutFile;
	HRESULT hres;
	IShellLink *psl;
	char szGotPath [MAX_PATH];
	WIN32_FIND_DATA wfd;
	int result = 1; // error

	pszShortcutFile = pathname;

	// Get a pointer to the IShellLink interface.
	hres = CoCreateInstance(CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER,
											IID_IShellLink, (void **)&psl);
	if(SUCCEEDED (hres))
	{
		IPersistFile *ppf;

		// Get a pointer to the IPersistFile interface.
		hres = psl->QueryInterface (IID_IPersistFile, (void **)&ppf);

		if(SUCCEEDED(hres))
		{
			WORD wsz[MAX_PATH + MAX_PATH]; // buffer for Unicode string

			// Ensure that the string consists of Unicode characters.
			MultiByteToWideChar(CP_ACP, 0, pszShortcutFile, -1, (LPWSTR)&wsz[0], MAX_PATH);

			// Load the shortcut.
			hres = ppf->Load((LPCOLESTR)&wsz[0], STGM_READ);

			if(SUCCEEDED(hres))
			{
				// Resolve the shortcut.
				hres = psl->Resolve(NULL, SLR_ANY_MATCH);
				if(SUCCEEDED(hres))
				{
					strcpy(szGotPath, pszShortcutFile);
					// Get the path to the shortcut target.
					hres = psl->GetPath(szGotPath, MAX_PATH, (WIN32_FIND_DATA *)&wfd, SLGP_SHORTPATH);
					if(!SUCCEEDED (hres))
						;
					else
					{
						if(::GetLongPathName(szGotPath, pathname, MAX_PATH)) // returns length
							result = 0; // ok
					}
				}
			}
			// Release the pointer to IPersistFile.
			ppf->Release();
		}
		// Release the pointer to IShellLink.
		psl->Release();
	}
	
	return result;

} // maWinResolveShortcut
#endif


#if defined(MAINSTANCES) || (defined(__MAOSX__) && !defined(MAUSEMPSERVICES)) || defined(__MACUNIX__)
int maNamedSemaphoreOpen(void **ppsem, char *name)
{
 #if defined(__MAWIN__) || defined(__MAOS9__)
	int err;
#endif	
	char cstr[256];

	if(!ppsem)
	{
		maLog("maNamedSemaphoreOpen ppsem null");
		return 1;
	}
	
	if(!name || !name[0])
	{
		maLog("maSemaphoreOpen name null");
		return 1;
	}
	
	// signaled
 #if defined(__MAOSX__) || defined(__MAUNIX__)
	if((*ppsem = sem_open(name, O_CREAT, 0666, 1)) == (sem_t *)SEM_FAILED)
	{
		*ppsem = NULL;
		sprintf(cstr, "maNamedSemaphoreOpen sem_open '%s' errno=%d", name, errno);
		maLog(cstr);
		return 1;
	}
 #endif

 #if defined(__MAWIN__)
	*ppsem = CreateMutex(NULL, FALSE, name);
	err = GetLastError();
	if(err && err != ERROR_ACCESS_DENIED && err != ERROR_ALREADY_EXISTS)
 	{
		*ppsem = NULL;
		sprintf(cstr, "maNamedSemaphoreOpen CreateMutex '%s' err=%d", name, err);
		maLog(cstr);
		return 1;
	}
 #endif

 #if defined(__MAOS9__)
	if((err = MPCreateBinarySemaphore((MPSemaphoreID*)ppsem)) != noErr) // MP semaphores not system-wide
	{
		*ppsem = NULL;
		sprintf(cstr, "maNamedSemaphoreOpen MPCreateBinarySemaphore '%s' err=%d", name, err);
		maLog(cstr);
		return 1;
	}
 #endif

	return 0;
	
} // maNamedSemaphoreOpen


# if(defined(__MAOSX__) && !defined(MAUSEMPSERVICES)) || defined(__MACUNIX__)
int maNamedSemaphoreUnlink(void *psem, char *name)
{
#ifdef MAUNUSED
 (void)psem;
#endif

	if(!name || !name[0])
	{
		maLog("maNamedSemaphoreUnlink name null");
		return 1;
	}
	
	/* EINVAL 22 Invalid argument
	if(sem_unlink(name) == -1)
	{
		char cstr[256];
		sprintf(cstr, "maNamedSemaphoreUnlink sem_unlink '%s' errno=%d", name, errno);
		maLog(cstr);
		return 1;
	} */

	return 0;
	
} // maNamedSemaphoreUnlink
#endif
 

int maNamedSemaphorePost(void *psem, char *name)
{
#ifdef MAUNUSED
 (void)name;
#endif

	if(!psem)
	{
		maLog("maNamedSemaphorePost psem null");
		return 1;
	}

#if defined(__MAWIN__)
	ReleaseMutex(psem);
#endif

#if defined(__MAOS9__)
	MPSignalSemaphore((MPSemaphoreID)psem);
#endif

#if defined(__MAOSX__) || defined(__MAUNIX__)	
	sem_post((sem_t *)psem);
#endif

	return 0;
	
} // maNamedSemaphorePost


int maNamedSemaphoreWait(void *psem, char *name)
{
#ifdef MAUNUSED
 (void)name;
#endif

	if(!psem)
	{
		maLog("maNamedSemaphoreWait psem null");
		return 1;
	}

#if defined(__MAWIN__)
	switch (WaitForSingleObject(psem, INFINITE))
	{
		case WAIT_OBJECT_0:
			return 0;
		break;

		case WAIT_ABANDONED:
			//maLog("maNamedSemaphoreWait WaitForSingleObject abandoned");
			return 0;
		break;

		case WAIT_TIMEOUT:
			maLog("maNamedSemaphoreWait WaitForSingleObject timeout");
		break;

		case WAIT_FAILED:
		default:
			maLog("maNamedSemaphoreWait WaitForSingleObject failed");
		break;
	}
		return 1;
#endif

#if defined(__MAOS9__)
	MPWaitOnSemaphore((MPSemaphoreID)psem, kDurationForever);
	return 0;
#endif
 	
#if defined(__MAOSX__) || defined(__MAUNIX__)	
	sem_wait((sem_t *)psem);
	return 0;
#endif

} // maNamedSemaphoreWait
#endif // MAINSTANCES ...


/*
int maSemaphoreUnlink(void *psem, char *name)
{
#ifdef MAUNUSED
 (void)name;
#endif
	char cstr[256];
	
	if(!psem)
	{
		maLog("maSemaphoreUnlink null psem");
		return 1;
	}
	
	if(maSemaphoreDelete(psem))
	{
		sprintf(cstr, "maSemaphoreUnlink maSemaphoreDelete '%s' failed", name);
		maLog(cstr);
		return 1;
	}

	return 0;
	
} // maSemaphoreUnlink
*/


int maSemaphoreNew(void **ppsem)
{
	if(!ppsem)
	{
		maLog("maSemaphoreNew ppsem null");
		return 1;
	}
	
	if(*ppsem)
	{
		maLog("maSemaphoreNew *ppsem not null");
		return 1;
	}
	
#if defined(__MAWIN__)
	if((*ppsem = CreateEvent(0, FALSE, TRUE, 0)) == NULL)
	{
		maLog("maSemaphoreNew CreateEvent failed");
		return 1;
	}
#endif

#if(defined(__MAOSX__) && defined(MAUSEMPSERVICES)) || defined(__MAOS9__)
	if(MPCreateBinarySemaphore((MPSemaphoreID*)ppsem) != noErr)
	{
		*ppsem = NULL;
		maLog("maSemaphoreNew MPCreateBinarySemaphore failed");
		return 1;
	}
#endif

#if(defined(__MAOSX__) && !defined(MAUSEMPSERVICES)) || defined(__MAUNIX__)
	if((*ppsem = MAMALLOC(sizeof(sem_t))) == NULL)
	{
		maLog("maSemaphoreNew MAMALLOC failed");
		return 1;
	}
	sem_init((sem_t *)*ppsem, 0, 0);
#endif

	return 0;
	
} // maSemaphoreNew


int maSemaphoreDelete(void *psem)
{
	if(!psem)
	{
		maLog("maSemaphoreDelete psem null");
		return 1;
	}

#if defined(__MAWIN__)
	CloseHandle(psem);
#endif

#if(defined(__MAOSX__) && defined(MAUSEMPSERVICES)) || defined(__MAOS9__)
	MPDeleteSemaphore((MPSemaphoreID)psem);
#endif

#if(defined(__MAOSX__) && !defined(MAUSEMPSERVICES)) || defined(__MAUNIX__)
	sem_destroy((sem_t *)psem); // sem_close
	MAFREE(psem);
#endif

	return 0;
	
} // maSemaphoreDelete


int maSemaphorePost(void *psem)
{
	if(!psem)
	{
		maLog("maSemaphorePost psem null");
		return 1;
	}

#if defined(__MAWIN__)
	SetEvent(psem);
#endif

#if(defined(__MAOSX__) && defined(MAUSEMPSERVICES)) || defined(__MAOS9__)
	MPSignalSemaphore((MPSemaphoreID)psem);
#endif

#if(defined(__MAOSX__) && !defined(MAUSEMPSERVICES)) || defined(__MAUNIX__)
	sem_post((sem_t *)psem);
#endif

	return 0;
	
} // maSemaphorePost


int g_errMPWaitOnSemaphore = 0;
int maSemaphoreWait(void *psem)
{
	if(!psem)
	{
		maLog("maSemaphoreWait psem null");
		return 1;
	}

#if defined(__MAWIN__)
	switch (WaitForSingleObject(psem, INFINITE))
	{
		case WAIT_OBJECT_0:
			return 0;
		break;

		case WAIT_ABANDONED:
			if(!g_errMPWaitOnSemaphore++) // once only
				maLog("maSemaphoreWait WaitForSingleObject abandoned");
		break;

		case WAIT_TIMEOUT:
			if(!g_errMPWaitOnSemaphore++)
				maLog("maSemaphoreWait WaitForSingleObject timeout");
		break;

		case WAIT_FAILED:
		default:
			if(!g_errMPWaitOnSemaphore++)
				maLog("maSemaphoreWait WaitForSingleObject failed");
		break;
	}
		return 1;
#endif

#if(defined(__MAOSX__) && defined(MAUSEMPSERVICES)) || defined(__MAOS9__)
	MPWaitOnSemaphore((MPSemaphoreID)psem, kDurationForever);
	return 0;
#endif
 	
#if(defined(__MAOSX__) && !defined(MAUSEMPSERVICES)) || defined(__MAUNIX__)
	sem_wait((sem_t *)psem);
	return 0;
#endif

} // maSemaphoreWait


int maSemaphoreWaitTimeout(void *psem, int timeout)
{
	if(!psem)
	{
		maLog("maSemaphoreWaitTimeout psem null");
		return 1;
	}
	
#if defined(__MAWIN__)
	switch (WaitForSingleObject(psem, timeout))
	{
		case WAIT_OBJECT_0:
			return 0;
		break;

		case WAIT_ABANDONED:
			if(!g_errMPWaitOnSemaphore++) // once only
				maLog("maSemaphoreWaitTimeout WaitForSingleObject abandoned");
		break;

		case WAIT_TIMEOUT:
			if(!g_errMPWaitOnSemaphore++)
				maLog("maSemaphoreWaitTimeout WaitForSingleObject timeout");
		break;

		case WAIT_FAILED:
		default:
			if(!g_errMPWaitOnSemaphore++)
				maLog("maSemaphoreWaitTimeout WaitForSingleObject failed");
		break;
	}
	return 1;
#endif

#if(defined(__MAOSX__) && defined(MAUSEMPSERVICES)) || defined(__MAOS9__)
	MPWaitOnSemaphore((MPSemaphoreID)psem, timeout);
	return 0;
#endif
	
#if(defined(__MAOSX__) && !defined(MAUSEMPSERVICES)) || defined(__MAUNIX__)
	maLog("maSemaphoreWaitTimeout not implemented");
	return 1;
#endif

} // maSemaphoreWaitTimeout


void ThreadYield() // the calling thread wants to give up its time slice
{
#if(defined(__MAOSX__) && defined(MAUSEMPSERVICES)) || defined(__MAOS9__)
	MPYield();
#endif

#ifdef __MAWIN__
	Sleep(0);
#endif

#if(defined(__MAOSX__) && !defined(MAUSEMPSERVICES)) || defined(__MAUNIX__)
	usleep(0);
#endif

} // ThreadYield


void AppYield() 
{
	guiAppYield();
	
} // AppYield


int maSleep(unsigned int millis)
{
#if defined(__MAOSX__) || defined(__MAOS9__)
	MPEventID event;
	char cstr[256];

	OSStatus status = MPCreateEvent(&event);	
	if(status)
	{
		sprintf(cstr, "maSleep %d MPCreateEvent error %d", millis, (int)status);
		maLog(cstr);
		return 1;
	}
	status = MPWaitForEvent(event, NULL, millis);
	if(status && status != kMPTimeoutErr) // -29296)
	{
		sprintf(cstr, "maSleep %d MPWaitForEvent error %d", millis, (int)status);
		maLog(cstr);
	}
	MPDeleteEvent(event); 
 #ifdef __MAOS9__
	AppYield();
 #endif
 	if(status)
		return 1;
#endif

#ifdef __MAWIN__
	Sleep(millis);
#endif

#if defined(__MAUNIX__)
	if(usleep(millis * 1000) != 0
 #if defined(MAGUINONE)
		&& errno != EINTR // 4, when we ctl-C out
 #endif
		) //
	{
		char cstr[256];
		sprintf(cstr, "maSleep %d usleep errno=%d", millis * 1000, errno);
		maLog(cstr);
		return 1;
	}
#endif

	return 0;
	
} // maSleep


#if defined(MAINSTANCES)
int maNamedLock(MALOCK *lock)
{
#ifdef __MAOS9__ // MP semaphores not system-wide
 (void)lock;
#else
	char cstr[256];

	if(!lock)
	{
		ShowErr("maNamedLock null lock");
		return 1;
	}

	if(!lock->psem)
	{
		if(!lock->name[0])
		{
			ShowErr("maNamedLock null lock name");
			return 1;
		}

		if(maNamedSemaphoreOpen(&(lock->psem), lock->name))
		{   
			sprintf(cstr, "maNamedLock '%s' maNamedSemaphoreOpen failed", lock->name);
			ShowErr(cstr);
			return 1;
		}
	}

	if(maNamedSemaphoreWait(lock->psem, lock->name))
	{
		sprintf(cstr, "maNamedLock '%s' maSemaphoreWait failed", lock->name);
		ShowErr(cstr);
		return 1;
	}
	
#endif

	return 0;

} // maNamedLock


int maNamedUnlock(MALOCK *lock)
{
#ifdef __MAOS9__ // MP semaphores not system-wide
 (void)lock;
#else
	char cstr[256];
	
	if(!lock || !lock->psem)
	{
		ShowErr("maNamedUnlock null lock");
		return 1;
	}

	if(maNamedSemaphorePost(lock->psem, lock->name))
	{
		sprintf(cstr, "maNamedUnlock '%s' maSemaphorePost failed", lock->name);
		ShowErr(cstr);
		return 1;
	}
#endif
	
	return 0;
	
} // maNamedUnlock
#endif // MAINSTANCES


#if defined(__MAOS9__)
// from MoreFiles
OSErr maFSMakeFSSpecCompat(short vRefNum, long dirID, ConstStr255Param fileName, FSSpec *spec);
OSErr maFSpGetFullPath(const FSSpec *spec, short *fullPathLength, Handle *fullPath);
OSErr maFSpLocationFromFullPath(short fullPathLength, const void *fullPath, FSSpec *spec);

OSErr maFSMakeFSSpecCompat(short vRefNum, long dirID, ConstStr255Param fileName, FSSpec *spec)
{
	OSErr result;
	
	/* Let the file system create the FSSpec if it can since it does the job */
	/* much more efficiently than I can. */
	result = FSMakeFSSpec(vRefNum, dirID, fileName, spec);

	/* Fix a bug in Macintosh PC Exchange's MakeFSSpec code where 0 is */
	/* returned in the parID field when making an FSSpec to the volume's */
	/* root directory by passing a full pathname in MakeFSSpec's */
	/* fileName parameter. Fixed in Mac OS 8.1 */
	if( (result == noErr) && (spec->parID == 0) )
		spec->parID = fsRtParID;

	return (result);
	
} // maFSMakeFSSpecCompat


OSErr maFSpGetFullPath(const FSSpec *spec, short *fullPathLength, Handle *fullPath)
{
	OSErr result;
	OSErr realResult;
	FSSpec tempSpec;
	CInfoPBRec pb;

	*fullPathLength = 0;
	*fullPath = NULL;

	/* Default to noErr */
	realResult = result = noErr;

	/* work around Nav Services "bug" (it returns invalid FSSpecs with empty names) */
	if(spec->name[0] == 0)
		result = maFSMakeFSSpecCompat(spec->vRefNum, spec->parID, spec->name, &tempSpec);
	else /* Make a copy of the input FSSpec that can be modified */
		BlockMoveData(spec, &tempSpec, sizeof(FSSpec));

	if(result == noErr)
	{
		if(tempSpec.parID == fsRtParID)
		{
			/* The object is a volume */

			/* Add a colon to make it a full pathname */
			++tempSpec.name[0];
			tempSpec.name[tempSpec.name[0]] = ':';

			/* We're done */
			result = PtrToHand(&tempSpec.name[1], fullPath, tempSpec.name[0]);
		}
		else
		{
			/* The object isn't a volume */

			/* Is the object a file or a directory? */
			pb.dirInfo.ioNamePtr = tempSpec.name;
			pb.dirInfo.ioVRefNum = tempSpec.vRefNum;
			pb.dirInfo.ioDrDirID = tempSpec.parID;
			pb.dirInfo.ioFDirIndex = 0;
			result = PBGetCatInfoSync(&pb);
			// Allow file/directory name at end of path to not exist.
			realResult = result;
			if((result == noErr) || (result == fnfErr))
			{
				/* if the object is a directory, append a colon so full pathname ends with colon */
				if( (result == noErr) && (pb.hFileInfo.ioFlAttrib & kioFlAttribDirMask) != 0 )
				{
					++tempSpec.name[0];
					tempSpec.name[tempSpec.name[0]] = ':';
				}

				/* Put the object name in first */
				result = PtrToHand(&tempSpec.name[1], fullPath, tempSpec.name[0]);
				if( result == noErr )
				{
					/* Get the ancestor directory names */
					pb.dirInfo.ioNamePtr = tempSpec.name;
					pb.dirInfo.ioVRefNum = tempSpec.vRefNum;
					pb.dirInfo.ioDrParID = tempSpec.parID;
					do /* loop until we have an error or find the root directory */
					{
						pb.dirInfo.ioFDirIndex = -1;
						pb.dirInfo.ioDrDirID = pb.dirInfo.ioDrParID;
						result = PBGetCatInfoSync(&pb);
						if(result == noErr)
						{
							/* Append colon to directory name */
							++tempSpec.name[0];
							tempSpec.name[tempSpec.name[0]] = ':';

							/* Add directory name to beginning of fullPath */
							(void)Munger(*fullPath, 0, NULL, 0, &tempSpec.name[1], tempSpec.name[0]);
							result = MemError();
						}
					} while((result == noErr) && (pb.dirInfo.ioDrDirID != fsRtDirID));
				}
			}
		}
	}

	if(result == noErr)
	{
		/* Return the length */
		*fullPathLength = GetHandleSize(*fullPath);
		result = realResult;    // return realResult in case it was fnfErr
	}
	else
	{
		/* Dispose of the handle and return NULL and zero length */
		if( *fullPath != NULL )
			DisposeHandle(*fullPath);
		*fullPath = NULL;
		*fullPathLength = 0;
	}
	
	return (result);
	
} // maFSpGetFullPath


OSErr maFSpLocationFromFullPath(short fullPathLength, const void *fullPath, FSSpec *spec)
{
	AliasHandle alias;
	OSErr result;
	Boolean wasChanged;
	Str32 nullString;
	
	/* Create a minimal alias from the full pathname */
	nullString[0] = 0;      /* null string to indicate no zone or server name */
	result = NewAliasMinimalFromFullPath(fullPathLength, fullPath, nullString, nullString, &alias);
	if(result == noErr)
	{
		/* Let the Alias Manager resolve the alias. */
		result = ResolveAlias(NULL, alias, spec, &wasChanged);
			
		/* work around Alias Mgr sloppy volume matching bug */
		if(spec->vRefNum == 0)
		{
			/* invalidate wrong FSSpec */
			spec->parID = 0;
			spec->name[0] = 0;
			result = nsvErr;
		}
		DisposeHandle((Handle)alias);   /* Free up memory used */
	}
	return (result);
}


int maMacFSSpec2MacFilename(char *pname, void *pspec)
{
	FSSpec *spec = (FSSpec *)pspec;

	Handle myPath;
	short length;
	// get length of path and allocate handle
	maFSpGetFullPath(spec , &length , &myPath);
	::SetHandleSize(myPath , length + 1);
	::HLock(myPath);
	(*myPath)[length] = 0;
	if((length > 0) && ((*myPath)[length-1] == ':'))
		(*myPath)[length-1] = 0;
	strcpy(pname, (char*)*myPath);
	// free allocated handle
	::HUnlock(myPath);
	::DisposeHandle(myPath);
	
	return 0;
	
} // maMacFSSpec2MacFilename // __MAOS9__


int maMacFilename2FSSpec(char *pname, void *pspec)
{
	OSStatus err = noErr;
	char cstr[MAX_PATH + 2];
	if(strchr(pname , ':' ) == NULL)
	{
		// try whether it is a volume / or a mounted volume
		strncpy(cstr, pname, MAX_PATH + 2);
		cstr[MAX_PATH] = 0;
		strcat(cstr , ":");
		err = maFSpLocationFromFullPath((short)strlen(cstr), (const void *)cstr, (FSSpec *)pspec);
	}
	else
		err = maFSpLocationFromFullPath((short)strlen(pname), (const void *)pname, (FSSpec *)pspec);

	return (int)err;
	
} // maMacFilename2FSSpec // __MAOS9__
#endif


#if defined(__MAOSX__) || defined(__MAOS9__)
int HaveMPServices()
{
	char cstr[256];
	
	if(MPLibraryIsLoaded())
	{
		UInt32 major;
		UInt32 minor;
		UInt32 release;
		UInt32 revision;
  		char versionCString[256];
  		char **p = (char **)&versionCString;
		
		_MPLibraryVersion((const char **)p, &major, &minor, &release, &revision);
		
		if(major >= 2 && minor >= 3)	// maj 2 min 2 fails on OS9.0, maj 2 min 3 works on OSX 10.3
			return 1;					// maj 2 min 4 works on OS9.2.2, fails on OSX 10.3 others untested

		sprintf(cstr,
"MPLibrary version='%s' maj=%d min=%d rel=%d rev=%d. MultiProcessing will not be used. Get MPLibrary 2.4 or higher (OS9.2.2)",
											*p, (int)major, (int)minor, (int)release, (int)revision);
		maLog(cstr);
	}
	
	return 0;
	
} // HaveMPServices
#endif


/* #if defined(__MAOSX__) || defined(__MACUNIX__)
void maBSDpath(char *path)
{
	char itemdir[MAX_PATH];
	char itemname[MAX_PATH];
	// item can have path and/or name in osx or bsd format
	PathnameFromFilename(path, itemdir);
	FilenameFromPathname(path, itemname);
	StrBSDtoStrOSX(itemdir);
	StrBSDtoStrOSX(itemname);
	sprintf(path, "%s%c%s", itemdir, FILESEP, itemname);
}
#endif */


#if defined(__MAOSX__) || defined(__MAOS9__)
 MPTaskID l_threadmac;
#endif
#if defined(__MAUNIX__)
 pthread_t l_threadux;
 struct sched_param l_prioritynormal;
 int l_policynormal;
#endif
#if defined(__MAWIN__)
 HANDLE l_threadwin;
#endif

int maThreadCreate(void *threadfunction, void **ppthreadid, int threadstacksz, int threadpri)
{
	char cstr[256];

#if defined(__MAOSX__) || defined(__MAOS9__)
	int err;
	*ppthreadid = &l_threadmac;
	if((err = MPCreateTask((long (*)(void*))threadfunction,
		NULL,			// the parameter passed to the task
		threadstacksz,	// stack size
		NULL,			// no notification queue
		NULL, NULL,		// result parameters - unused
		0,				// no special task flags
		(MPTaskID *)(*ppthreadid))) != 0)	
	{
		*ppthreadid = NULL;
		sprintf(cstr, "maThreadCreate MPCreateTask error %d", err);
		maLog(cstr);
		return 1;
	}
	return maThreadSetPriority(*ppthreadid, threadpri);
#endif

#if defined(__MAWIN__)
	DWORD thread_id;
	l_threadwin = CreateThread(0, 0, (LPTHREAD_START_ROUTINE)threadfunction, 0, 0, &thread_id);
    if(!l_threadwin)
	{
		*ppthreadid = NULL;
		sprintf(cstr, "maThreadCreate CreateThread error %08X", GetLastError());
		maLog(cstr);
		return 1;
	}
	*ppthreadid = &l_threadwin;
	return maThreadSetPriority(*ppthreadid, threadpri);
#endif

#if defined(__MAUNIX__)
	int err;
    if(pthread_create(&l_threadux, NULL, (void*(*)(void*))threadfunction, NULL))
	{
		*ppthreadid = NULL;
		maLog("maThreadCreate pthread_create failed");
		return 1;
    }
	*ppthreadid = &l_threadux;
	// save 'normal' sched policy and priority
	if((err = pthread_getschedparam(l_threadux, &l_policynormal, &l_prioritynormal)))
	{
		sprintf(cstr, "maThreadCreate pthread_getschedparam error %d", err);
		maLog(cstr);
		return 1;
	}
	// set the new sched policy, here 'threadpri' is really policy
	maThreadSetPriority(&l_threadux, threadpri);
	return 0;
#endif

} // maThreadCreate


int maThreadSetPriority(void *pthreadid, int pri)
{	
#if defined(__MAOSX__) || defined(__MAOS9__)
	int err;
	char cstr[256];
	if(!pthreadid)
	{
		sprintf(cstr, "maThreadSetPriority %d failed, null pthreadid", pri);
		maLog(cstr);
		return 1;
	}
	if((err = MPSetTaskWeight(*((MPTaskID*)pthreadid), pri)) != 0)	
	{
 #if !TARGET_API_MAC_CARBON
		sprintf(cstr, "maThreadSetPriority MPSetTaskWeight error %d", err);
		maLog(cstr);
 #else
		err = 0;
 #endif
	}
	return err;
#endif

#if defined(__MAWIN__)
 #if !defined(_DEBUG)
	char cstr[256];
	if(!pthreadid)
	{
		sprintf(cstr, "maThreadSetPriority %d failed, null pthreadid", pri);
		maLog(cstr);
		return 1;
	}
	if(!SetThreadPriority(*((HANDLE *)pthreadid), pri))
	{
		sprintf(cstr, "maThreadSetPriority %d failed on '%s'", pri, GetLastError());
		maLog(cstr);
		return 1;
	}
	//SetPriorityClass(GetCurrentProcess(), NORMAL_PRIORITY_CLASS); // HIGH_PRIORITY_CLASS
 #endif
	return 0;
#endif

#if defined(__MAUNIX__)
	int err;
    struct sched_param priority;
	int policy = pri; // pri here is really sched policy
	char cstr[256];
	memcpy(&priority, &l_prioritynormal, sizeof(struct sched_param));
 #ifdef __MACUNIX__ // darwin, maybe other unices
	if(pri != MATHREADPRILO)
  #ifdef PTHREAD_MAX_PRIORITY
		priority.sched_priority = PTHREAD_MAX_PRIORITY; // where is PTHREAD_MAX_PRIORITY ?
  #else
		priority.sched_priority = 63; // 'normal' is 31
  #endif
	if((err = pthread_setschedparam(*((pthread_t*)pthreadid), policy, &priority)))
	{
		sprintf(cstr, "maThreadSetPriority pthread_setschedparam error %d", err);
		maLog(cstr);
		return 1;
	}
 #else
	if(pri != MATHREADPRILO)
		priority.sched_priority = sched_get_priority_max(policy);
	if((err = sched_setscheduler(0, policy, &priority)))
	{
		sprintf(cstr, "maThreadSetPriority sched_setscheduler error %d", err);
		maLog(cstr);
		return 1;
	}
 #endif
	return 0;
#endif
} // maThreadSetPriority


#if defined(MAUSESHF)
 // instance table, shared file instead of shm
void *mashmopen(size_t shmsize, char *keyfile)
{
	char *pinstance;
	char cstr[256];
	
	if(!keyfile)
	{
		ShowErr("mashmopen null keyfile");	
		return NULL;
	}

	if(maFileExists(keyfile))
		return (void *)keyfile; // ok
	
	if((pinstance = (char *)MAMALLOC(shmsize)) == NULL)
	{
		sprintf(cstr, "mashmopen MAMALLOC failed on '%s'", keyfile);
		ShowErr(cstr);	
		return NULL;
	}
	
	memset(pinstance, 0, shmsize);
	
	if(mashmwrite(pinstance, 1, shmsize, keyfile) != (int)shmsize)
	{
		MAFREE(pinstance);
		sprintf(cstr, "mashmopen mashmwrite failed on '%s'", keyfile);
		ShowErr(cstr);	
		return NULL;
	}

	MAFREE(pinstance);
	
	return (void *)keyfile; // ok
	
} // mashmopen


int mashmclose(void *pmashm)
{
#ifdef MAUNUSED
 (void)pmashm;
#endif

	return 0;
	
} // mashmclose


int mashmwrite(char *ptr, size_t size, size_t nmemb, void *pmashm)
{
	int ret;
	int data_len;
	FILE *fp;
	char cstr[256];

	if(!ptr || !pmashm || !((char *)pmashm)[0])
	{   
		ShowErr("mashmwrite null pointer to shared file");
		return 0;
	}
	
	if((data_len = size * nmemb) <= 0)
	{   
		ShowErr("mashmwrite data length is 0");
		return 0;
	}
		
	if((fp = mafopen((char *)pmashm, "wb")) == NULL)
	{
		sprintf(cstr, "mashmwrite mafopen failed on '%s'", (char *)pmashm);
		ShowErr(cstr);
		return NULL;
	}
	
	ret = fwrite(ptr, size, nmemb, fp);
		
	fclose(fp);
	
	if(ret != (int)nmemb)
	{
		sprintf(cstr, "mashmwrite fwrite failed on '%s'", (char *)pmashm);
		ShowErr(cstr);	
		return NULL;
	}
	
	return nmemb;
	
} // mashmwrite


int mashmread(char *ptr, size_t size, size_t nmemb, void *pmashm)
{
	int ret;
	int data_len;
	FILE *fp;
	char cstr[256];

	if(!ptr || !pmashm || !((char *)pmashm)[0])
	{   
		ShowErr("mashmread null pointer to shared file");
		return 0;
	}
	
	if((data_len = size * nmemb) <= 0)
	{   
		ShowErr("mashmread data length is 0");
		return 0;
	}
		
	if((fp = mafopen((char *)pmashm, "rb")) == NULL)
	{
		sprintf(cstr, "mashmread mafopen failed on '%s'", (char *)pmashm);
		ShowErr(cstr);
		return NULL;
	}
	
	ret = fread(ptr, size, nmemb, fp);
	
	fclose(fp);
	
	if(ret != (int)nmemb)
	{
		sprintf(cstr, "mashmread fread failed on '%s'", (char *)pmashm);
		ShowErr(cstr);	
		return NULL;
	}
	
	return nmemb;
	
} // mashmread


int mashmdelete(void *pmashm)
{
	char cstr[256];

	if(!pmashm || !((char *)pmashm)[0])
	{   
		ShowErr("mashmdelete null pointer to shared file");
		return 0;
	}

	if(maunlink((char *)pmashm))
	{
		sprintf(cstr, "mashmdelete failed to delete '%s'", (char *)pmashm);
		ShowErr(cstr);
		return 1;
	}		
	
	return 0;
}
#endif // MAUSESHF

#if defined(MAUSESHM)
 // shared mem
void *mashmopen(size_t shmsize, char *keyfile)
{
	MASHM *pmashm;	
	char cstr[256];

	if((pmashm = (MASHM *)MAMALLOC(sizeof(MASHM))) == NULL)
	{
		ShowErr("mashmopen MAMALLOC failed");
		return NULL;
	}
	memset(pmashm, 0, sizeof(MASHM));
	pmashm->size = shmsize;

#if defined(__MAWIN__)
	// on win we share mamory mapped file 'keyfile'
	if((pmashm->file = CreateFile(keyfile, GENERIC_WRITE | GENERIC_READ,
									FILE_SHARE_WRITE | FILE_SHARE_READ,
						NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL))
											== INVALID_HANDLE_VALUE)
	{
		MAFREE(pmashm);
		sprintf(cstr, "mashmopen CreateFile error %d on '%s'",
											GetLastError(), keyfile);
		ShowErr(cstr);
		return NULL;
	}
	if((pmashm->map = CreateFileMapping(pmashm->file, NULL,
					PAGE_READWRITE, 0, shmsize, "mMaAkKeEyYfFiIlLeE")) == NULL)
	{
		CloseHandle(pmashm->file);
		MAFREE(pmashm);
		sprintf(cstr, "mashmopen CreateFileMapping error %d on '%s'",
											GetLastError(), keyfile);
		ShowErr(cstr);
		return NULL;
	}
	pmashm->alone = (GetLastError() == ERROR_ALREADY_EXISTS) ? 0 : 1; // maNumProcesses == 1
	pmashm->view = MapViewOfFile(pmashm->map, FILE_MAP_WRITE, 0, 0, 0);
	if(pmashm->view == NULL)
	{
		CloseHandle(pmashm->map);
		CloseHandle(pmashm->file);
		MAFREE(pmashm);
		sprintf(cstr, "mashmopen MapViewOfFile error %d on '%s'",
											GetLastError(), keyfile);
		ShowErr(cstr);
		return NULL;
	}
	pmashm->addr = (char *)pmashm->view;
	if(pmashm->alone)
		memset(pmashm->addr, 0, pmashm->size); // we are the only instance
#endif
#if defined(__MAOSX__) || defined(__MAUNIX__)
	// on unix we share a memory segment
	struct shmid_ds shm;
	if((pmashm->key = ftok(keyfile, (int)getuid())) == -1) // each user gets his own shm
	{
		MAFREE(pmashm);
		sprintf(cstr, "mashmopen ftok('%s', 0) failed", keyfile);
		ShowErr(cstr);
		return NULL;
	}
	if((pmashm->shmid = shmget(pmashm->key, pmashm->size,  0666 | IPC_CREAT)) == -1)
	{
		MAFREE(pmashm);
		ShowErr("mashmopen shmget failed");
		return NULL;
	}
	if(shmctl(pmashm->shmid, IPC_STAT, &shm) == -1)
	{
		MAFREE(pmashm);
		sprintf(cstr, "mashmopen shmctl IPC_STAT failed errno=%d", errno);
		ShowErr(cstr);
		return NULL;
	}	
	pmashm->size = shm.shm_segsz;
	pmashm->addr = (char *)shmat(pmashm->shmid, 0, 0);
	if(pmashm->addr == (char *)-1)
	{
		MAFREE(pmashm);
		ShowErr("mashmopen shmat failed"); // errno
		return NULL;
	}
	if(maNumProcesses(0) == 1)
	{
		pmashm->alone = 1;
		memset(pmashm->addr, 0, pmashm->size); // we are the only instance
	}
	else
		pmashm->alone = 0;
#endif

	return pmashm;
	
} // mashmopen


int mashmclose(void *pmashm)
{
	if(!pmashm)
	{   
		ShowErr("mashmclose null pmashm");
		return 1;
	}

#if defined(__MAWIN__) // mashmdelete
	UnmapViewOfFile(((MASHM *)pmashm)->view);
	CloseHandle(((MASHM *)pmashm)->map);
	CloseHandle(((MASHM *)pmashm)->file);
#endif

#if defined(__MAOSX__) || defined(__MAUNIX__)
	if(shmdt(((MASHM *)pmashm)->addr) == -1)
	{
		ShowErr("mashmclose shmdt failed"); // errno
		return 1;
	}
#endif
	
	return 0;
}


int mashmwrite(char *ptr, size_t size, size_t nmemb, void *pmashm)
{
	int data_len;
	char cstr[256];

	if(!pmashm)
	{   
		ShowErr("mashmwrite null pointer to shared mem");
		return 0;
	}
	
	if((data_len = size * nmemb) <= 0)
	{   
		ShowErr("mashmwrite data length is 0");
		return 0;
	}
		
	if(data_len > ((MASHM *)pmashm)->size)
	{
		sprintf(cstr, "mashmwrite data size %d greater than shared mem size %d",
								data_len, ((MASHM *)pmashm)->size);
		ShowErr(cstr);
		return 0;
	}

	memcpy(((MASHM *)pmashm)->addr, ptr, data_len);

	return nmemb;
	
} // mashmwrite


int mashmread(char *ptr, size_t size, size_t nmemb, void *pmashm)
{
	int data_len;

	if(!pmashm)
	{   
		ShowErr("mashmread null pointer to shared mem");
		return 0;
	}
	
	if((data_len = size * nmemb) <= 0)
	{   
		ShowErr("mashmread data length is 0");
		return 0;
	}
		
	if(data_len > ((MASHM *)pmashm)->size)
	{
		ShowErr("mashmread data size greater than shared mem");
		return 0;
	}

	memcpy(ptr, ((MASHM *)pmashm)->addr, data_len);

	return nmemb;
	
} // mashmread


int mashmdelete(void *pmashm)
{
	if(!pmashm)
	{   
		ShowErr("mashmdelete null pmashm");
		return 1;
	}

#if defined(__MAWIN__)
	UnmapViewOfFile(((MASHM *)pmashm)->view);
	CloseHandle(((MASHM *)pmashm)->map);
	CloseHandle(((MASHM *)pmashm)->file);
#endif

#if defined(__MAOSX__) || defined(__MAUNIX__)
	if(shmctl(((MASHM *)pmashm)->shmid, IPC_RMID, NULL))
	{
		ShowErr("mashmdelete shmctl IPC_RMID failed");
		return 1;
	}
#endif

	return 0;
}
#endif // MAUSESHM


char *magetmoreampdir(char *buf, int size)
{
#ifdef __MAWIN__
	// moreampdir is the dir with the MoreAmp binary
	return magetbindir(buf, size);
#endif

#ifdef __MAOS9__
	// moreampdir is the dir with the MoreAmp binary
	return magetbindir(buf, size);
#endif

#if defined(__MAOSX__) && !defined(MAOSXHOME)
	// moreampdir is the dir with the MoreAmp binary
	return magetbindir(buf, size);
#endif

#if defined(__MAUNIX__) || defined(MAOSXHOME)
	// moreampdir is MOREAMP_DIRNAME in the user's home dir
	ssize_t len;
	char *p;
	char path[MAX_PATH];
	char cstr[MAX_PATH + 256];
	if(!(p = getenv("HOME")))
	{
		ShowErr("magetmoreampdir getenv(HOME) failed");
		return NULL;
	}
	strcpy(buf, p);
	// resolve any links to home dir
	if((len = readlink(buf, path, MAX_PATH - 2)) == -1)
	{
		if(errno != EINVAL) // buf is not a symlink
		{
			sprintf(cstr, "magetmoreampdir readlink '%s' errno=%d", buf, errno);
			maLog(cstr);
		}
	}
	else
	{
		path[len] = 0;
		strcpy(buf, path);
	}
	sprintf(cstr, "%c%s", FILESEP, MOREAMP_DIRNAME);
	strcat(buf, cstr);
	return buf;
#endif

} // magetmoreampdir


char *magetcwd(char *buf, int size)
{
#if defined(__MAOS9__)
	// cwd is the dir with the MoreAmp binary
	return (magetbindir(buf, size));
#endif

#if defined(__MAOSX__) || defined(__MAUNIX__)
	// cwd is set by the shell
	return getcwd(buf, (size_t)size);
#endif

#if defined(__MAWIN__)	
	return _getcwd(buf, size); // 'starting dir' property ?
#endif
} // magetcwd


char *magetbindir(char *buf, int size)
{
#if defined(__MAOS9__)
	// this gets the dir with the MoreAmp binary
	(void)size;
	FSSpec cwdSpec;
	FCBPBRec pb;
	OSErr err;
	Str255 fileName;
	char cstr[32];
	pb.ioNamePtr = (StringPtr)&fileName;
	pb.ioVRefNum = 0;
	pb.ioRefNum = LMGetCurApRefNum();
	pb.ioFCBIndx = 0;
	err = PBGetFCBInfoSync(&pb);
	if(err == noErr)
	{
		cwdSpec.vRefNum = pb.ioFCBVRefNum;
		cwdSpec.parID = pb.ioFCBParID;
		cwdSpec.name[0] = 0;
		maMacFSSpec2MacFilename(buf, &cwdSpec);
		return buf;
	}
	sprintf(cstr, "magetcwd PBGetFCBInfoSync error=%d", err);
	maLog(cstr);
	return NULL;
#endif

#if defined(__MAOSX__)
	// this gets the dir with the MoreAmp.app bundle
	char *p, *q, *r;
	strcpy(buf, g_argv[0]);						// "/.../adir/MoreAmp.app/Contents/MacOS/MOREAMP_NAME"
	if((p = strstr(buf, "/Contents/MacOS")))   // we're in the bundle  
		*p = 0;									// "/.../adir/MoreAmp.app"
	else
	{
		ShowErr("magetmoreampdir unable to determine working dir (bad .app file ?)");
		return NULL;
	}
	q = 0;										// remove last '/MoreAmp.app'
	r = &buf[1];
	while((p = strchr(r, FILESEP)))
	{
		r = p + 1;
		q = p;
	}
	if(q)
		*q = 0;									// "/.../adir"
	return buf;
#endif

#if defined(__MAUNIX__)
	// this gets the command line
	char *p, *q, *r;
	strcpy(buf, g_argv[0]); // "/.../adir/moreamp"
	q = 0;					// remove last '/moreamp'
	r = &buf[1];
	while((p = strchr(r, FILESEP)))
	{
		r = p + 1;
		q = p;
	}
	if(q)
		*q = 0;				// "/.../adir"
	return buf;
#endif

#if defined(__MAWIN__)
	char *p, *q, *r;
	strcpy(buf, g_argv[0]); // "C:\...\adir\MOREAMP_NAME.EXE"
	q = 0;					// remove last '\MOREAMP_NAME.EXE'
	r = &buf[1];
	while((p = strchr(r, FILESEP)))
	{
		r = p + 1;
		q = p;
	}
	if(q)
		*q = 0;				// "C:\...\adir"
	return buf;
#endif
	
} // magetbindir


int masetcwd(char *dir)
{
	int err;
	char cstr[256];
	
#if defined(__MAOS9__) || defined(__MAOSX__) || defined(__MAUNIX__)
	if((err = chdir(dir)) == 0)
		return 0;
#endif

#if defined(__MAWIN__)
	if(SetCurrentDirectory(dir)) // or _chdir
		return 0;
	err = 0;
#endif

	sprintf(cstr, "masetcwd SetCurrentDirectory err %d on '%s'", err, dir);
	maLog(cstr);	
	return 1;
	
} // masetcwd


#if defined(__MAWIN__)
int maIsWin9x()
{
	OSVERSIONINFOA info;

	info.dwOSVersionInfoSize = sizeof(info);

	if(GetVersionExA(&info))
		if((info.dwPlatformId == 2 && info.dwMajorVersion == 4) || info.dwMajorVersion >= 5) // NT, WDM
			return 0;

	return 1;

} // maIsWin9x
#endif


#if defined(MAINSTANCES)
 #ifdef __MAWIN__
// return the number of 'moreamp' processes running
// or if 'moreamp' process 'pid' is running
// code from ms pviewer sample
#define INDEX_STR_LEN 10
#define PN_PROCESS TEXT("Process")
#define PN_PROCESS_ID TEXT("ID Process")
// ss #ifndef W2A
// ss #define W2A(w, a, cb) WideCharToMultiByte(CP_ACP, 0, w, -1, a, cb, NULL, NULL)
// ss#endif
typedef PERF_DATA_BLOCK PERF_DATA, *PPERF_DATA;
typedef PERF_OBJECT_TYPE PERF_OBJECT, *PPERF_OBJECT;
typedef PERF_INSTANCE_DEFINITION PERF_INSTANCE, *PPERF_INSTANCE;
typedef PERF_COUNTER_DEFINITION PERF_COUNTER, *PPERF_COUNTER;
DWORD GetPerfTitleSz(HKEY hKeyMachine, HKEY hKeyPerf, LPTSTR *TitleBuffer, LPTSTR *TitleSz[], DWORD *TitleLastIdx);
TCHAR INDEX_PROCTHRD_OBJ[2*INDEX_STR_LEN];

int maNumProcessesWin(int *pid)
{
	int i;
	int *p = pid;
	int lpid = 0;
	int ninstances = 0;
	INT InstanceIndex = 0;
	DWORD gPerfDataSize = 50*1024;
	DWORD *pdwProcID;
	DWORD Last;
	DWORD Index;
	DWORD dwR;
	DWORD PX_PROCESS;
	DWORD PX_PROCESS_ID;
	PPERF_OBJECT gpProcessObject;
	PPERF_DATA gpPerfData = NULL;
	PPERF_INSTANCE pInstance;
	PPERF_COUNTER pCounterProcID;
	PERF_COUNTER_BLOCK *pCounterBlock;
	LPTSTR TitleBuffer;
	LPTSTR *Title;
	HKEY ghMachineKey = HKEY_LOCAL_MACHINE;
	HKEY ghPerfKey = HKEY_PERFORMANCE_DATA;
	char maname[MAX_PATH];
	char cstr[256];

	dwR = GetPerfTitleSz(ghMachineKey, ghPerfKey, &TitleBuffer, &Title, &Last);
	if(dwR != ERROR_SUCCESS)
	{
		sprintf(cstr, "maNumProcessesWin error GetPerfTitleSz %d", dwR);
		ShowErr(cstr);
		return 0;
	}

	for(Index = 0; Index <= Last; Index++)
		if(Title[Index])
			if(!lstrcmpi (Title[Index], PN_PROCESS))
				break;
	if(Index > Last)
	{
		ShowErr("maNumProcessesWin cannot find process name index");
		return 0;
	}
	PX_PROCESS = Index;

	for(Index = 0; Index <= Last; Index++)
		if(Title[Index])
			if(!lstrcmpi (Title[Index], PN_PROCESS_ID))
				break;
	if(Index > Last)
	{
		ShowErr("maNumProcessesWin cannot find process id index");
		return 0;
	}
	PX_PROCESS_ID = Index;

	DWORD DataSize;
	DWORD Type;

	if(!gpPerfData)
		gpPerfData = (PPERF_DATA)LocalAlloc(LMEM_FIXED, gPerfDataSize);

	do
	{
		DataSize = gPerfDataSize;
		dwR = RegQueryValueEx(ghPerfKey, INDEX_PROCTHRD_OBJ, NULL, &Type, (BYTE *)gpPerfData, &DataSize);

		if(dwR == ERROR_MORE_DATA)
		{
			LocalFree (gpPerfData);
			gPerfDataSize += 1024;
			gpPerfData = (PPERF_DATA)LocalAlloc (LMEM_FIXED, gPerfDataSize);
		}

		if(!gpPerfData)
		{
			LocalFree (gpPerfData);
			ShowErr("maNumProcessesWin LocalAlloc failed");
			return 0;
		}

	} while(dwR == ERROR_MORE_DATA);

	if(dwR != ERROR_SUCCESS)
	{
		ShowErr("maNumProcessesWin RegQueryValueEx failed");
		return 0;
	}

	if((gpProcessObject = ((PPERF_OBJECT)((PBYTE)gpPerfData + gpPerfData->HeaderLength))) == NULL)
	{
		ShowErr("maNumProcessesWin process not found");
		return 0;
	}
	
	i = 0;
	while(i < (int)gpPerfData->NumObjectTypes)
	{
		if(gpProcessObject->ObjectNameTitleIndex == PX_PROCESS)
			break;
		gpProcessObject = ((PPERF_OBJECT)((PBYTE)gpProcessObject + gpProcessObject->TotalByteLength));
		i++;
	}
	if(i >= (int)gpPerfData->NumObjectTypes)
	{
		ShowErr("maNumProcessesWin process not found");
		return 0;
	}

	if((pCounterProcID = (PPERF_COUNTER)((PCHAR)gpProcessObject + gpProcessObject->HeaderLength)) == NULL)
	{
		ShowErr("maNumProcessesWin process not found");
		return 0;
	}
	
	i = 0;
	while(i < (int)gpProcessObject->NumCounters)
	{
		if(pCounterProcID->CounterNameTitleIndex == PX_PROCESS_ID)
			break;		
		pCounterProcID = (PPERF_COUNTER)((PCHAR)pCounterProcID + pCounterProcID->ByteLength);
		i++;
	}
	if(i >= (int)gpProcessObject->NumCounters)
	{
		ShowErr("maNumProcessesWin process not found");
		return 0;
	}

	pInstance = (PPERF_INSTANCE)((PCHAR)gpProcessObject + gpProcessObject->DefinitionLength);

	if(p)
		lpid = *p;
	while(pInstance && InstanceIndex < gpProcessObject->NumInstances)
	{
		// ss W2A((const unsigned short *)(LPTSTR)((PCHAR)pInstance + pInstance->NameOffset),
		// ss (LPWSTR)&maname[0], MAX_PATH);
		// ss #define W2A(w, a, cb)
		WideCharToMultiByte(CP_ACP, 0,
			// (const unsigned short *)(LPTSTR)((PCHAR)pInstance + pInstance->NameOffset),
			(LPCWSTR)((PCHAR)pInstance + pInstance->NameOffset),
			-1,
			// (LPWSTR)&maname[0],
			(LPSTR)&maname[0],
			MAX_PATH, NULL, NULL);
		if(strstr(maname, MOREAMP_NAME) == maname)
		{
			PPERF_COUNTER_BLOCK pCounterBlock;

			pCounterBlock = (PPERF_COUNTER_BLOCK)((PCHAR)pInstance + pInstance->ByteLength);
			if((pdwProcID = (DWORD *)(PVOID)((PCHAR)pCounterBlock + pCounterProcID->CounterOffset)))
			{
				if(*pdwProcID)
				{
					if(lpid == (int)(*pdwProcID))
						return 1;
					*p++ = (int)(*pdwProcID);
					ninstances++;
				}
			}
		}
		pCounterBlock = (PERF_COUNTER_BLOCK *)((PCHAR)pInstance + pInstance->ByteLength);
		pInstance = (PPERF_INSTANCE)((PCHAR)pCounterBlock + pCounterBlock->ByteLength);
		InstanceIndex++;
	}
	
	if(lpid)
		return 0;

	return ninstances;
	
} // maNumProcessesWin

/*
#ifdef UNICODE
 #define atoi atoiW
 INT atoiW (LPTSTR s)
 {
	INT i = 0;
	while(iswdigit (*s))
	{
		i = i*10 + (BYTE)*s - L'0';
		s++;
	}
	return i;
 }
#endif
*/

DWORD GetPerfTitleSz(HKEY hKeyMachine, HKEY hKeyPerf, LPTSTR *TitleBuffer,
									LPTSTR *TitleSz[], DWORD *TitleLastIdx)
{
	HKEY hKey1;
	HKEY hKey2;
	DWORD Type;
	DWORD DataSize;
	DWORD dwR;
	DWORD Len;
	DWORD Index;
	DWORD dwTemp;
	BOOL bNT10;
	LPTSTR szCounterValueName;
	LPTSTR szTitle;

	// Initialize
	hKey1 = NULL;
	hKey2 = NULL;
	*TitleBuffer = NULL;
	*TitleSz 	= NULL;

	// Open the perflib key to find out the last counter's index and system version.
	dwR = RegOpenKeyEx(hKeyMachine,
						TEXT("software\\microsoft\\windows nt\\currentversion\\perflib"),
						0, KEY_READ, &hKey1);
	if(dwR != ERROR_SUCCESS)
		goto done;

	// Get the last counter's index so we know how much memory to allocate for TitleSz
	DataSize = sizeof (DWORD);
	dwR = RegQueryValueEx(hKey1, TEXT("Last Counter"), 0, &Type, (LPBYTE)TitleLastIdx, &DataSize);
	if(dwR != ERROR_SUCCESS)
		goto done;

	// Find system version, for system earlier than 1.0a, there's no version value.
	dwR = RegQueryValueEx(hKey1, TEXT("Version"), 0, &Type, (LPBYTE)&dwTemp, &DataSize);

	if(dwR != ERROR_SUCCESS)
		// unable to read the value, assume NT 1.0
		bNT10 = TRUE;
	else
		// found the value, so, NT 1.0a or later
		bNT10 = FALSE;

	// Now, get ready for the counter names and indexes.
	if(bNT10)
	{
		// NT 1.0, so make hKey2 point to ...\perflib\009 and get
		// the counters from value "Counters"
		szCounterValueName = TEXT("Counters");
		dwR = RegOpenKeyEx(hKeyMachine,
						 TEXT("software\\microsoft\\windows nt\\currentversion\\perflib\\009"),
						 0, KEY_READ, &hKey2);
		if(dwR != ERROR_SUCCESS)
			goto done;
	}
	else
	{
		// NT 1.0a or later. Get the counters in key HKEY_PERFORMANCE_KEY
		// and from value "Counter 009"
		szCounterValueName = TEXT("Counter 009");
		hKey2 = hKeyPerf;
	}

	// Find out the size of the data.
	dwR = RegQueryValueEx (hKey2, szCounterValueName, 0, &Type, 0, &DataSize);
	if(dwR != ERROR_SUCCESS)
		goto done;

	// Allocate memory
	*TitleBuffer = (LPTSTR)LocalAlloc (LMEM_FIXED, DataSize);
	if(!*TitleBuffer)
	{
		dwR = ERROR_NOT_ENOUGH_MEMORY;
		goto done;
	}

	*TitleSz = (LPTSTR *)LocalAlloc (LPTR, (*TitleLastIdx+1) * sizeof (LPTSTR));
	if(!*TitleSz)
	{
		dwR = ERROR_NOT_ENOUGH_MEMORY;
		goto done;
	}

	// Query the data
	dwR = RegQueryValueEx (hKey2, szCounterValueName, 0, &Type, (BYTE *)*TitleBuffer, &DataSize);
	if(dwR != ERROR_SUCCESS)
		goto done;

	// Setup the TitleSz array of pointers to point to beginning of each title string.
	// TitleBuffer is type REG_MULTI_SZ.
	szTitle = *TitleBuffer;

	while(Len = lstrlen (szTitle))
	{
		Index = atoi (szTitle);

		szTitle = szTitle + Len +1;

		if(Index <= *TitleLastIdx)
			(*TitleSz)[Index] = szTitle;

		szTitle = szTitle + lstrlen (szTitle) +1;
	}

done:

	// Done. Now cleanup!
	if(dwR != ERROR_SUCCESS)
	{
		// There was an error, free the allocated memory
		if(*TitleBuffer) LocalFree (*TitleBuffer);
		if(*TitleSz) LocalFree (*TitleSz);
	}

	// Close the hKeys.
	if(hKey1) RegCloseKey (hKey1);
	if(hKey2 && hKey2 != hKeyPerf) RegCloseKey (hKey2);

	return dwR;

} // GetPerfTitleSz
 #endif
#endif


unsigned long maGetLocalTimeMillis()
{
#if defined(__MAWIN__)
    struct timeb tp;
    (void)ftime(&tp);
    return (tp.time * 1000 + tp.millitm);
#endif
#if defined(__MAOS9__)
    UInt64 gMilliAtStart;
    AbsoluteTime upTime = AbsoluteToNanoseconds(UpTime());
	time_t start = time(NULL);
	gMilliAtStart = ((UInt64) start) * 1000000L;
	gMilliAtStart -= upTime.lo / 1000 ;
	gMilliAtStart -= (((UInt64) upTime.hi) << 32 ) / (1000 * 1000);
	UInt64 millival = gMilliAtStart;
	millival += upTime.lo / (1000 * 1000);
	millival += (((UInt64) upTime.hi) << 32 ) / (1000 * 1000);
	return millival;
#endif
#if defined(__MAOSX__) || defined(__MAUNIX__)
	struct timeval t;
	struct tm dt;
	gettimeofday(&t, NULL);
	memset(&dt, 0, sizeof(dt));
	dt.tm_isdst = -1;
	dt.tm_hour = t.tv_sec / 3600;
	dt.tm_min = t.tv_sec / 60 - dt.tm_hour * 60;
	dt.tm_sec = t.tv_sec - dt.tm_hour * 3600 - dt.tm_min * 60;
	return mktime(&dt) * 1000 + t.tv_usec / 1000;
#endif
}


#if defined(MAMEMTRACE)
 #include <stdlib.h>
 int g_nmamallocs = 0;
 int g_nmacallocs = 0;
 int g_ndupallocs = 0;
 int g_nmafrees = 0;
 int g_ndupfrees = 0;
 void *mamalloc(int a)
 { 
	void *p = NULL;
	if(a && (p = malloc(a)) != NULL)
	{
		char cstr[256];
		sprintf(cstr, "mamalloc %d 0x%x", a, (int)p);
		maLog(cstr);
		g_nmamallocs++;
	}
	return p;
 }


 void *macalloc(int a, int b)
 { 
	void *p = NULL;
	if(a && b && (p = macalloc(a, b)) != NULL)
	{
		char cstr[256];
		sprintf(cstr, "mamalloc %d 0x%x", a, (int)p);
		maLog(cstr);
		g_nmacallocs++;
	}
	return p;
 }


 void mafree(void *a)
 { 
	if(a)
	{
		char cstr[256];
		free(a);
		sprintf(cstr, "mafree 0x%x", (int)a);
		maLog(cstr);
		g_nmafrees++;
	}
 }


 void madupfree(void *a)
 { 
	if(a)
	{
		char cstr[256];
		free(a);
		sprintf(cstr, "madupfree 0x%x", (int)a);
		maLog(cstr);
		g_ndupfrees++;
	}
 }
#endif // MAMEMTRACE


int string2int(char* digit, int* pval) // values >= 0
{
	char *p = digit;
	
	*pval = 0;

	while(*p >= '0' && *p <='9')
	{
		*pval = (*pval * 10) + (*p - '0');
		p++;
	}

	if (*pval >= 0)
		return 0;

	return 1;
   
} // string2int


int maIsFullPath(char *path)
{
#if defined(__MAOS9__)
	if(strchr(path, VOLSEP) != NULL) // drive:dir:dir:file. We should check for drive...
		return 1;
#endif

#if defined(__MAOSX__) || defined(__MAUNIX__) || defined(__MACUNIX__)
	if(path[0] == VOLSEP) // (rootdir)/dir/dir/file.
		return 1;
#endif

#if defined(__MAWIN__)
	if(strchr(path, VOLSEP) != NULL) // drive:dir\dir\file.
		return 1;
#endif

	return 0; // must be relative
	
} // maIsFullPath


/*
#if defined(MAGUINONE) && defined(__MAOS9__)
//#include <SIOUX.h>
//#include <console.h>
#include "abort_exit.h"
//#include "console_io.h"
#include "critical_regions.h"
#include "misc_io.h"

enum console_status {
	no_console,
	console_open,
	console_broken
};

int maread_console(unsigned char *buffer, size_t *count);
int mawrite_console(unsigned char *buffer, size_t *count);
int maclose_console();

static int console_status = no_console;

static int check_console(void)
{
	int	result = 0;
	
	if (console_status == console_open)
		return(1);
	
	__begin_critical_region(console_status_access);
	
	if (console_status != no_console)
		goto exit;
	
	__stdio_atexit();
	
	if (InstallConsole(0))
	{
		console_status = console_broken;
		goto exit;
	}
	__console_exit = RemoveConsole;
	
	console_status = console_open;
	
	result = 1;
	
exit:
	
	__end_critical_region(console_status_access);
	
	return(result);
}


int maread_console(unsigned char *buffer, size_t *count)
{
	if (!check_console())
		return(1);
	fflush(stdout);
	*count = ReadCharsFromConsole((char *)buffer, *count);
	
	if (*count == -1)
		return(1);
	
	return(0);
}


int mawrite_console(unsigned char *buffer, size_t *count)
{
	if (!check_console())
		return(1);
	
	*count = WriteCharsToConsole((char *) buffer, *count);
	
	if (*count == -1)
		return(1);
	
	return(0);
}


int maclose_console()
{
	return(0);
}
#endif


#if defined(__MAOSX__) || defined(__MAUNIX__)
// use of unix process timer as one-shot
int l_matimersig = 0;
int matimer_set_timeout(long ms, void *callback)
{
	struct itimerval t_r;
	sigset_t sigset;

	if(callback && !l_matimersig)
	{
		l_matimersig = 1;
		signal(SIGALRM, (sig_t)callback);
		sigemptyset(&sigset);
		sigaddset(&sigset, SIGALRM);
		sigprocmask(SIG_UNBLOCK, &sigset, NULL);
	}

	t_r.it_value.tv_usec = 1000 * ms;
	t_r.it_value.tv_sec = t_r.it_interval.tv_sec = t_r.it_interval.tv_usec = 0;

	if(setitimer(ITIMER_REAL, &t_r, NULL != 0))
	{
		char cstr[256];
		sprintf(cstr, "matimer_set_timeout setitimer ms=%ld errno=%d", ms, errno);
		maLog(cstr);
		return 1;
	}	
	
	return 0;
	
} // matimer_set_timeout


int matimer_unset_timeout()
{
	struct itimerval no_timeout = {0};
	// no_timeout.it_value.tv_sec = no_timeout.it_value.tv_usec = 0;
	// no_timeout.it_interval.tv_sec = no_timeout.it_interval.tv_usec = 0;
	if(setitimer(ITIMER_REAL, &no_timeout, NULL))
	{
		char cstr[256];
		sprintf(cstr, "matimer_unset_timeout setitimer errno=%d", errno);
		maLog(cstr);
		return 1;
	}	
	
	return 0;
	
} // matimer_unset_timeout

#endif // defined(__MAOSX__) || defined(__MAUNIX__)
*/


#ifdef __cplusplus
 }
#endif

// eof
