// maUtils.h

/*
 * MoreAmp
 * Copyright (C) 2004-2008 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef __MAUTILS_H__
#define __MAUTILS_H__

#ifdef __cplusplus
 extern "C"
 {
#endif

void ShowErr(const char *msg);

double SliderDBFromPos(int value);
int SliderPosFromDB(double db);

int SongIsInMemory(char *filepath);
int SongListIsCda();
int SongFileExists(char *songfile);
int PathnameFromFilename(char *filename, char *pathname);
int FilenameFromPathname(char *pathname, char *filename);
int FileNameRemoveExtension(char *filename);
int FileNameHasExtension(char *fname, const char *ext);
int FileNameHasExtensionPlayable(char *fname);
int FileIsPlayable(char *filepath);
int maCreateDir(char *from);

int maFileExists(char *filepath);
int maDirExists(char *dirpath);
int maClearDir(char *dirpath);
int maFileCopy(char *frompath, char *topath, int concat);
int maFileConcat(char *frompath, char *topath);

int IsAudioCd(char *fname);
char *SecondsToStrHMS(char *strhms, int s);
char *MillisecondsToStrHMS(char *strhms, int ms);
int StrHMSToMilliSeconds(char *strhms);

FILE *mafopen(char *filename, const char *mode);
int machmod(char *filename, int mode);
int mamkdir(char *filename, int mode);
int maunlink(char *filename);
char *mafgets(char *s, int n, FILE *fp);

int mastrcasecmp(char *s1, char *s2);
int mastrncasecmp(char *s1, char *s2, int n);
char *mastrcasestr(char *s1, char *s2);
int StrOSXtoStrBSD(char *pfilepath);
int StrBSDtoStrOSX(char *pfilepath);

#ifdef __MAWIN__
 int maWinResolveShortcut(char *shortcut, char *pathname);
 int maGetLongPathName(char *infname, char *outfname, int len);
#endif

int maNamedSemaphoreOpen(void **ppsem, char *name);
int maNamedSemaphoreUnlink(void *psem, char *name);
int maNamedSemaphorePost(void *psem, char *name);
int maNamedSemaphoreWait(void *psem, char *name);
int maSemaphoreNew(void **ppsem);
int maSemaphoreDelete(void *psem);
int maSemaphorePost(void *psem);
int maSemaphoreWait(void *psem);
int maSemaphoreWaitTimeout(void *psem, int timeout);

void ThreadYield();
void AppYield();
int maSleep(unsigned int millis);

#if defined(MAUSESHM) || defined(MAUSESHF)
 typedef struct
 {
	char name[32];
	void *psem;
 } MALOCK;
 int maNamedLock(MALOCK *lock);
 int maNamedUnlock(MALOCK *lock);
#endif

#if defined(__MAOSX__)
 int maGetRootVolumeName(char *name);
#endif

#if defined(__MAOS9__)
 int maMacFSSpec2MacFilename(char *pname, void *pspec);
 int maMacFilename2FSSpec(char *pname, void *pspec);
#endif

#if defined(__MAOSX__) || defined(__MAOS9__)
 int HaveMPServices();
#endif

/*
#if defined(__MAOSX__) || defined(__MACUNIX__)
 void maBSDpath(char *path);
#endif
*/
 
int maThreadCreate(void *threadfunction, void **ppthreadid, int threadstacksz, int threadpri);
int maThreadSetPriority(void *pthreadid, int pri);

#if defined(MAUSESHM) || defined(MAUSESHF)
 void *mashmopen(size_t shmsize, char *keyfile);
 int mashmclose(void *pmashm);
 int mashmread(char *ptr, size_t size, size_t nmemb, void *pmashm);
 int mashmwrite(char *ptr, size_t size, size_t nmemb, void *pmashm);
 int mashmdelete(void *pmashm);
#endif

#ifdef MAINSTANCES
 int magetpid();
 int maNumProcesses(int *pid);
#endif

char *magetmoreampdir(char *buf, int size);
char *magetcwd(char *buf, int size);
char *magetbindir(char *buf, int size);
int masetcwd(char *dir);
#if defined(__MAWIN__)
 int maIsWin9x();
#endif
	 
unsigned long maGetLocalTimeMillis();

int matimer_set_timeout(long ms, void *callback);
int matimer_unset_timeout();

#if defined(MAMEMTRACE)
 extern int g_nmamallocs;
 extern int g_nmacallocs;
 extern int g_nmafrees;
 void *mamalloc(int a);
 void *macalloc(int a, int b);
 void mafree(void *a);
 void maTrace(char *str);
#endif

int string2int(char *digit, int *pval);

int maIsFullPath(char *path);

#ifdef __cplusplus
 }
#endif

#endif // __MAUTILS_H__

// eof
