// maWriteAac.c

/*
 * MoreAmp
 * Copyright (C) 2004-2008 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

 // with code from faac 1.24 frontend
 
#include "maConfig.h" // included by all moreamp files

#ifdef __MAWIN__
 #ifdef MAUNUSEDWIN
  #pragma warning( disable : 4100 ) // unreferenced formal parameter
 #endif
 #pragma warning( disable : 4206 ) // nonstandard extension used : translation unit is empty
 #pragma warning( disable : 4514 ) // 'mad_f_mul_inline' : unreferenced inline function has been removed
 #pragma warning( disable : 4702 ) // unreachable code
#endif

#ifdef ENCODEAAC

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include <errno.h>

#ifdef __MAOSX__
 #include <sys/stat.h>
#endif

#if defined(__MAOS9__)
 #include <stat.h>
#endif

#ifdef __MAWIN__
 typedef long int32_t;
#endif

#include "maDefs.h"
#include "maPlayer.h"
#include "maUtils.h"
#include "faac.h"
#include "maWriteAac.h"


#define DEFAULT_TNS 0
faacEncHandle g_aachEncoder = NULL;
unsigned long g_aacsamplesInput, g_aacmaxBytesOutput;
unsigned char *g_aacbitbuf = NULL;
short *g_aacpcmbuf= NULL;
short *g_aacend;

int OpenWriteAac()
{
	int channels = 2;
	faacEncConfigurationPtr myFormat;
	unsigned int useAdts = g_sets.fileout[g_k[kOUTAAC]].mode & kACCADTS ? 1 : 0;
	unsigned long quantqual;

	char cstr[256];

	switch (g_sets.fileout[g_k[kOUTAAC]].quality)
	{
		case 0  : quantqual = 130; break;
		case 1  : quantqual = 120; break;
		case 2  : quantqual = 110; break;
		case 4  : quantqual =  90; break;
		case 5  : quantqual =  80; break;
		case 6  : quantqual =  70; break;
		default : quantqual = 100; break;
	}

	// open the encoder library
	g_aachEncoder = faacEncOpen(g_state.insamplerate, channels,
							&g_aacsamplesInput, &g_aacmaxBytesOutput);

	if(g_aachEncoder == NULL)
	{
		maLog("OpenWriteAac faacEncOpenfailed");
		return 1;
	}
	
	g_aacpcmbuf = (short *)MAMALLOC(2 * g_aacsamplesInput * sizeof(short));
	
	if(g_aacpcmbuf == NULL)
	{
		sprintf(cstr, "OpenWriteAac MAMALLOC failed for %d bytes",
											(int)(2 * g_aacsamplesInput * sizeof(short)));
		maLog(cstr);
		return 1;
	}

	g_aacbitbuf = (unsigned char*)MAMALLOC(g_aacmaxBytesOutput * sizeof(unsigned char));
	
	if(g_aacbitbuf == NULL)
	{
		sprintf(cstr, "OpenWriteAac MAMALLOC failed for %d bytes",
									(int)(g_aacmaxBytesOutput * sizeof(unsigned char)));
		maLog(cstr);
		return 1;
	}
	
	// put the options in the configuration struct
	myFormat = faacEncGetCurrentConfiguration(g_aachEncoder);
	myFormat->aacObjectType = LOW; //objectType;
	myFormat->mpegVersion = MPEG2; //mpegVersion;
	myFormat->useTns = 0; //useTns;
	
	myFormat->shortctl = SHORTCTL_NORMAL;
	
	myFormat->useLfe = 0;
	
	myFormat->allowMidside = 1; //useMidSide;

	myFormat->bitRate = 0;

	myFormat->bandWidth = 0; //cutOff;

	if(quantqual > 0)
		myFormat->quantqual = quantqual;

	myFormat->outputFormat = useAdts;
	
	myFormat->inputFormat = FAAC_INPUT_16BIT;

	if(!faacEncSetConfiguration(g_aachEncoder, myFormat))
	{
		sprintf(cstr, "OpenWriteAac unsupported output format!");
		maLog(cstr);
		return 1;
	}

	g_aacend = g_aacpcmbuf;

	g_stopB = 0;

	return 0;

} // OpenWriteAac


int WriteAac(void* pcmblock, unsigned long frames, unsigned long len)
{
#ifdef MAUNUSED
 (void)frames;
#endif

	unsigned long bytesToFile;
	unsigned long samplesToEncode;
	short *pcm;
	char cstr[256];
	
	samplesToEncode = len / sizeof(short);
	pcm = (short *)pcmblock;
	
	// encoding loop

	while(!g_stopB && samplesToEncode >= g_aacsamplesInput)
	{
		// concat pcm to any previous incomplete block
		memcpy(g_aacend, pcm, g_aacsamplesInput * sizeof(short));
		
		// encode exactly g_aacsamplesInput samples
		bytesToFile = faacEncEncode(g_aachEncoder, (int32_t *)g_aacpcmbuf,
							g_aacsamplesInput, g_aacbitbuf, g_aacmaxBytesOutput);

		if(bytesToFile < 0)
		{
			sprintf(cstr, "WriteAac faacEncEncode() failed");
			maLog(cstr);
			return 1;
		}

		// write to output file
		if(bytesToFile > 0)
		{
			fwrite(g_aacbitbuf, 1, bytesToFile, g_outputfilefp);
		} // if(bytesToFile > 0)
				
		// shift incomplete block
		if(g_aacend != g_aacpcmbuf)
			memmove(g_aacpcmbuf, g_aacpcmbuf + g_aacsamplesInput,
										(g_aacend - g_aacpcmbuf) * sizeof(short));

		// adjust pointers
		samplesToEncode -= g_aacsamplesInput;
		pcm += g_aacsamplesInput;
		
	} // while
	
	// save incomplete block
	if(samplesToEncode > 0)
	{
		memcpy(g_aacend, pcm, samplesToEncode * sizeof(short));
		g_aacend += samplesToEncode;
	}
	
	// until saved block is big enough to encode
	if(g_aacend >= g_aacpcmbuf + g_aacsamplesInput)
	{
		// encode exactly g_aacsamplesInput samples
		bytesToFile = faacEncEncode(g_aachEncoder, (int32_t *)g_aacpcmbuf,
							g_aacsamplesInput, g_aacbitbuf, g_aacmaxBytesOutput);

		if(bytesToFile < 0)
		{
			sprintf(cstr, "WriteAac faacEncEncode() failed");
			maLog(cstr);
			return 1;
		}

		// write to output file
		if(bytesToFile > 0)
		{
			fwrite(g_aacbitbuf, 1, bytesToFile, g_outputfilefp);
		} // if(bytesToFile > 0)
		
		g_aacend -= g_aacsamplesInput;
		
		// shift incomplete block
		if(g_aacend != g_aacpcmbuf)
			memmove(g_aacpcmbuf, g_aacpcmbuf + g_aacsamplesInput,
										(g_aacend - g_aacpcmbuf) * sizeof(short));
	}
	
	return 0;

} // WriteAac


int CloseWriteAac()
{
	unsigned long bytesToFile;
	int32_t dummy;
	int i;

	/*
	char cstr[256];
	// pad to 0 and encode last saved blocks
	if(g_aacend != g_aacpcmbuf)
	{
		memset(g_aacend, 0, (g_aacend - g_aacpcmbuf) * sizeof(short));
		
		// encode exactly g_aacsamplesInput samples
		bytesToFile = faacEncEncode(g_aachEncoder, (int32_t *)g_aacpcmbuf,
							g_aacsamplesInput, g_aacbitbuf, g_aacmaxBytesOutput);

		if(bytesToFile < 0)
		{
			sprintf(cstr, "CloseWriteAac faacEncEncode() failed");
			maLog(cstr);
			return 1;
		}

		// write to output file
		if(bytesToFile > 0)
			fwrite(g_aacbitbuf, 1, bytesToFile, g_outputfilefp);
	}
	*/

	// final calls with 0 bytes
	if(g_aachEncoder && g_aacbitbuf && g_outputfilefp)
	{
		i = 64; // arbitrary
		while(i-- > 0)
		{
			if((bytesToFile = faacEncEncode(g_aachEncoder, &dummy, 0, g_aacbitbuf,
														g_aacmaxBytesOutput)) > 0)
				fwrite(g_aacbitbuf, 1, bytesToFile, g_outputfilefp);

			if(bytesToFile <= 0)
				break;
		}
	}

	if(g_aachEncoder)
	{
 		faacEncClose(g_aachEncoder);
		g_aachEncoder = NULL;
	}
	
	if(g_aacpcmbuf)
	{
		MAFREE(g_aacpcmbuf);
		g_aacpcmbuf = NULL;
	}
		
	if(g_aacbitbuf)
	{
		MAFREE(g_aacbitbuf);
		g_aacbitbuf = NULL;
	}
		
	return 0;

} // CloseWriteAac


#ifdef __cplusplus
}
#endif

#endif

// eof
