// maWriteFlac.c

/*
 * MoreAmp
 * Copyright (C) 2004-2008 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "maConfig.h" // included by all moreamp files

#ifdef __MAWIN__
 #ifdef MAUNUSEDWIN
  #pragma warning( disable : 4100 ) // unreferenced formal parameter
 #endif
 #pragma warning( disable : 4206 ) // nonstandard extension used : translation unit is empty
 #pragma warning( disable : 4514 ) // 'mad_f_mul_inline' : unreferenced inline function has been removed
#endif

#ifdef ENCODEFLAC

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include <errno.h>

#ifdef __MAOSX__
 #include <sys/stat.h>
#endif

#ifdef __MAOS9__
 #include <stat.h>
#endif

#include "maDefs.h"
#include "maPlayer.h"
#include "maUtils.h"
#include "maWriteFlac.h"

#include "FLAC/all.h"

#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT < 8
 #ifndef LEGACY_FLAC
  #define LEGACY_FLAC 1
 #endif
 // #include "FLAC/file_encoder.h"
#else
 #undef LEGACY_FLAC
#endif

#if defined(LEGACY_FLAC)
 FLAC__FileEncoder *l_flacencoder = NULL;
 FLAC__FileEncoderState l_flacencstate = (FLAC__FileEncoderState)NULL;
#else
 void encoder_progress_callback(const FLAC__StreamEncoder *encoder, FLAC__uint64 bytes_written, FLAC__uint64 samples_written,
											unsigned frames_written, unsigned total_frames_estimate, void *client_data);
 FLAC__StreamEncoder *l_flacencoder = NULL;
 FLAC__StreamEncoderState l_flacencstate = (FLAC__StreamEncoderState)NULL;
 static void stream_encoder_progress_callback_(const FLAC__StreamEncoder *encoder, FLAC__uint64 bytes_written, FLAC__uint64 samples_written,
																unsigned frames_written, unsigned total_frames_estimate, void *client_data)
 {
	(void)encoder, (void)bytes_written, (void)samples_written, (void)frames_written, (void)total_frames_estimate, (void)client_data;
 }
#endif


int OpenWriteFlac()
{
	char cstr[256];
#if defined(LEGACY_FLAC)
#else
	FLAC__StreamEncoderInitStatus init_status;	
	FLAC__StreamMetadata *metadata[1];
	FLAC__StreamMetadata_VorbisComment_Entry entry;
#endif

	if(l_flacencoder)
	{
		sprintf(cstr, "OpenWriteFlac l_flacencoder already opened");
		maLog(cstr);
		return 1;
	}

#if defined(LEGACY_FLAC)
	if((l_flacencoder = FLAC__file_encoder_new()) == NULL)
	{
		sprintf(cstr, "OpenWriteFlac FLAC__file_encoder_new failed");
#else
	if((l_flacencoder = FLAC__stream_encoder_new()) == NULL)
	{
		sprintf(cstr, "OpenWriteFlac FLAC__stream_encoder_new failed");
#endif
		maLog(cstr);
		return 1;
	}

#if defined(LEGACY_FLAC)
	if(!FLAC__file_encoder_set_sample_rate(l_flacencoder, g_state.insamplerate))
	{
		sprintf(cstr, "OpenWriteFlac FLAC__file_encoder_set_sample_rate failed");
#else
	if(!FLAC__stream_encoder_set_sample_rate(l_flacencoder, g_state.insamplerate))
	{
		sprintf(cstr, "OpenWriteFlac FLAC__stream_encoder_set_sample_rate failed");
#endif
		maLog(cstr);
		return 1;
	}
	


#if defined(LEGACY_FLAC)
#else
	sprintf(cstr, "%s %s", MOREAMP_NAME, MOREAMP_VERSION_STRING);
	if((metadata[0] = FLAC__metadata_object_new(FLAC__METADATA_TYPE_VORBIS_COMMENT)) == NULL ||
			!FLAC__metadata_object_vorbiscomment_entry_from_name_value_pair(&entry, "ENCODER", cstr) ||
			!FLAC__metadata_object_vorbiscomment_append_comment(metadata[0], entry, false))
	{
		sprintf(cstr, "OpenWriteFlac metadata out of memory or tag error\n");
		maLog(cstr);
	}
	FLAC__stream_encoder_set_metadata(l_flacencoder, metadata, 1);
#endif



#if defined(LEGACY_FLAC)
 #if defined(__MAOSX__)
	char tobsdname[MAX_PATH];
	strcpy(tobsdname, g_outputfile);
	StrOSXtoStrBSD(tobsdname);
 	if(!FLAC__file_encoder_set_filename(l_flacencoder, tobsdname))
 #else
 	if(!FLAC__file_encoder_set_filename(l_flacencoder, g_outputfile))
 #endif
	{
		sprintf(cstr, "OpenWriteFlac FLAC__file_encoder_set_filename failed");
		maLog(cstr);
		return 1;
	}

	if((l_flacencstate = FLAC__file_encoder_init(l_flacencoder)) != FLAC__FILE_ENCODER_OK)
	{
		sprintf(cstr, "OpenWriteFlac FLAC__file_encoder_init failed, state=%d", l_flacencstate);
		maLog(cstr);
		return 1;
	}
#else
 #if defined(__MAOSX__)
	char tobsdname[MAX_PATH];
	strcpy(tobsdname, g_outputfile);
	StrOSXtoStrBSD(tobsdname);
	init_status = FLAC__stream_encoder_init_file(l_flacencoder, tobsdname, stream_encoder_progress_callback_, /*client_data=*/NULL);
 #else
	init_status = FLAC__stream_encoder_init_file(l_flacencoder, g_outputfile, stream_encoder_progress_callback_, /*client_data=*/NULL);
 #endif

	if(init_status != FLAC__STREAM_ENCODER_INIT_STATUS_OK)
	{
		sprintf(cstr, "OpenWriteFlac FLAC__stream_encoder_init_file failed, init_status=%d", init_status);
		maLog(cstr);
		return 1;
	}
#endif

	return 0;

} // OpenWriteFlac


int WriteFlac(void *pcmblock, unsigned long frames, unsigned long len)
{
#ifdef MAUNUSED
 (void)frames;
#endif

	int i;
	FLAC__int32 *p;
	short *s = (short *)pcmblock;
	unsigned int samplesperchannel = len >> 2; // / 4;
	int bytes = len << 1;
	char cstr[256];
	
	// copy pcmblock to an array of FLAC__int32's
	if((p = (FLAC__int32 *)MAMALLOC(bytes)) == NULL)
	{
#if defined(LEGACY_FLAC)
		l_flacencstate = FLAC__file_encoder_get_state(l_flacencoder);
#else
		l_flacencstate = FLAC__stream_encoder_get_state(l_flacencoder);
#endif
		sprintf(cstr, "WriteFlac MAMALLOC failed for %d bytes", bytes);
		maLog(cstr);
		return 1;
	}
	
	for(i = 0; i < (int)(len >> 1); i++)
		p[i] = s[i];
		
#if defined(LEGACY_FLAC)
	if(!FLAC__file_encoder_process_interleaved(l_flacencoder, p, samplesperchannel))
	{
		MAFREE(p);
		l_flacencstate = FLAC__file_encoder_get_state(l_flacencoder);
		sprintf(cstr, "WriteFlac FLAC__file_encoder_process_interleaved failed state=%d", l_flacencstate);
#else
	if(!FLAC__stream_encoder_process_interleaved(l_flacencoder, p, samplesperchannel))
	{
		MAFREE(p);
		l_flacencstate = FLAC__stream_encoder_get_state(l_flacencoder);
		sprintf(cstr, "WriteFlac FLAC__stream_encoder_process_interleaved failed state=%d", l_flacencstate);
#endif
		maLog(cstr);
		return 1;
	}
	
	MAFREE(p);
	return 0;
	
} // WriteFlac


int CloseWriteFlac()
{
	if(l_flacencoder)
	{
#if defined(LEGACY_FLAC)
		FLAC__file_encoder_finish(l_flacencoder);
		FLAC__file_encoder_delete(l_flacencoder);
#else
		FLAC__stream_encoder_finish(l_flacencoder);
		FLAC__stream_encoder_delete(l_flacencoder);
#endif
		l_flacencoder = NULL;
	}
	
	return 0;
		
} // CloseWriteFlac

#ifdef __cplusplus
}
#endif

#endif

// eof
