// maPlayer.c

/*
 * MoreAmp
 * Copyright (C) 2004-2008 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "maConfig.h" // included by all moreamp files

#ifdef __MAWIN__
 #ifdef MAUNUSEDWIN
  #pragma warning( disable : 4100 ) // unreferenced formal parameter
 #endif
 #pragma warning( disable : 4054 ) // 'type cast' from function pointer 'long (__cdecl *)(void *)' to data pointer 'void *'
 #pragma warning( disable : 4127 ) // conditional expression is constant
 #pragma warning( disable : 4152 ) // onstandard extension, function/data pointer conversion in expression
 #pragma warning( disable : 4244 ) // conversion from 'x ' to 'y ', possible loss of data
 #pragma warning( disable : 4702 ) // unreachable code
 #pragma warning( disable : 4706 ) // assignment within conditional expression
 #pragma warning( disable : 4761 ) // integral size mismatch in argument; conversion supplied
 #define __RPCASYNC_H__ // rpcasync.h(45) : warning C4115: '_RPC_ASYNC_STATE' : named type definition in parentheses
#endif

#ifdef __cplusplus
extern "C"
{
#endif

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include <errno.h>

#ifndef __MAWIN__
 #include <unistd.h>
#endif

#if defined(__MAOS9__)
 #include <unix.h>
 #include <stat.h>
 #include <MacMemory.h>
 #include <MacErrors.h>
#endif

#if defined(__MAOSX__)
 #include <sys/stat.h>
 #include <MacMemory.h>
#endif

#if defined(__MAOSX__) || defined(__MAOS9__)
 #include <AEDataModel.h>
 #include <AppleEvents.h>
 #include <CarbonEvents.h>
 #include <Power.h>
 #include <MacTypes.h>
#endif

#ifdef __MAUNIX__
 #include <sys/stat.h>
 #include <pthread.h>
#endif

#ifdef __MAWIN__
 #include <wtypes.h>
 #include <io.h>
#endif

// Application includes

#include "maDefs.h"
#include "maGUI.h"
#include "maMemFile.h"
#include "maEqualizer.h"
#include "maPlayer.h"
#include "maPlayMp3.h"
#include "maPlayOgg.h"
#include "maPlayWma.h"
#include "maPlayAac.h"
#include "maPlayMp4.h"
#include "maPlayFlac.h"
#include "maPlayAif.h"
#include "maEffects.h"
#include "maSongList.h"
#include "maUtils.h"

#if defined(__MAOS9__)
 #include "maAudioMac.h"
#endif

#if defined(__MAOSX__)
 #include "maAudioMac.h"
#endif

#ifdef __MAUNIX__
 #include "maAudioUnix.h"
#endif

#ifdef __MAWIN__
 #include "maAudioWin.h"
#endif

#ifndef MANOCDA
 #include "maCda.h"
 #include "maPlayCda.h"
#endif

#ifdef ENCODEMP3
 #include "maWriteMp3.h"
#endif

#ifdef ENCODEOGG
 #include "maWriteOgg.h"
#endif

#ifdef ENCODEFLAC
 #include "maWriteFlac.h"
#endif

#ifdef ENCODEAAC
 #include "maWriteAac.h"
#endif

#ifdef ENCODEMP4
 #include "maWriteMp4.h"
#endif

#ifdef ENCODEWMA
 #include "maWriteWma.h"
#endif

#ifdef ENCODEWAV
 #include "maWriteWav.h"
#endif

#ifdef ENCODEAIF
 #include "maWriteAif.h"
#endif

//////////////////////////////////////////////////////////////////////////////////////////////
// defs used in this module only
#define INDEXLEN 6		// length of index field in songlist (incl space) e.g. "00123 MacIntosh HD:mp3:..."
#define POSINCR 100			// increment value in ms for beg and end position little arrows
#define POSDECR POSINCR		// decrement value in ms for beg and end position little arrows
#define MINPOSDELTA 100 	// ms minimum g_posend - g_posbeg
#define SEEKPERIOD 100		// ms to wait in runloop when seeking
#define TITLEPERIOD 1000	// ms to wait in runloop when not seeking
#define SPINDOWNPERIOD 30	// 30 titleperiods between attempts to spindown the hard disk
#define FXBUFFERSIZE (FIFOSIZE * 2) 
// values for l_outputfilter
#define OUTPUT_STEREO	  1
#define OUTPUT_MONO		  2
#define OUTPUT_LEFT		  4
#define OUTPUT_RIGHT	  8
#define OUTPUT_DIFF		 16
#define OUTPUT_REVERSE	512
// values for g_sets.monodiff
#define MONOCHECKBOXL	1
#define MONOCHECKBOXR	2
#define MONOCHECKBOXSUB	4
#define MONOCHECKBOXREV	8

// multiple instances
#if defined(MAINSTANCES)
 #define MAINSTANCEMAX 32 
 typedef struct // shared among all instances, in memory or on disk
 {
	int pid[MAINSTANCEMAX];
	int multipleinst;
 } MAINSTANCE;
 #define MAINSTANCEFILE (char *)"maInstances.mai"
 #if defined(MAUSESHF)
  #define MAKEYFILE l_mainstancefile
 #endif
 #if defined(MAUSESHM)
  #if defined(__MAWIN__)
   #define MAKEYFILE l_mainstancefile // memory-mapped file
 #else
  #define MAKEYFILE (char *)"/tmp" // for ftok
 #endif
#endif
#endif

#if defined(__MAUNIX__)
 #define MAREADMEDIR l_mareadmedir
#else
 #define MAREADMEDIR g_madir
#endif

#define MAVERSIONISCURRENT 0
#define MAVERSIONISPREVIOUS 1
#define MAVERSIONISUNKNOWN 2

//////////////////////////////////////////////////////////////////////////////////////////////
// g_... globals also used in other modules
char g_prognametitle[32];
int g_fifoready;
int g_effectschange;
#if defined(__MAWIN__) || defined(__MAUNIX__)
 #define dskFulErr -34
#endif
unsigned long g_nframes[NSNDBUFS];
int g_cataloguing;
int g_posbeg;
int g_posend = MINPOSDELTA;
int g_posbegon;
int g_posendon;
int g_reptsong;
int g_running;
int g_noreload;
int g_cancel;
int g_stopB;
int g_loadednum;
int g_loadedcurrent;
int g_listbusy;
int g_loadedmemsize;
#ifdef AUDIO_MAC_SOUNDMGR
 int g_buftoplay = 0;
#endif
int g_listselect;
int g_songsnum;
int g_bufready[NSNDBUFS];
void *g_w[WINDOWLAST];
void *g_menubar;
#ifdef AUDIO_WIN_WAVE
 void *g_semaudiowin;
#endif
#if defined(__MAOSX__) || defined(__MAOS9__)
 Handle g_memhandle;
#else
 char *g_memhandle;
#endif
char g_madir[MAX_PATH];
#if defined(__MAUNIX__)
 char l_mareadmedir[MAX_PATH];
#endif
char g_logpath[MAX_PATH];
char g_songlistsdir[MAX_PATH];
char g_settingsdir[MAX_PATH];
char g_fileoutdir[MAX_PATH];
char g_loadedname[SONGSLOADMAX][MAX_PATH];
char g_hdsongpath[SONGSLOADMAX][MAX_PATH];
char g_settingspath[MAX_PATH];
#if !defined(MANOCDA) && defined(__MAUNIX__)
 MASETSCDROM g_setscdrom;
#endif

#if defined(MAKEYBOARD)
 char g_keybindingspath[MAX_PATH];
#endif

char g_settingsdef[MAX_PATH];
char g_songlist[MAX_PATH];
char g_songlistdrop[MAX_PATH];
char g_songlistcda[MAX_PATH];
#if !defined(NODECODEAUDIOIN)
 char g_songlistaudioin[MAX_PATH];
 char g_songfileaudioin[MAX_PATH];
#endif
char g_songlisttemp[MAX_PATH];
char g_readmepath[MAX_PATH];
char g_readmefind[256];
char g_catalogfind[256];
#if defined(MAKEYBOARD)
 char g_keybindingsfind[256];
#endif
char g_logfind[256];
FILE *g_fpsonglist;
MASETS g_sets;
MASETS g_setsbu;
int g_pause = 0;
int g_buffersize;
int g_argc;
char **g_argv;
unsigned char *g_fxbuffer = NULL;
char *g_songmemptr[SONGSLOADMAX + 1];
char *g_sndbuf[NSNDBUFS] = {NULL};
#if defined(MAINSTANCES)
 int g_thisinstance;
#endif
#if defined(MAKEYBOARD)
 int g_maOnKeySkip = 0;
#endif
#if defined(MAINSTANCES)
  void *g_pmashm = NULL;
#endif

int g_waitaudiodone; // not needed if AUDIO_WIN_WAVE

// input file extensions (order matches playable files enum in Defs.h)
char *g_extstr[] =
{
 (char *)EXTMP1,
 (char *)EXTMP2,
 (char *)EXTMP3,
 (char *)EXTOGG,
 (char *)EXTWMA,
 (char *)EXTAIF,
 (char *)EXTAIFF,
 (char *)EXTWAV,
 /* these could be added to maPlayAif
 (char *)EXTAU,		// Sun/NeXT AU format (big endian)
 (char *)EXTRAW,	// RAW PCM data
 (char *)EXTPAF,	// Ensoniq PARIS file format
 (char *)EXTSVX,	// Amiga IFF / SVX8 / SV16 format
 (char *)EXTNIST,	// Sphere NIST format
 (char *)EXTVOC,	// VOC fil
 (char *)EXTIRCAM,  // Berkeley/IRCAM/CARL
 (char *)EXTW64,	// Sonic Foundry's 64 bit RIFF/WAV
 (char *)EXTMAT4,	// Matlab (tm) V4.2 / GNU Octave 2.0
 (char *)EXTMAT5,	// Matlab (tm) V5.0 / GNU Octave 2.1
 (char *)EXTPVF,	// Portable Voice Format
 (char *)EXTXI,		// Fasttracker 2 Extended Instrument
 (char *)EXTHTK,	// HMM Tool Kit format
 */
 (char *)EXTAAC,
 (char *)EXTM2A,
 (char *)EXTM4A,
 (char *)EXTMP4,
 (char *)EXTFLA,
 (char *)EXTFLAC,
 (char *)EXTMAA,
 (char *)EXTMAF,
 (char *)EXTMAS,
 (char *)EXTM3U,
 (char *)EXTCDA // keep this here
};

// output file extensions (order matches outputs enum in Defs.h)
char *g_extstrout[] =
{
 (char *)MASTRSOUND,
 
#ifdef ENCODEMP3
 (char *)EXTMP3,
#endif

#ifdef ENCODEOGG
 (char *)EXTOGG,
#endif

#ifdef ENCODEFLAC
 (char *)EXTFLAC,
#endif

#ifdef ENCODEAAC
 (char *)EXTAAC,
#endif

#ifdef ENCODEMP4
 (char *)EXTMP4,
#endif

#ifdef ENCODEWMA
 (char *)EXTWMA,
#endif

#ifdef ENCODEWAV
 (char *)EXTWAV,
#endif

#ifdef ENCODEAIF
 (char *)EXTAIF
#endif
};

MASTATE g_state;

int g_k[kOUTMAX];

//////////////////////////////////////////////////////////////////////////////////////////////
// functions used only in this module 
void ShowListWindow();
int	LoadLoop();
int FillSongBuffer(int dest);
int AllocSongRec(SONGRECORD **ppsongrec);
int ZeroSongRec(SONGRECORD **ppsongrec);
int FileToMem(char *from, char *tomem, int maxmem, long *bytesread);
int FileToVDisk(char *from, char *tomem, int maxmem, long *bytesread);
void PlayerSeek(int pos);
void SetAppTitle2();
int PlayLoop();
int GetInFiletype(char *playpath);
int LaunchPlayThread(char *playpath);
int EqDoParamChange();
int GetSongMem();
int GetRecentSongListName();
int SetSettingsPath();
int OpenSongList();
void FileOutWindowDisable();
void FileOutWindowEnable();
void DisableButtonsMenus();
void EnableButtonsMenus();
void SetLogMenus();
void SetAppWindowMsg(char *msg);
void SetListWindowTitle(char *msg);
void Init_g_k();
void CheckFileoutSettings();
int CheckSettings(MASETS *psets);
int LoadSettings();
int SaveSettings();
#if !defined(MANOCDA) && defined(__MAUNIX__)
 int LoadFactorySettingsCDRom();
 int SaveSettingsCDRom();
#endif
int LoadFactorySettings(MASETS *psets);
int LoadFactorySettingsAudioIn(MASETS *psets);
int SaveSettingsAsDefault(MASETS *psets);
void SetPause(int pause);
void SetSliders(int slider);
void SetSliderCaption(int slider);
int SetGUIUseMem();
void SetGUIPopupButtons();
int ShowLoaded();
void DoBeginEndEdit(int panelid, int param);
void SetGUIMonoStereo();
void SetGUITitleInfo();
void SetPosSlider(int value);
void DoWindowEq(int show);
void DoWindowEq2(int show);
void DoWindowList(int show);
void DoWindowEffects(int show);
void DoWindowFileOut(int show);
void DoWindowPromptNum(int show);
int DumpSongInfo(char *songfpath, FILE *dumpfp, int nomem);
int ShowPlayerInfo();
int ShowSongList();
void ListWindowSetHeader(char *header);
int ListWindowGetValue();
void ListWindowAddString(char *str);
void ListWindowShowRow(int inrow);
void ListWindowClear();
void ListWindowCopyRow(char *str, int row);
void ListWindowShowRowPlaying();
int ShowSongInfo();
int GetRecentSonglist();
void ShowMessageValue(int value);
void OnMsgQuit();
void ApplyChannelSelection(char *soundbuf, unsigned long numFrames);
int MakeOutputFileName();
int maCloseOutputFile();
int maOpenOutputFile();
int maOutputToFile(void* pcmblock, unsigned long frames, unsigned long len, int whichbuf);
int maAllocBuffers();
int maFreeBuffers();
int maEraseBuffers();
void maOutputClose(int wait);	
int maOutputCanWrite();
maFifo *maFifoCreate(maFifo* pfifo, char *storage, unsigned long size);
void maFifoDelete(maFifo* pfifo);	
void maFifoClear(maFifo* pfifo);
char *maFifoGetStorage(maFifo* pfifo);
void maFifoErase(maFifo* pfifo);
int WriteFiFo(maFifo* pfifo, unsigned char *data, unsigned int len);
int WriteFiFoBlocking(maFifo* pfifo, unsigned char *data, unsigned int len);
int maFifoToOutput();
void SpinDown();
void AppQuit();
int SetVDisk();
#if !defined(MANOCDA) && defined(__MAUNIX__)
 int SetAudioCD();
 int ClearAudioCD();
#endif
int GetNextSong(int *pm, SONGRECORD *lsongrec);
#if defined(MAINSTANCES)
 int InstanceInit();
 int InstanceClearThis();
 int InstanceSet(int which, int value);
 int InstanceWrite(MAINSTANCE *pinstance);
 int InstanceSet(int which, int value);
 int InstanceRead(MAINSTANCE *pinstance);
 int SetMultipleInstances();
#endif
int GetVDisk();
void WindowsMoveDelta(int skipwindow, int xdelta, int ydelta);
void WindowMoveAndSetSize(int window, int xpos, int ypos, int xsize, int ysize);
void WindowsMoveAndSetSize();
void WindowsCopyPositions();
void WindowSetFactoryValues(int winconf);
void SetFileoutType(int which);
void LoadFactoryFileout(int which);
#if defined(MAKEYBOARD)
 void UpdateEffectsKbLabels();
 int SaveKeyBindings(int blank);
 int VerifyKeyBindings(int showerr);
 void GetFreeKeyBindings(char *p);
 void SetFactoryKeyBindings();
 void OnKeyAltSize();
 void OnKeySavedSize();
 void OnKeyDefaultSize();
 void OnKeyFactorySize();
 void OnKeyShowFileOut();
 void OnKeyEffectsWindow();
 void OnKeyCheckTitleInfo();
 void OnKeyCheckAltSize();
 void OnKeyCheckRandom(); 
 void OnKeyCheckReptPlayed();
 void OnKeyCheckLoopLoaded();
 void OnKeyCheckLoopList(); 
 #ifdef __MAOS9__
  void OnKeyCheckSpindown();
 #endif
 void OnKeyCheckHighPriority(); 
 void OnKeyCheckSinglePlay();
 void OnKeyCheckEqWindow(); 
 void OnKeyCheckListWindow(); 
 void OnKeyCheckEqualizer();
 void OnKeyCheckEffects();
 void OnKeyCheckRepeat(); 
 void OnKeyCheckBalance();
 void OnKeyCheckMonoLeft();
 void OnKeyCheckMonoRight();
 void OnKeyCheckMonoDiff();
 void OnKeyCheckLimiter();
 void OnKeyCheckEqExtra();
 void OnKeyCheckEqZoom();
 void OnKeyCheckEqLink();
 void OnKeyCheckInvertRight();
 void OnKeyCheckEq31Bands();
 void OnKeyCheckPitch();
 void OnKeyCheckTempo();
 void OnKeyCheckSpeed();
 void OnKeyCheckQualSpeed();
 void OnKeyCheckQualFilter();
 #if !defined(MANOCDA) && defined(__MAUNIX__)
  void OnKeyCheckCdParanoia();
 #endif
 void OnKeyCheckBoxBeg();
 void OnKeyCheckBoxEnd();
 void OnKeyCheckBoxM4A();
 void OnKeyCheckShowIndex();
 void OnKeyCheckShowDir();
 void OnKeyCheckShowPath();
 void ShowKeyStatus(char *status);
 void OnKeyButtonQuit();
 void OnKeyButtonStop();
 void OnKeyButtonPause();
 void OnKeyButtonPlay();
 void OnKeyButtonEqRestore();
 void OnKeyButtonEqInvert();
 void OnKeyPopupEqPreset1();
 void OnKeyPopupEqPreset2();
 void OnKeyPopupEqPreset3();
 void OnKeyPopupEqPreset4();
 void OnKeyResetSliderPitch();
 void OnKeyResetSliderTempo();
 void OnKeyResetSliderSpeed();
 void OnKeyPitchUp();
 void OnKeyPitch2Up();
 void OnKeyTempoUp();
 void OnKeySpeedUp();
 void OnKeyPitchDown();
 void OnKeyPitch2Down();
 void OnKeyTempoDown();
 void OnKeySpeedDown();
 void OnKeyOpenList();
 void OnKeyMakeList();
 void OnKeySaveListAs();
 void OnKeyMenuZero();
 void OnKeyPlayFile();
 void OnKeyPlayCD();
 void OnKeyEjectCD();
 void OnKeyViewAbout();
 void OnKeyViewCatalog();
 void OnKeyViewLog();
 void OnKeyClearLog();
 void OnKeyEditBindings();
 void OnKeyViewReadme();
 void OnKeyButtonInfo();
 void OnKeyButtonList();
 void OnKeyButtonLoaded();
 void OnKeyButtonSong();
 void OnKeyUpArrow();
 void OnKeyDownArrow();
 void OnKeyVolumeDown();
 void OnKeyVolumeUp();
 void OnKeyPlayIndex();
#endif
int maCheckSettingsVersion(char *settingsversion);

//////////////////////////////////////////////////////////////////////////////////////////////
// l_... globals used only in this module
int l_posend;
int l_poscurrent;
int l_sliderposms;
int l_sliderposmsbu;
int l_quit;
int l_songsnumplayed;
int l_songcurrent = -1; // 0-based index of songlist song currently playing
int l_loadednew;
int l_previous;
int l_next;
int l_loadedquit;
int l_songsindex[SONGSLOADMAX];
int l_droplaunch;
int l_outputfilter = OUTPUT_STEREO;
int l_diskeventpending;
int l_ispaused;
int l_posslidervalue;
int l_possliderchanged;
int l_posslidersetvalue;
int l_buftofill;
int l_mnextfile = -1;
int l_playerseeking = 0;
int l_pan = SEPARATIONMAX / 2;
int l_separation = SEPARATIONMAX;
char l_recentlist[MAX_PATH];
unsigned long l_endtime;
float l_separationch = 1.;
float l_separationmix = 0.;
float l_panleft = 0.5;
float l_panright = 0.5;
char *l_songmem;
char *l_songmemend;
char *l_songmempos;
unsigned char *l_fxoutdata = NULL;
unsigned long l_fxoutlen = 0;
char l_strclicked[LISTLINELEN + 2];
int l_songmemsize;
char l_settingsdrop[MAX_PATH];
char l_settingscda[MAX_PATH];
char l_settingsaudioin[MAX_PATH];
char l_settingspathlast[MAX_PATH];
maFifo l_mafifo;
maFifo *l_pmafifo;
SONGRECORD *l_songrec = NULL;
unsigned char *l_monobuf;
int l_monobytes;
unsigned long l_logdelay;
char l_cwd[MAX_PATH];
char l_bindir[MAX_PATH] = {0};
char *l_allhaveplayedstr = (char *) "All songs in the list have played\n\n"
									"To play the list again, either :\n\n"
									"- Zero List from the Tools menu, or\n\n"
									"- Set the LoopList checkbox.";
EQUPRESET l_eqpresettemp;
int l_noshowrow;

#if !defined(MANOCDA) && defined(__MAUNIX__)
 char l_setscdrompath[MAX_PATH];
#endif

#if defined(MAINSTANCES)
 #define MALOGLOCK &l_maloglock
 #define MAINITLOCK &l_mainitlock
 #define MAINSTANCELOCK &l_mainstancelock
  MALOCK l_maloglock = {"maloglock", NULL};
  MALOCK l_mainitlock = {"mainitlock", NULL};
  MALOCK l_mainstancelock = {"mainstancelock", NULL};
  #if defined(MAUSESHF) || (defined(MAUSESHM) && defined(__MAWIN__))
   char l_mainstancefile[MAX_PATH];
  #endif
#endif

#if defined(BINARYDISTRIB)
 int l_binbuild = 119;
#else
 int l_binbuild = 0;
#endif

int volminmaxL();
int volminmaxR();
void volumevalstrL(char *str);
void volumevalstrR(char *str);
#if !defined(MAGUINONE)
 int l_temptitle;
#endif
char l_songlist[MAX_PATH];
int l_keyindex = 0;
char l_loadeddir[MAX_PATH]; // path to vdisk dir to hold loaded songs

FILE *l_fplogpath;	// log file
char l_logcmdclear; // clear the log
char l_logcmdexist; // test for existance of a log
int l_logoff;		// no logging flag
int l_logtotal;		// log event count per session
int l_logcount;		// log event count per period
int l_havelog;		// have a log file

//////////////////////////////////////////////////////////////////////////


int volminmaxL()
{
	if(g_sets.volumesliderleft >= VOLUMEMAX)
		return VOLUMEMAX;
	if(g_sets.volumesliderleft <= VOLUMEMIN)
		return VOLUMEMIN;
	return g_sets.volumesliderleft;

} // volminmaxL


int volminmaxR()
{
	if(g_sets.volumesliderright >= VOLUMEMAX)
		return VOLUMEMAX;
	if(g_sets.volumesliderright <= VOLUMEMIN)
		return VOLUMEMIN;
	return g_sets.volumesliderright;

} // volminmaxR


#define VOLSCALEDB 100.0	// if defined, volume sliders scale is in dB
//#define VOLSCALEAU 79.9	// if defined, volume sliders scale is log audio taper, attenuation at mid-scale = 90%
							// else volume sliders scale is linear

int volumevalL()
{
	if(g_sets.volumesliderleft >= VOLUMEMAX)
		return VOLUMEMAX;
	if(g_sets.volumesliderleft <= VOLUMEMIN)
		return VOLUMEMIN;
#if defined(VOLSCALEDB)
	return (int)(0.5 + (VOLUMEMAX * exp(((g_sets.volumesliderleft - VOLUMEMAX) / (((double)VOLUMEMAX / VOLSCALEDB) * 20.0)))));
#elif defined(VOLSCALEAU)
	return (int)(0.5 + (((double)VOLUMEMAX / VOLSCALEAU) * (pow(VOLSCALEAU + 1.0, (double)g_sets.volumesliderleft / VOLUMEMAX) - 1.0)));	
#else
	return g_sets.volumesliderleft;
#endif

} // volumevalL()


int volumevalR()
{
	if(g_sets.volumesliderright >= VOLUMEMAX)
		return VOLUMEMAX;
	if(g_sets.volumesliderright <= VOLUMEMIN)
		return VOLUMEMIN;
#if defined(VOLSCALEDB)
	return (int)(0.5 + (VOLUMEMAX * exp(((g_sets.volumesliderright - VOLUMEMAX) / (((double)VOLUMEMAX / VOLSCALEDB) * 20.0)))));
#elif defined(VOLSCALEAU)
	return (int)(0.5 + (((double)VOLUMEMAX / VOLSCALEAU) * (pow(VOLSCALEAU + 1.0, (double)g_sets.volumesliderright / VOLUMEMAX) - 1.0)));
#else
	return g_sets.volumesliderright;
#endif

} // volumevalR()


void volumevalstrL(char *str)
{
#if defined(VOLSCALEAU)
	if(!g_sets.balance) // linear scale for left slider
	{
 #if defined(__MAOS9__)
		sprintf(str, "%d", (100 * volminmaxL()) / kVOLMAX); // 0-100
 #else
		sprintf(str, "%d", volminmaxL()); // 0-1000
 #endif
		return;
	}
#endif
	
#if defined(VOLSCALEDB) || defined(VOLSCALEAU)
	if(g_sets.volumesliderleft >= VOLUMEMAX)
	{
		strcpy(str, "max");
		return;
	}
	
	if(g_sets.volumesliderleft <= VOLUMEMIN)
	{
		strcpy(str, "-inf");
		return;
	}
#endif
	
#if defined(VOLSCALEDB)
	sprintf(str, "%0.1f", (((double)g_sets.volumesliderleft - VOLUMEMAX) / (VOLUMEMAX / VOLSCALEDB)));
#elif defined(VOLSCALEAU)
	sprintf(str, "%0.1f", 20.0 * log((double)volumevalL() / VOLUMEMAX));
#else
	sprintf(str, "%d", g_sets.volumesliderleft);
#endif

} // volumevalstrL


void volumevalstrR(char *str)
{
#if defined(VOLSCALEDB) || defined(VOLSCALEAU)
	if(g_sets.volumesliderright >= VOLUMEMAX)
	{
		strcpy(str, "max");
		return;
	}
	
	if(g_sets.volumesliderright <= VOLUMEMIN)
	{
		strcpy(str, "-inf");
		return;
	}
#endif
	
#if defined(VOLSCALEDB)
	sprintf(str, "%0.1f", (((double)g_sets.volumesliderright - VOLUMEMAX) / (VOLUMEMAX / VOLSCALEDB)));
#elif defined(VOLSCALEAU)
	sprintf(str, "%0.1f", 20.0 * log((double)volumevalR() / VOLUMEMAX));
#else
	sprintf(str, "%d", g_sets.volumesliderright);
#endif

} // volumevalstrR


int ChangeSongList(char *songlist)
{
	int ret;
	
	strcpy(g_songlist, songlist);
		
	if(g_w[WINDOWAPP])
		SetAppWindowTitle((char *)"");

	// invalidate current vdisk contents
	g_loadednum = 0;

	// count songs and set g_settingspath	
	if((ret = OpenSongList()) && ret != 3)  // 3 means empty songlist 
	{
		char cstr[256];
		sprintf(cstr, "ChangeSongList OpenSongList failed on '%s'", g_songlist);
		maLog(cstr);
		return 1;
	}
	
	UpdateRecent(g_songlist, 0);
	
	if(g_w[WINDOWAPP])
	{
		ShowListWindow();		
		UpdateDisplay();
	}
					
	return 0;
		
} // ChangeSongList


int DeleteSongList(char *songlist)
{
	char name[MAX_PATH];
	char setsfile[MAX_PATH];
	char cstr[MAX_PATH + 256];
	
	if(!strcmp(g_songlist, songlist))
	{
		sprintf(cstr, "This is the current songlist, it cannot be deleted:\n\n%s", g_songlist);
		ShowErr(cstr);
		return 1;
	}
	if(!strcmp(g_songlistdrop, songlist) || !strcmp(g_songlistcda, songlist)
#if !defined(NODECODEAUDIOIN)
		|| !strcmp(g_songlistaudioin, songlist) || !strcmp(g_songfileaudioin, songlist)
#endif
		)
	{
		sprintf(cstr, "This is an internal %s file, it cannot be deleted:\n\n%s", MOREAMP_NAME, songlist);
		ShowErr(cstr);
		return 1;
	}

	// delete songlist
	if(maFileExists(songlist))
	{
		if(maunlink(songlist))
		{
			sprintf(cstr, "DeleteSongList maunlink errno=%d '%s'", errno, songlist);
			maLog(cstr);
			return 1;
		}		
	}
	
	// delete settings file
	if(FilenameFromPathname(songlist, name))
	{	
		sprintf(cstr, "DeleteSongList failed to get filename from '%s'", songlist);
		maLog(cstr);
		return 1;
	}					
	FileNameRemoveExtension(name);
	sprintf(setsfile, "%s%c%s%s", g_settingsdir, FILESEP, name, EXTMAS);
	if(maFileExists(setsfile))
	{
		if(maunlink(setsfile))
		{
			sprintf(cstr, "DeleteSongList maunlink errno=%d '%s'", errno, setsfile);
			maLog(cstr);
		}		
	}
	
	// update the recentsonglist file
	UpdateRecent(songlist, 1);
						
	return 0;
		
} // DeleteSongList


int PlaySonglist(char *songlist)
{
	if(!g_w[WINDOWAPP])
		return PlaySonglistB(songlist);
	
	strcpy(g_newsonglist, songlist);
	if(g_running)
	{
		DoTimerStop(100);
		DoTimerPlaySonglist(3000); 
	}
	else
		DoTimerPlaySonglist(100);
			
	return 1;
	
} // PlaySonglist
		

int PlaySonglistB(char *songlist)
{
	int err;
	l_mnextfile = -1;
	l_droplaunch = 1;
	l_songsindex[0] = -1;

	err = ChangeSongList(songlist);

	if(!g_w[WINDOWAPP])
	{
		if(err)
			LoadDefaultSettings(&g_sets);
		AppWindowMakeA();
	}			
	else
		maLog(&l_logcmdexist);
		
	if(err)
		return 1;
		
	// start
	if(g_sets.output == kOUTSOUND)
	{
		if(strcmp(g_songlist, g_songlistdrop) || g_sets.autostart)
			DoTimerPlay(100);
	}
		
	return 0;
	
} // PlaySonglistB


void ShowListWindow()
{
	switch (g_sets.listwindow & (LBOXON | LBOXINFO | LBOXLIST | LBOXLOADED | LBOXID3))
	{
		case LBOXON | LBOXINFO :
			ShowPlayerInfo();
		break;

		case LBOXON | LBOXLIST :
			ShowSongList();
		break;

		case LBOXON | LBOXLOADED :
			ShowLoaded();
		break;

		case LBOXON | LBOXID3 :
			ShowSongInfo();
		break;

		default :
		break;
	}
	
	SetGUIUseMem();
	
	ListSetColumnWidth();

} // ShowListWindow


int l_runloop = 0;
// this is called when timerrunloop fires from the gui event loop,
// or if nogui, from the 'main' loop, 
signed long RunLoop(void *parameter)
{		
#ifdef MAUNUSED
 (void)parameter;
#endif

	int ret;
	
	if(l_runloop == 0)
	{
		l_runloop = 1;	
		DisableButtonsMenus();
		l_quit = 0;
		g_running = 1;
		SetAppWindowTitle((char *)"");
	}

	if(l_runloop == 1)
	{
		if(!g_noreload)
			g_loadednum = 0;
	}
	
	if(l_runloop >= 1)
	{
		if((ret = LoadLoop()) >= 1 && ret <= 4)	// 1 failure 		2 no player
		{										// 3 no songlist	4 no memory
			g_running = 0;
			l_runloop = 0;		
		}
	}

	if(l_runloop == 0) // stop
	{
		if(l_quit)
			AppQuit();

		SetAppWindowTitle((char *)"Stopped");

#ifndef MANOCDA
		if(l_diskeventpending) // a cd was mounted or unmounted
		{
			l_diskeventpending = 0;
			CdaInitCDRom();
			if(g_sets.listwindow & LBOXON && g_sets.listwindow & LBOXINFO)
				ShowPlayerInfo();
		}
#endif
	
		l_droplaunch = 0;
		EnableButtonsMenus();
	}
	else // keep going
	{
#if defined(MAGUINONE) && (defined(__MAUNIX__) || defined(__MAWIN__))
		return (g_posendon ? SEEKPERIOD : TITLEPERIOD);
#else
		DoTimerRunLoop(g_posendon ? SEEKPERIOD : TITLEPERIOD);
		return 0;
#endif
	}
	
	return 0;
	
} // RunLoop


void OnButtonPlay() // Play
{
#ifndef MANOCDA
	int i;
	char cstr[256];
#endif
	
	if(g_running)
		return;

	l_songsindex[0] = -1;

#ifndef MANOCDA
	l_mnextfile = -1;
	if(!l_droplaunch)
	{
		// if(re)starting a cda make a new songlistcda
		if(SongListIsCda())
		{
			int nitems;
			char *pitems[CDAMAXNUMDRIVES];

			if(CdaInitCDRom()) // OnButtonPlay
			{
				ShowErr("Insert an Audio CD or open another song list");
				return;
			}
			
			nitems = 0;
			for(i = 1; i <= g_cdanumdrives; i++)
			{
				if(g_cdadrive[i].exists && g_cdadrive[i].isready)
					pitems[nitems++] = g_cdadrive[i].name;
			}
			
			if(nitems <= 0)
			{
				ShowErr("Insert an Audio CD or open another song list");
				return;
			}
			
			// write a new cda songlist
			if(maSongListMake(nitems, pitems, NULL)) // OnButtonPlay
			{
				sprintf(cstr, "OnButtonPlay maSongListMake failed on '%s'", g_songlistcda);
				maLog(cstr);
				return;
			}
			
			if(ChangeSongList(g_songlistcda))
				return;
		}
	}
#endif // !MANOCDA

	DoTimerRunLoop(100);

} // OnButtonPlay


void OnButtonPause()
{
	if(l_ispaused) // toggle
		SetPause(0);
	else
		SetPause(1);

} // OnButtonPause


void OnButtonStop()
{
	if(g_running)
		SetAppWindowTitle((char *)"Stopping...");
	else
		SetAppWindowTitle((char *)"Stopped");
		
	g_stopB = 1;
	SetPause(0);
	g_cancel = 1;
	g_running = 0;
	g_noreload = 0;
	g_state.stop |= STOP_FLAG;
	l_droplaunch = 0;

} // OnButtonStop


void OnButtonPrevious() // Previous
{	
	PushButtonDisable(WINDOWAPP, ID_BTN_PREVIOUS);
					
	SetPause(0);
			
	if(!g_running) // from Stopped state
	{
		if(g_sets.buffertype == BUFUSEDIRECT && g_sets.reptplayed && !g_sets.random)
		{
			PushButtonEnable(WINDOWAPP, ID_BTN_PREVIOUS);
			if((l_droplaunch = l_songcurrent + 1) < 2)	// 0-based row to play + 2
				l_droplaunch = g_songsnum + 1;			// recirculate to last
			DoTimerPlay(100);
			return;
		}

		if(g_songsnum)
		{
			PushButtonEnable(WINDOWAPP, ID_BTN_PREVIOUS);
			g_noreload = 1;
			EnableButtonsMenus();
			DoTimerPlay(100);
		}
		PushButtonEnable(WINDOWAPP, ID_BTN_PREVIOUS);
	}
	
	if(g_sets.buffertype == BUFUSEDIRECT && g_sets.reptplayed && !g_sets.random)
	{
			l_previous = 1;
			l_next = 0;
			l_loadednew = 0;
	}
	else
	{
		if((l_loadednew = g_loadedcurrent) <= 0)
			l_loadednew = 1;
	}
	
	l_loadedquit = 1;
					
	PushButtonEnable(WINDOWAPP, ID_BTN_PREVIOUS);		

} // OnButtonPrevious


void OnButtonNext() // Next
{	
	PushButtonDisable(WINDOWAPP, ID_BTN_NEXT);
	
	SetPause(0);
	
	if(!g_running) // restart from Stopped state
	{
		if(g_sets.buffertype == BUFUSEDIRECT && g_sets.reptplayed && !g_sets.random)
		{
			PushButtonEnable(WINDOWAPP, ID_BTN_NEXT);
			if((l_droplaunch = l_songcurrent + 3) > g_songsnum + 1)	// 0-based row to play + 2
				l_droplaunch = 2;									// recirculate to first
			DoTimerPlay(100);
			return;
		}

		if(g_songsnum) // some songs are loaded
		{
			PushButtonEnable(WINDOWAPP, ID_BTN_NEXT);
			if(l_loadednew)
				g_noreload = 1;
			EnableButtonsMenus();
			DoTimerPlay(100);
		}
		else
			PushButtonEnable(WINDOWAPP, ID_BTN_NEXT);
	}

	if(g_sets.buffertype == BUFUSEDIRECT && g_sets.reptplayed && !g_sets.random)
	{
			l_previous = 0;
			l_next = 1;
			l_loadednew = 0;
	}
	else
	{
		if((l_loadednew = g_loadedcurrent + 2) > g_loadednum)
			l_loadednew = 0; // reloads
	}
	
	l_loadedquit = 1;

	PushButtonEnable(WINDOWAPP, ID_BTN_NEXT);

} // OnButtonNext


void OnMenuPlayCD() // Play CD
{
	if(g_running)
		return;
		
#ifndef MANOCDA
	if(CdaInitCDRom()) // OnMenuPlayCD
	{
		ShowErr("Insert an Audio CD");
		return;
	}
	
	if(g_fpsonglist)
	{
		fclose(g_fpsonglist);
		g_fpsonglist = NULL;
	}
	
	strcpy(g_songlist, g_songlistcda);
	SetAppWindowTitle((char *)"");
	OnButtonPlay();
#endif

} // OnMenuPlayCD


void OnButtonQuit() // Exit
{
	PushButtonDisable(WINDOWAPP, ID_BTN_QUIT);
					
	if(g_running)
	{
		g_sets.spindown = 0;
		l_quit = 1;
		OnButtonStop();
	}			
	else
		AppQuit();
			
	PushButtonEnable(WINDOWAPP, ID_BTN_QUIT);

} // OnButtonQuit


int LoadLoop()
{
	int err = 0;
				
	if(l_runloop == 1)
		if(g_cancel)
			l_runloop = 0;

	if(l_runloop == 1)
	{
		l_runloop = 2;

		if(g_loadednum <= 0 || (!g_sets.looploaded && !g_noreload))
		{
			if(!g_fpsonglist) // get songlist and its settings
				if(OpenSongList())
					return 3;	
	
			// get mem or vdisk
			if(g_memhandle)
#if defined(__MAOSX__) || defined(__MAOS9__)
				DisposeHandle(g_memhandle);
#else
				MAFREE(g_memhandle);
#endif
			g_memhandle = NULL; 
			l_songmem = NULL;
			l_songmemend = NULL;
			l_songmempos = NULL;
			g_loadedmemsize = 0;
			
			if(g_sets.buffertype == BUFUSEMEM)
			{
				if(GetSongMem())
					return 4;
			}
			else
				if(g_sets.buffertype == BUFUSEVDISK && GetVDisk())
					if(SetVDisk() || GetVDisk()) // prompt for vdisk
						return 1;
		}
		
	} // if(l_runloop == 1)

	if(l_runloop >= 1)
	{
		if(l_runloop == 2)
		{
			if(g_cancel)
				l_runloop = 1;
		}			

		if(l_runloop == 2)
		{
			if(g_loadednum <= 0 || (!g_sets.looploaded && !g_noreload))
			{
				if(g_buffersize != g_sets.buffersize)
				{
					g_buffersize = g_sets.buffersize;
					if(g_sets.buffertype == BUFUSEMEM)
						if(GetSongMem())
							return 4;
				}
			
				if((err = FillSongBuffer(g_sets.buffertype)))
				{
					if(err == 1 || g_sets.buffertype == BUFUSEMEM || g_sets.buffertype == BUFUSEVDISK)
						return 1;
				}
			}
		
			if((g_sets.listwindow & (LBOXON | LBOXLOADED)) == (LBOXON | LBOXLOADED) && !g_noreload)
				ShowListWindow();
				
			g_noreload = 0;
		}

		// play the songs
		if(l_runloop)
		{
			if((err = PlayLoop()))
				l_runloop = 2;
		}

		if(l_runloop == 2)
		{
			if(!g_sets.looploaded && !SongListIsCda())
				SaveSettings();
				
			if(g_cancel)
				l_runloop = 1;
			
			if(!(g_sets.looploaded /* && !g_sets.singleplay */ && !l_droplaunch))
				l_runloop = 1;
		}		
		if(err)
			l_runloop = 1;
				
	} // if(l_runloop >= 1)
	
	if(l_runloop == 1)
		if(!SongListIsCda())
			SaveSettings();
			
	return err;
		
} // LoadLoop


void OnMenuMakeCatalog()
{		
	int i;
	int size;
	int bigsize = 0;
	FILE *fpsonglist;
	FILE *fpcatalogfile = NULL;
	int numsongs = 0;
	char *p;
	char name[256];
	char bigfile[MAX_PATH];
	char catalogfile[MAX_PATH];
	char songsource[MAX_PATH] = {0};
	char cstr[256];
	
	DisableButtonsMenus();
	g_cataloguing = 1;
	bigfile[0] = 0;
	
	if(FilenameFromPathname(g_songlist, name))
	{
		sprintf(cstr, "OnMenuMakeCatalog FilenameFromPathname failed on '%s'", g_songlist);
		maLog(cstr);
		goto catalogret;
	}		
	if((p = strstr(name, g_extstr[kMAMAF])))
		*p = 0;

	sprintf(catalogfile, "%s%c%s.txt", g_madir, FILESEP, name);

	if((fpcatalogfile = mafopen(catalogfile, "w")) == NULL)
	{
		sprintf(cstr, "OnMenuMakeCatalog mafopen failed on '%s'", catalogfile);
		maLog(cstr);
		goto catalogret;
	}
	
#if defined(__MAOSX__) || defined(__MAUNIX__)
	machmod(catalogfile, ACCESSPERMS); // 0777
#endif

	if((fpsonglist = mafopen(g_songlist, "rb")) == NULL)
	{
		fclose(fpcatalogfile);
		sprintf(cstr, "OnMenuMakeCatalog mafopen '%s'", g_songlist);
		maLog(cstr);
		goto catalogret;
	}

	fprintf(fpcatalogfile, "\nSonglist : %s\n", g_songlist);
	
	l_posslidersetvalue = 1;
	SetPosSlider(-1);
	g_cancel = 0;
	
	sprintf(cstr, "%s Catalog... %d", name, g_songsnum);
	WindowSetTitle(WINDOWAPP, cstr);

	i = 0;
	while(mafgets(songsource, MAX_PATH, fpsonglist) != NULL)
	{
		i++;
		
		ThreadYield();
		AppYield();
		if(g_cancel)
		{
			fclose(fpsonglist);
			fclose(fpcatalogfile);
			SetAppWindowMsg((char *)"Catalog cancelled");
			goto catalogret;
		}
			
		// skip lines with '#' or files that are not playable or don't exist
		if(songsource[0] == '#')
			continue;

#if defined(__MAOSX__)
		StrBSDtoStrOSX(songsource);
#endif

		if(!SongFileExists(songsource))
		{
			fprintf(fpcatalogfile, "\n----------------------------------------------------\n");
			sprintf(cstr, "file not playable or not found:\n'%s'\n", songsource);
			fprintf(fpcatalogfile, cstr);
			continue;
		}
			
		numsongs++;		
			
		fprintf(fpcatalogfile, "\n%05d -----------------------------------------------\n", numsongs);
				
		fprintf(fpcatalogfile, "%s +%d\n", songsource, l_songrec[numsongs - 1]);
		
		if((size = DumpSongInfo(songsource, fpcatalogfile, 1)) > bigsize)
		{
			bigsize = size;
			strcpy(bigfile, songsource);
		}		
		
#if(1) // this is a major slowup...
		if(g_songsnum && (numsongs % 100 == 0))
		{
			//SetPosSlider(LROUND((-(numsongs * (double)POSMAX) / g_songsnum)));
			sprintf(cstr, "%s Catalog... %d/%d", name, i, g_songsnum);
			WindowSetTitle(WINDOWAPP, cstr);
		}
#endif
		
	} // while

	if(bigsize > 0)
	{
		if(bigsize >= 1000000)
		{
			int millions = bigsize / 1000000;
			int thousands = (bigsize - millions * 1000000) / 1000;
			sprintf(cstr, "%d,%03d,%03d", millions, thousands, (bigsize - millions * 1000000) - thousands * 1000);
		}
		else if(bigsize >= 1000)
		{
			int thousands = bigsize / 1000;
			sprintf(cstr, "%d,%03d", thousands, bigsize - thousands * 1000);
		}
		else
			sprintf(cstr, "%d", bigsize);
		fprintf(fpcatalogfile, "\nLargest file %d MB (%s) '%s'", 1 + bigsize / MEGABYTE, cstr, bigfile);
	}

	fclose(fpsonglist);
	fclose(fpcatalogfile);
	
	if(numsongs == 0)
	{
		sprintf(cstr, "OnMenuMakeCatalog no playable songs in song list '%s'", g_songlist);
		maLog(cstr);
	}
	
	SetAppWindowMsg((char *)"Catalog done");
	
catalogret:	
	SetPosSlider(-1);	
	g_cataloguing = 0;
	EnableButtonsMenus();

} // OnMenuMakeCatalog


int GetNextSong(int *pm, SONGRECORD *lsongrec)
{
	int j, k, m, n, p;

	if(!lsongrec)
		return -1;

	// get song m, 0 <= m < g_songsnum
	if(g_sets.random && l_droplaunch < 2)
	{				
		if(l_mnextfile >= 0) // previously rejected 'large' file
		{
			m = l_mnextfile;
			l_mnextfile = -1;
		}
		else // randomly pick a song from the songlist, index m | 0 <= m < g_songsnum
		{
			if(g_sets.reptplayed)
				m = LROUND((rand() * (double)g_songsnum) / RAND_MAX); 
			else
			{
				if(g_songsnum == l_songsnumplayed) // all have played, recirculate
				{
					if(!g_sets.looplist)
						return -2;
					ZeroSongRec(&lsongrec); // clear played settings
					ZeroSongRec(&l_songrec);
					l_songsnumplayed = 0;
					for(j = 0; j < SONGSLOADMAX; j++) // except those just selected
						if(l_songsindex[j] >= 0)
							lsongrec[l_songsindex[j]] = 1;
				}
			
				p = 0;
				for(j = 0; j < SONGSLOADMAX; j++) // count songs already picked for playing
					if(l_songsindex[j] >= 0)
						p++;
				if((n = g_songsnum - l_songsnumplayed - p) <= 0)
					return -2; // no unplayed or unpicked songs, else n unplayed and unpicked songs to pick from
				k = LROUND((rand() * (double)n) / RAND_MAX); // kth unplayed and unpicked song, 0 <= k < n

				p = 0;
				for(m = 0; m < g_songsnum; m++) // get songlist index m of the kth unplayed and unpicked song
				{
					if(!lsongrec[m]) // song m is unplayed
					{
						for(j = 0; j < SONGSLOADMAX; j++)
							if(l_songsindex[j] == m)
								break;
						if(j >= SONGSLOADMAX) // song m is unpicked
							if(p++ == k)
								break; // m is the kth unplayed and unpicked song
					}
				}
				
			} // !g_sets.reptplayed
			
		} // l_mnextfile < 0
		
	} // if g_sets.random
	else													// sequential m if not random
	{	
		if(l_droplaunch >= 2)
			m = l_droplaunch - 2; 							// doubleclicked on songlist item or entered song index to play
		else
		{
			if(!pm)
				return -1;

			m = *pm + 1;
			if(!g_sets.reptplayed)							// next unplayed m
				while(m < g_songsnum && lsongrec[m])
					m++;
		
			if(m >= g_songsnum)								// all have played, recirculate
			{
				if(!g_sets.looplist)
					return -2;
				ZeroSongRec(&lsongrec);
				ZeroSongRec(&l_songrec);
				l_songsnumplayed = 0;
				m = 0;
			}
		}
		
	} // !g_sets.random

	*pm = m;

	return 0;

} // GetNextSong


int FillSongBuffer(int dest)
{
	int j, k, m;
	int ret;
	int err;
	int once = 0;
	long memleft;
	long bytescopied = 0;
 	char songsource[MAX_PATH];
	char cstr[256];
	SONGRECORD *lsongrec = NULL;

	g_buffersize = g_sets.buffersize;
	
	if(dest == BUFUSEMEM)
	{
		// clear mem pointers
		memset(g_songmemptr, 0, sizeof(g_songmemptr));
		g_songmemptr[0] = l_songmem;
		memleft = g_loadedmemsize;
	}
	else if(dest == BUFUSEVDISK)
	{
		memleft = g_buffersize * MEGABYTE;	
		// clear l_loadeddir			
		if(maDirExists(l_loadeddir))
		{
			if(maClearDir(l_loadeddir))
			{
				sprintf(cstr, "FillSongBuffer maClearDir failed on '%s'", l_loadeddir);
				maLog(cstr);
				return 1;
			}
		}
		else if(maCreateDir(l_loadeddir))
		{
			sprintf(cstr, "maCreateDir failed on '%s'", l_loadeddir);
			maLog(cstr);
			return 1;
		}
	}
	else if(dest == BUFUSEDIRECT)
	{
		// do one song file
		memleft = 0;
		g_loadednum = 0;				
		m = l_songsindex[0];
	}
	else
	{
		maLog("FillSongBuffer error bad dest");
		return 1;
	}

	// use a temp local copy of l_songrec :	
	if(!l_songrec)
	{
		maLog("FillSongBuffer error NULL l_songrec");
		return 1;
	}
	
	if(g_songsnum <= 0)
	{
		maLog("FillSongBuffer no playable songs in song list");
		return 1;
	}
	
	if(AllocSongRec(&lsongrec))
		return 1;
		
	memcpy(lsongrec, l_songrec, g_songsnum * sizeof(SONGRECORD));
			
	// fill songbuffer with song files :	
	g_loadednum = j = 0; // count of songs loaded
				
	err = ret = 0;
		
	if(g_sets.buffertype == BUFUSEDIRECT && g_sets.reptplayed && !g_sets.random) 
	{
		if(l_previous)
		{
			l_previous = 0;
			if((m = l_songsindex[0] - 2) < -1) // previous
				m = g_songsnum - 2;
		}
		else if(l_next)
		{
			l_next = 0;
			if((m = l_songsindex[0]) >= g_songsnum) // next
				m = -1;
		}
	}
	else
		m = -1; // -1 gets first song
		
	for(k = 0; k < SONGSLOADMAX; k++)
		l_songsindex[k] = -1;
	
	while(ret != dskFulErr && j < g_songsnum && j < SONGSLOADMAX)
	{
		ThreadYield();
		AppYield();
		if(g_cancel)
			goto errorrets;

#if(0) // this is a major slowup...
		if(!g_cataloguing)
			SetPosSlider(LROUND((-((g_loadedmemsize - memleft) * (double)POSMAX) / g_loadedmemsize)));
#endif
		// get song m
		if((err = GetNextSong(&m, lsongrec)))
		{
			if(err == -2)
				break;
			sprintf(cstr, "FillSongBuffer GetNextSong failed for '%s'", g_songlist);
			maLog(cstr);
			goto errorrets;
		}
					
		// gets here with song m
					
		// read songname m from songlist :	
		if(!g_fpsonglist)
		{
			sprintf(cstr, "FillSongBuffer null file pointer for '%s'", g_songlist);
			maLog(cstr);
			goto errorrets;
		}					

		rewind(g_fpsonglist);
			
		k = 0;
			
		while(k <= m)
		{
			if(mafgets(songsource, MAX_PATH, g_fpsonglist) == NULL) // should never eof
			{
				sprintf(cstr, "FillSongBuffer song file\n\n'%s'\n\ncould not be found for index %d in\n\n'%s'\n\n"
								"Re-open a song list or restart MoreAmp", songsource, m + 1, g_songlist);
				ShowErr(cstr);
				goto errorrets;
			}
						
			// skip lines with '#' or files that are not playable or don't exist
			if(songsource[0] == '#')
				continue;

			if(!SongFileExists(songsource))
			{
				sprintf(cstr, "FillSongBuffer file not playable or not found: '%s'", songsource);
				maLog(cstr);
				continue;
			}
						
			k++;
		}

		// save song name		
		strcpy(g_hdsongpath[j], songsource);
		
		bytescopied = 0;

		if(FileNameHasExtensionPlayable(songsource))
		{
			if(FilenameFromPathname(songsource, g_loadedname[j]))
			{
				sprintf(cstr, "FillSongBuffer failed to get filename from '%s'", songsource);
				maLog(cstr);
				goto errorrets;
			}

 #if defined(__MAOSX__)
			StrBSDtoStrOSX(g_loadedname[j]);			
 #endif
			if(dest == BUFUSEMEM)
			{
				if(!once++)
					SetAppWindowTitle((char *)"loading memory...");
				ret = FileToMem(songsource, g_songmemptr[j], memleft, &bytescopied);
			}
			else if(dest == BUFUSEVDISK)
			{
				if(!once++)
					SetAppWindowTitle((char *)"loading Virtual Disk...");
				ret = FileToVDisk(songsource, g_loadedname[j], memleft, &bytescopied);
			}
			else // direct
				ret = 0;

			if(ret)
			{
				if(ret == dskFulErr)
				{
					if(bytescopied > 0 && bytescopied <= g_loadedmemsize) // bytescopied holds filesize
						l_mnextfile = m; // 'large' file rejected now will play next time
					else
						l_mnextfile = -1;
					break;
				}
				
				if(dest == BUFUSEMEM)
					sprintf(cstr, "FillSongBuffer FileToMem failed '%s'", songsource);
				else if(dest == BUFUSEVDISK)
					sprintf(cstr, "FillSongBuffer FileToVDisk failed '%s'", songsource);
				else
					sprintf(cstr, "FillSongBuffer FileDirect failed '%s'", songsource);
				maLog(cstr);
				goto errorrets;
			}

			memleft -= bytescopied;
		}
		else // is cda
			strcpy(g_loadedname[j], songsource);
		
		// save song index
		l_songsindex[j] = m;						
		
		// locally mark song as played
		lsongrec[l_songsindex[j]] += 1;
		
		g_songmemptr[j + 1] = g_songmemptr[j] + bytescopied;
		j++;
		
		if(l_droplaunch >= 2) // started by a doubleclick on songlist item
		{
			l_droplaunch = 0;
			break;
		}
		
		if(dest == BUFUSEDIRECT)
			break;
			
	} // while ret != dskFulErr

	if((g_loadednum = j) <= 0)
	{
		if(err == -2)
			ShowErr(l_allhaveplayedstr);
		else if(ret == dskFulErr)
		{
			if(dest == BUFUSEMEM)
			{
				sprintf(cstr,
				"There is not enough memory to load\n\n'%s'\n\nEither :\n\n \
- Increase the Mem Buffer to at least %d Mb, or\n\n \
- Set the Buffer mode to Direct.", g_loadedname[j], (int)(1 + bytescopied / MEGABYTE));
				ShowErr(cstr);
			}
			else if(dest == BUFUSEVDISK)
			{
				sprintf(cstr,
				"There is not enough virtual disk space to load\n\n'%s'\n\nEither :\n\n \
- Increase the virtual disk size to at least %d Mb, or\n\n \
- Set the Buffer mode to Direct.", g_loadedname[j], (int)(1 + bytescopied / MEGABYTE));
				ShowErr(cstr);
			}
		}
		goto errorrets;
	}
	
	goto okret;
	
errorrets:
	if(lsongrec)
		MAFREE(lsongrec);
		
	g_cancel = 1;
	g_running = 0;
	g_noreload = 0;
	
	return 1;
	
okret:

	if(lsongrec)
		MAFREE(lsongrec);
		
	for(k = 0; k < g_loadednum; k++)
		if(l_songsindex[k] < 0 || l_songsindex[k] >= g_songsnum)
		{
			sprintf(cstr, "FillSongBuffer bad l_songsindex[%d]=%d", k, l_songsindex[k]);
			maLog(cstr);
			return 1;
		}	
	
	return 0;
			
} // FillSongBuffer


int AllocSongRec(SONGRECORD **ppsongrec)
{
	char cstr[256];
	
	if(!ppsongrec)
	{
		maLog("AllocSongRec null ppsongrec");
		return 1;
	}
	
	if(*ppsongrec)
	{
		MAFREE(*ppsongrec);
		*ppsongrec = NULL;
	}
							
	if((g_songsnum <= 0) || (*ppsongrec = (SONGRECORD *)MAMALLOC(g_songsnum * sizeof(SONGRECORD))) == NULL)
	{
		if(SongListIsCda())
			return 0;
		
		sprintf(cstr, "AllocSongRec failed");
		maLog(cstr);
		return 1;
	}
	
	if(ZeroSongRec(ppsongrec))
		return 2;
	
	return 0;
	
} // AllocSongRec


int ZeroSongRec(SONGRECORD **ppsongrec)
	 
{
	char cstr[256];
		
	if(*ppsongrec == NULL)
	{
		sprintf(cstr, "cannot reset settings, null settings pointer");
		maLog(cstr);
		return 1;
	}
		
	if(g_songsnum <= 0)
	{
		sprintf(cstr, "ZeroSongRec cannot reset settings");
		maLog(cstr);
		return 1;
	}
		
	memset(*ppsongrec, 0, g_songsnum * sizeof(SONGRECORD));
	
	return 0;
	
} // ZeroSongRec


int FileToVDisk(char *from, char *tomem, int maxmem, long *bytesread)
{
	int l;
	int ret;
	long filesize;
	FILE *filep;
	char c;
	char playpath[MAX_PATH];
	char cstr[256];
	
	if(!from || !from[0])
	{
		maLog("FileToVDisk null from filename");
		return 1;
	}
	
	if(!tomem || !tomem[0])
	{
		maLog("FileToVDisk null tomem");
		return 1;
	}
	
	// get file size
	if((filep = mafopen(from, "rb")) == NULL)
	{
		sprintf(cstr, "FileToVDisk failed to open '%s'", from);
		maLog(cstr);
		return 1;
	}

	fseek(filep, 0, SEEK_END);		
	filesize = ftell(filep);
	fclose(filep);
	*bytesread += filesize;
	if(*bytesread > maxmem)
		return dskFulErr;
		
	if(!FileNameHasExtensionPlayable(from))
		return 1;

	// rename song if the name already exists in vdisk
	sprintf(playpath, "%s%c%s", l_loadeddir, FILESEP, tomem);

	c = '1';
		
	while(maFileExists(playpath))
	{
		if((l = strlen(tomem)) < NAMELENMAX)	// add a char '1', '2', ...
		{
			if(c == '1')
			{
				tomem[l] = c++;
				tomem[l + 1] = 0;
			}
			else
				tomem[l - 1] = c++;
		}
		else // max length, so just bump the last char to next ascii value
		{
			(tomem[l - 1])++;
			
			sprintf(playpath, "%s%c%s", l_loadeddir, FILESEP, tomem);
		}
	}
		
	// copy song to l_loadeddir
	if((ret = maFileCopy(from, playpath, 0)))
	{
		if(ret == dskFulErr)
			return dskFulErr;
					
		sprintf(cstr, "FileToVDisk maFileCopy error %d '%s'", ret, from);
		maLog(cstr);
		return 1;
	}

	return(0);
	
} // FileToVDisk


int FileToMem(char *from, char *tomem, int maxmem, long *bytesread)
{
	int ret;
	long fsize;
	FILE *fp;
	char cstr[256];
	
	*bytesread = 0;
	
	if(!from || !from[0])
	{
		maLog("FileToMem null from filename");
		return 1;
	}
	
	if(!tomem)
	{
		maLog("FileToMem null tomem");
		return 1;
	}
	
	if(!maxmem)
	{
		maLog("FileToMem null maxmem");
		return 1;
	}
	
	// open file	
	if((fp = mafopen(from, "rb")) == NULL)
	{
		sprintf(cstr, "FileToMem failed on mafopen '%s'", from);
		maLog(cstr);
		return 1;
	}
			
	// get the file size		
	fseek(fp, 0, SEEK_END);
	
	if((fsize = ftell(fp)) > maxmem)
	{
		fclose(fp);
		*bytesread = fsize;
		return dskFulErr;
	}

	rewind(fp);
					
	// read the file into the passed pointer	
	if((ret = fread(tomem, 1, fsize, fp)) != fsize)
	{
		sprintf(cstr, "FileToMem fread '%s' failed to read %d bytes, ret=%d",
												from, (int)fsize, (int)ret);		
		maLog(cstr);
		fclose(fp);
		return 1;
	}
			
	fclose(fp);
	
	*bytesread = fsize;
				
	return(0);

} // FileToMem


void PlayerSeek(int pos)
{
	if(l_playerseeking)
		return;
	
	l_playerseeking = 1;

	switch (g_state.infiletype)
	{
#ifndef NODECODEMP3
		case kMAMP1 :
		case kMAMP2 :
		case kMAMP3 :
			mp3seek(pos);
		break;
#endif
		
#ifndef NODECODEOGG
		case kMAOGG :
			oggseek(pos, g_state.duration);
		break;
#endif
		
#ifndef NODECODEWMA
		case kMAWMA :
			l_playerseeking = 0; // don't seek on wma wmaseek(pos, g_state.duration);
		break;
#endif
		
#ifndef NODECODEAIF
		case kMAAIF :
		case kMAAIFF :
		/* these could be added to maPlayAif
		case kMAAU :
		case kMARAW :
		case kMAPAF :
		case kMASVX :
		case kMANIST :
		case kMAVOC :
		case kMAIRCAM :
		case kMAW64 :
		case kMAMAT4 :
		case kMAMAT5 :
		case kMAPVF :
		case kMAXI :
		case kMAHTK :
		*/
			aifseek(pos);
		break;
#endif

#ifndef NODECODEWAV
		case kMAWAV :
			aifseek(pos); // wavseek
		break;
#endif
		
#ifndef NODECODEAAC
		case kMAAAC :
		case kMAM2A :
			aacseek(pos);
		break;
#endif

#ifndef NODECODEMP4
		case kMAMP4 :
		case kMAM4A :
			mp4seek(pos);
		break;
#endif

#ifndef NODECODEFLAC
		case kMAFLAC:
		case kMAFLA:
			flacseek(pos);
		break;
#endif

#ifndef MANOCDA
		case kMACDA :
			cdaseek(pos);
		break;
#endif

		default:
			l_playerseeking = 0;
		break;
	}

} // PlayerSeek


void SetAppTitle2()
{
	int s;
	char tstr[32];
	char cstr[256];

	s = (g_state.duration - g_state.position) / MSPERSEC; // time left in secs	
	SecondsToStrHMS(tstr, s);

	if(g_sets.buffertype == BUFUSEDIRECT)		// do not show number of songs loaded
	{
		if(g_sets.reptplayed)					// do not show number of songs played
		{
			// if(g_sets.random)					// do not show index nor numplayed
			//	sprintf(cstr, "%s %d %s", l_songlist, g_songsnum, tstr);
			// else								// show index
				sprintf(cstr, "%s #%d/%d %s", l_songlist, l_songsindex[0] + 1, g_songsnum, tstr);
		}
		else									// show numplayed instead of index
			sprintf(cstr, "%s %d/%d %s", l_songlist, l_songsnumplayed, g_songsnum, tstr);
	}
	else										// show number of songs loaded
	{
		if(g_sets.reptplayed)					// do not show number of songs played
		{
			// if(g_sets.random)					// do not show index nor numplayed
			//	sprintf(cstr, "%s %d (%d/%d) %s", l_songlist, g_songsnum,
			//									g_loadedcurrent + 1, g_loadednum, tstr);
			// else								// show index
				sprintf(cstr, "%s #%d/%d (%d/%d) %s", l_songlist, l_songsindex[g_loadedcurrent] + 1,
											g_songsnum, g_loadedcurrent + 1, g_loadednum, tstr);
		}
		else									// show numplayed instead of index
			sprintf(cstr, "%s %d/%d (%d/%d) %s", l_songlist, l_songsnumplayed, g_songsnum,
												g_loadedcurrent + 1, g_loadednum, tstr);
	}
	
	WindowSetTitle(WINDOWAPP, cstr);

} // SetAppTitle2


void ResetAppTitle()
{
	if(g_sets.titleinfo == 1)
		WindowSetTitle(WINDOWAPP, g_loadedname[g_loadedcurrent]);
	else if(g_sets.titleinfo == 2)
		SetAppTitle2();
	else
		SetAppWindowTitle((char *)"");
		
	if(g_sets.listwindow & LBOXON)
	{
		if(g_sets.listwindow & (LBOXLOADED | LBOXLIST))
		{
			if(!l_noshowrow) // scrolling up or down the list sets this
				ListWindowShowRowPlaying();
		}
		else
			ShowListWindow(); // redraws player or song info lists
	}
	
#if !defined(MAGUINONE)
	l_temptitle = 0;
#endif

	maLog(&l_logcmdexist);
	
} // ResetAppTitle


int l_titletime = 0;
int l_retval;
double l_attenuation;
#ifdef __MAOS9__
 unsigned long l_spindowntime;
#endif
#if !defined(NODECODEAUDIOIN)
	time_t l_stTime;
#endif	
int PlayLoop()
{
#ifndef NODECODEMP3
	float mpegattenuationdb;
#endif
	char *p;
	char playpath[MAX_PATH];
	char cstr[256];
#if !defined(NODECODEAUDIOIN)
	time_t stTime;
	struct tm *tmTime;
#endif	
	l_retval = 0;

	if(l_runloop == 2)
	{
		l_runloop = 3;
		l_attenuation = 123.0;
#ifdef __MAOS9__
		l_spindowntime = 0;
#endif

		FilenameFromPathname(g_songlist, l_songlist);
		if((p = strstr(l_songlist, g_extstr[kMAMAF])))
			*p = 0;

		l_separation = g_sets.separationslider;
		l_separationch = 0.5 + ((0.5 * l_separation) / SEPARATIONMAX);
		l_separationmix = 1. - l_separationch;
		l_pan = g_sets.panslider;
		l_panright =                  l_pan  / (double)SEPARATIONMAX;
		l_panleft  = (SEPARATIONMAX - l_pan) / (double)SEPARATIONMAX;
		EqDoParamChange();

		g_loadedcurrent = 0;
	}

	if(l_runloop == 3)
	{
		l_runloop = 4;

		g_state.stop = 0;

		if(l_loadednew) // doubleclick in loaded list or next or previous
		{
			g_loadedcurrent = l_loadednew - 1;
			l_loadednew = 0;
			l_previous = 0;
			l_next = 0;
		}
		
		sprintf(playpath, "%s", g_hdsongpath[g_loadedcurrent]);		
		
		if(l_songrec[l_songsindex[g_loadedcurrent]] <= 0)
		{
			l_songsnumplayed++;
			l_songrec[l_songsindex[g_loadedcurrent]] += 1; // mark song as played					
		}
		
		SetPause(0);
		l_possliderchanged = l_posslidervalue = 0;
		SetPosSlider(-1);

		g_effectschange = g_sets.effects & EFFECTSON &&
					(g_sets.effects & (EFFECTSTEMPO | EFFECTSPITCH | EFFECTSSPEED)) ? 1 : 0;

		// reset limiter
		g_state.mpegattenuation = 1.0;

		// output file
		if(g_sets.output != kOUTSOUND)
		{
			// output file name
			if(MakeOutputFileName())
				return 1; // stops
		}
		
		if(LaunchPlayThread(playpath))
		{
			sprintf(cstr, "PlayLoop LaunchPlayThread failed '%s'", playpath);
			l_runloop = 2;
			goto errorret1;
		}
		
		// now playing g_loadedcurrent
		l_songcurrent = l_songsindex[g_loadedcurrent];
		
		// show new song info		
		l_noshowrow = 0;
		ResetAppTitle();
					
		if(!g_posbegon)
		{
			g_posbeg = 0;
			StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEBEG, ZEROMSSTR);
		}
		
		l_playerseeking = 0;
		l_loadedquit = 0; // gets set by Next, Previous, listbox doubleclick

#if !defined(NODECODEAUDIOIN)
		if(g_state.infiletype == kMAMAA)
		{
			l_stTime = time(NULL);
			StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEBEG, NOTIMESTR);
		}
#endif
		
	} // if(l_runloop == 3)

	// playing the song		 
	if(l_runloop == 4)
	{
		if(g_cancel)
		{
			l_runloop = 3;
			goto errorret;
		}

		// seek	to posbeg or end of song		
		if(!l_ispaused)
		{
			if(g_posbegon && g_state.position <= g_posbeg)
				PlayerSeek(g_posbeg);
			else if(g_posendon && g_state.position > g_posend)
			{
				if(g_reptsong)
					PlayerSeek(g_posbeg);
				else
					PlayerSeek(g_state.duration);
			}
			else
				l_playerseeking = 0;
			
			// every titleperiod
			if(l_titletime++ >= (g_posendon ? TITLEPERIOD / SEEKPERIOD : 1))
			{
				l_titletime = 0;

#if !defined(NODECODEAUDIOIN)
				if(g_state.infiletype == kMAMAA)
				{
					stTime = time(NULL) - l_stTime;
					tmTime = gmtime((time_t *)&stTime);
					sprintf(cstr, "%2.2d:%2.2d:%2.2d", tmTime->tm_hour, tmTime->tm_min, tmTime->tm_sec);
					cstr[24] = 0;	
					StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEPOS, cstr);
				}
				else
#endif
				{
					if(!g_posendon && l_posend != g_posend)
					{
						l_posend = g_posend;
						MillisecondsToStrHMS(cstr, g_posend);
						StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEEND, cstr);
					}
				
					if(l_possliderchanged) // user moved the pos slider, seek to new pos 
					{
						l_possliderchanged = 0;
						if(!l_ispaused)
							PlayerSeek(LROUND((g_state.duration * (double)l_posslidervalue) / POSMAX));
					}
					else if(!l_ispaused) // update position slider
						SetPosSlider(LROUND(((g_state.position * (double)POSMAX) / g_state.duration)));

					l_playerseeking = 0;
				
					// update display of song duration				
					if(g_sets.titleinfo == 2)
						SetAppTitle2();
#if !defined(MAGUINONE)
					else if(l_temptitle) // clear temp title info (key pressed or volume)
					{
						if(--l_temptitle <= 0)
							ResetAppTitle();
					}
#endif
			
#ifndef NODECODEMP3
					// update clipping				
					if(g_sets.limiter == (PREAMP | CLIPPING) &&
						(g_state.infiletype == kMAMP3 || g_state.infiletype == kMAMP2 || g_state.infiletype == kMAMP1))
					{
						if(l_attenuation != g_state.mpegattenuation)
						{
							mpegattenuationdb = (g_state.mpegattenuation == 1.0) ? 0.0 : 20.0 * log10(g_state.mpegattenuation);
							sprintf(cstr, "%.1f", mpegattenuationdb);
							CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_LIMITER, cstr);
							l_attenuation = g_state.mpegattenuation;
							// sudden attenuation more than about 20dB is likely due to a spike from bad encoding, cancel it
							if((mpegattenuationdb + SliderDBFromPos(g_sets.preampslider)) < -20.0)
								g_state.mpegattenuation = MP3ATTENUATIONFLAG; // flag for maPlayMp3() to reset attenuation
						}
					}
#endif
				}
#ifdef __MAOS9__
				// spindown the hard disk
				if(g_sets.spindown)
				{
					if(l_spindowntime++ > SPINDOWNPERIOD)
					{
						l_spindowntime = 0;
						SpinDown();										
					}
				}
#endif
			} // l_titletime
		}
		else  // paused
			if(l_titletime++ >= 1)
				l_titletime = 0;
		
		if(l_loadedquit || g_state.stop)
			l_runloop = 3;

	} // if(l_runloop  == 4)

	if(l_runloop == 3)
	{
		// close		
		g_state.stop |= STOP_FLAG;
		// wait for decoder or sound to stop
		while(!(g_state.stop & (STOP_PLAYDECODE | STOP_SNDCHANNEL))) 
		{
			ThreadYield();
			AppYield();
		}
		if(l_loadedquit)
			maOutputClose(0);
		else
			maOutputClose(2000); // wait for buffered sound to finish
		g_state.stop |= STOP_SNDCHANNEL;

		while(!(g_state.stop & (STOP_FLAG | STOP_PLAYDECODE)))
		{
			ThreadYield();
			AppYield();
			if(g_cancel)
				l_runloop = 2;
		}
		
		g_state.stop |= STOP_INPUT;

		if(/* g_sets.singleplay || */ g_cancel)
			l_runloop = 2;
		else if(g_sets.singleplay) 
		{
			// OnButtonStop();
			SetPause(0);
			SetAppWindowTitle((char *)"Stopping...");
			g_stopB = 1;
			g_cancel = 1;
			g_running = 0;
			g_noreload = 0;
			g_state.stop |= STOP_FLAG;
			l_droplaunch = 0;

			l_runloop = 2;
		}
		else
		{
			g_loadedcurrent++;
			if(g_loadedcurrent >= g_loadednum && !l_loadednew)
				l_runloop = 2;
		}
	} // if(l_runloop == 3)
					
	if(l_runloop == 2)
	{
		goto allret;	
	
	// error returns			
errorret1:
		maLog(cstr);
		return 0;

errorret:
		l_retval = 1;
	
		// all returns	
allret:

		if(g_state.stop & STOP_INPUT)
			return l_retval;
		
		// close
		g_state.stop |= STOP_FLAG;		
		// wait for decoder or sound to stop
		while(!(g_state.stop & (STOP_PLAYDECODE | STOP_SNDCHANNEL))) 
		{
			ThreadYield();
			AppYield();
			maSleep(100);		
		}
		maOutputClose(0);
		g_state.stop |= STOP_SNDCHANNEL;

		while(g_loadednum && !(g_state.stop & (STOP_FLAG | STOP_PLAYDECODE)))
		{
			ThreadYield();
			AppYield();
			if(g_cancel)
				break;
			maSleep(100);		
		}
		
	} // if(l_runloop == 2)
	
	return l_retval;
		
} // PlayLoop


int GetInFiletype(char *playpath)
{
	int i;
	
	for(i = kMAFIRSTPLAYBLE; i < kMALAST; i++)
		if(FileNameHasExtension(playpath, g_extstr[i]))
			return i;
			
#ifndef MANOCDA
	if(IsAudioCd(playpath))
		return kMACDA;
#endif
			
	return kMALAST; // not playable

} // GetInFiletype


#if !defined(NODECODEAUDIOIN)
 signed long ThreadPlayAudioIn(void *parameter);
 signed long ThreadPlayAudioIn(void *parameter)
 {
 #ifdef MAUNUSED
  (void)parameter;
 #endif

	g_waitaudiodone = 1;

	g_state.insamplerate = 44100;
	// format of this decoder output
 #ifdef WORDS_BIGENDIAN
	g_state.inwordsbigendian = 1;
 #else
	g_state.inwordsbigendian = 0;
 #endif
	g_state.ininterleaved = 1;
	g_state.inchannels = 2;

	if(maOutputOpen() < 0)
	{
		maLog("ThreadPlayAudioIn maOutputOpen failed");
		return 0;
	}

	while(!g_state.stop && !g_stopB) // should instead block and wait for user input here
		maSleep(1000);

	g_state.stop |= STOP_PLAYDECODE;
	
	return 0;

 } // ThreadPlayAudioIn
#endif


int LaunchPlayThread(char *playpath)
{
	int err;
	int infiletype;
	char cstr[256];

	if((infiletype = GetInFiletype(playpath)) >= kMALAST)
		return -1; // not playable

	strcpy(g_state.inpath, playpath);
	g_state.infiletype = infiletype;
	g_state.nomem = (g_sets.buffertype == BUFUSEMEM) ? 0 : 1;

	switch(infiletype)
	{
#ifndef NODECODEMP3
		case kMAMP3 :
		case kMAMP2 :
		case kMAMP1 :
			g_state.threadplay = (void *)ThreadPlayMp3;
			g_state.threadplayprilo = MATHREADPRILO;
			g_state.threadplayprihi = MATHREADPRIHI2;
		break;
#endif
		
#ifndef NODECODEOGG
		case kMAOGG :
			g_state.threadplay = (void *)ThreadPlayOgg;
			g_state.threadplayprilo = MATHREADPRILO;
			g_state.threadplayprihi = MATHREADPRIHI1;
		break;
#endif

#ifndef NODECODEWMA
		case kMAWMA :
			g_state.threadplay = (void *)ThreadPlayWma;
			g_state.threadplayprilo = MATHREADPRILO;
			g_state.threadplayprihi = MATHREADPRIHI1;
		break;
#endif

#ifndef NODECODEAAC
		case kMAAAC :
		case kMAM2A :
			g_state.threadplay = (void *)ThreadPlayAac;
			g_state.threadplayprilo = MATHREADPRILO;
			g_state.threadplayprihi = MATHREADPRIHI2;
		break;
#endif

#ifndef NODECODEMP4
		case kMAM4A :
		case kMAMP4 :
			g_state.threadplay = (void *)ThreadPlayMp4;
			g_state.threadplayprilo = MATHREADPRILO;
			g_state.threadplayprihi = MATHREADPRIHI2;
		break;
#endif

#ifndef NODECODEFLAC
		case kMAFLAC :
		case kMAFLA :
			g_state.threadplay = (void *)ThreadPlayFlac;
			g_state.threadplayprilo = MATHREADPRILO;
			g_state.threadplayprihi = MATHREADPRIHI1;
		break;
#endif

#if !defined(NODECODEAUDIOIN)
		case kMAMAA :
			g_state.threadplay = (void *)ThreadPlayAudioIn;
			g_state.threadplayprilo = MATHREADPRILO;
			g_state.threadplayprihi = MATHREADPRIHI1;
		break;
#endif

#ifndef NODECODEAIF
		case kMAAIF :
		case kMAAIFF :
		
		/* these could be added to maPlayAif
		case kMAAU :
		case kMARAW :
		case kMAPAF :
		case kMASVX :
		case kMANIST :
		case kMAVOC :
		case kMAIRCAM :
		case kMAW64 :
		case kMAMAT4 :
		case kMAMAT5 :
		case kMAPVF :
		case kMAXI :
		case kMAHTK :
		*/
			g_state.threadplay = (void *)ThreadPlayAif;
			g_state.threadplayprilo = MATHREADPRILO;
			g_state.threadplayprihi = MATHREADPRIHI1;
		break;
#endif

#ifndef NODECODEWAV
		case kMAWAV :
			g_state.threadplay = (void *)ThreadPlayAif; // ThreadPlayWav
			g_state.threadplayprilo = MATHREADPRILO;
			g_state.threadplayprihi = MATHREADPRIHI1;
		break;
#endif

#ifndef MANOCDA
		case kMACDA :
			g_state.nomem = 1; // always for cda
			g_state.threadplay = (void *)ThreadPlayCda;
			g_state.threadplayprilo = MATHREADPRILO;
			g_state.threadplayprihi = MATHREADPRIHI2;
		break;
#endif

		default :
		return 1;
		
	} // case

	g_stopB = 0;
	
	if((err = maThreadCreate(g_state.threadplay, &g_state.pthreadplayid, MATHREADSTACKSIZE,
			g_sets.highpriority ? g_state.threadplayprihi : g_state.threadplayprilo)))
	{
		g_state.pthreadplayid = NULL;
		sprintf(cstr, "LaunchPlayThread maThreadCreate error %d", err);
		maLog(cstr);
	}
	
	return err;

} // LaunchPlayThread


int MadDoParamChange()
{	
#ifndef NODECODEMP3
	// mad equalizer as preamp
	mp3eqset(g_sets.limiter, SliderDBFromPos(g_sets.preampslider));
#endif
		
	return 0;
					
} // MadDoParamChange


int EqDoParamChange()
{	
	int i;
	char cstr[256];
		
	if(g_sets.eqpreset < 0 || g_sets.eqpreset >= EQNUMPRESETS)
	{
		sprintf(cstr, "EqDoParamChange eqpreset=%d", g_sets.eqpreset + 1);
		maLog(cstr);
		return 1;
	}

	if(g_sets.eqflags & MAEQACTIVE)
	{
		for(i = 0; i < EQBANDS31; i++)
		{
			g_EqualizerSettings.Levels[i] = g_sets.eqpresets[g_sets.eqpreset].eqslider[i];
			g_EqualizerSettings.Levels2[i] = g_sets.eq2presets[g_sets.eqpreset].eqslider[i];
		}
		maEqualizerSetLevels(g_sets.eqflags & MAEQ31BANDS, g_sets.eqpresets[g_sets.eqpreset].eqoffset);
		g_EqualizerSettings.Enabled = 1;
	}
	else
		g_EqualizerSettings.Enabled = 0;

	return 0;
					
} // EqDoParamChange


#if defined(__MAOSX__) || defined(__MAOS9__)
int GetSongMem() // mac
{
	OSErr err;
	int avail;
	char cstr[256];

	// free current song memory
	if(g_memhandle)
	{
		DisposeHandle(g_memhandle);
		g_memhandle = NULL; 
		l_songmem = NULL;
		l_songmemend = NULL;
		l_songmempos = NULL;
		g_loadedmemsize = 0;
	}
	
	g_buffersize = g_sets.buffersize;
	
#ifdef __MAOSX__
	avail = g_buffersize * MEGABYTE; // the user hopes the system will eventually give him that much real mem
#else
	long grow;
	// find largest contiguous block
	avail = TempMaxMem(&grow);

	// leave some for others
	if((avail - (SYSMEMMIN * MEGABYTE)) <= 0)
	{
		sprintf(cstr, "GetSongMem got %d, cannot leave %d", avail, (SYSMEMMIN * MEGABYTE));
		maLog(cstr);
		return 1;	
	}
	
	if((avail -= (SYSMEMMIN * MEGABYTE)) > (SONGMEMMAX * MEGABYTE))
		avail = (SONGMEMMAX * MEGABYTE);
		
	grow = g_buffersize;
		
	if(avail > (grow * MEGABYTE))
		avail = (grow * MEGABYTE);
#endif
		
	g_memhandle = TempNewHandle(avail, &err);	

	if(err != noErr || g_memhandle == NULL)
	{
		g_memhandle = NULL; 
		sprintf(cstr, "%d GetSongMem TempNewHandle(%d)", err, avail);
		maLog(cstr);
		return 1;	
	}

	if((l_songmem = *g_memhandle) == NULL)
	{
		sprintf(cstr, "GetSongMem null l_songmem (avail=%d)", avail);
		maLog(cstr);
		return 1;	
	}
	
	g_loadedmemsize = avail;
	l_songmempos = l_songmem;
	l_songmemend = l_songmem + avail;
	
	return 0;
	
} // GetSongMem
#endif

#ifdef __MAWIN__
int GetSongMem() // win
{
	long grow;
	int avail;
	MEMORYSTATUS memstatus;
	char cstr[256];

	// free current song memory
	if(g_memhandle)
	{
		MAFREE(g_memhandle);
		g_memhandle = NULL; 
		l_songmem = NULL;
		l_songmemend = NULL;
		l_songmempos = NULL;
		g_loadedmemsize = 0;
	}

	// find largest contiguous block
	GlobalMemoryStatus(&memstatus);

	avail = memstatus.dwAvailPhys;

	// leave some for others
	if((avail - (SYSMEMMIN * MEGABYTE)) <= 0)
	{
		sprintf(cstr, "GetSongMem got %d, cannot leave %d", avail, (SYSMEMMIN * MEGABYTE));
		maLog(cstr);
		return 1;	
	}
	
	if((avail -= (SYSMEMMIN * MEGABYTE)) > (SONGMEMMAX * MEGABYTE))
		avail = (SONGMEMMAX * MEGABYTE);
		
	grow = g_buffersize;
		
	if(avail > (grow * MEGABYTE))
		avail = (grow * MEGABYTE);
		
	// get it : todo : try VirtualAlloc + VirtualLock
	l_songmem = g_memhandle = (char *)MAMALLOC(avail);
	
	if(g_memhandle == NULL)
	{
		sprintf(cstr, "GetSongMem MAMALLOC failed for %d bytes", avail);
		maLog(cstr);
		return 1;	
	}
	
	g_loadedmemsize = avail;
	l_songmempos = l_songmem;
	l_songmemend = l_songmem + avail;
	
	return 0;
	
} // GetSongMem
#endif

#ifdef __MAUNIX__
int GetSongMem() // unix
{
	int avail;
	char cstr[256];

	// free current song memory
	if(g_memhandle)
	{
		MAFREE(g_memhandle);
		g_memhandle = NULL; 
		l_songmem = NULL;
		l_songmemend = NULL;
		l_songmempos = NULL;
		g_loadedmemsize = 0;
	}
	
	g_buffersize = g_sets.buffersize;
	
	avail = g_buffersize * MEGABYTE; // the user hopes the system will eventually give him that much real mem
	
	l_songmem = g_memhandle = (char *)MAMALLOC(avail);
	
	if(g_memhandle == NULL)
	{
		sprintf(cstr, "GetSongMem MAMALLOC failed for %d bytes", avail);
		maLog(cstr);
		return 1;	
	}
	
	g_loadedmemsize = avail;
	l_songmempos = l_songmem;
	l_songmemend = l_songmem + avail;
	
	return 0;
	
} // GetSongMem
#endif


int SetSettingsPath()
{
	char name[256];
	char cstr[256];
	
	if(!g_songlist[0])
		return 1;
	
	if(FilenameFromPathname(g_songlist, name))
	{	
		sprintf(cstr, "SetSettingsPath failed to get filename from '%s'", g_songlist);
		maLog(cstr);
		return 2;
	}					
	FileNameRemoveExtension(name);
	sprintf(g_settingspath, "%s%c%s%s", g_settingsdir, FILESEP, name, EXTMAS);

	return 0;
		
} // SetSettingsPath


int OpenSongList()
{
 	char cstr[256];
	char songsource[MAX_PATH] = {0};

#if !defined(MAGUINONE)
	MASETS setscurrent;
	int modkeydown = 0;
	if(g_w[WINDOWAPP] && maModKeyDown())
		modkeydown = 1;
#endif
		
	if(g_fpsonglist)
	{
		fclose(g_fpsonglist);
		g_fpsonglist = NULL;
		ListWindowClear();
	}
	
	g_songsnum = 0;

	if(!g_songlist[0])
		return 1;					
	
	SetSettingsPath();
	
	// open the songlist file	
	if((g_fpsonglist = mafopen(g_songlist, "rb")) == NULL)
	{
		sprintf(cstr, "OpenSongList mafopen failed on '%s'", g_songlist);
		maLog(cstr);
		return 1;
	}
	
	// count the songs
	while(mafgets(songsource, MAX_PATH, g_fpsonglist) != NULL)
	{
		// skip lines with '#' or files that are not playable or don't exist
		if(songsource[0] != '#')
		{
			if(SongFileExists(songsource))
				g_songsnum++;		
			else
			{
				sprintf(cstr, "OpenSongList file not playable or not found: '%s'", songsource);
				maLog(cstr);
#if !defined(NODECODEAUDIOIN)
#else
				if(FileNameHasExtension(songsource, EXTMAA))
					maLog("OpenSongList : audio input not implemented");
#endif
			}
		} // if !#
	} // while
			
	if(g_songsnum == 0)
	{
		fclose(g_fpsonglist);
		g_fpsonglist = NULL;

		if(SongListIsCda())
			return 0;

		sprintf(cstr, "OpenSongList no playable songs in song list '%s'", g_songlist);
		maLog(cstr);
		return 3;
	}
			
	rewind(g_fpsonglist);

#if !defined(MAGUINONE)
	if(modkeydown) // copy current settings
		memcpy(&setscurrent, &g_sets, sizeof(MASETS));
#endif
	
	if(LoadSettings())
	{
		sprintf(cstr, "OpenSongList LoadSettings failed on '%s'", g_songlist);
		maLog(cstr);
		return 1;
	}

#if !defined(MAGUINONE)
	if(modkeydown) // update loaded settings with current settings
		memcpy(&g_sets, &setscurrent, sizeof(MASETS));
#endif
	
	return 0;
	
} // OpenSongList


int GetRecentSongListName()
{
	char *ret;
	char cstr[256];
	FILE *fp;
	
	if((fp = mafopen(l_recentlist, "rb")) == NULL)
	{
		g_songlist[0] = 0;
		sprintf(cstr, "GetRecentSongListName mafopen '%s'", l_recentlist);
		maLog(cstr);
		return 1;
	}
		
	ret = mafgets(g_songlist, MAX_PATH, fp);
	
	fclose(fp);
	
	if(!ret || strlen(g_songlist) < 1)
	{
		g_songlist[0] = 0;
		sprintf(cstr, "GetRecentSongListName mafgets '%s'", l_recentlist);
		maLog(cstr);	
		return 2;
	}

	return 0;
	
} // GetRecentSongListName
	

int UpdateRecent(char *item, int del)
{
	int i;
	char litem[OPENRECENTLENGTH];
	char items[MAXOPENRECENT][OPENRECENTLENGTH];
	char cstr[256];
	FILE *fp;
	
	// read the file into the array
	if(GetRecent(items))
		return 1;
		
	// update the array
	if(del) // delete item
	{
		if(!items[0][0]) // empty list
			return 0;
		strcpy(litem, item);
#if defined(__MAOSX__)
		StrBSDtoStrOSX(litem);
#endif
		for(i = 0; i < MAXOPENRECENT; i++)
			if(!strcmp(litem, items[i]))
				break; // item to delete is at index i
		if(i >= MAXOPENRECENT)
			return 0; // item is not in the list
		for(; i < MAXOPENRECENT - 1; i++) // shift items up one
		{
			if(!items[i + 1][0]) // end of list
			{
				items[i][0] = 0;
				break;
			}
			strcpy(items[i], items[i + 1]);
		}
	}
	else // add item
	{
		strcpy(litem, item);
#if defined(__MAOSX__)
		StrBSDtoStrOSX(litem);
#endif
		for(i = 0; i < MAXOPENRECENT; i++)
			if(!strcmp(litem, items[i]))
				break; // already have the current songlist at index i
		if(i == 0) // the current songlist is already at the top of the array
			return 0;
		if(i == MAXOPENRECENT)
			i--;
		for(; i > 0; i--) // shift items down one
			strcpy(items[i], items[i - 1]);
		strcpy(items[0], litem);
	}
	
	// rewrite the array to the file
	if((fp = mafopen(l_recentlist, "w")) == NULL)
	{
		sprintf(cstr, "UpdateRecent mafopen '%s'", l_recentlist);
		maLog(cstr);
		return 1;
	}
	
	for(i = 0; i < MAXOPENRECENT; i++)
	{
		if(!items[i][0])
			break;
		if(fprintf(fp, "%s\n", items[i]) < 0)
		{
			sprintf(cstr, "UpdateRecent fprintf '%s' failed",  items[i]);
			maLog(cstr);
		}
	}
	
	fclose(fp);
		
#if defined(__MAOSX__) || defined(__MAUNIX__)
	machmod(l_recentlist, ACCESSPERMS); // 0777
#endif

	UpdateSubMenusRecent();

	return 0;
	
} // UpdateRecent


int GetRecent(char items[][OPENRECENTLENGTH])
{
	int i, j;
	char fpath[MAX_PATH];
	char cstr[256];
	FILE *fp;
	
	// clear the array	
	for(i = 0; i < MAXOPENRECENT; i++)
		items[i][0] = 0;

	// read the file into the array
	if(maFileExists(l_recentlist))
	{
		if((fp = mafopen(l_recentlist, "rb")) == NULL)
		{
			sprintf(cstr, "GetRecent mafopen '%s'", l_recentlist);
			maLog(cstr);
			return 1;
		}

		for(i = 0, j = 0; i < MAXOPENRECENT; i++)
		{
			if(mafgets(fpath, OPENRECENTLENGTH, fp) == NULL)
				break;
			if(maFileExists(fpath))
				strcpy(items[j++], fpath);
		}
		fclose(fp);
	}
		
	return 0;

} // GetRecent


void FileOutWindowDisable()
{
	if(WINDOWOUTCUR < WINDOWOUTFIRST || WINDOWOUTCUR > WINDOWOUTLAST || !g_w[WINDOWOUTCUR])
		return;
		
	PushButtonDisable(WINDOWOUTCUR, ID_BTN_FILEOUTDIR);

#if defined(ENCODEMP3) || defined(ENCODEOGG) || defined(ENCODEAAC) || defined(ENCODEMP4) 

	switch (g_sets.output)
	{
		
 #ifdef ENCODEMP3
		case kOUTMP3 :
			PopupButtonDisable(WINDOWOUTCUR, ID_POPUP_QUALITY);
			PopupButtonDisable(WINDOWOUTCUR, ID_POPUP_BITRATE);
			PopupButtonDisable(WINDOWOUTCUR, ID_POPUP_MODE);
			PopupButtonDisable(WINDOWOUTCUR, ID_POPUP_SAMPLERATE);
		break;
 #endif
 
 #ifdef ENCODEOGG
		case kOUTOGG :
			// PopupButtonDisable(WINDOWOUTCUR, ID_POPUP_QUALITY); // not used
			PopupButtonDisable(WINDOWOUTCUR, ID_POPUP_BITRATE);
			PopupButtonDisable(WINDOWOUTCUR, ID_POPUP_MODE);
		break;
 #endif
 
 #ifdef ENCODEAAC
		case kOUTAAC :
			PopupButtonDisable(WINDOWOUTCUR, ID_POPUP_QUALITY);
			CheckBoxDisable(WINDOWOUTCUR, ID_CHK_ADTS);
		break;
 #endif
 
 #ifdef ENCODEMP4
		case kOUTMP4 :
			PopupButtonDisable(WINDOWOUTCUR, ID_POPUP_QUALITY);
			CheckBoxDisable(WINDOWOUTCUR, ID_CHK_M4A);
		break;
 #endif
 
		default :
		break;
	} // switch
		
#endif

} // FileOutWindowDisable


void FileOutWindowEnable()
{
	if(WINDOWOUTCUR < WINDOWOUTFIRST || WINDOWOUTCUR > WINDOWOUTLAST || !g_w[WINDOWOUTCUR])
		return;
		
	PushButtonEnable(WINDOWOUTCUR, ID_BTN_FILEOUTDIR);

#if defined(ENCODEMP3) || defined(ENCODEOGG) || defined(ENCODEAAC) || defined(ENCODEMP4) 

	switch (g_sets.output)
	{
 #ifdef ENCODEMP3
		case kOUTMP3 :
			PopupButtonEnable(WINDOWOUTCUR, ID_POPUP_QUALITY);
			PopupButtonEnable(WINDOWOUTCUR, ID_POPUP_BITRATE);
			PopupButtonEnable(WINDOWOUTCUR, ID_POPUP_MODE);
			PopupButtonEnable(WINDOWOUTCUR, ID_POPUP_SAMPLERATE);
		break;
 #endif
 
 #ifdef ENCODEOGG
		case kOUTOGG :
			// PopupButtonEnable(WINDOWOUTCUR, ID_POPUP_QUALITY); // not used
			PopupButtonEnable(WINDOWOUTCUR, ID_POPUP_BITRATE);
			PopupButtonEnable(WINDOWOUTCUR, ID_POPUP_MODE);
		break;
 #endif
 
 #ifdef ENCODEAAC
		case kOUTAAC :
			PopupButtonEnable(WINDOWOUTCUR, ID_POPUP_QUALITY);
			CheckBoxEnable(WINDOWOUTCUR, ID_CHK_ADTS);
		break;
 #endif
 
 #ifdef ENCODEMP4
		case kOUTMP4 :
			PopupButtonEnable(WINDOWOUTCUR, ID_POPUP_QUALITY);
			CheckBoxEnable(WINDOWOUTCUR, ID_CHK_M4A);
		break;
 #endif
 
		default :
		break;
	} // switch
		
#endif

} // FileOutWindowEnable


void DisableButtonsMenus()
{
	if(g_running)
		return;
		
	g_cancel = 0;
	l_logoff = g_sets.logflags & kMALOGOFF;
	l_logcount = 0;
	
	PushButtonDisable(WINDOWAPP, ID_BTN_PLAY);			// Play
	MenuDisable(MENUMACONTROLS, ID_MENU_PLAY);			// Play

	MenuDisable(MENUMACONTROLS, ID_MENU_PLAYCD);		// Play CD
	MenuDisable(MENUMACONTROLS, ID_MENU_EJECTCD);		// Eject CD

	MenuDisable(MENUMACONTROLS, ID_MENU_AUDIOIN);		// Audio In mode

	MenuDisable(MENUMASETS, ID_MENU_SETVDISK);			// Set Vdisk
#if !defined(MANOCDA) && defined(__MAUNIX__)
	MenuDisable(MENUMASETS, ID_MENU_SETAUDIOCD);		// Set Audio CD	
#endif
	MenuDisable(MENUMASETS, ID_MENU_SETFACTORY);		// Factory setttings
	MenuDisable(MENUMASETS, ID_MENU_SETDEFAULT);		// Default setttings
	MenuDisable(MENUMASETS, ID_MENU_SETSAVED);			// Saved setttings
	MenuDisable(MENUFILES, ID_MENU_MAKELIST);			// Make List
	MenuDisable(MENUFILES, ID_MENU_SAVELISTAS);			// Save List As
	MenuDisable(MENUFILES, ID_MENU_OPENLIST);			// Select List
	MenuDisable(MENUFILES, ID_MENU_PLAYFILE);			// Play Song File
	MenuDisable(MENUFILES, ID_MENU_OPENRECENT);			// Open Recent
	MenuDisable(MENUTOOLS, ID_MENU_MAKECATALOG);		// Make Catalog
	PopupButtonDisable(WINDOWAPP, ID_POPUP_OUTPUT);		// Output
	PopupButtonDisable(WINDOWAPP, ID_POPUP_BUFFER);		// Mem/VDisk/Direct
	// MenuDisable(MENUMASETS, ID_MENU_ADVANCED);		// Advanced setttings
	
	FileOutWindowDisable();
	
} // DisableButtonsMenus


void EnableButtonsMenus()
{	
	if(g_running)
		return;
		
	PushButtonEnable(WINDOWAPP, ID_BTN_PLAY);			// Play
	MenuEnable(MENUMACONTROLS, ID_MENU_PLAY);			// Play
#ifndef MANOCDA
	MenuEnable(MENUMACONTROLS, ID_MENU_PLAYCD);			// Play CD
	MenuEnable(MENUMACONTROLS, ID_MENU_EJECTCD);		// Eject CD
#endif
#if !defined(NODECODEAUDIOIN)
	MenuEnable(MENUMACONTROLS, ID_MENU_AUDIOIN);		// Audio In mode
#endif
	MenuEnable(MENUMASETS, ID_MENU_SETVDISK);			// Set Vdisk
#if !defined(MANOCDA) && defined(__MAUNIX__)
	MenuEnable(MENUMASETS, ID_MENU_SETAUDIOCD);			// Set Audio CD	
#endif
	MenuEnable(MENUMASETS, ID_MENU_SETFACTORY);			// Factory setttings
	MenuEnable(MENUMASETS, ID_MENU_SETDEFAULT);			// Default setttings
	MenuEnable(MENUMASETS, ID_MENU_SETSAVED);			// Saved setttings
	MenuEnable(MENUFILES, ID_MENU_MAKELIST);			// Make List
	MenuEnable(MENUFILES, ID_MENU_SAVELISTAS);			// Save List As
	MenuEnable(MENUFILES, ID_MENU_OPENLIST);			// Select List
	MenuEnable(MENUFILES, ID_MENU_PLAYFILE);			// Play Song File
	MenuEnable(MENUFILES, ID_MENU_OPENRECENT);			// Open Recent
	MenuEnable(MENUTOOLS, ID_MENU_MAKECATALOG);			// Make Catalog
	PopupButtonEnable(WINDOWAPP, ID_POPUP_OUTPUT);		// Output
	PopupButtonEnable(WINDOWAPP, ID_POPUP_BUFFER);		// Mem/VDisk/Direct
	// MenuEnable(MENUMASETS, ID_MENU_ADVANCED);		// Advanced setttings
	
	FileOutWindowEnable();
	
} // EnableButtonsMenus


void SetLogMenus()
{	
	if(g_w[WINDOWAPP])
	{
		if(l_havelog)
		{
			MenuEnable(MENUTOOLS, ID_MENU_VIEWLOG);
			MenuEnable(MENUTOOLS, ID_MENU_CLEARLOG);
		}
		else
		{
			MenuDisable(MENUTOOLS, ID_MENU_VIEWLOG);
			MenuDisable(MENUTOOLS, ID_MENU_CLEARLOG);
		}
	}
	
} // SetLogMenus


void OnMenuSave()
{
	if(g_songlist == NULL || !g_songlist[0])
	{
		OnMenuAsDefaults();
		return;
	}
	
	DisableButtonsMenus();

	memcpy(&g_setsbu, &g_sets, sizeof(MASETS));
	if(!SaveSettings())
		if(!g_running)
			SetAppWindowTitle((char *)"Settings saved");

	EnableButtonsMenus();
	
} // OnMenuSave


void OnMenuSpindownNow()
{
	SpinDown();
	
} // OnMenuSpindownNow


void OnMenuEjectCD()
{
#ifndef MANOCDA
	int i;

	if(g_running)
	{
		ShowErr("Eject CD : Stop first");
		return;
	}
	
	//CdaCloseFile(g_cdaactive);
		
	for(i = 1; i <= CDAMAXNUMDRIVES; i++)
		if(g_cdadrive[i].exists && g_cdadrive[i].isready)
			CdaEject(i);
	
#endif
	
} // OnMenuEjectCD


void OnMenuAudioIn()
{
#if !defined(NODECODEAUDIOIN)
	if(g_running)
	{
		ShowErr("Audio In : Stop first");
		return;
	}
	
	if(g_fpsonglist)
	{
		fclose(g_fpsonglist);
		g_fpsonglist = NULL;
	}
	
	if(strcmp(g_songlist, g_songlistaudioin))
	{
		strcpy(g_songlist, g_songlistaudioin);
		if(ChangeSongList(g_songlist))
			return; // error
	}

	OnButtonPlay();
#endif	
} // OnMenuAudioIn


void OnButtonList()
{
	if(!g_w[WINDOWLIST] || g_listbusy)
		return;
	g_listbusy = 1;
		
	PushButtonDisable(WINDOWLIST, ID_BTN_SONGLIST);
					
	g_sets.listwindow &= ~(LBOXON | LBOXINFO | LBOXLIST | LBOXLOADED | LBOXID3);
	g_sets.listwindow |= (LBOXON | LBOXLIST);
	ShowListWindow();
		
	PushButtonEnable(WINDOWLIST, ID_BTN_SONGLIST);
	
	g_listbusy = 0;
	
} // OnButtonList


void OnButtonSong()
{
	if(!g_w[WINDOWLIST] || g_listbusy)
		return;
	g_listbusy = 1;
		
	PushButtonDisable(WINDOWLIST, ID_BTN_SONGINFO);
					
	DisableButtonsMenus();

	g_sets.listwindow &= ~(LBOXON | LBOXINFO | LBOXLIST | LBOXLOADED | LBOXID3);
	g_sets.listwindow |= (LBOXON | LBOXID3);
	ShowListWindow();

	EnableButtonsMenus();

	PushButtonEnable(WINDOWLIST, ID_BTN_SONGINFO);		
	
	g_listbusy = 0;
		
} // OnButtonSong


void OnButtonInfo()
{
	if(!g_w[WINDOWLIST] || g_listbusy)
		return;
	g_listbusy = 1;
		
	PushButtonDisable(WINDOWLIST, ID_BTN_INFO);
					
	DisableButtonsMenus();

#ifndef MANOCDA
	if(!g_running)
		CdaInitCDRom(); // OnButtonInfo
#endif

	g_sets.listwindow &= ~(LBOXON | LBOXINFO | LBOXLIST | LBOXLOADED | LBOXID3);
	g_sets.listwindow |= (LBOXON | LBOXINFO);
	ShowListWindow();
			
	EnableButtonsMenus();

	PushButtonEnable(WINDOWLIST, ID_BTN_INFO);		
	
	g_listbusy = 0;
		
} // OnButtonInfo


void OnButtonLoaded()
{
	if(!g_w[WINDOWLIST] || g_listbusy)
		return;
	g_listbusy = 1;
		
	PushButtonDisable(WINDOWLIST, ID_BTN_LOADED);
					
	DisableButtonsMenus();

	g_sets.listwindow &= ~(LBOXON | LBOXINFO | LBOXLIST | LBOXLOADED | LBOXID3);
	g_sets.listwindow |= (LBOXON | LBOXLOADED);
	ShowListWindow();
		
	EnableButtonsMenus();

	PushButtonEnable(WINDOWLIST, ID_BTN_LOADED);
	
	g_listbusy = 0;
		
} // OnButtonLoaded


void OnMenuZero()
{
	int ret;
	char cstr[256];

	DisableButtonsMenus();

	if((ret = ZeroSongRec(&l_songrec)) == 0)
	{	
		l_songsnumplayed = 0;	
		sprintf(cstr, "Zero - all %d songs marked as unplayed", g_songsnum);
		SetAppWindowTitle(cstr);
	}
	
	EnableButtonsMenus();

} // OnMenuZero


void OnCheckBoxBeg(int checked)
{
	char cstr[32];

	if(checked)
	{
		g_posbegon = 1;
		
		if(l_ispaused)
			g_posbeg = LROUND(((g_state.duration * (double)l_posslidervalue) / POSMAX));
		else
			g_posbeg = g_state.position;
		
		if(g_posbeg >= g_posend)
			g_posbeg = g_posend - MINPOSDELTA;
		if(g_posbeg < 0)
			g_posbeg = 0;
			
		MillisecondsToStrHMS(cstr, g_posbeg);
	}
	else
	{
		g_posbegon = 0;
		g_posbeg = 0;
		strcpy(cstr, ZEROMSSTR);
	}
	
	l_playerseeking = 0;
	
	StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEBEG, cstr);
	
} // OnCheckBoxBeg


void OnCheckBoxEnd(int checked)
{
	char cstr[32];

	if(checked)
	{
		g_posendon = 1;
		
		if(l_ispaused)
			g_posend = l_poscurrent;
		else
			g_posend = g_state.position;
		
		if(g_posend <= g_posbeg)
			g_posend = g_posbeg + MINPOSDELTA;		
		if(g_posend < MINPOSDELTA)
			g_posend = MINPOSDELTA;
		
		MillisecondsToStrHMS(cstr, g_posend);
	}
	else
	{
		g_posendon = 0;
		g_posend = g_state.duration;
		MillisecondsToStrHMS(cstr, g_posend);
	}
	
	l_playerseeking = 0;
	
	l_posend = g_posend;
	StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEEND, cstr);

} // OnCheckBoxEnd


void OnCheckBoxM4A(int checked)
{
	if(checked)
		g_sets.fileout[g_k[g_sets.output]].mode |= kACCM4A;
	else
		g_sets.fileout[g_k[g_sets.output]].mode &= ~kACCM4A;

} // OnCheckBoxM4A


void OnCheckBoxADTS(int checked)
{
	if(checked)
		g_sets.fileout[g_k[g_sets.output]].mode |= kACCADTS;
	else
		g_sets.fileout[g_k[g_sets.output]].mode &= ~kACCADTS;

} // OnCheckBoxM4A


void OnCheckCloseReopen(int checked)
{
	if(checked)
		g_sets.fileout[kOUTSOUND].mode = 1;
	else
		g_sets.fileout[kOUTSOUND].mode = 0;

} // OnCheckCloseReopen


void OnCheckUseM3u(int checked)
{
	if(checked)
		g_sets.usem3u = 1;
	else
		g_sets.usem3u = 0;

} // OnCheckUseM3u


void OnMenuOpenList()
{
	char cstr[256];
	char file[MAX_PATH];
	
	DisableButtonsMenus();
	
	sprintf(cstr, "%s (*.maf)|*.maf|Any file (*)|*", g_prognametitle);
	if(!maFileDialog(file, WINDOWAPP, (char *)"Open a song list", g_songlistsdir, (char *)"", cstr, maOPEN | maFILE_MUST_EXIST))
		ChangeSongList(file);
	
	EnableButtonsMenus();
	
} // OnMenuOpenList


void OnMenuDeleteList()
{
	int ok;
	char cstr[256];
	char file[MAX_PATH];
	
	DisableButtonsMenus();
	
	sprintf(cstr, "%s (*.maf)|*.maf|(*.maa)|*.maa|Any file (*)|*", g_prognametitle);
	ok = 0;
	while(1)
	{
		if(!maFileDialog(file, WINDOWAPP, (char *)"Delete a song list", g_songlistsdir, (char *)"", cstr, maOPEN | maFILE_MUST_EXIST))
			DeleteSongList(file);
		else
			break;
	}
	
	EnableButtonsMenus();
	
} // OnMenuDeleteList


char g_lastsongdir[MAX_PATH];
void OnMenuPlayFile()
{
	int err = 1;
	int i, j;
	char newsonglist[MAX_PATH];
	char cstr[MAX_PATH + MAX_PATH];
	maArray ar;
		
	DisableButtonsMenus();
	
	if(maFileDialogM(&ar, WINDOWAPP, (char *)"Open files to play", g_lastsongdir, (char *)"", (char *)
		"Any file|*|mp3|*.mp3|ogg|*.ogg|wma|*.wma|wav|*.wav|aif|*.aif|aiff|*.aiff|cda|*.cda|aac|*.aac|mp4|*.mp4|m2a|*.m2a|m4a|*.m4a",
				maOPEN | maFILE_MUST_EXIST | maMULTIPLE))
		goto onmenuplayfileret;
				
	// save dir for next time	
	j = 0;
	strcpy(g_lastsongdir, ar.pitems[0]);
	for(i = 0; i < (int)strlen(g_lastsongdir); i++)
		if(g_lastsongdir[i] == FILESEP)
			j = i;
	g_lastsongdir[j] = 0;
				
	newsonglist[0] = 0;
	err = maSongListMake(ar.nitems, ar.pitems, newsonglist); // OnMenuPlayFile
	maFileDialogMFree(&ar);
	if(err)
	{
		sprintf(cstr, "OnMenuPlayFile maSongListMake failed on '%s'", newsonglist);
		maLog(cstr);
		goto onmenuplayfileret;
	}
				
	err = ChangeSongList(newsonglist);

onmenuplayfileret:
	EnableButtonsMenus();
	
	if(!err)
		DoTimerPlay(100);
	
} // OnMenuPlayFile


void OnMenuOpenRecent(int index)
{
	char items[MAXOPENRECENT][OPENRECENTLENGTH];
	if(!GetRecent(items))
		if(items[index][0])
			ChangeSongList(items[index]);

} // OnMenuOpenRecent


void OnMenuSetVDisk()
{
	DisableButtonsMenus();
	
	if(!SetVDisk())
		GetVDisk();
		
	ShowListWindow();
			
	EnableButtonsMenus();
	
} // OnMenuSetVDisk


#if defined(MAINSTANCES)
void OnMenuMultipleInst(int checked)
{
	if(g_thisinstance == 1) // always true here
	{
		g_sets.multipleinst = checked;
		MenuSetChecked(MENUMASETS, ID_MENU_MULTIPLEINST, (g_sets.multipleinst) ? 1 : 0);
		SetMultipleInstances();
	}
} // OnMenuMultipleInst
#endif


#if defined(MAKEYBOARD)
void OnMenuKeyboard(int checked)
{
	g_sets.keyboard = checked;
	MenuSetChecked(MENUMASETS, ID_MENU_KEYBOARD, (g_sets.keyboard) ? 1 : 0);
	if(!checked)
		OnCheckTitleInfo(g_sets.titleinfo + 1);
	
	UpdateMenuKbLabels();
	UpdateEffectsKbLabels();
	UpdateButtonLabels();
			
} // OnMenuKeyboard
#endif


#if defined(MAMOUSEWHEEL)
void OnMenuMouseWheel(int checked)
{
	g_sets.mousewheel = checked;
	MenuSetChecked(MENUMASETS, ID_MENU_MOUSEWHEEL, (g_sets.mousewheel) ? 1 : 0);
			
} // OnMenuMouseWheel
#endif


#if !defined(MANOCDA) && defined(__MAUNIX__)
void OnButtonAudioCDSet()
{
	DisableButtonsMenus();
	
	SetAudioCD();
		
	EnableButtonsMenus();
	
} // OnButtonAudioCDSet


void OnButtonAudioCDClear()
{
	ClearAudioCD();
		
} // OnButtonAudioCDClear


void OnButtonAudioCDSave()
{
	int item;	

	PopupButtonGetSelection(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, &item);
	if(item > -1)
		g_setscdrom.item = item;

	WindowGetCoordinates(WINDOWAUDIOCD, &g_setscdrom.winx[g_sets.winconf], &g_setscdrom.winy[g_sets.winconf],
										&g_setscdrom.winw[g_sets.winconf], &g_setscdrom.winh[g_sets.winconf]);
	SaveSettingsCDRom();
		
} // OnButtonAudioCDSave
#endif



void OnMenuAsDefaults()
{
	if(maMessageBox((char *)"Really save current settings as Default settings ?",
					(char *)"NEW DEFAULT SETTINGS", maYES_NO | maICON_QUESTION) != maYES)
		return;
	
	DisableButtonsMenus();
	
	if(!SaveSettingsAsDefault(&g_sets))
		if(!g_running)
			SetAppWindowTitle((char *)"Settings saved");
		
	EnableButtonsMenus();

} // OnMenuAsDefaults


void OnMenuSetFactory()
{
	DisableButtonsMenus();
	
	LoadFactorySettings(&g_sets);
	UpdateDisplay();
	MadDoParamChange();
	EqDoParamChange();
		
	EnableButtonsMenus();
	
} // OnMenuSetFactory


void OnMenuSetDefault()
{
	DisableButtonsMenus();
	
	LoadDefaultSettings(&g_sets);
	UpdateDisplay();
	MadDoParamChange();
	EqDoParamChange();
		
	EnableButtonsMenus();

} // OnMenuSetDefault


void OnMenuSetSaved()
{
	DisableButtonsMenus();
	
	l_settingspathlast[0] = 0;
	LoadSettings();
	UpdateDisplay();
	MadDoParamChange();
	EqDoParamChange();
		
	EnableButtonsMenus();

} // OnMenuSetSaved


void OnMenuMakeList()
{
	char *pitems[1];
	char dirpath[MAX_PATH];
	char newsonglist[MAX_PATH];
	
	DisableButtonsMenus();

	dirpath[0] = 0;

	// prompt for items for a new song list (dirs, files, links, cd, ...)
	if(maDirSelector(dirpath, (char *)
#ifdef __MAWIN__
		"Select a FOLDER containing song files and/or shortcuts, or an Audio CD"
#endif

#if defined(__MAUNIX__)
		"Select a DIRECTORY containing song files and/or links, or an Audio CD"
#endif

#ifdef __MAOS9__
		"Select a FOLDER containing song files and/or aliases, or an Audio CD"
#endif

#ifdef __MAOSX__
		"Select a FOLDER containing song files and/or aliases" // wxDirSelector can't handle Audio CD
#endif
		))
		goto OnButtonMakeListRet; // cancelled

	// make a new songlist
	strcpy(newsonglist, dirpath); // prompt
	pitems[0] = dirpath;
	if(maSongListMake(1, pitems, newsonglist)) // OnMenuMakeList
		goto OnButtonMakeListRet;
	
	// new songlist becomes current song list
	ChangeSongList(newsonglist);
	
OnButtonMakeListRet:

	SetAppWindowTitle((char *)"Stopped");

	EnableButtonsMenus();
	
} // OnMenuMakeList


void OnMenuSaveListAs()
{
	DisableButtonsMenus();

	SaveSongListAs();
	
	EnableButtonsMenus();
	
} // OnMenuSaveListAs


double g_eqzoomval = -1.0;
EQUPRESET l_eqpresetstmp[EQNUMPRESETS];
EQUPRESET l_eq2presetstmp[EQNUMPRESETS];
void OnButtonEqRaise();
void OnButtonEqLower();

void OnButtonEqRaise()
{
	int band;
	
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQPLUS);
					
	// first time in tmpsave eq settings
	if(g_eqzoomval < 0.0)
	{
		g_eqzoomval = 1.0;
		memcpy( l_eqpresetstmp,  g_sets.eqpresets, EQNUMPRESETS * sizeof(EQUPRESET));
		memcpy(l_eq2presetstmp, g_sets.eq2presets, EQNUMPRESETS * sizeof(EQUPRESET));
	}
	
	// do nothing if all >= max
	for(band = 0; band < EQBANDS31; band++)
		if(g_sets.eqpresets[g_sets.eqpreset].eqslider[band] < DBSLIDERMAX ||
			g_sets.eq2presets[g_sets.eqpreset].eqslider[band] < DBSLIDERMAX)
			break;
	
	if(band >= EQBANDS31) // all >= max
	{
		for(band = 0; band < EQBANDS31; band++)
		{
			g_sets.eqpresets[g_sets.eqpreset].eqslider[band] =
			g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = DBSLIDERMAX;
		}
		if(g_w[WINDOWEQ])
			PushButtonEnable(WINDOWEQ, ID_BTN_EQPLUS);		
		return;
	}
				
	for(band = 0; band < EQBANDS31; band++)
	{		
		g_sets.eqpresets[g_sets.eqpreset].eqslider[band] += SLIDERINCR;
		g_sets.eq2presets[g_sets.eqpreset].eqslider[band] += SLIDERINCR;
		if(g_sets.eqpresets[g_sets.eqpreset].eqslider[band] > DBSLIDERMAX)
			g_sets.eqpresets[g_sets.eqpreset].eqslider[band] = DBSLIDERMAX;
		if(g_sets.eqpresets[g_sets.eqpreset].eqslider[band] < DBSLIDERMIN)
			g_sets.eqpresets[g_sets.eqpreset].eqslider[band] = DBSLIDERMIN;
		if(g_sets.eq2presets[g_sets.eqpreset].eqslider[band] > DBSLIDERMAX)
			g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = DBSLIDERMAX;
		if(g_sets.eq2presets[g_sets.eqpreset].eqslider[band] < DBSLIDERMIN)
			g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = DBSLIDERMIN;
	}			
				
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
		SetSliders(EQ2SLIDERS);
	
	if(g_w[WINDOWEQ])
	{
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQPLUS);		
	}
	
} // OnButtonEqRaise


void OnButtonEqLower()
{
	int band;
	
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQMINUS);
	
	// first time in tmpsave eq settings	
	if(g_eqzoomval < 0.0)
	{
		g_eqzoomval = 1.0;
		memcpy(l_eqpresetstmp,   g_sets.eqpresets, EQNUMPRESETS * sizeof(EQUPRESET));
		memcpy(l_eq2presetstmp, g_sets.eq2presets, EQNUMPRESETS * sizeof(EQUPRESET));
	}
	
	// do nothing if all <= min			
	for(band = 0; band < EQBANDS31; band++)
		if(g_sets.eqpresets[g_sets.eqpreset].eqslider[band] > DBSLIDERMIN ||
		   g_sets.eq2presets[g_sets.eqpreset].eqslider[band] > DBSLIDERMIN)
			break;
	
	if(band >= EQBANDS31) // all <= min
	{
		for(band = 0; band < EQBANDS31; band++)
		{
			g_sets.eqpresets[g_sets.eqpreset].eqslider[band] =
			g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = DBSLIDERMIN;
		}
		if(g_w[WINDOWEQ])
			PushButtonEnable(WINDOWEQ, ID_BTN_EQMINUS);		
		return;
	}
			
	for(band = 0; band < EQBANDS31; band++)
	{		
		g_sets.eqpresets[g_sets.eqpreset].eqslider[band] -= SLIDERDECR;
		g_sets.eq2presets[g_sets.eqpreset].eqslider[band] -= SLIDERDECR;
		if(g_sets.eqpresets[g_sets.eqpreset].eqslider[band] > DBSLIDERMAX)
			g_sets.eqpresets[g_sets.eqpreset].eqslider[band] = DBSLIDERMAX;
		if(g_sets.eqpresets[g_sets.eqpreset].eqslider[band] < DBSLIDERMIN)
			g_sets.eqpresets[g_sets.eqpreset].eqslider[band] = DBSLIDERMIN;
		if(g_sets.eq2presets[g_sets.eqpreset].eqslider[band] > DBSLIDERMAX)
			g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = DBSLIDERMAX;
		if(g_sets.eq2presets[g_sets.eqpreset].eqslider[band] < DBSLIDERMIN)
			g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = DBSLIDERMIN;
	}
			
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
		SetSliders(EQ2SLIDERS);
	
	if(g_w[WINDOWEQ])
	{
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQMINUS);		
	}
	
} // OnButtonEqLower


void OnButtonEqPlus()
{
	int band;
	int val[EQBANDS31];
	int val2[EQBANDS31];
	int mid = (DBSLIDERMAX - DBSLIDERMIN) / 2;
	int imax = 0;
	int nmax = 0;
	
	if(!(g_sets.eqflags & MAEQZOOM))
	{
		OnButtonEqRaise();
		return;
	}
					
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQPLUS);

	// first time in tmpsave eq settings	
	if(g_eqzoomval < 0.0)
	{
		g_eqzoomval = 1.0;
		memcpy(l_eqpresetstmp,   g_sets.eqpresets, EQNUMPRESETS * sizeof(EQUPRESET));
		memcpy(l_eq2presetstmp, g_sets.eq2presets, EQNUMPRESETS * sizeof(EQUPRESET));
	}
	
	// 5% increase	
	g_eqzoomval = g_eqzoomval + 0.05;
	
	for(band = 0; band < EQBANDS31; band++)
	{
		imax = 0;
		if((val[band] = LROUND(
			mid + g_eqzoomval * (l_eqpresetstmp[g_sets.eqpreset].eqslider[band] - mid))) > DBSLIDERMAX || 
																			val[band] < DBSLIDERMIN)
			imax++;
		if((val2[band] = LROUND(
			mid + g_eqzoomval * (l_eq2presetstmp[g_sets.eqpreset].eqslider[band] - mid))) > DBSLIDERMAX || 
																	val2[band] < DBSLIDERMIN || imax)
			nmax++;
	}
	
	if(nmax >= EQBANDS31) // all sliders on limits
	{
		if(g_w[WINDOWEQ])
			PushButtonEnable(WINDOWEQ, ID_BTN_EQPLUS);
		return;
	}
			
	for(band = 0; band < EQBANDS31; band++)
	{	
	
		if(val[band] > DBSLIDERMAX)
			val[band] = DBSLIDERMAX;
		else if(val[band] < DBSLIDERMIN)
			val[band] = DBSLIDERMIN;
		g_sets.eqpresets[g_sets.eqpreset].eqslider[band] = val[band];

		if(val2[band] > DBSLIDERMAX)
			val2[band] = DBSLIDERMAX;
		else if(val2[band] < DBSLIDERMIN)
			val2[band] = DBSLIDERMIN;
		g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = val2[band];
	}
				
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
		SetSliders(EQ2SLIDERS);
	
	if(g_w[WINDOWEQ])
	{
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQPLUS);		
	}
	
} // OnButtonEqPlus


void OnButtonEqMinus()
{
	int band;
	int mid = (DBSLIDERMAX - DBSLIDERMIN) / 2;
	
	if(!(g_sets.eqflags & MAEQZOOM))
	{
		OnButtonEqLower();
		return;
	}
					
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQMINUS);
	
	// first time in tmpsave eq settings	
	if(g_eqzoomval < 0.0)
	{
		g_eqzoomval = 1.0;
		memcpy(l_eqpresetstmp,  g_sets.eqpresets,  EQNUMPRESETS * sizeof(EQUPRESET));
		memcpy(l_eq2presetstmp, g_sets.eq2presets, EQNUMPRESETS * sizeof(EQUPRESET));
	}
	
	// 5% decrease	
	if((g_eqzoomval = g_eqzoomval - 0.05) < 0.0)
		g_eqzoomval = 0.0;
	
	for(band = 0; band < EQBANDS31; band++)
	{
		 g_sets.eqpresets[g_sets.eqpreset].eqslider[band] = LROUND(
			mid + g_eqzoomval * ( l_eqpresetstmp[g_sets.eqpreset].eqslider[band] - mid));
		g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = LROUND(
			mid + g_eqzoomval * (l_eq2presetstmp[g_sets.eqpreset].eqslider[band] - mid));
	}
	
	for(band = 0; band < EQBANDS31; band++)
	{		
		if(g_sets.eqpresets[g_sets.eqpreset].eqslider[band] > DBSLIDERMAX)
			g_sets.eqpresets[g_sets.eqpreset].eqslider[band] = DBSLIDERMAX;
		if(g_sets.eqpresets[g_sets.eqpreset].eqslider[band] < DBSLIDERMIN)
			g_sets.eqpresets[g_sets.eqpreset].eqslider[band] = DBSLIDERMIN;

		if(g_sets.eq2presets[g_sets.eqpreset].eqslider[band] > DBSLIDERMAX)
			g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = DBSLIDERMAX;
		if(g_sets.eq2presets[g_sets.eqpreset].eqslider[band] < DBSLIDERMIN)
			g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = DBSLIDERMIN;
	}

	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
		SetSliders(EQ2SLIDERS);
	
	if(g_w[WINDOWEQ])
	{
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQMINUS);		
	}
	
} // OnButtonEqMinus


void OnButtonEqCopy()
{			
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQCOPY);
					
	memcpy(&l_eqpresettemp, &g_sets.eqpresets[g_sets.eqpreset], sizeof(EQUPRESET));
	
	if(g_w[WINDOWEQ])
		PushButtonEnable(WINDOWEQ, ID_BTN_EQCOPY);		
	
} // OnButtonEqCopy


void OnButtonEqPaste()
{			
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQPASTE);
					
	g_eqzoomval = -1.0;

	memcpy(&g_sets.eqpresets[g_sets.eqpreset], &l_eqpresettemp, sizeof(EQUPRESET) - EQNAMELEN);
	
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ])
	{
		StaticTextSetValue(WINDOWEQ, ID_TEXT_EQOFFSETVAL, g_sets.eqpresets[g_sets.eqpreset].eqoffset);
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQPASTE);		
	}
	
} // OnButtonEqPaste


void OnButtonEqRestore()
{
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQRESTORE);
	
	g_eqzoomval = -1.0;
	
	memcpy(&g_sets.eqpresets[g_sets.eqpreset], &g_setsbu.eqpresets[g_sets.eqpreset], sizeof(EQUPRESET) - EQNAMELEN);
						
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ])
	{
		StaticTextSetValue(WINDOWEQ, ID_TEXT_EQOFFSETVAL, g_sets.eqpresets[g_sets.eqpreset].eqoffset);
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQRESTORE);		
	}
	
} // OnButtonEqRestore


void OnButtonEqMid()
{
	int band;
	
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQMID);
					
	// first time in tmpsave eq settings	
	if(g_eqzoomval < 0.0)
		memcpy(l_eqpresetstmp, g_sets.eqpresets, EQNUMPRESETS * sizeof(EQUPRESET));
	
	g_eqzoomval = 0.0;
	
	for(band = 0; band < EQBANDS31; band++)
		g_sets.eqpresets[g_sets.eqpreset].eqslider[band] = DBSLIDERMAX / 2; // mid == 31
	
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ])
	{
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQMID);		
	}
	
} // OnButtonEqMid


void OnButtonEqShiftL()
{
	int band;
	
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQSHIFTL);
					
	// first time in tmpsave eq settings	
	if(g_eqzoomval < 0.0)
		memcpy(l_eqpresetstmp, g_sets.eqpresets, EQNUMPRESETS * sizeof(EQUPRESET));
	
	for(band = 0; band < EQBANDS31 - 1; band++)
	{
		g_sets.eqpresets[g_sets.eqpreset].eqslider[band] =
								g_sets.eqpresets[g_sets.eqpreset].eqslider[band + 1];
		l_eqpresetstmp[g_sets.eqpreset].eqslider[band] =
								l_eqpresetstmp[g_sets.eqpreset].eqslider[band + 1];
		g_sets.eq2presets[g_sets.eqpreset].eqslider[band] =
								g_sets.eq2presets[g_sets.eqpreset].eqslider[band + 1];
		l_eq2presetstmp[g_sets.eqpreset].eqslider[band] =
								l_eq2presetstmp[g_sets.eqpreset].eqslider[band + 1];
	}
	
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
		SetSliders(EQ2SLIDERS);
	
	if(g_w[WINDOWEQ])
	{
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQSHIFTL);		
	}
	
} // OnButtonEqShiftL


void OnButtonEqShiftR()
{
	int band;
	
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQSHIFTR);
					
	// first time in tmpsave eq settings	
	if(g_eqzoomval < 0.0)
		memcpy(l_eqpresetstmp, g_sets.eqpresets, EQNUMPRESETS * sizeof(EQUPRESET));
	
	for(band = EQBANDS31 - 1; band > 0; band--)
	{
		g_sets.eqpresets[g_sets.eqpreset].eqslider[band] =
								g_sets.eqpresets[g_sets.eqpreset].eqslider[band - 1];
		l_eqpresetstmp[g_sets.eqpreset].eqslider[band] =
								l_eqpresetstmp[g_sets.eqpreset].eqslider[band - 1];
		g_sets.eq2presets[g_sets.eqpreset].eqslider[band] =
								g_sets.eq2presets[g_sets.eqpreset].eqslider[band - 1];
		l_eq2presetstmp[g_sets.eqpreset].eqslider[band] =
								l_eq2presetstmp[g_sets.eqpreset].eqslider[band - 1];
	}
									
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
		SetSliders(EQ2SLIDERS);
	
	if(g_w[WINDOWEQ])
	{
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQSHIFTR);		
	}
	
} // OnButtonEqShiftR


void OnButtonEqInvert()
{
	int band;
	
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQINVERT);
					
	// first time in tmpsave eq settings	
	if(g_eqzoomval < 0.0)
		memcpy(l_eqpresetstmp, g_sets.eqpresets, EQNUMPRESETS * sizeof(EQUPRESET));
	
	for(band = 0; band < EQBANDS31; band++)
	{
		g_sets.eqpresets[g_sets.eqpreset].eqslider[band] =
						DBSLIDERMAX - g_sets.eqpresets[g_sets.eqpreset].eqslider[band];
		l_eqpresetstmp[g_sets.eqpreset].eqslider[band] =
						DBSLIDERMAX - l_eqpresetstmp[g_sets.eqpreset].eqslider[band];
		g_sets.eq2presets[g_sets.eqpreset].eqslider[band] =
						DBSLIDERMAX - g_sets.eq2presets[g_sets.eqpreset].eqslider[band];
		l_eq2presetstmp[g_sets.eqpreset].eqslider[band] =
						DBSLIDERMAX - l_eq2presetstmp[g_sets.eqpreset].eqslider[band];
	}
									
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
		SetSliders(EQ2SLIDERS);
	
	if(g_w[WINDOWEQ])
	{
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQINVERT);		
	}
	
} // OnButtonEqInvert


void OnButtonEq2Copy()
{			
	if(g_w[WINDOWEQ2])
		PushButtonDisable(WINDOWEQ2, ID_BTN_EQ2COPY);
					
	memcpy(&l_eqpresettemp, &g_sets.eq2presets[g_sets.eqpreset], sizeof(EQUPRESET));
	
	if(g_w[WINDOWEQ2])
		PushButtonEnable(WINDOWEQ2, ID_BTN_EQ2COPY);		
	
} // OnButtonEq2Copy


void OnButtonEq2Paste()
{			
	if(g_w[WINDOWEQ2])
		PushButtonDisable(WINDOWEQ2, ID_BTN_EQ2PASTE);
					
	g_eqzoomval = -1.0;

	memcpy(&g_sets.eq2presets[g_sets.eqpreset], &l_eqpresettemp, sizeof(EQUPRESET) - EQNAMELEN);
	
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
	{
		SetSliders(EQ2SLIDERS);
		PushButtonEnable(WINDOWEQ2, ID_BTN_EQ2PASTE);		
	}
	
} // OnButtonEq2Paste


void OnButtonEq2Restore()
{
	if(g_w[WINDOWEQ2])
		PushButtonDisable(WINDOWEQ2, ID_BTN_EQ2RESTORE);
	
	g_eqzoomval = -1.0;
	
	memcpy(&g_sets.eq2presets[g_sets.eqpreset], &g_setsbu.eq2presets[g_sets.eqpreset], sizeof(EQUPRESET) - EQNAMELEN);
						
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
	{
		SetSliders(EQ2SLIDERS);
		PushButtonEnable(WINDOWEQ2, ID_BTN_EQ2RESTORE);		
	}
	
} // OnButtonEq2Restore


void OnButtonEq2Mid()
{
	int band;
	
	if(g_w[WINDOWEQ2])
		PushButtonDisable(WINDOWEQ2, ID_BTN_EQ2MID);
					
	// first time in tmpsave eq settings	
	if(g_eqzoomval < 0.0)
		memcpy(l_eqpresetstmp, g_sets.eq2presets, EQNUMPRESETS * sizeof(EQUPRESET));
	
	g_eqzoomval = 0.0;
	
	for(band = 0; band < EQBANDS31; band++)
		g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = DBSLIDERMAX / 2; // mid == 31
	
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
	{
		SetSliders(EQ2SLIDERS);
		PushButtonEnable(WINDOWEQ2, ID_BTN_EQ2MID);		
	}
	
} // OnButtonEq2Mid


void OnButtonFileOutDir()
{
	char dirpath[MAX_PATH];

	if(g_sets.fileout[g_k[g_sets.output]].outdir && g_sets.fileout[g_k[g_sets.output]].outdir[0])
		strcpy(dirpath, g_sets.fileout[g_k[g_sets.output]].outdir); // default dir for dirSelector
	else
		dirpath[0] = 0;
	
	if(!maDirSelector(dirpath, (char *)
#if defined(__MAUNIX__)
		"Select a DIRECTORY in which to put the new files"
#else
		"Select a FOLDER in which to put the new files"
#endif
	)) // else cancelled
		strcpy(g_sets.fileout[g_k[g_sets.output]].outdir, dirpath);

	if(WINDOWOUTCUR >= WINDOWOUTFIRST && WINDOWOUTCUR <= WINDOWOUTLAST)
		StaticTextSetText(WINDOWOUTCUR, ID_TEXT_FILEOUTDIR, g_sets.fileout[g_k[g_sets.output]].outdir);
	
	if(!maDirExists(g_sets.fileout[g_k[g_sets.output]].outdir))
	{
		char cstr[MAX_PATH];
		sprintf(cstr, "File output folder does not exist:\n\n '%s'", g_sets.fileout[g_k[g_sets.output]].outdir);
		ShowErr(cstr);
	}

} // OnButtonFileOutDir


void OnCheckEqZoom(int checked) 
{
	if(checked)
		g_sets.eqflags |= MAEQZOOM;
	else
		g_sets.eqflags &= ~MAEQZOOM;

} // OnCheckEqZoom


void OnCheckEqLink(int checked) 
{
	if(checked)
		g_sets.eqflags |= MAEQLINK;
	else
		g_sets.eqflags &= ~MAEQLINK;
		
} // OnCheckEqLink


void OnCheckTitleInfo(int menu)
{
	char *p;
	
	if(menu)
		g_sets.titleinfo = menu - 1;
	else if(++g_sets.titleinfo > 2)
		g_sets.titleinfo = 0;
		
	SetGUITitleInfo();
	
	if(g_sets.titleinfo == 1)
	{
		if(g_running)
			WindowSetTitle(WINDOWAPP, g_loadedname[g_loadedcurrent]);
		else
			WindowSetTitle(WINDOWAPP, (char *)"(Song name)");
	}
	else if(g_sets.titleinfo == 2)
	{
		if(g_running)
		{
			FilenameFromPathname(g_songlist, l_songlist);
			if((p = strstr(l_songlist, g_extstr[kMAMAF])))
				*p = 0;				
			SetAppTitle2();
		}
		else
			WindowSetTitle(WINDOWAPP, (char *)"(Numbers)");
	}
	else
		SetAppWindowTitle((char *)"");
	
} // OnCheckTitleInfo


void OnCheckAltSize(int checked, int kbcommand) 
{
	g_sets.winconf = checked;

	MenuSetChecked(MENUSHOW, ID_MENU_ALTSIZE, g_sets.winconf ? 1 : 0);

	if(!kbcommand && maModKeyDown())
		WindowsCopyPositions();
	WindowsMoveAndSetSize();
	
} // OnCheckAltSize


void OnCheckRandom(int checked) 
{
	g_sets.random = checked;
	MenuSetChecked(MENUMASETS, ID_MENU_RANDOM, g_sets.random ? 1 : 0);

} // OnCheckRandom


void OnCheckReptPlayed(int checked) 
{
	g_sets.reptplayed = checked;
	MenuSetChecked(MENUMASETS, ID_MENU_REPTPLAYED, g_sets.reptplayed ? 1 : 0);
		
} // OnCheckReptPlayed


void OnCheckLoopLoaded(int checked) 
{
	g_sets.looploaded = checked;
	MenuSetChecked(MENUMASETS, ID_MENU_LOOPLOADED, g_sets.looploaded ? 1 : 0);

} // OnCheckLoopLoaded


void OnCheckLoopList(int checked) 
{
	g_sets.looplist = checked;
	MenuSetChecked(MENUMASETS, ID_MENU_LOOPLIST, g_sets.looplist ? 1 : 0);

} // OnCheckLoopList


#ifdef __MAOS9__
void OnCheckSpindown(int checked)
{
	g_sets.spindown = checked;
	MenuSetChecked(MENUMASETS, ID_MENU_SPINDOWN, g_sets.spindown ? 1 : 0);
	
} // OnCheckSpindown
#endif


void OnCheckHighPriority(int checked) 
{
	if(checked)
	{
		g_sets.highpriority = 1;
		MenuSetChecked(MENUMASETS, ID_MENU_HIPRIORITY, 1);
		if(g_running && g_state.pthreadplayid)
			maThreadSetPriority(g_state.pthreadplayid, g_state.threadplayprihi);
	}
	else
	{
		g_sets.highpriority = 0;
		MenuSetChecked(MENUMASETS, ID_MENU_HIPRIORITY, 0);
		if(g_running && g_state.pthreadplayid)
			maThreadSetPriority(g_state.pthreadplayid, g_state.threadplayprilo);
	}
		
} // OnCheckHighPriority


void OnCheckSinglePlay(int checked) 
{
	g_sets.singleplay = checked;
	MenuSetChecked(MENUMASETS, ID_MENU_SINGLEPLAY, g_sets.singleplay ? 1 : 0);

} // OnCheckSinglePlay


void OnPopupMem()
{
	if(g_sets.buffertype == BUFUSEMEM)
		return;

	g_loadednum = 0;
	g_sets.buffertype = BUFUSEMEM;	
	ShowListWindow();
		
} // OnPopupMem


void OnPopupVDisk()
{
	if(g_sets.buffertype == BUFUSEVDISK)
		return;
		
	g_sets.buffertype = BUFUSEVDISK;
	ShowListWindow();
		
} // OnPopupVDisk


void OnPopupDirect()
{
	if(g_sets.buffertype == BUFUSEDIRECT)
		return;

	if(g_sets.listwindow & LBOXLOADED)
	{
		g_sets.listwindow &= ~LBOXLOADED;
		g_sets.listwindow |= LBOXLIST;
	}
	g_sets.buffertype = BUFUSEDIRECT;

	ShowListWindow();
		
} // OnPopupDirect


void OnCheckEqualizer(int checked)
{
	if(checked)
	{
		g_sets.eqflags |= MAEQACTIVE;
		if(g_w[WINDOWEQ])
			CheckBoxSetValue(WINDOWEQ, ID_CHK_EQUALIZERB, 1);
		MenuSetChecked(MENUMASETS, ID_MENU_EQUALIZER, 1);
	}
	else
	{
		g_sets.eqflags &= ~MAEQACTIVE;
		if(g_w[WINDOWEQ])
			CheckBoxSetValue(WINDOWEQ, ID_CHK_EQUALIZERB, 0);
		MenuSetChecked(MENUMASETS, ID_MENU_EQUALIZER, 0);
	}
	
	EqDoParamChange();
	
} // OnCheckEqualizer


void OnCheckEqualizerB(int checked)
{
	if(checked)
	{
		g_sets.eqflags |= MAEQACTIVE;
		CheckBoxSetValue(WINDOWAPP, ID_CHK_EQUALIZER, 1);
		MenuSetChecked(MENUMASETS, ID_MENU_EQUALIZER, 1);
	}
	else
	{
		g_sets.eqflags &= ~MAEQACTIVE;
		CheckBoxSetValue(WINDOWAPP, ID_CHK_EQUALIZER, 0);
		MenuSetChecked(MENUMASETS, ID_MENU_EQUALIZER, 0);
	}
	
	EqDoParamChange();
	
} // OnCheckEqualizerB


void OnCheckEffects(int checked)
{
	if(checked)
	{
		g_sets.effects |= EFFECTSON;
#if !defined(MAGUINONE)
		if(!g_w[WINDOWEFFECTS])
			DoWindowEffects(1);
		else
			CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_EFFECTSB, 1);
#endif
		if(g_sets.effects & (EFFECTSTEMPO | EFFECTSPITCH | EFFECTSSPEED))
			g_effectschange = 1;
	}
	else
	{
		g_sets.effects &= ~EFFECTSON;
		if(g_w[WINDOWEFFECTS])
			CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_EFFECTSB, 0);
	}

	MenuSetChecked(MENUMASETS, ID_MENU_EFFECTS, (g_sets.effects & EFFECTSON) ? 1 : 0);
		
} // OnCheckEffects


void OnCheckEffectsB(int checked)
{
	if(checked)
	{
		g_sets.effects |= EFFECTSON;
		if(g_sets.effects & (EFFECTSTEMPO | EFFECTSPITCH | EFFECTSSPEED))
			g_effectschange = 1;
	}
	else
		g_sets.effects &= ~EFFECTSON;
		
	CheckBoxSetValue(WINDOWAPP, ID_CHK_EFFECTS, (g_sets.effects & EFFECTSON) ? 1 : 0);
	MenuSetChecked(MENUMASETS, ID_MENU_EFFECTS, (g_sets.effects & EFFECTSON) ? 1 : 0);
	
} // OnCheckEffectsB


void OnCheckReptSong(int checked) 
{
	g_reptsong = checked;
	
} // OnCheckReptSong


void OnCheckBalance(int checked)
{ 
	if(!checked)
	{
		g_sets.volumesliderleft = volminmaxL();
		g_sets.volumesliderright = volminmaxR();
	}
	g_sets.balance = checked;
	SetSliders(VOLUMESLIDERS);
	
} // OnCheckBalance


void OnCheckMonoLeft(int checked)
{ 
	if(checked)
		g_sets.monodiff |= MONOCHECKBOXL;
	else
		g_sets.monodiff &= ~MONOCHECKBOXL;

	RefreshMonoDiff();
	SetGUIMonoStereo();
	
} // OnCheckMonoLeft


void OnCheckMonoRight(int checked)
{ 
	if(checked)
		g_sets.monodiff |= MONOCHECKBOXR;
	else
		g_sets.monodiff &= ~MONOCHECKBOXR;

	RefreshMonoDiff();
	SetGUIMonoStereo();
	
} // OnCheckMonoRight


void OnCheckMonoDiff(int checked) // Mono Diff or Stereo Reverse
{ 
	if(checked)
	{
		if((g_sets.monodiff & MONOCHECKBOXL) && (g_sets.monodiff & MONOCHECKBOXR))  // mono LR
			g_sets.monodiff |= MONOCHECKBOXSUB;										// mono diff
		else if((!(g_sets.monodiff & MONOCHECKBOXL)) && (!(g_sets.monodiff & MONOCHECKBOXR))) // stereo
			g_sets.monodiff |= MONOCHECKBOXREV;										// stereo reverse
	}
	else
	{
		if((g_sets.monodiff & MONOCHECKBOXL) && (g_sets.monodiff & MONOCHECKBOXR))
			g_sets.monodiff &= ~MONOCHECKBOXSUB;
		else if((!(g_sets.monodiff & MONOCHECKBOXL)) && (!(g_sets.monodiff & MONOCHECKBOXR)))
			g_sets.monodiff &= ~MONOCHECKBOXREV;
	}

	SetGUIMonoStereo();
	
} // OnCheckMonoDiff


void RefreshMonoDiff() // for Mono Diff or Stereo Reverse
{ 
	if((  g_sets.monodiff & MONOCHECKBOXL &&
		   g_sets.monodiff & MONOCHECKBOXR &&		// mono LR
		   g_sets.monodiff & MONOCHECKBOXSUB) ||	// mono diff
		(!(g_sets.monodiff & MONOCHECKBOXL) &&
		 !(g_sets.monodiff & MONOCHECKBOXR) &&		// stereo	
		   g_sets.monodiff & MONOCHECKBOXREV))		// stereo reverse
		CheckBoxSetValue(WINDOWAPP, ID_CHK_MONODIFF, 1);
	else
		CheckBoxSetValue(WINDOWAPP, ID_CHK_MONODIFF, 0);

} // OnCheckMonoDiff


void OnCheckLimiter(int checked)
{
	if(checked)
	{
		g_sets.limiter = PREAMP | CLIPPING;
		CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_LIMITER, (char *)"0.0");
	}
	else
	{
		g_sets.limiter = PREAMP;
		CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_LIMITER, (char *)"Limit");
	}
		
	g_state.mpegattenuation = 1.0;
	
	MadDoParamChange();
	
} // OnCheckLimiter


void OnCheckEqExtra(int checked)
{
	if(checked)
	{
		if(g_w[WINDOWEQ])
			CheckBoxSetValue(WINDOWEQ, ID_CHK_EQEXTRAB, 1);
		MenuSetChecked(MENUMASETS, ID_MENU_EQEXTRA, 1);
		g_sets.eqflags |= MAEQEXTRA;
		eqcfg.extra_filtering = 1;
	}
	else
	{
		if(g_w[WINDOWEQ])
			CheckBoxSetValue(WINDOWEQ, ID_CHK_EQEXTRAB, 0);
		MenuSetChecked(MENUMASETS, ID_MENU_EQEXTRA, 0);
		g_sets.eqflags &= ~MAEQEXTRA;
		eqcfg.extra_filtering = 0;
	}
	
} // OnCheckEqExtra


void OnCheckEqExtraB(int checked)
{
	if(checked)
	{
		CheckBoxSetValue(WINDOWAPP, ID_CHK_EQEXTRA, 1);
		MenuSetChecked(MENUMASETS, ID_MENU_EQEXTRA, 1);
		g_sets.eqflags |= MAEQEXTRA;
		eqcfg.extra_filtering = 1;
	}
	else
	{
		CheckBoxSetValue(WINDOWAPP, ID_CHK_EQEXTRA, 0);
		MenuSetChecked(MENUMASETS, ID_MENU_EQEXTRA, 0);
		g_sets.eqflags &= ~MAEQEXTRA;
		eqcfg.extra_filtering = 0;
	}
	
} // OnCheckEqExtraB


void OnCheckSeparateLR(int checked)
{
	char cstr[256];
	
	if(checked)
	{
		CheckBoxSetValue(WINDOWEQ, ID_CHK_SEPARATELR, 1);
		g_sets.eqflags |= MASEPARATELR;
		DoWindowEq2(1);
	}
	else
	{
		CheckBoxSetValue(WINDOWEQ, ID_CHK_SEPARATELR, 0);
		g_sets.eqflags &= ~MASEPARATELR;
		// DoWindowEq2(0);
	}

	EqDoParamChange();

	if(g_w[WINDOWEQ])
	{
		if(g_sets.eqflags & MASEPARATELR)
			sprintf(cstr, "%s Equalizer Left Channel", g_prognametitle);
		else
			sprintf(cstr, "%s Equalizer", g_prognametitle);
		WindowSetTitle(WINDOWEQ, cstr);
	}
	
	if(g_w[WINDOWEQ2])
	{
		if(g_sets.eqflags & MASEPARATELR)
			sprintf(cstr, "%s Equalizer Right Channel", g_prognametitle);
		else
			sprintf(cstr, "%s Equalizer Right Channel (Off)", g_prognametitle);
		WindowSetTitle(WINDOWEQ2, cstr);
	}
	
} // OnCheckSeparateLR


void SetAppWindowTitle(char *msg)
{
	char name[256];
	char cstr[512];
	char *p;
	
	if(g_songlist[0])
	{
		FilenameFromPathname(g_songlist, name);
		if((p = strstr(name, g_extstr[kMAMAF])))
			*p = 0;
	}
	else
		name[0] = 0;
		
	sprintf(cstr, "%s %s  %s", g_prognametitle, name, msg);
	
	WindowSetTitle(WINDOWAPP, cstr);

} // SetAppWindowTitle


void SetAppWindowMsg(char *msg) // SetAppWindowTitle without argv0
{
	char name[256];
	char cstr[512];
	char *p;
	
	if(g_songlist[0])
	{
		FilenameFromPathname(g_songlist, name);
		if((p = strstr(name, g_extstr[kMAMAF])))
			*p = 0;
	}
	else
		name[0] = 0;
		
	sprintf(cstr, "%s  %s", name, msg);
	
	WindowSetTitle(WINDOWAPP, cstr);

} // SetAppWindowMsg


void SetListWindowTitle(char *msg)
{
	char cstr[512];
	
	sprintf(cstr, "%s %s", g_prognametitle, msg);
	
	WindowSetTitle(WINDOWLIST, cstr);

} // SetListWindowTitle


void Init_g_k()
{
	int i;

	for(i = 0; i < kOUTMAX; i++)
		g_k[i] = 0;
	
	// init indices for g_sets.fileout[g_k[]]

	i = 1; // kOUTFIRST to kOUTLAST

 #ifdef ENCODEMP3
	g_k[i++] = 1;
 #endif
 
#ifdef ENCODEOGG
	g_k[i++] = 2;
#endif
 
 #ifdef ENCODEFLAC
	g_k[i++] = 3;
 #endif
 
 #ifdef ENCODEAAC
	g_k[i++] = 4;
 #endif
 
 #ifdef ENCODEMP4
	g_k[i++] = 5;
 #endif
 
 #ifdef ENCODEWMA
	g_k[i++] = 6;
 #endif

 #ifdef ENCODEWAV
	g_k[i++] = 7;
 #endif
 
 #ifdef ENCODEAIF
	g_k[i++] = 8;
 #endif
 
} // void Init_g_k()


void CheckFileoutSettings()
{
 #if(defined(ENCODEMP3) || defined(ENCODEOGG) || defined(ENCODEFLAC) || defined(ENCODEAAC) \
  || defined(ENCODEMP4) || defined(ENCODEWMA) || defined(ENCODEWAV)  || defined(ENCODEAIF))
	int i = 1; // kOUTFIRST
 #endif

 #ifdef ENCODEMP3
	if(g_sets.fileout[g_k[i]].type != kMAMP3)
		LoadFactoryFileout(i);
	i++;
 #endif
 
#ifdef ENCODEOGG
	if(g_sets.fileout[g_k[i]].type != kMAOGG)
		LoadFactoryFileout(i);
	i++;
 #endif
 
 #ifdef ENCODEFLAC
	if(g_sets.fileout[g_k[i]].type != kMAFLAC)
		LoadFactoryFileout(i);
	i++;
 #endif
 
 #ifdef ENCODEAAC
	if(g_sets.fileout[g_k[i]].type != kMAAAC)
		LoadFactoryFileout(i);
	i++;
 #endif
 
 #ifdef ENCODEMP4
	if(g_sets.fileout[g_k[i]].type != kMAMP4)
		LoadFactoryFileout(i);
	i++;
 #endif
 
 #ifdef ENCODEWMA
	if(g_sets.fileout[g_k[i]].type != kMAWMA)
		LoadFactoryFileout(i);
	i++;
 #endif

 #ifdef ENCODEWAV
	if(g_sets.fileout[g_k[i]].type != kMAWAV)
		LoadFactoryFileout(i);
	i++;
 #endif
 
 #ifdef ENCODEAIF
	if(g_sets.fileout[g_k[i]].type != kMAAIF)
		LoadFactoryFileout(i);
	i++;
 #endif
 
} // CheckFileoutSettings


int CheckSettings(MASETS *psets)
{
	int i, j;
	int mx; // e.g. 1024
	int my; // e.g.  768

	DisplayGetSize(&mx, &my);
	
	if(psets->aboutxpos < -20) psets->aboutxpos = ABOUTXPOSDEF;
	if(psets->aboutypos < -20) psets->aboutypos = ABOUTYPOSDEF;
	if(psets->viewerxpos < -20) psets->viewerxpos = VIEWERXPOSDEF;
	if(psets->viewerypos < -20) psets->viewerypos = VIEWERYPOSDEF;
	if(psets->keybindingsxpos < -20) psets->keybindingsxpos = KEYBINDINGSXPOSDEF;
	if(psets->keybindingsypos < -20) psets->keybindingsypos = KEYBINDINGSYPOSDEF;
	if(psets->advancedxpos < -20) psets->advancedxpos = ADVANCEDXPOSDEF;
	if(psets->advancedypos < -20) psets->advancedypos = ADVANCEDYPOSDEF;
	
	for(i = 0; i < NUMWINCONF; i++)
	{
		if(psets->xpos[i]      < -20)   psets->xpos[i]		= i == 0 ? XPOSDEF : XPOSDEF2;
		if(psets->ypos[i] < 0 /*-20*/)	psets->ypos[i]		= i == 0 ? YPOSDEF : YPOSDEF2;
		if(psets->eqxpos[i]	   < -20)   psets->eqxpos[i]	= i == 0 ? EQXPOSDEF : EQXPOSDEF2;
		if(psets->eqypos[i]	   < -20)   psets->eqypos[i]	= i == 0 ? EQYPOSDEF : EQYPOSDEF2;
		if(psets->listxpos[i]  < -20)	psets->listxpos[i]  = i == 0 ? LISTXPOSDEF : LISTXPOSDEF2;
		if(psets->listypos[i]  < -20)	psets->listypos[i]  = i == 0 ? LISTYPOSDEF : LISTYPOSDEF2;
	
		if(psets->xsize[i]    < 20)	psets->xsize[i]		= i == 0 ? XSIZEDEF : XSIZEDEF2;
		if(psets->ysize[i]    < 20)	psets->ysize[i]		= i == 0 ? YSIZEDEF : YSIZEDEF2;
		if(psets->eqxsize[i]   < 0)	psets->eqxsize[i]   = i == 0 ? EQXSIZEDEF : EQXSIZEDEF2;
		if(psets->eqysize[i]   < 0)	psets->eqysize[i]   = i == 0 ? EQYSIZEDEF : EQYSIZEDEF2;
		if(psets->eq2xsize[i]  < 0)	psets->eq2xsize[i]  = i == 0 ? EQ2XSIZEDEF : EQ2XSIZEDEF2;
		if(psets->eq2ysize[i]  < 0)	psets->eq2ysize[i]  = i == 0 ? EQ2YSIZEDEF : EQ2YSIZEDEF2;
		if(psets->listxsize[i] < 0)	psets->listxsize[i] = i == 0 ? LISTXSIZEDEF : LISTXSIZEDEF2;
		if(psets->listysize[i] < 0)	psets->listysize[i] = i == 0 ? LISTYSIZEDEF : LISTYSIZEDEF2;

		if(psets->xpos[i]		> mx)							 psets->xpos[i]		 = i == 0 ? XPOSDEF : XPOSDEF2;
		if(psets->ypos[i]		> my)							 psets->ypos[i]		 = i == 0 ? YPOSDEF : YPOSDEF2;
		if(psets->xsize[i]		>= abs(mx - psets->xpos[i]))	 psets->xsize[i]	 = i == 0 ? XSIZEDEF : XSIZEDEF2;
		if(psets->ysize[i]		>= abs(my - psets->ypos[i]))	 psets->ysize[i]	 = i == 0 ? YSIZEDEF : YSIZEDEF2;
		if(psets->eqxsize[i]	>= abs(mx - psets->eqxpos[i]))   psets->eqxsize[i]   = i == 0 ? EQXSIZEDEF : EQXSIZEDEF2;
		if(psets->eqysize[i]	>= abs(my - psets->eqypos[i]))   psets->eqysize[i]   = i == 0 ? EQYSIZEDEF : EQYSIZEDEF2;
		if(psets->eq2xsize[i]	>= abs(mx - psets->eq2xpos[i]))  psets->eq2xsize[i] = i == 0 ? EQ2XSIZEDEF : EQ2XSIZEDEF2;
		if(psets->eq2ysize[i]	>= abs(my - psets->eq2ypos[i]))  psets->eq2ysize[i] = i == 0 ? EQ2YSIZEDEF : EQ2YSIZEDEF2;
		if(psets->listxsize[i]  >  abs(mx - psets->listxpos[i])) psets->listxsize[i] = i == 0 ? LISTXSIZEDEF : LISTXSIZEDEF2;
		if(psets->listysize[i]  >= abs(my - psets->listypos[i])) psets->listysize[i] = i == 0 ? LISTYSIZEDEF : LISTYSIZEDEF2;

		for(j = 0; j < 4; j++)
			if(psets->listcolxsize[i][j] < psets->listxsize[i] - SCROLLERWIDTH)
				psets->listcolxsize[i][j] = psets->listxsize[i] - SCROLLERWIDTH;
		
	} // for i

	// check parameter sanity
	if(psets->balance)
	{
	 if(psets->volumesliderleft  < (VOLUMEMIN*2) || psets->volumesliderleft  > (VOLUMEMAX*2))	psets->volumesliderleft	 = (VOLUMEMAX - VOLUMEMIN) / 2;
	 if(psets->volumesliderright < (VOLUMEMIN*2) || psets->volumesliderright > (VOLUMEMAX*2))	psets->volumesliderright = (VOLUMEMAX - VOLUMEMIN) / 2;
	}
	else
	{
	 if(psets->volumesliderleft  < VOLUMEMIN	|| psets->volumesliderleft	>  VOLUMEMAX)		psets->volumesliderleft	 = (VOLUMEMAX - VOLUMEMIN) / 2;
	 if(psets->volumesliderright < VOLUMEMIN	|| psets->volumesliderright	>  VOLUMEMAX)		psets->volumesliderright = (VOLUMEMAX - VOLUMEMIN) / 2;
	}
	if(psets->separationslider  < SEPARATIONMIN	|| psets->separationslider  >  SEPARATIONMAX)	psets->separationslider	= (SEPARATIONMAX - SEPARATIONMIN) / 2;
	if(psets->panslider			< SEPARATIONMIN	|| psets->panslider			>  SEPARATIONMAX)	psets->panslider		= (SEPARATIONMAX - SEPARATIONMIN) / 2;
	if(psets->preampslider		< PREAMPMIN		|| psets->preampslider		>  PREAMPMAX)		psets->preampslider		= (PREAMPMAX - PREAMPMIN) / 2;
	if(psets->buffersize		< 0				|| psets->buffersize		>  SONGMEMMAX)		psets->buffersize		= SONGMEMDEF;
	if(psets->buffertype		< 0				|| psets->buffertype		>= BUFUSEMAX)		psets->buffertype		= BUFUSEMEM;
	if(psets->output			< 0				|| psets->output			>  kOUTLAST)		psets->output			= kOUTSOUND;
	if(psets->eqpreset			< 0				|| psets->eqpreset			>= EQNUMPRESETS)	psets->eqpreset			= 0;

	for(i = 0; i < EQNUMPRESETS; i++)
	{
		if(psets->eqpresets[i].eqoffset > DBRANGE || psets->eqpresets[i].eqoffset < -DBRANGE)
			psets->eqpresets[i].eqoffset = 0;
		if(psets->eq2presets[i].eqoffset > DBRANGE || psets->eq2presets[i].eqoffset < -DBRANGE)
			psets->eq2presets[i].eqoffset = 0;
			
		for(j = 0; j < EQBANDS31; j++)
		{
			if(psets->eqpresets[i].eqslider[j]  < DBSLIDERMIN || psets->eqpresets[i].eqslider[j]  > DBSLIDERMAX)
				psets->eqpresets[i].eqslider[j] = (DBSLIDERMAX - DBSLIDERMIN) / 2;
			if(psets->eq2presets[i].eqslider[j] < DBSLIDERMIN || psets->eq2presets[i].eqslider[j] > DBSLIDERMAX)
				psets->eq2presets[i].eqslider[j] = (DBSLIDERMAX - DBSLIDERMIN) / 2;
		}
	}
	
#if defined(MAKEYBOARD) && defined(MAGUINONE)
	psets->keyboard = 1;
#endif

	CheckFileoutSettings();
	
	return 0;

} // CheckSettings()


// for updating settings from a previous settings version
/*
 #define MOREAMP_PREVIOUS_VERSION_STRING "0.1.22" // settings version that needs updating to this settings version
 #define MOREAMP_PREVIOUS_VERSION_LEN 6
 #define PREVIOUSMASETS MASETS_022
 #define FixSettingsPrevious FixSettings_02x
 void FixSettings_02x(FILE *fp);
 void FixSettings_02x(FILE *fp)
 {
	PREVIOUSMASETS prev_sets;
	fseek(fp, 0L - sizeof(PREVIOUSMASETS), SEEK_END);	
	fread(&prev_sets, 1, sizeof(PREVIOUSMASETS), fp);
	// copy/fix prev_sets to g_sets
	memcpy(&g_sets, &prev_sets, sizeof(PREVIOUSMASETS));
	// ...
 } // FixSettings_02x
*/
 #define MOREAMP_PREVIOUS_VERSION_STRING "0.1.22" // settings version that needs updating to this settings version
 #define MOREAMP_PREVIOUS_VERSION_LEN 6
 #define PREVIOUSMASETS MASETS_022
 #define FixSettingsPrevious FixSettings_023
 void FixSettings_023(FILE *fp, MASETS *psets);
 typedef struct
 {
	int autostart;					// start player when app is launched
	int spindown;					// spin down the hard disk
	int random;						// get songs from songlist randomly
	int reptplayed;					// allow to repeat a song when in random mode 
	int looploaded;					// loop on songs that have been loaded in mem or vdisk
	int looplist;					// loop on song list else stop when all have played 
	int highpriority;				// decoder task riority
	int effects; 					// effects active
	int limiter; 					// mad limiter
	int balance;					// free or joined left and right volume controls
	int monodiff;					// L-R in mono; reverse L,R in stereo
	int invertright;				// invert right channel
	int eqflags;					// equalizer flags
	int eqpreset;					// eq preset index
	int listwindow;					// show list window | which to show
	int effectswindow;				// show effects window
	int fileoutwindow;				// show file output window
	int volumesliderleft; 			// volume L value
	int volumesliderright; 			// volume R value
	int separationslider; 			// separation value (stereo) 
	int panslider;					// pan value (mono)
	int preampslider;   			// preamp value (mad)
	EQUPRESET eqpresets[EQNUMPRESETS];  // equalizer values
	int pitchslider;				// pitch effect value
	int pitchslider2;				// pitch effect value (fine)
	int temposlider;				// tempo effect value
	int speedslider;				// speed effect value
	int xpos[NUMWINCONF];   		// player window coordinates
	int ypos[NUMWINCONF];
	int xsize[NUMWINCONF];
	int ysize[NUMWINCONF];
	int eqxpos[NUMWINCONF]; 		// eq window coordinates
	int eqypos[NUMWINCONF];
	int eqxsize[NUMWINCONF];
	int eqysize[NUMWINCONF];
	int effectsxpos[NUMWINCONF];	// effects window coordinates
	int effectsypos[NUMWINCONF];
	int effectsxsize[NUMWINCONF];
	int effectsysize[NUMWINCONF];
	int listxpos[NUMWINCONF];		// list window coordinates
	int listypos[NUMWINCONF];
	int listxsize[NUMWINCONF];
	int listysize[NUMWINCONF];
	char vdiskname[MAX_PATH]; 		// virtual disk name
	int buffertype;					// type of buffering for songs (mem, vdisk, direct)
	int output;						// output to use from Output popup
	int buffersize;					// max mem or vdisk used, in Mbytes
	int titleinfo;					// put info in window's title bar
	int winconf;					// player window configuration
	int singleplay;					// stop afer every song
	FILEOUT fileout[kOUTMAX];		// file output parameters	
	int logflags;					// stop or limit logging
	int logmax;						// max log entries per session
	int logevents;					// max log entries per fast log period
	int logperiod;					// fast log period
	int multipleinst;				// multiple instances
	int keyboard;					// use keyboard
	int listcolxsize[NUMWINCONF][4];// h scrollable if wider than list window 
	int aboutxpos;					// about window coordinates
	int aboutypos;
	int aboutxsize;
	int aboutysize;
	int viewerxpos;					// viewer window coordinates
	int viewerypos;
	int viewerxsize;
	int viewerysize;
	int keybindingsxpos;			// keybindings window coordinates
	int keybindingsypos;
	int keybindingsxsize;
	int keybindingsysize;
	int fileoutxpos[NUMWINCONF];	// fileout window coordinates
	int fileoutypos[NUMWINCONF];
	int fileoutxsize[NUMWINCONF];
	int fileoutysize[NUMWINCONF];
	int advancedxpos;				// advanced window coordinates
	int advancedypos;
	int advancedxsize;
	int advancedysize;
	int mousewheel;					// use mousewheel
	int usem3u;						// use m3u files
	int future1;					// for future use
	int future2;
	int future3;
	int future4;
	char settingsversion[VERSIONSTRINGSIZE]; // first version to use this settings format, keep this here (last)
 } MASETS_022;
 void FixSettings_023(FILE *fp, MASETS *psets)
 {
	PREVIOUSMASETS prev_sets;
	fseek(fp, 0L - sizeof(PREVIOUSMASETS), SEEK_END);	
	fread(&prev_sets, 1, sizeof(PREVIOUSMASETS), fp);
	// copy/fix prev_sets to psets
	memcpy(psets, &prev_sets, sizeof(PREVIOUSMASETS));
	psets->addmode = ADDMODEDEF;
	psets->eq2xpos[0] = psets->eqxpos[0] + EQ2POSDELTA;
	psets->eq2ypos[0] = psets->eqypos[0] + EQ2POSDELTA;;
	psets->eq2xsize[0] = psets->eqxsize[0];
	psets->eq2ysize[0] = psets->eqysize[0];
	psets->eq2xpos[1] = psets->eqxpos[1] + EQ2POSDELTA;
	psets->eq2ypos[1] = psets->eqypos[1] + EQ2POSDELTA;;
	psets->eq2xsize[1] = psets->eqxsize[1];
	psets->eq2ysize[1] = psets->eqysize[1];
	memcpy(&psets->eq2presets, &prev_sets.eqpresets, sizeof(psets->eq2presets));
	psets->future1 = 0;
	psets->future2 = 0;
	psets->future3 = 0;
	psets->future4 = 0;
	strcpy(psets->settingsversion, MOREAMP_SETTINGS_VERSION_STRING);
	// FixDefaultSettings_022();
 } // FixSettings_023


int maCheckSettingsVersion(char *settingsversion)
{
	// currrent settings version
	if(!strncmp(settingsversion, MOREAMP_SETTINGS_VERSION_STRING, (size_t)MOREAMP_SETTINGS_VERSION_LEN))
		return MAVERSIONISCURRENT; // current settings don't change

#ifdef MOREAMP_PREVIOUS_VERSION_STRING
	// previous settings version
	if(!strncmp(settingsversion, MOREAMP_PREVIOUS_VERSION_STRING, (size_t)MOREAMP_PREVIOUS_VERSION_LEN))
		return MAVERSIONISPREVIOUS; // current settings will be copied from previous version
#endif

	// any other unknown (older or newer) settings version
	return MAVERSIONISUNKNOWN; // current settings will be set to factory defaults
}


int LoadSettings()  // load settings from a settings file, recreate one if none exist or if error
{					// if dropping songs try to keep previous settings even though g_songsnum may have changed
	int i;
	int ret = 0;
	int ret2 = 0;
	int isdrop = 0;
	int version;
	char settingsversion[VERSIONSTRINGSIZE];
	char cstr[256];
	long s;
	FILE *fp;
		
	if(g_songsnum <= 0 && !SongListIsCda())
	{
		sprintf(cstr, "LoadSettings cannot load settings");
		maLog(cstr);
		return 1;	
	}
	
	l_songsnumplayed = 0;
	
	if(!g_settingspath[0])
	{
		maLog("LoadSettings cannot load settings, no file name");
		return 2;	
	}
			
	if(AllocSongRec(&l_songrec) && !SongListIsCda())
		return 3;
	
	if(SongListIsCda())
	{
		if(!mastrcasecmp(g_settingspath, l_settingscda))
			isdrop = 1;
	}
	else
	{
		if(!mastrcasecmp(g_settingspath, l_settingsdrop))
			isdrop = 1;
	}
	
	if(isdrop && !strcmp(l_settingspathlast, g_settingspath)) // don't reread settings
	{
		ret = g_songsnum;
		ret2 = sizeof(MASETS);
	}
	else if((fp = mafopen(g_settingspath, "rb")) != NULL) // read song records and settings
	{
		// check version
		fseek(fp, 0L - sizeof(g_sets.settingsversion), SEEK_END);
		fread(settingsversion, 1, sizeof(g_sets.settingsversion), fp);
		if((version = maCheckSettingsVersion(settingsversion)) == MAVERSIONISCURRENT)
		{
			// check size, don't care for dropped songs
			if(!isdrop)
			{
				fseek(fp, 0, SEEK_END);
				if(ftell(fp) != (long)(sizeof(SONGRECORD) * g_songsnum + sizeof(MASETS)))
					ret = -1; // bad size
			}
			if(ret != -1) // size ok or don't care
			{
				// get song records
				rewind(fp);	
				ret = fread(l_songrec, sizeof(SONGRECORD), g_songsnum, fp);

				// get settings
				s = sizeof(MASETS);
				fseek(fp, -s, SEEK_END);
				ret2 = fread(&g_sets, 1, sizeof(MASETS), fp);
				memcpy(&g_setsbu, &g_sets, sizeof(MASETS));
				strcpy(l_settingspathlast, g_settingspath);
			}
		}
#ifdef MOREAMP_PREVIOUS_VERSION_STRING
		else if(version == MAVERSIONISPREVIOUS)
		{
			// check size, don't care for dropped songs
			if(!isdrop)
			{
				fseek(fp, 0, SEEK_END);
				if(ftell(fp) != (long)(sizeof(SONGRECORD) * g_songsnum + sizeof(PREVIOUSMASETS)))
					ret = -1; // bad size
			}
			if(ret != -1) // size ok or don't care
			{
				char cstr2[256];
				// get song records
				rewind(fp);	
				ret = fread(l_songrec, sizeof(SONGRECORD), g_songsnum, fp);
					
				FixSettingsPrevious(fp, &g_sets);

				ret2 = sizeof(MASETS);
				memcpy(&g_setsbu, &g_sets, sizeof(MASETS));
				strcpy(l_settingspathlast, g_settingspath);
				fclose(fp);
				fp = NULL;
				SaveSettings();
				if(FilenameFromPathname(g_settingspath, cstr2))
				{
					sprintf(cstr, "LoadSettings FilenameFromPathname failed on '%s'", g_settingspath);
					maLog(cstr);
					strcpy(cstr2, g_settingspath);
				}
				sprintf(cstr, "LoadSettings updated settings '%s' from settings version %s to %s",
											cstr2, settingsversion, MOREAMP_SETTINGS_VERSION_STRING);
				maLog(cstr);
			}
			else // MAVERSIONISUNKNOWN
			{
				sprintf(cstr, "LoadSettings unable to determine if previous settings version is %s",
																	MOREAMP_SETTINGS_VERSION_STRING);
				maLog(cstr);
			}
		}
#endif
		if(fp)
			fclose(fp);		
	}
	else
		ret = ret2 = -2; // cannot open settings file

	// error or wrong size, make a new settings file :	
	if(ret != g_songsnum || ret2 != sizeof(MASETS))
	{			
		// if dropping songs try to keep previous settings
		if(ret2 != sizeof(MASETS))
		{
#if !defined(NODECODEAUDIOIN)
			if(!mastrcasecmp(g_settingspath, l_settingsaudioin))
			{
				if(LoadFactorySettingsAudioIn(&g_sets)) // get audioin factory settings
					return 4;
			}
			else
#endif
			{
				if(LoadDefaultSettings(&g_sets)) // get settings from defaults
					return 4;
			}
		}
					
		if(l_songrec)
			if(ZeroSongRec(&l_songrec))
				return 5;
		
		memcpy(&g_setsbu, &g_sets, sizeof(MASETS));

		if(SaveSettings())
			return 6;

		strcpy(l_settingspathlast, g_settingspath);
		
	} // if error or wrong size
	
	// count number of songs played so far
	l_songsnumplayed = 0;
	for(i = 0; i < g_songsnum; i++)
	{
		if(isdrop)
			l_songrec[i] = 0;
		else
		{
			if(l_songrec[i])
				l_songsnumplayed++;
		}
	}
	
	CheckSettings(&g_sets);

#if defined(MAINSTANCES)
	if(g_thisinstance == 1)
		if(ret2 == sizeof(MASETS)) 
			SetMultipleInstances();
		// else was done in LoadDefaultSettings or LoadFactorySettings
#endif

	return 0;

} // LoadSettings


int SaveSettings()
{
	int ret;
	char cstr[256];
	FILE *fp;
	
	if(g_songsnum <= 0 && !SongListIsCda())
	{
		sprintf(cstr, "SaveSettings failed");
		maLog(cstr);
		return 1;
	}

	if(l_songrec == NULL && !SongListIsCda())
	{
		sprintf(cstr, "SaveSettings failed, NULL l_songrec pointer");
		maLog(cstr);
		return 2;
	}
		
	if((fp = mafopen(g_settingspath, "wb")) == NULL)
	{
		sprintf(cstr, "SaveSettings mafopen failed on '%s'", g_settingspath);
		maLog(cstr);
		return 3;
	}
	
#if defined(__MAOSX__) || defined(__MAUNIX__)
	machmod(g_settingspath, ACCESSPERMS); // 0777
#endif

	// song record		
	if(l_songrec && g_songsnum > 0)
	{
		ret = fwrite(l_songrec, sizeof(SONGRECORD), g_songsnum, fp);
	
		if(ret != g_songsnum)
		{
			sprintf(cstr, "SaveSettings fwrite %d records instead of %d in '%s'", ret, g_songsnum, g_settingspath);
			maLog(cstr);
			fclose(fp);
			return 4;
		}
	}

	strcpy(g_sets.settingsversion, MOREAMP_SETTINGS_VERSION_STRING);
	strcpy(g_setsbu.settingsversion, MOREAMP_SETTINGS_VERSION_STRING);

	ret = fwrite((char *)&g_setsbu, 1, sizeof(MASETS), fp);
		
	fclose(fp);
	
	if(ret != sizeof(MASETS))
	{
		sprintf(cstr, "SaveSettings blocks=%d fwrite '%s'", ret, g_settingspath);
		maLog(cstr);	
		return 5;
	}
	
	return 0;
	
} // SaveSettings


#if !defined(MANOCDA) && defined(__MAUNIX__)
int LoadSettingsCDRom()
{
	int ret = 0;
	FILE *fp;
	char cstr[256];
		
	if(!l_setscdrompath[0])
	{
		maLog("LoadSettingsCDRom cannot load settings, no file name");
		return 1;	
	}
			
	if((fp = mafopen(l_setscdrompath, "rb")) == NULL)
	{
		if(errno != ENOENT) // 2
		{
			sprintf(cstr, "LoadSettingsCDRom mafopen errno=%d '%s'", errno, l_setscdrompath);
			maLog(cstr);
		}
		maLog("LoadSettingsCDRom setting factory cdrom settings");
		LoadFactorySettingsCDRom();
		SaveSettingsCDRom();
		return 1;
	}

	if((ret = fread(&g_setscdrom, 1, sizeof(MASETSCDROM), fp)) != sizeof(MASETSCDROM))
	{			
		fclose(fp);
		sprintf(cstr, "LoadSettingsCDRom fread errno=%d '%s'", errno, l_setscdrompath);
		maLog(cstr);
		maLog("LoadSettingsCDRom setting factory cdrom settings");
		LoadFactorySettingsCDRom();
		SaveSettingsCDRom();
		return 1;
	}
	
	fclose(fp);

	return 0;
	
} // LoadSettingsCDRom


int SaveSettingsCDRom()
{
	int ret;
	char cstr[256];
	FILE *fp;
	
	if((fp = mafopen(l_setscdrompath, "wb")) == NULL)
	{
		sprintf(cstr, "SaveSettingsCDRom mafopen failed on '%s'", l_setscdrompath);
		maLog(cstr);
		return 6;
	}
	machmod(l_setscdrompath, ACCESSPERMS); // 0777
	ret = fwrite((char *)&g_setscdrom, 1, sizeof(MASETSCDROM), fp);
	fclose(fp);
	if(ret != sizeof(MASETSCDROM))
	{
		sprintf(cstr, "SaveSettingsCDRom blocks=%d fwrite '%s'", ret, l_setscdrompath);
		maLog(cstr);	
		return 7;
	}

	return 0;
	
} // SaveSettingsCDRom
#endif


 void OnEditTextPromptNum() // response from the dialog text field (Enter)
 {
	int i;
	long outlen;
	char cstr[MAPROMPTMAX + 2];

	if (!g_w[WINDOWPROMPTNUM])
		return;
	
	EditTextGetText(WINDOWPROMPTNUM, ID_EDITTEXT_PROMPTNUM, cstr, MAPROMPTMAX, &outlen);
	
	if(strlen(cstr) > 0)
	{
		if(!string2int(cstr, &i))
		{
			if(i >= 1 && i <= g_songsnum) // 1-based index of song to play
			{
				l_droplaunch = i + 1; // 0-based row to play + 2

				if(g_running) // exit song loop...
				{
					g_loadedcurrent = g_loadednum = 0;
					l_loadedquit = 1;
				}
				else
					DoTimerPlay(100);
			}
		}
	}
	
	WindowDoClose(WINDOWPROMPTNUM);
	
 } // OnEditTextPromptNum


#if defined(MAGUIOSX) || defined(MAGUIWX) || defined(MAGUIGTK)
 void OnButtonPromptNum(int ok) // response from the dialog buttons
 {
	if(ok)
		OnEditTextPromptNum(); // Ok
	else
		WindowDoClose(WINDOWPROMPTNUM); // Cancel	
 }
#endif


#if defined(MAKEYBOARD)
 // #define MAKEYBOARDTRACE

 void ShowKeyStatus(char *status)
 {
	char cstr[256];
	sprintf(cstr, "key : %s", status);
	WindowSetTitle(WINDOWAPP, cstr);
 #if !defined(MAGUINONE)
	l_temptitle = 1;
 #endif
 }


 void OnKeyButtonQuit()
 {
	ShowKeyStatus((char *)"Quit");
	OnButtonQuit();
 }


 void OnKeyButtonStop()
 {
	ShowKeyStatus((char *)"Stop");
	OnButtonStop();
 }


 void OnKeyButtonPause()
 {
	OnButtonPause();
	if(l_ispaused)
		ShowKeyStatus((char *)"Pause on");
	else
		ShowKeyStatus((char *)"Pause off");
 }


 void OnKeyButtonPrevious()
 {
	ShowKeyStatus((char *)"Previous");
	OnButtonPrevious();
 }


 void OnKeyButtonPlay()
 {
	ShowKeyStatus((char *)"Play");
	OnButtonPlay();
 }


 void OnKeyButtonNext()
 {
	ShowKeyStatus((char *)"Next");
	OnButtonNext();
 }


 void OnKeyButtonEqRestore()
 {
	ShowKeyStatus((char *)"Eq reset");
	OnButtonEqRestore();
 }


 void OnKeyButtonEqInvert()
 {
	ShowKeyStatus((char *)"Eq invert");
	OnButtonEqInvert();
 }


 void OnKeyPopupEqPreset1()
 {
	ShowKeyStatus((char *)"Eq Preset 1");
	OnPopupEqPreset(0);
 }


 void OnKeyPopupEqPreset2()
 {
	ShowKeyStatus((char *)"Eq Preset 2");
	OnPopupEqPreset(1);
 }


 void OnKeyPopupEqPreset3()
 {
	ShowKeyStatus((char *)"Eq Preset 3");
	OnPopupEqPreset(2);
 }


 void OnKeyPopupEqPreset4()
 {
	ShowKeyStatus((char *)"Eq Preset 4");
	OnPopupEqPreset(3);
 }


 void OnKeyResetSliderPitch()
 {
	ShowKeyStatus((char *)"Pitch effect reset");
	if(g_w[WINDOWEFFECTS])
		SliderSetValue(WINDOWEFFECTS, ID_SLIDER_PITCH, g_setsbu.pitchslider);
	OnSliderPitch(g_setsbu.pitchslider);
	if(g_w[WINDOWEFFECTS])
		SliderSetValue(WINDOWEFFECTS, ID_SLIDER_PITCH2, g_setsbu.pitchslider2);
	OnSliderPitch2(g_setsbu.pitchslider2);
 }


 void OnKeyResetSliderTempo()
 {
	ShowKeyStatus((char *)"Tempo effect reset");
	if(g_w[WINDOWEFFECTS])
		SliderSetValue(WINDOWEFFECTS, ID_SLIDER_TEMPO, g_setsbu.temposlider);
	OnSliderTempo(g_setsbu.temposlider);
 }


 void OnKeyResetSliderSpeed()
 {
	ShowKeyStatus((char *)"Speed effect reset");
	if(g_w[WINDOWEFFECTS])
		SliderSetValue(WINDOWEFFECTS, ID_SLIDER_SPEED, g_setsbu.speedslider);
	OnSliderSpeed(g_setsbu.speedslider);
 }


 void OnKeyPitchUp()
 {
	ShowKeyStatus((char *)"Pitch semitone up");
	OnLittleArrowPitch(1);
 }


 void OnKeyPitch2Up()
 {
	ShowKeyStatus((char *)"Pitch up");
	OnLittleArrowPitch2(1);
 }


 void OnKeyTempoUp()
 {
	ShowKeyStatus((char *)"Tempo up");
	OnLittleArrowTempo(1);
 }


 void OnKeySpeedUp()
 {
	ShowKeyStatus((char *)"Speed up");
	OnLittleArrowSpeed(1);
 }


 void OnKeyPitchDown()
 {
	ShowKeyStatus((char *)"Pitch semitone down");
	OnLittleArrowPitch(-1);
 }


 void OnKeyPitch2Down()
 {
	ShowKeyStatus((char *)"Pitch down");
	OnLittleArrowPitch2(-1);
 }


 void OnKeyTempoDown()
 {
	ShowKeyStatus((char *)"Tempo down");
	OnLittleArrowTempo(-1);
 }


 void OnKeySpeedDown()
 {
	ShowKeyStatus((char *)"Speed down");
	OnLittleArrowSpeed(-1);
 }


 void OnKeyOpenList()
 {
 #if !defined(MAGUINONE)
	if(g_running)
		ShowKeyStatus((char *)"OpenList... Stop first");
	else
	{
		ShowKeyStatus((char *)"OpenList");
		OnMenuOpenList();
	}
 #endif
 }


 void OnKeyMakeList()
 {
 #if !defined(MAGUINONE)
	if(g_running)
		ShowKeyStatus((char *)"MakeList... Stop first");
	else
	{
		ShowKeyStatus((char *)"MakeList");
		OnMenuMakeList();
	}
 #endif
 }


 void OnKeySaveListAs()
 {
 #if !defined(MAGUINONE)
	if(g_running)
		ShowKeyStatus((char *)"SaveListAs... Stop first");
	else
	{
		ShowKeyStatus((char *)"SaveListAs");
		OnMenuSaveListAs();
	}
 #endif
 }



 void OnKeyMenuZero()
 {
	ShowKeyStatus((char *)"ZeroList");
	OnMenuZero();
 }


 void OnKeyPlayFile()
 {
 #if !defined(MAGUINONE)
	if(g_running)
		ShowKeyStatus((char *)"PlayFile... Stop first");
	else
	{
		ShowKeyStatus((char *)"PlayFile");
		OnMenuPlayFile();
	}
 #endif
 }


 void OnKeyPlayCD()
 {
	if(g_running)
		ShowKeyStatus((char *)"PlayCD... Stop first");
	else
	{
		ShowKeyStatus((char *)"PlayCD");
		OnMenuPlayCD();
	}
 }


 void OnKeyEjectCD()
 {
	if(g_running)
		ShowKeyStatus((char *)"EjectCD... Stop first");
	else
	{
		ShowKeyStatus((char *)"EjectCD");
		OnMenuEjectCD();
	}
 }


 void OnKeyViewAbout()
 {
 #if !defined(MAGUINONE)
	if(g_w[WINDOWABOUT])
	{
		ShowKeyStatus((char *)"About window off");
		WindowDoClose(WINDOWABOUT);
		g_w[WINDOWABOUT] = NULL;
	}
	else
	{
		ShowKeyStatus((char *)"About window on");
		maViewAbout();
	}
 #endif
 }


 void OnKeyViewCatalog()
 {
 #if !defined(MAGUINONE)
	if(g_w[WINDOWCATALOG])
	{
		ShowKeyStatus((char *)"Catalog window off");
		WindowDoClose(WINDOWCATALOG);
		g_w[WINDOWCATALOG] = NULL;
	}
	else
	{
		ShowKeyStatus((char *)"Catalog window on");
		OnMenuViewCatalog();
	}
 #endif
 }


 void OnKeyViewLog()
 {
 #if !defined(MAGUINONE)
	if(g_w[WINDOWLOG])
	{
		ShowKeyStatus((char *)"Log window off");
		WindowDoClose(WINDOWLOG);
		g_w[WINDOWLOG] = NULL;
	}
	else
	{
		ShowKeyStatus((char *)"Log window on");
		OnMenuViewLog();
	}
 #endif
 }

 void OnKeyClearLog()
 {
	ShowKeyStatus((char *)"Clear log");
	OnMenuClearLog();
 }


 void OnKeyEditBindings()
 {
 #if defined(MAGUINONE)
	// dump the key bindings to stdout
	OnMenuEditKeyBindings();
 #else
	if(g_w[WINDOWKEYBINDINGS])
	{
		ShowKeyStatus((char *)"Key Bindings window off");
		WindowDoClose(WINDOWKEYBINDINGS);
		g_w[WINDOWKEYBINDINGS] = NULL;
	}
	else
	{
		ShowKeyStatus((char *)"Key Bindings window on");
		OnMenuEditKeyBindings();
	}
 #endif
 }


 void OnKeyViewReadme()
 {
 #if !defined(MAGUINONE)
	if(g_w[WINDOWREADME])
	{
		ShowKeyStatus((char *)"README window off");
		WindowDoClose(WINDOWREADME);
		g_w[WINDOWREADME] = NULL;
	}
	else
	{
		ShowKeyStatus((char *)"README window on");
		OnMenuReadme();
	}
 #endif
 }


 void OnKeyButtonInfo()
 {
	ShowKeyStatus((char *)"List playerinfo");
	OnButtonInfo();
 }


 void OnKeyButtonList()
 {
	ShowKeyStatus((char *)"List songlist");
	OnButtonList();
 }


 void OnKeyButtonLoaded()
 {
	ShowKeyStatus((char *)"List songs loaded");
	OnButtonLoaded();
 }


 void OnKeyButtonSong()
 {
	ShowKeyStatus((char *)"List song info");
	OnButtonSong();
 }


 void OnKeyUpArrow() // from kb shortcut 'i', not from kb arrow key
 {
	ShowKeyStatus((char *)"List scroll up");
	l_noshowrow = 1;
	maOnKey(WINDOWAPP, MAWXK_UP);
 }


 void OnKeyDownArrow() // from kb shortcut 'k', not from kb arrow key
 {
	ShowKeyStatus((char *)"List scroll down");
	l_noshowrow = 1;
	maOnKey(WINDOWAPP, MAWXK_DOWN);
 }


 void OnKeyShowFileOut()
 {
	if(MenuIsChecked(MENUSHOW, ID_MENU_SHOWFILEOUT))
	{
		ShowKeyStatus((char *)"FileOut window off");
		DoWindowFileOut(0);
	}
	else
	{
		if(WINDOWOUTCUR >= WINDOWOUTFIRST && WINDOWOUTCUR <= WINDOWOUTLAST)
		{
			ShowKeyStatus((char *)"FileOut window on");
			DoWindowFileOut(1);
		}
	}
 }


 void OnKeyEffectsWindow()
 {
	if(MenuIsChecked(MENUSHOW, ID_MENU_SHOWEFFECTS))
	{
		ShowKeyStatus((char *)"Effects window off");
		DoWindowEffects(0);
	}
	else
	{
		ShowKeyStatus((char *)"Effects window on");
		DoWindowEffects(1);
	}
 }


 void OnKeyAltSize()
 {
	ShowKeyStatus((char *)"Alt window sizes");
	OnMenuAltSize(g_sets.winconf ? 0 : 1, 1);
 }


 void OnKeyFactorySize()
 {
	ShowKeyStatus((char *)"Factory window sizes");
	OnMenuFactorySize(1);
 }


 void OnKeySavedSize()
 {
	ShowKeyStatus((char *)"Saved window sizes");
	OnMenuSavedSize(1);
 }


 void OnKeyDefaultSize()
 {
	ShowKeyStatus((char *)"Default window sizes");
	OnMenuDefaultSize(1);
 }


 void OnKeyCheckTitleInfo()
 {
	OnCheckTitleInfo(0);
 }


 void OnKeyCheckAltSize()
 {
	CheckBoxSetValue(WINDOWAPP, ID_CHK_ALTSIZE, g_sets.winconf ? 0 : 1);
	OnCheckAltSize(g_sets.winconf ? 0 : 1, 1);
 }


 void OnKeyCheckRandom()
 {
	if(g_sets.random)
		ShowKeyStatus((char *)"Random off");
	else
		ShowKeyStatus((char *)"Random on");
	CheckBoxSetValue(WINDOWAPP, ID_CHK_RANDOM, g_sets.random ? 0 : 1);
	OnCheckRandom(g_sets.random ? 0 : 1);
 }

 
 void OnKeyCheckReptPlayed()
 {
	if(g_sets.reptplayed)
		ShowKeyStatus((char *)"RepeatPlayed off");
	else
		ShowKeyStatus((char *)"RepeatPlayed on");
	CheckBoxSetValue(WINDOWAPP, ID_CHK_REPTPLAYED, g_sets.reptplayed ? 0 : 1);
	OnCheckReptPlayed(g_sets.reptplayed ? 0 : 1);
 }


 void OnKeyCheckLoopLoaded()
 {
	if(g_sets.looploaded)
		ShowKeyStatus((char *)"LoopLoaded off");
	else
		ShowKeyStatus((char *)"LoopLoaded on");
	CheckBoxSetValue(WINDOWAPP, ID_CHK_LOOPLOADED, g_sets.looploaded ? 0 : 1);
	OnCheckLoopLoaded(g_sets.looploaded ? 0 : 1);
 }


 void OnKeyCheckLoopList()
 {
	if(g_sets.looplist)
		ShowKeyStatus((char *)"LoopList off");
	else
		ShowKeyStatus((char *)"LoopList on");
	CheckBoxSetValue(WINDOWAPP, ID_CHK_LOOPLIST, g_sets.looplist ? 0 : 1);
	OnCheckLoopList(g_sets.looplist ? 0 : 1);
 }

 
 #ifdef __MAOS9__
 void OnKeyCheckSpindown()
 {
	if(g_sets.spindown)
		ShowKeyStatus((char *)"Spindown off");
	else
		ShowKeyStatus((char *)"Spindown on");
	CheckBoxSetValue(WINDOWAPP, ID_CHK_SPINDOWN, g_sets.spindown ? 0 : 1);
	OnCheckSpindown(g_sets.spindown ? 0 : 1);
 }
 #endif


 void OnKeyCheckHighPriority()
 {
	if(g_sets.highpriority)
		ShowKeyStatus((char *)"HighPriority off");
	else
		ShowKeyStatus((char *)"HighPriority on");
	CheckBoxSetValue(WINDOWADVANCED, ID_CHK_HIGHPRIORITY, g_sets.highpriority ? 0 : 1);
	OnCheckHighPriority(g_sets.highpriority ? 0 : 1);
 }

 
 void OnKeyCheckSinglePlay()
 {
	if(g_sets.singleplay)
		ShowKeyStatus((char *)"SinglePlay off");
	else
		ShowKeyStatus((char *)"SinglePlay on");
	CheckBoxSetValue(WINDOWAPP, ID_CHK_SINGLEPLAY, g_sets.singleplay ? 0 : 1);
	OnCheckSinglePlay(g_sets.singleplay ? 0 : 1);
 }


 void OnKeyCheckEqWindow()
 {
 #if !defined(MAGUINONE)
	if(g_sets.eqflags & MAEQWINDOW)
		ShowKeyStatus((char *)"Equalizer window off");
	else
		ShowKeyStatus((char *)"Equalizer window on");
	CheckBoxSetValue(WINDOWAPP, ID_CHK_EQWINDOW, g_sets.eqflags & MAEQWINDOW ? 0 : 1);
	OnCheckEqWindow(g_sets.eqflags & MAEQWINDOW ? 0 : 1);
 #endif
 }

 
 void OnKeyCheckListWindow()
 {
 #if defined(MAGUINONE)
	// dump the list to stdout
	ShowSongList();
 #else
	if(g_sets.listwindow & LBOXON)
		ShowKeyStatus((char *)"List window off");
	else
		ShowKeyStatus((char *)"List window on");
	CheckBoxSetValue(WINDOWAPP, ID_CHK_LISTWINDOW, g_sets.listwindow & LBOXON ? 0 : 1);
	OnCheckListWindow(g_sets.listwindow & LBOXON ? 0 : 1);
 #endif
 }

 
 void OnKeyCheckEqualizer()
 {
	if(g_sets.eqflags & MAEQACTIVE)
		ShowKeyStatus((char *)"Equalizer off");
	else
		ShowKeyStatus((char *)"Equalizer on");
	CheckBoxSetValue(WINDOWAPP, ID_CHK_EQUALIZER, g_sets.eqflags & MAEQACTIVE ? 0 : 1);
	OnCheckEqualizer(g_sets.eqflags & MAEQACTIVE ? 0 : 1);
 }


 void OnKeyCheckEffects()
 {
	if(g_sets.effects & EFFECTSON)
		ShowKeyStatus((char *)"Effects off");
	else
		ShowKeyStatus((char *)"Effects on");
	CheckBoxSetValue(WINDOWAPP, ID_CHK_EFFECTS, g_sets.effects & EFFECTSON ? 0 : 1);
	OnCheckEffects(g_sets.effects & EFFECTSON ? 0 : 1);
 }


 void OnKeyCheckRepeat()
 {
	if(g_reptsong)
		ShowKeyStatus((char *)"Repeat off");
	else
		ShowKeyStatus((char *)"Repeat on");
	CheckBoxSetValue(WINDOWAPP, ID_CHK_REPTSONG, g_reptsong ? 0 : 1);
	OnCheckReptSong(g_reptsong ? 0 : 1);
 }

 
 void OnKeyCheckBalance()
 {
	if(g_sets.balance)
		ShowKeyStatus((char *)"Balance off");
	else
		ShowKeyStatus((char *)"Balance on");
	CheckBoxSetValue(WINDOWAPP, ID_CHK_BALANCE, g_sets.balance ? 0 : 1);
	OnCheckBalance(g_sets.balance ? 0 : 1);
 }


 void OnKeyCheckMonoLeft()
 {
	if(g_sets.monodiff & MONOCHECKBOXL)
		ShowKeyStatus((char *)"MonoLeft off");
	else
		ShowKeyStatus((char *)"MonoLeft on");
	CheckBoxSetValue(WINDOWAPP, ID_CHK_MONOLEFT, g_sets.monodiff & MONOCHECKBOXL ? 0 : 1);
	OnCheckMonoLeft(g_sets.monodiff & MONOCHECKBOXL ? 0 : 1);
 }


 void OnKeyCheckMonoRight()
 {
	if(g_sets.monodiff & MONOCHECKBOXR)
		ShowKeyStatus((char *)"MonoRight off");
	else
		ShowKeyStatus((char *)"MonoRight on");
	CheckBoxSetValue(WINDOWAPP, ID_CHK_MONORIGHT, g_sets.monodiff & MONOCHECKBOXR ? 0 : 1);
	OnCheckMonoRight(g_sets.monodiff & MONOCHECKBOXR ? 0 : 1);
 }


 void OnKeyCheckMonoDiff()
 {
	if(g_sets.monodiff & (MONOCHECKBOXSUB | MONOCHECKBOXREV))
		ShowKeyStatus((char *)"Mono L+R");
	else
		ShowKeyStatus((char *)"Mono L-R");
	CheckBoxSetValue(WINDOWAPP, ID_CHK_MONODIFF, g_sets.monodiff & (MONOCHECKBOXSUB | MONOCHECKBOXREV) ? 0 : 1);
	OnCheckMonoDiff(g_sets.monodiff & (MONOCHECKBOXSUB | MONOCHECKBOXREV) ? 0 : 1);
 }


 void OnKeyCheckLimiter()
 {
	if(g_sets.limiter)
		ShowKeyStatus((char *)"Limiter off");
	else
		ShowKeyStatus((char *)"Limiter on");
	CheckBoxSetValue(WINDOWAPP, ID_CHK_LIMITER, g_sets.limiter ? 0 : 1);
	OnCheckLimiter(g_sets.limiter ? 0 : 1);
 }


 void OnKeyCheckEqExtra()
 {
	if(g_sets.eqflags & MAEQEXTRA)
		ShowKeyStatus((char *)"EqExtra off");
	else
		ShowKeyStatus((char *)"EqExtra on");
	CheckBoxSetValue(WINDOWAPP, ID_CHK_EQEXTRA, g_sets.eqflags & MAEQEXTRA ? 0 : 1);
	OnCheckEqExtra(g_sets.eqflags & MAEQEXTRA ? 0 : 1);
 }


 void OnKeyCheckEqZoom()
 {
	if(g_w[WINDOWEQ])
		CheckBoxSetValue(WINDOWEQ, ID_CHK_EQZOOM, g_sets.eqflags & MAEQZOOM ? 0 : 1);
	OnCheckEqZoom(g_sets.eqflags & MAEQZOOM ? 0 : 1);
 }


 void OnKeyCheckEqLink()
 {
	if(g_w[WINDOWEQ])
		CheckBoxSetValue(WINDOWEQ, ID_CHK_EQLINK, g_sets.eqflags & MAEQLINK ? 0 : 1);
	OnCheckEqLink(g_sets.eqflags & MAEQLINK ? 0 : 1);
 }


 void OnKeyCheckInvertRight()
 {
	if(g_sets.invertright)
		ShowKeyStatus((char *)"InvertRight off");
	else
		ShowKeyStatus((char *)"InvertRight on");
	CheckBoxSetValue(WINDOWAPP, ID_CHK_INVERTRIGHT, g_sets.invertright ? 0 : 1);
	OnCheckInvertRight(g_sets.invertright ? 0 : 1);
 }


 void OnKeyCheckEq31Bands()
 {
	if(g_sets.eqflags & MAEQ31BANDS)
		ShowKeyStatus((char *)"Eq31Bands off");
	else
		ShowKeyStatus((char *)"Eq31Bands on");
	CheckBoxSetValue(WINDOWAPP, ID_CHK_EQ31BANDS, g_sets.eqflags & MAEQ31BANDS ? 0 : 1);
	OnCheckEq31Bands(g_sets.eqflags & MAEQ31BANDS ? 0 : 1);
 }


 void OnKeyCheckPitch()
 {
	if(g_sets.effects & EFFECTSPITCH)
		ShowKeyStatus((char *)"Pitch effect off");
	else
		ShowKeyStatus((char *)"Pitch effect on");
	if(g_w[WINDOWEFFECTS])
		CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_PITCH, g_sets.effects & EFFECTSPITCH ? 0 : 1);
	OnCheckPitch(g_sets.effects & EFFECTSPITCH ? 0 : 1);
 }


 void OnKeyCheckTempo()
 {
	if(g_sets.effects & EFFECTSTEMPO)
		ShowKeyStatus((char *)"Tempo effect off");
	else
		ShowKeyStatus((char *)"Tempo effect on");
	if(g_w[WINDOWEFFECTS])
		CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_TEMPO, g_sets.effects & EFFECTSTEMPO ? 0 : 1);
	OnCheckTempo(g_sets.effects & EFFECTSTEMPO ? 0 : 1);
 }


 void OnKeyCheckSpeed()
 {
	if(g_sets.effects & EFFECTSSPEED)
		ShowKeyStatus((char *)"Speed effect off");
	else
		ShowKeyStatus((char *)"Speed effect on");
	if(g_w[WINDOWEFFECTS])
		CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_SPEED, g_sets.effects & EFFECTSSPEED ? 0 : 1);
	OnCheckSpeed(g_sets.effects & EFFECTSSPEED ? 0 : 1);
 }


void OnKeyCheckQualSpeed()
{
	if(g_sets.effects & EFFECTSQUALSPEED)
		ShowKeyStatus((char *)"Speed effect quality off");
	else
		ShowKeyStatus((char *)"Speed effect quality on");
	if(g_w[WINDOWEFFECTS])
		CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_QUALSPEED, g_sets.effects & EFFECTSQUALSPEED ? 0 : 1);
	OnCheckQualSpeed(g_sets.effects & EFFECTSQUALSPEED ? 0 : 1);
}


 void OnKeyCheckQualFilter()
 {
	if(g_sets.effects & EFFECTSQUALFILTER)
		ShowKeyStatus((char *)"Effects filter quality off");
	else
		ShowKeyStatus((char *)"Effects filter quality on");
	if(g_w[WINDOWEFFECTS])
		CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_QUALFILTER, g_sets.effects & EFFECTSQUALFILTER ? 0 : 1);
	OnCheckQualFilter(g_sets.effects & EFFECTSQUALFILTER ? 0 : 1);
 }


 #if !defined(MANOCDA) && defined(__MAUNIX__)
 void OnKeyCheckCdParanoia()
 {
	if(g_setscdrom.cdparanoia)
		ShowKeyStatus((char *)"CD Paranoia off");
	else
		ShowKeyStatus((char *)"CD Paranoia on");
	if(g_w[WINDOWAUDIOCD])
		CheckBoxSetValue(WINDOWAUDIOCD, ID_CHK_CDPARANOIA, g_setscdrom.cdparanoia ? 0 : 1);
	OnCheckCdParanoia(g_setscdrom.cdparanoia ? 0 : 1);
 }
 #endif


 void OnKeyCheckBoxBeg()
 {
 #if !defined(MAGUINONE)
	if(g_posbegon)
		ShowKeyStatus((char *)"Begin position off");
	else
		ShowKeyStatus((char *)"Begin position on");
 #endif
	CheckBoxSetValue(WINDOWAPP, ID_CHK_BEG, g_posbegon ? 0 : 1);
	OnCheckBoxBeg(g_posbegon ? 0 : 1);
 }


 void OnKeyCheckBoxEnd()
 {
 #if !defined(MAGUINONE)
	if(g_posendon)
		ShowKeyStatus((char *)"End position off");
	else
		ShowKeyStatus((char *)"End position on");
 #endif
	CheckBoxSetValue(WINDOWAPP, ID_CHK_END, g_posendon ? 0 : 1);
	OnCheckBoxEnd(g_posendon ? 0 : 1);
 }


 void OnKeyCheckBoxM4A()
 {
	if(g_w[WINDOWOUTCUR])
		CheckBoxSetValue(WINDOWOUTCUR, ID_CHK_M4A, g_sets.fileout[g_k[g_sets.output]].mode & kACCM4A ? 0 : 1);
	OnCheckBoxM4A(g_sets.fileout[g_k[g_sets.output]].mode & kACCM4A ? 0 : 1);
 }


 void OnKeyCheckShowIndex()
 {
	if(g_w[WINDOWLIST])
		CheckBoxSetValue(WINDOWLIST, ID_CHK_SHOWINDEX, g_sets.listwindow & LBOXINDEX ? 0 : 1);
	OnCheckShowIndex(g_sets.listwindow & LBOXINDEX ? 0 : 1);
 }


 void OnKeyCheckShowDir()
 {
	if(g_w[WINDOWLIST])
		CheckBoxSetValue(WINDOWLIST, ID_CHK_SHOWDIR, g_sets.listwindow & LBOXDIR ? 0 : 1);
	OnCheckShowDir(g_sets.listwindow & LBOXDIR ? 0 : 1);
 }


 void OnKeyCheckShowPath()
 {
	if(g_w[WINDOWLIST])
		CheckBoxSetValue(WINDOWLIST, ID_CHK_SHOWPATH, g_sets.listwindow & LBOXPATH ? 0 : 1);
	OnCheckShowPath(g_sets.listwindow & LBOXPATH ? 0 : 1);
 }


 void OnKeyVolumeDown()
 {
	ChangeVolume(-MAKEYVOLUMEDELTA);
 }


 void OnKeyVolumeUp()
 {
	ChangeVolume(MAKEYVOLUMEDELTA);
 }


 void OnKeyPlayIndex()
 {
	char cstr[256];

	if(l_keyindex)
		return; // don't reenter
	l_keyindex = 1;
		
 #if defined(MAKEYBOARD)
	g_maOnKeySkip = 1;
 #endif

	ShowKeyStatus((char *)"Play index");
	
	sprintf(cstr, "Enter the index of a song to play (1 to %d)", g_songsnum);

	DoWindowPromptNum(1);
	
	if(!g_w[WINDOWPROMPTNUM])
	{
#if defined(MAKEYBOARD)
		g_maOnKeySkip = 0;
#endif
		maLog("OnKeyPlayIndex failed");
		l_keyindex = 0;
		return;
	}
	
	StaticTextSetText(WINDOWPROMPTNUM, ID_TEXT_PROMPTNUM, cstr);

#if defined(MAGUINONE)
	OnEditTextPromptNum();
#endif

 } // OnKeyPlayIndex


 MAKEYBINDINGS g_keybindings[MANUMKEYS]; // initialized by SetFactoryKeyBindings()

 char l_keybindingstext[] =
 "This file is created by MoreAmp and will be\n"
 "restored with factory settings if deleted.\n"
 "\nEdit between the double-quotes the keyboard\n"
 "character that you wish to use as a shortcut\n"
 "to the function next to it.\n"
 "\nDo not use the double-quote character as a shortcut.\n"
 "\nTo disable a shortcut to a function put nothing\n"
 "(no space) between the double-quotes.\n"
 "\nDo not change the order of the functions.";

 int SaveKeyBindings(int blank)
 {
	int i;
	int ret;
	FILE *fp;
	char cstr[256];
	
	if((fp = mafopen(g_keybindingspath, "wb")) == NULL)
	{
		sprintf(cstr, "SaveSettingsCDRom mafopen failed on '%s'", g_keybindingspath);
		maLog(cstr);
		return 1;
	}

 #if defined(__MAOSX__) || defined(__MAUNIX__)
	machmod(g_keybindingspath, ACCESSPERMS); // 0777
 #endif
	
	if(blank)
		for(i = 0; i < MANUMKEYS; i++)
			g_keybindings[i].key = 0;

	for(i = 0; i < MANUMKEYS; i++)
	{
		if(i == 0)
		{
			sprintf(cstr, "CONTROLS  ");
			fwrite(cstr, 1, strlen(cstr), fp);
		}
		
		if(i == 8)
		{
			sprintf(cstr, "\n\nSETTINGS  ");
			fwrite(cstr, 1, strlen(cstr), fp);
		}
		
		if(i == 16)
		{
			sprintf(cstr, "\n\nLIST  ");
			fwrite(cstr, 1, strlen(cstr), fp);
		}

		if(i == 23)
		{
			sprintf(cstr, "PlaySelection(Enter)  ");
			fwrite(cstr, 1, strlen(cstr), fp);
		}

		if(i == 24)
		{
			sprintf(cstr, "\n\nEQUALIZER  ");
			fwrite(cstr, 1, strlen(cstr), fp);
		}
		
		if(i == 34)
		{
			sprintf(cstr, "\n\nEFFECTS  ");
			fwrite(cstr, 1, strlen(cstr), fp);
		}
		
		if(i == 36)
		{
			sprintf(cstr, "\n\nPITCH  ");
			fwrite(cstr, 1, strlen(cstr), fp);
		}
		
		if(i == 42)
		{
			sprintf(cstr, "\n\nTEMPO  ");
			fwrite(cstr, 1, strlen(cstr), fp);
		}
		
		if(i == 46)
		{
			sprintf(cstr, "\n\nSPEED  ");
			fwrite(cstr, 1, strlen(cstr), fp);
		}
		
		if(i == 50)
		{
			sprintf(cstr, "\n\nFILE OUT  ");
			fwrite(cstr, 1, strlen(cstr), fp);
		}
		
		if(i == 51)
		{
			sprintf(cstr, "\n\nVIEW  ");
			fwrite(cstr, 1, strlen(cstr), fp);
		}
		
		if(i == 57)
		{
			sprintf(cstr, "\n\nWINDOW SIZES  ");
			fwrite(cstr, 1, strlen(cstr), fp);
		}

		if(i == 61)
		{
			sprintf(cstr, "\n\nFILE  ");
			fwrite(cstr, 1, strlen(cstr), fp);
		}

		if(i == 66)
		{
			sprintf(cstr, "\n\nVOLUME  ");
			fwrite(cstr, 1, strlen(cstr), fp);
		}

		if(!g_keybindings[i].key)
		{
			if(g_keybindings[i].labeld)
				sprintf(cstr, "%s%c%c  ",
					g_keybindings[i].labeld, MAKEYDELIM, MAKEYDELIM);
			else
				sprintf(cstr, "%s%c%c  ",
					g_keybindings[i].label, MAKEYDELIM, MAKEYDELIM);
		}
		else
		{
			if(g_keybindings[i].labeld)
				sprintf(cstr, "%s%c%c%c  ",
					g_keybindings[i].labeld, MAKEYDELIM, g_keybindings[i].key, MAKEYDELIM);
			else
				sprintf(cstr, "%s%c%c%c  ",
					g_keybindings[i].label, MAKEYDELIM, g_keybindings[i].key, MAKEYDELIM);
		}
		
		ret = fwrite(cstr, 1, strlen(cstr), fp);
		if(ret != (int)strlen(cstr))
		{
			fclose(fp);
			sprintf(cstr, "SaveKeyBindings blocks=%d fwrite '%s'", ret, g_keybindingspath);
			maLog(cstr);	
			return 1;
		}
	}
	
	sprintf(cstr, "\n\n%s\n", g_keybindingspath);
	ret = fwrite(cstr, 1, strlen(cstr), fp);
	if(ret != (int)strlen(cstr))
	{
		fclose(fp);
		sprintf(cstr, "SaveKeyBindings blocks=%d fwrite '%s'", ret, g_keybindingspath);
		maLog(cstr);	
		return 1;
	}

	ret = fwrite(l_keybindingstext, 1, strlen(l_keybindingstext), fp);
	if(ret != (int)strlen(l_keybindingstext))
	{
		fclose(fp);
		sprintf(cstr, "SaveKeyBindings blocks=%d fwrite '%s'", ret, g_keybindingspath);
		maLog(cstr);	
		return 1;
	}

	fclose(fp);

	if(blank)
		sprintf(cstr, "SaveKeyBindings blank file '%s'", g_keybindingspath);
	else
		sprintf(cstr, "SaveKeyBindings factory settings '%s'", g_keybindingspath);
	
	maLog(cstr);	
	
	return 0;
	
 } // SaveKeyBindings


 int LoadKeyBindings(int showerr)
 {
	int i;
	int ret = 0;
	long filesize;
	FILE *fp;
	char *p;
	char *q;
	char *r;
	char cstr[256];

	if(!g_keybindingspath[0])
	{
		maLog("LoadKeyBindings cannot load settings, no file name");
		return 1;	
	}
			
	if((fp = mafopen(g_keybindingspath, "rb")) == NULL)
	{
		SetFactoryKeyBindings();
		SaveKeyBindings(0);
		return 2;
	}

	fseek(fp, 0, SEEK_END);		
	filesize = ftell(fp);
	fseek(fp, 0, SEEK_SET);		
	
	if((p = (char *)MAMALLOC(filesize + 1)) == NULL)
	{
		fclose(fp);
		sprintf(cstr, "LoadKeyBindings MAMALLOC %d failed", (int)filesize);
		maLog(cstr);
		return 1;
	}			

	if((ret = fread(p, 1, filesize, fp)) != filesize)
	{			
		fclose(fp);
		MAFREE(p);
		sprintf(cstr, "LoadKeyBindings fread errno=%d '%s'", errno, g_keybindingspath);
		maLog(cstr);
		return 1;
	}

	fclose(fp);
	
	for(q = p; q < p + filesize; q++)
		if(*q == 0)
			*q = ' '; // in case someone stuck a 0 in there
	q = p;
	i = 0;
	p[filesize] = 0;
	
	while ((r = strchr(q, MAKEYDELIM))) // first delimiter of a pair
	{
		r++;
		if(!(q = strchr(r, MAKEYDELIM))) // second delimiter
			break;
		if(r == q) // "" empty
			g_keybindings[i].key = 0;
		else
			g_keybindings[i].key = *r & 0xff; // first char after first delimiter
		if(++i >= MANUMKEYS)
			break;
		q++;
	}
	
	MAFREE(p);
	
	if(showerr)
	{
		if(i < 1)
		{
			sprintf(cstr, "LoadKeyBindings found no entries (no double-quote pairs)");
			ShowErr(cstr);
			for(i = 0; i < MANUMKEYS; i++)
				g_keybindings[i].key = 0;
		}
		else if(i < MANUMKEYS)
		{
			sprintf(cstr, "LoadKeyBindings only found %d entries (double-quote pairs) instead of %d", i, MANUMKEYS);
			ShowErr(cstr);
			for(; i < MANUMKEYS; i++)
				g_keybindings[i].key = 0;
		}
		else
			VerifyKeyBindings(1);
	}
	else
	{
		if(i < 1)
		{
			SetFactoryKeyBindings();
			SaveKeyBindings(0);
			sprintf(cstr, "LoadKeyBindings found no entries in '%s', set factory bindings", g_keybindingspath);
			maLog(cstr);
			return 3;
		}
		if(i < MANUMKEYS)
		{
			sprintf(cstr, "LoadKeyBindings only found %d entries instead of %d in '%s'", i, MANUMKEYS, g_keybindingspath);
			maLog(cstr);
		}
		VerifyKeyBindings(0);
	}
	
	UpdateMenuKbLabels();
	UpdateEffectsKbLabels();
	UpdateButtonLabels();
	
	return 0;
	
 } // LoadKeyBindings


 int VerifyKeyBindings(int showerr)
 {
	int i, j;
	char unused[256];
	char cstr[512];

	for(i = 0; i < MANUMKEYS; i++)
	{
		if(!g_keybindings[i].key)
			continue;
		for(j = i + 1; j < MANUMKEYS; j++)
			if(g_keybindings[i].key == g_keybindings[j].key)
			{
				sprintf(cstr, "VerifyKeyBindings duplicate key '%c' (0x%x) for %s and %s",
										g_keybindings[i].key, (char)g_keybindings[i].key,
										g_keybindings[i].label, g_keybindings[j].label);
				if(showerr)
				{
					GetFreeKeyBindings(unused);
					if(unused[0])
					{
						strcat(cstr, "\n\nunused keys :\n");
						strcat(cstr, unused);
					}
					ShowErr(cstr);
				}
				else
					maLog(cstr);
				return 1;
			}
	}
	
	return 0;
	
 } // VerifyKeyBindings


 void GetFreeKeyBindings(char *p)
 {
	int i;
	char c[2];

	if(p == NULL)
		return;
		
	c[1] = 0;
	p[0] = 0;
	for(c[0] = 33; c[0] < 127; c[0]++) // ascii '!' to tilde ~
	{
		if(c[0] == MAKEYDELIM)
			continue;
		for(i = 0; i < MANUMKEYS; i++)
			if(g_keybindings[i].key == c[0])
				break;
		if(i >= MANUMKEYS)
			strcat(p, c);
	}
 } // GetFreeKeyBindings


 void SetFactoryKeyBindings()
 {
	MAKEYBINDINGS keybindings[MANUMKEYS] =
	{
/* default, xmms, requested, preferred */
				//"CONTROLS  " 0-7 yxcvbnm lower left row of keyboard
/* a, q, Q,   */ {'Q', (char *)"Quit", NULL, OnKeyButtonQuit, ID_MENU_QUIT, ID_TEXT_BUTTON_QUIT},
/* b, z, y,   */ {'y', (char *)"Prev", NULL, OnKeyButtonPrevious, ID_MENU_PREVIOUS, ID_TEXT_BUTTON_PREVIOUS},
/* c, x,  ,   */ {'x', (char *)"Play", NULL, OnKeyButtonPlay, ID_MENU_PLAY, ID_TEXT_BUTTON_PLAY},
/* d, c,  ,   */ {'c', (char *)"Pause", NULL, OnKeyButtonPause, ID_MENU_PAUSE, ID_TEXT_BUTTON_PAUSE},
/* e, v,  ,   */ {'v', (char *)"Stop", NULL, OnKeyButtonStop, ID_MENU_STOP, ID_TEXT_BUTTON_STOP},
/* f, b,  ,   */ {'b', (char *)"Next", NULL, OnKeyButtonNext, ID_MENU_NEXT, ID_TEXT_BUTTON_NEXT},
/* g,  ,  ,   */ {'N', (char *)"PlayCd", NULL, OnKeyPlayCD, ID_MENU_PLAYCD, 0},
/* h,  ,  ,   */ {'M', (char *)"EjectCd", NULL, OnKeyEjectCD, ID_MENU_EJECTCD, 0},

				//"\nSETTINGS" 8
/* i, r,  ,   */ {'r', (char *)"Repeat", NULL, OnKeyCheckRepeat, 0, 0},
/* j, s,  ,   */ {'s', (char *)"Random", NULL, OnKeyCheckRandom, ID_MENU_RANDOM, 0},
/* k,  ,  ,   */ {'j', (char *)"ReptPlayed", NULL, OnKeyCheckReptPlayed, ID_MENU_REPTPLAYED, 0},
/* l,  ,  ,   */ {'n', (char *)"LooopLoaded", NULL, OnKeyCheckLoopLoaded, ID_MENU_LOOPLOADED, 0},
/* m,  ,  ,   */ {'o', (char *)"LoopList", NULL, OnKeyCheckLoopList, ID_MENU_LOOPLIST, 0},
/* n,  , 1,   */ {'1', (char *)"SinglePlay", NULL, OnKeyCheckSinglePlay, ID_MENU_SINGLEPLAY, 0},
/* o,  ,  ,   */ {'g', (char *)"HiPriority", NULL, OnKeyCheckHighPriority, ID_MENU_HIPRIORITY, 0},
/* p,  ,  ,   */ {'a', (char *)"TitleInfo", NULL, OnKeyCheckTitleInfo, ID_MENU_TITLEINFO, 0},

				//"\nLIST" 16
/* q,  ,  ,   */ {'q', (char *)"ListWindow", (char *)"Window", OnKeyCheckListWindow, ID_MENU_SHOWLIST, 0},
/* r,  ,  ,   */ {'w', (char *)"PlayerInfo", (char *)"Info", OnKeyButtonInfo, 0, 0},
/* s,  ,  ,   */ {'t', (char *)"SongList", (char *)"List", OnKeyButtonList, 0, 0},
/* t,  ,  ,   */ {'u', (char *)"SongsLoaded", (char *)"Load", OnKeyButtonLoaded, 0, 0},
/* u,  ,  ,   */ {'p', (char *)"SongInfo", (char *)"Song", OnKeyButtonSong, 0, 0},
/* v,  ,  , i */ {'i', (char *)"ScrollUp", NULL, OnKeyUpArrow, 0, 0},
/* w,  ,  , k */ {'k', (char *)"ScrollDown", NULL, OnKeyDownArrow, 0, 0},
				//"PlaySelection(Enter)" 23
/* x,  ,  ,   */ {'P', (char *)"ZeroList", NULL, OnKeyMenuZero, ID_MENU_ZEROLIST, 0},

				//"\nEQUALIZER" 24
/* y,  ,  , E */ {'E', (char *)"Equalizer", (char *)"", OnKeyCheckEqualizer, ID_MENU_EQUALIZER, 0},
/* z,  ,  , e */ {'e', (char *)"EqWindow", (char *)"Window", OnKeyCheckEqWindow, ID_MENU_SHOWEQ, 0},
/* A,  ,  ,   */ {'A', (char *)"Eq31Bands", (char *)"31Bands", OnKeyCheckEq31Bands, ID_MENU_EQ31BANDS, 0},
/* B,  ,  ,   */ {'B', (char *)"EqExtra", (char *)"Extra", OnKeyCheckEqExtra, ID_MENU_EQEXTRA, 0},
/* C,  ,  ,   */ {'C', (char *)"EqReset", (char *)"Reset", OnKeyButtonEqRestore, 0, 0},
/* D,  ,  ,   */ {'D', (char *)"EqInvert", (char *)"Invert", OnKeyButtonEqInvert, 0, 0},
/* E,  ,  ,   */ {'G', (char *)"EqPreset1", (char *)"Preset1", OnKeyPopupEqPreset1, 0, 0},
/* F,  ,  ,   */ {'H', (char *)"EqPreset2", (char *)"Preset2", OnKeyPopupEqPreset2, 0, 0},
/* G,  ,  ,   */ {'J', (char *)"EqPreset3", (char *)"Preset3", OnKeyPopupEqPreset3, 0, 0},
/* H,  ,  ,   */ {'K', (char *)"EqPreset4", (char *)"Preset4", OnKeyPopupEqPreset4, 0, 0},

				//"\nEFFECTS" 35
/* I,  ,  , F */ {'F', (char *)"Effects", (char *)"", OnKeyCheckEffects, ID_MENU_EFFECTS, ID_TEXT_KEYEFECTS},
/* J,  ,  , f */ {'f', (char *)"EffectsWindow", (char *)"Window", OnKeyEffectsWindow, ID_MENU_SHOWEFFECTS, 0},

				//"\nPITCH" 36 34567 upper left row of keaboard
/* K,  ,  ,   */ {'R', (char *)"Pitch", (char *)"", OnKeyCheckPitch, 0, ID_TEXT_KEYPITCHSET},
/* L,  ,  , 3 */ {'3', (char *)"PitchDown", (char *)"Down", OnKeyPitchDown, 0, ID_TEXT_KEYPITCHDOWN},
/* M,  ,  , 4 */ {'4', (char *)"PitchDown2", (char *)"Down2", OnKeyPitch2Down, 0, ID_TEXT_KEYPITCH2DOWN},
/* N,  ,  , 5 */ {'5', (char *)"PitchUp2", (char *)"Up2", OnKeyPitch2Up, 0, ID_TEXT_KEYPITCH2UP},
/* O,  ,  , 6 */ {'6', (char *)"PitchUp", (char *)"Up", OnKeyPitchUp, 0, ID_TEXT_KEYPITCHUP},
/* P,  ,  , 7 */ {'7', (char *)"PitchReset", (char *)"Reset", OnKeyResetSliderPitch, 0, ID_TEXT_KEYPITCHRESET},

				//"\nTEMPO" 42 890 upper right row of keaboard
/* Q,  , T,   */ {'T', (char *)"Tempo", (char *)"", OnKeyCheckTempo, 0, ID_TEXT_KEYTEMPOSET},
/* R,  , 8,   */ {'8', (char *)"TempoDown", (char *)"Down", OnKeyTempoDown, 0, ID_TEXT_KEYTEMPODOWN},
/* S,  , 9,   */ {'9', (char *)"TempoUp", (char *)"Up", OnKeyTempoUp, 0, ID_TEXT_KEYTEMPOUP},
/* T,  , 0,   */ {'0', (char *)"TempoReset", (char *)"Reset", OnKeyResetSliderTempo, 0, ID_TEXT_KEYTEMPORESET},

				//"\nSPEED" 46 UIO middle upper right row of keyboard
/* U,  ,  ,   */ {'Z', (char *)"Speed", (char *)"", OnKeyCheckSpeed, 0, ID_TEXT_KEYSPEEDSET},
/* W,  ,  , I */ {'U', (char *)"SpeedDown", (char *)"Down", OnKeySpeedDown, 0, ID_TEXT_KEYSPEEDDOWN},
/* V,  ,  , O */ {'I', (char *)"SpeedUp", (char *)"Up", OnKeySpeedUp, 0, ID_TEXT_KEYSPEEDUP},
/* X,  ,  , P */ {'O', (char *)"SpeedReset", (char *)"Reset", OnKeyResetSliderSpeed, 0, ID_TEXT_KEYSPEEDRESET},

				//"\nFILE OUT" 50
/* Y,  ,  ,   */ {'Y', (char *)"FileOutWindow", (char *)"Window", OnKeyShowFileOut, ID_MENU_SHOWFILEOUT, 0},

				//"\nVIEW" 51   
/* Z,  ,  ,   */ {'h', (char *)"EditKeyBindings", (char *)"KeyBindings", OnKeyEditBindings, ID_MENU_EDITKEYBINDINGS, 0},
/* 0,  ,  ,   */ {'2', (char *)"ViewCatalog", (char *)"Catalog", OnKeyViewCatalog, ID_MENU_VIEWCATALOG, 0},
/* 1,  ,  ,   */ {'d', MAREADME, (char *)"Readme", OnKeyViewReadme, ID_MENU_README, 0},
/* 2,  ,  ,   */ {'m', (char *)"About", NULL, OnKeyViewAbout, ID_MENU_ABOUT, 0},
/* 3,  ,  ,   */ {'z', (char *)"ViewLog", (char *)"Log", OnKeyViewLog, ID_MENU_VIEWLOG, 0},
/* 4,  ,  ,   */ {'S', (char *)"ClearLog", NULL, OnKeyClearLog, ID_MENU_CLEARLOG, 0},

				//"\nWINDOW SIZES" 57
/* 5,  ,  ,   */ {'X', (char *)"WindowAltSizes", (char *)"Alt", OnKeyAltSize, ID_MENU_ALTSIZE, 0},
/* 6,  ,  ,   */ {'V', (char *)"WindowSavedSizes", (char *)"Saved", OnKeySavedSize, ID_MENU_SAVEDSIZE, 0},
/* 7,  ,  ,   */ {'W', (char *)"WindowDefaultSizes", (char *)"Default", OnKeyDefaultSize, ID_MENU_DEFAULTSIZE, 0},
/* (,  ,  ,   */ {'(', (char *)"WindowFactorySizes", (char *)"Factory", OnKeyFactorySize, ID_MENU_FACTORYSIZE, 0},

				//"\nFILE" 61
/* 8,  ,  ,   */ {'-', (char *)"OpenList", NULL, OnKeyOpenList, ID_MENU_OPENLIST, 0},
/* 9, L,  ,   */ {'L', (char *)"MakeList", NULL, OnKeyMakeList, ID_MENU_MAKELIST, 0},
/* +,  ,  ,   */ {'+', (char *)"SaveListAs", NULL, OnKeySaveListAs, ID_MENU_SAVELISTAS, 0},
/* -, l,  ,   */ {'l', (char *)"PlayFile", NULL, OnKeyPlayFile, ID_MENU_PLAYFILE, 0},
/* ?,  ,  ,   */ {'?', (char *)"PlayIndex", NULL, OnKeyPlayIndex, 0, 0},

				//"\nVOLUME" 66
/* ,,dn,  ,   */ {',', (char *)"VolumeDown", NULL, OnKeyVolumeDown, 0, 0},
/* .,up,  ,   */ {'.', (char *)"VolumeUp", NULL, OnKeyVolumeUp, 0, 0},
	};

	memcpy(g_keybindings, keybindings, sizeof(g_keybindings));
	
 } // SetFactoryKeyBindings


 #if !defined(MAGUIGTK)
  void UpdateMenuKbLabels()
  {
	int i;
	int menuid;
	char *p;
	char nulabel[128];
	char key[16];
	
	if(!g_w[WINDOWAPP])
		return;

  #if defined(MAGUIWX)
	if(g_menubar)
	{
  #endif
		for(menuid = ID_MENU_FIRST; menuid <= ID_MENU_LAST; menuid++)
		{
			if(menuid >= ID_SUBMENU_OPENRECENT1 && menuid <= ID_SUBMENU_OPENRECENT1 + MAXOPENRECENT)
				continue; // these may not exist
			MenuGetLabel(menuid, nulabel);
			if((p = strchr(nulabel, '\'')))
				*(p - 1) = 0;
			if(g_sets.keyboard)
			{
				for(i = 0; i < MANUMKEYS; i++)
				{
					if(g_keybindings[i].key && menuid == g_keybindings[i].menuid)
					{
						sprintf(key, " '%c'", (char)g_keybindings[i].key);
						strcat(nulabel, key);
						break;
					}
				}
			}
  #if defined(__WXMAC__)
			if(menuid == ID_MENU_ABOUT || menuid == ID_MENU_README)
				; // do something because these two labels don't get updated...
  #endif
			MenuSetLabel(menuid, nulabel);
		}
  #if defined(MAGUIWX)
	}
  #endif
	
  } // UpdateMenuKbLabels
 #endif


 void UpdateEffectsKbLabels()
 {
	int i;
	int textid;
	char nulabel[16];
	
	if(!g_w[WINDOWEFFECTS])
		return;
	
	for(textid = ID_TEXT_KEYEFFECTS_FIRST; textid <= ID_TEXT_KEYEFFECTS_LAST; textid++)
	{
		for(i = 0; i < MANUMKEYS; i++)
			if(textid == g_keybindings[i].textid)
			{
				if(g_sets.keyboard && g_keybindings[i].key)
					sprintf(nulabel, "'%c'", (char)g_keybindings[i].key);
				else
					nulabel[0] = 0;
				StaticTextSetText(WINDOWEFFECTS, textid, nulabel);
				break;
			}
	}
	
 } // UpdateEffectsKbLabels


 void UpdateButtonLabels()
 {
	int i;
	int textid;
 #if defined(MAUSEMABUTTON)
	char *p;
 #endif
	char nulabel[128];
	char key[16];
	
	if(!g_w[WINDOWAPP])
		return;

	for(textid = ID_TEXT_BUTTON_FIRST; textid <= ID_TEXT_BUTTON_LAST; textid++)
	{
 #if defined(MAUSEMABUTTON)
		switch(textid)
		{
			case ID_TEXT_BUTTON_PREVIOUS :
				strcpy(nulabel, LABEL_PREV);
			break;
			case ID_TEXT_BUTTON_QUIT :
				strcpy(nulabel, LABEL_QUIT);
			break;
			case ID_TEXT_BUTTON_PLAY :
				strcpy(nulabel, LABEL_PLAY);
			break;
			case ID_TEXT_BUTTON_PAUSE :
				strcpy(nulabel, LABEL_PAUSE);
			break;
			case ID_TEXT_BUTTON_STOP :
				strcpy(nulabel, LABEL_STOP);
			break;
			case ID_TEXT_BUTTON_NEXT :
				strcpy(nulabel, LABEL_NEXT);
			break;
		}
		if((p = strchr(nulabel, '\'')))
			*p = 0;
  #if defined(__MAUNIX__) // s.b. __WXX11__
		else
		{
			p = nulabel + strlen(nulabel) - 1;
				while(*p == ' ') // initial labels need 3 spaces at end, remove them
					*(p--) = 0;
		}
  #endif
 #else
  #if defined(MAGUIOSX)
		nulabel[0] = 0;
  #else
		strcpy(nulabel, "   ");
  #endif
 #endif
		for(i = 0; i < MANUMKEYS; i++)
		{
			if(g_sets.keyboard && g_keybindings[i].key && textid == g_keybindings[i].textid)
			{
				sprintf(key, "'%c'", (char)g_keybindings[i].key);
				strcat(nulabel, key);
				break;
			}
		}
		StaticTextSetText(WINDOWAPP, textid, nulabel);
	}

 } // UpdateButtonLabels


 int maOnKey(int window, int key)
 {
 #ifdef MAUNUSED
  (void)window;
 #endif

	int i;
 #if !defined(MAGUINONE)
	int numrows;

  #if defined(MAKEYBOARD)
	if(g_maOnKeySkip)
		return 1;
  #endif

	if(key == MAWXK_UP || key == MAWXK_DOWN || key == MAWXK_RETURN || key == MAWXK_RETURN_NUMPAD)
	{
		if(!g_w[WINDOWLIST])
			return 1;
		if(key == MAWXK_RETURN || key == MAWXK_RETURN_NUMPAD)
		{
			OnListWindowDoubleClick();
			return 0;
		}
		switch (g_sets.listwindow & (LBOXLIST | LBOXLOADED))
		{
			case LBOXLOADED:
				numrows = g_loadednum;
			break;
			case LBOXLIST:
				numrows = g_songsnum;
			break;
			default:
				return 1;
			break;
		}
		if((i = ListGetRowSelected(WINDOWLIST, ID_LIST_CTRL)) < 0)
		{
			ListSelect(WINDOWLIST, ID_LIST_CTRL, 0);
			ListEnsureVisible(WINDOWLIST, ID_LIST_CTRL, 0);
			return 0;
		}
		if(key == MAWXK_UP)
		{
			if(--i < 0)
				return 1;
		}
		else // MAWXK_DOWN
		{
			if(++i > numrows)
				return 1;
		}
		ListSelect(WINDOWLIST, ID_LIST_CTRL, i);
		ListEnsureVisible(WINDOWLIST, ID_LIST_CTRL, i);
		return 0;
	}
 #endif
	
	if(!g_sets.keyboard || key < 0 || key > 255)
		return 1;
		
	for(i = 0; i < MANUMKEYS; i++)
		if(key == g_keybindings[i].key)
			break;
	
	if(i >= MANUMKEYS)
	{
 #if defined(MAKEYBOARDTRACE)
		char cstr[256];
		sprintf(cstr, "maOnKey window=%d, char=\"%c\" (0x%x)", window, key, key);
		maLog(cstr);
 #endif
		return 1;
	}		
	
 #if defined(MAKEYBOARDTRACE)
	{
		char cstr[256];
		sprintf(cstr, "maOnKey window=%d, char=\"%c\" (0x%x) '%s'",
										window, key, key, g_keybindings[i].label);
		maLog(cstr);
	}
 #endif

	if(g_keybindings[i].function)
		(*g_keybindings[i].function)();
		
	return 0;
	
 } // maOnKey
#else // !MAKEYBOARD
 int maOnKey(int window, int key)
 {
 #ifdef MAUNUSED
  (void)window;
 #endif

	int i;
 #if !defined(MAGUINONE)
	int numrows;

  #if defined(MAKEYBOARD) && defined(__MAOS9__)
	if(g_maOnKeySkip)
		return 1;
  #endif

	if(key == MAWXK_UP || key == MAWXK_DOWN || key == MAWXK_RETURN || key == MAWXK_RETURN_NUMPAD)
	{
		if(!g_w[WINDOWLIST])
			return 1;
		if(key == MAWXK_RETURN || key == MAWXK_RETURN_NUMPAD)
		{
			OnListWindowDoubleClick();
			return 0;
		}
		switch (g_sets.listwindow & (LBOXLIST | LBOXLOADED))
		{
			case LBOXLOADED:
				numrows = g_loadednum;
			break;
			case LBOXLIST:
				numrows = g_songsnum;
			break;
			default:
				return 1;
			break;
		}
		if((i = ListGetRowSelected(WINDOWLIST, ID_LIST_CTRL)) < 0)
		{
			ListSelect(WINDOWLIST, ID_LIST_CTRL, 0);
			ListEnsureVisible(WINDOWLIST, ID_LIST_CTRL, 0);
			return 0;
		}
		if(key == MAWXK_UP)
		{
			if(--i < 0)
				return 1;
		}
		else // MAWXK_DOWN
		{
			if(++i > numrows)
				return 1;
		}
		ListSelect(WINDOWLIST, ID_LIST_CTRL, i);
		ListEnsureVisible(WINDOWLIST, ID_LIST_CTRL, i);
		return 0;
	}
 #endif
	
	return 0;
	
} // maOnKey
#endif // !MAKEYBOARD


void WindowSetFactoryValues(int winconf)
{
	if(winconf == -1)
	{
		g_sets.aboutxpos = ABOUTXPOSDEF;
		g_sets.aboutypos = ABOUTYPOSDEF;
		g_sets.aboutxsize = ABOUTXSIZEDEF;
		g_sets.aboutysize = ABOUTYSIZEDEF;
		g_sets.viewerxpos = VIEWERXPOSDEF;
		g_sets.viewerypos = VIEWERYPOSDEF;
		g_sets.viewerxsize = VIEWERXSIZEDEF;
		g_sets.viewerysize = VIEWERYSIZEDEF;
		g_sets.keybindingsxpos = KEYBINDINGSXPOSDEF;
		g_sets.keybindingsypos = KEYBINDINGSYPOSDEF;
		g_sets.keybindingsxsize = KEYBINDINGSXSIZEDEF;
		g_sets.keybindingsysize = KEYBINDINGSYSIZEDEF;
		g_sets.advancedxpos = ADVANCEDXPOSDEF;
		g_sets.advancedypos = ADVANCEDYPOSDEF;
		g_sets.advancedxsize = ADVANCEDXSIZEDEF;
		g_sets.advancedysize = ADVANCEDYSIZEDEF;

		return;
	}
	
	if(winconf == 0)
	{
		g_sets.xpos[0] = XPOSDEF;
		g_sets.ypos[0] = YPOSDEF;
		g_sets.xsize[0] = XSIZEDEF;
		g_sets.ysize[0] = YSIZEDEF;
		g_sets.eqxpos[0] = EQXPOSDEF;
		g_sets.eqypos[0] = EQYPOSDEF;
		g_sets.eqxsize[0] = EQXSIZEDEF;
		g_sets.eqysize[0] = EQYSIZEDEF;
		g_sets.eq2xpos[0] = EQ2XPOSDEF;
		g_sets.eq2ypos[0] = EQ2YPOSDEF;
		g_sets.eq2xsize[0] = EQ2XSIZEDEF;
		g_sets.eq2ysize[0] = EQ2YSIZEDEF;
		g_sets.effectsxpos[0] = EFFECTSXPOSDEF;
		g_sets.effectsypos[0] = EFFECTSYPOSDEF;
		g_sets.effectsxsize[0] = EFFECTSXSIZEDEF;
		g_sets.effectsysize[0] = EFFECTSYSIZEDEF;
		g_sets.listxpos[0] = LISTXPOSDEF;
		g_sets.listypos[0] = LISTYPOSDEF;
		g_sets.listxsize[0]	= LISTXSIZEDEF;
		g_sets.listysize[0]	= LISTYSIZEDEF;
		g_sets.listcolxsize[0][0] = g_sets.listcolxsize[0][1] =
		g_sets.listcolxsize[0][2] = g_sets.listcolxsize[0][3] = LISTCOLXSIZEDEF;
	}
	else if(winconf >= 1 && winconf < NUMWINCONF)
	{
		g_sets.xpos[winconf] = XPOSDEF2;
		g_sets.ypos[winconf] = YPOSDEF2;
		g_sets.xsize[winconf] = XSIZEDEF2;
		g_sets.ysize[winconf] = YSIZEDEF2;
		g_sets.eqxpos[winconf] = EQXPOSDEF2;
		g_sets.eqypos[winconf] = EQYPOSDEF2;
		g_sets.eqxsize[winconf] = EQXSIZEDEF2;
		g_sets.eqysize[winconf] = EQYSIZEDEF2;
		g_sets.eq2xpos[winconf] = EQXPOSDEF2;
		g_sets.eq2ypos[winconf] = EQ2YPOSDEF2;
		g_sets.eq2xsize[winconf] = EQ2XSIZEDEF2;
		g_sets.eq2ysize[winconf] = EQ2YSIZEDEF2;
		g_sets.effectsxpos[winconf] = EFFECTSXPOSDEF2;
		g_sets.effectsypos[winconf] = EFFECTSYPOSDEF2;
		g_sets.effectsxsize[winconf] = EFFECTSXSIZEDEF2;
		g_sets.effectsysize[winconf] = EFFECTSYSIZEDEF2;
		g_sets.listxpos[winconf] = LISTXPOSDEF2;
		g_sets.listypos[winconf] = LISTYPOSDEF2;
		g_sets.listxsize[winconf]	= LISTXSIZEDEF2;
		g_sets.listysize[winconf]	= LISTYSIZEDEF2;
		g_sets.listcolxsize[winconf][0] = g_sets.listcolxsize[winconf][1] =
		g_sets.listcolxsize[winconf][2] = g_sets.listcolxsize[winconf][3] = LISTCOLXSIZEDEF2;
	}
	
	if(winconf == 0)
	{
		g_sets.fileoutxpos[0] = FILEOUTXPOSDEF;
		g_sets.fileoutypos[0] = FILEOUTYPOSDEF;
		g_sets.fileoutxsize[0] = FILEOUTXSIZEDEF;
		g_sets.fileoutysize[0] = FILEOUTYSIZEDEF;
	}
	else // if(winconf >= 1 && winconf < NUMWINCONF)
	{
		g_sets.fileoutxpos[winconf] = FILEOUTXPOSDEF2;
		g_sets.fileoutypos[winconf] = FILEOUTYPOSDEF2;
		g_sets.fileoutxsize[winconf] = FILEOUTXSIZEDEF2;
		g_sets.fileoutysize[winconf] = FILEOUTYSIZEDEF2;
	}

} // WindowSetFactoryValues


void SetFileoutType(int which)
{
	int i, first, last;
	
	if(which < 0)
	{
		first = kOUTFIRST;
		last = kOUTLAST;
		for(i = 1; i < kOUTMAX; i++) // 0 is kOUTSOUND
			g_sets.fileout[g_k[i]].type = -1;
	}
	else
		first = last = which;
			
	for(i = first; i <= last; i++)
	{
 #ifdef ENCODEMP3
		if(i == kOUTMP3)
		{
			g_sets.fileout[g_k[i]].type = kMAMP3;
			continue;
		}
 #endif
 #ifdef ENCODEOGG
		if(i == kOUTOGG)
		{
			g_sets.fileout[g_k[i]].type = kMAOGG;
			continue;
		}

 #endif
 #ifdef ENCODEFLAC
		if(i == kOUTFLAC)
		{
			g_sets.fileout[g_k[i]].type = kMAFLAC;
			continue;
		}
 #endif
 #ifdef ENCODEAAC
		if(i == kOUTAAC)
		{
			g_sets.fileout[g_k[i]].type = kMAAAC;
			continue;
		}
 #endif
 #ifdef ENCODEMP4
		if(i == kOUTMP4)
		{
			g_sets.fileout[g_k[i]].type = kMAMP4;
			continue;
		}
 #endif
 #ifdef ENCODEWMA
		if(i == kOUTWMA)
		{
			g_sets.fileout[g_k[i]].type = kMAWMA;
			continue;
		}
 #endif
 #ifdef ENCODEWAV
		if(i == kOUTWAV)
		{
			g_sets.fileout[g_k[i]].type = kMAWAV;
			continue;
		}
 #endif
 #ifdef ENCODEAIF
		if(i == kOUTAIF)
		{
			g_sets.fileout[g_k[i]].type = kMAAIF;
			continue;
		}
 #endif
	}

} // SetFileoutType


void LoadFactoryFileout(int which)
{
	int i, first, last;
	
	if(which < 0)
	{
		first = kOUTFIRST;
		last = kOUTLAST;
	}
	else
		first = last = which;
	
	SetFileoutType(which);

	for(i = first; i <= last; i++)
	{
		g_sets.fileout[g_k[i]].bitrate = FILEOUTBITRATEDEF;
		g_sets.fileout[g_k[i]].quality = FILEOUTQUALITYDEF;
		g_sets.fileout[g_k[i]].mode = FILEOUTMODEDEF;
		g_sets.fileout[g_k[i]].samplerate = FILEOUTSAMPLERATEDEF;
		strcpy(g_sets.fileout[g_k[i]].outdir, g_fileoutdir);

		switch(g_sets.fileout[g_k[i]].type)
		{
			case kMAMP3:
				g_sets.fileout[g_k[i]].quality = FILEOUTQUALITYMP3DEF;
			break;
			case kMAOGG:
				g_sets.fileout[g_k[i]].quality = FILEOUTQUALITYOGGDEF; // not used
			break;
			case kMAAAC:
				g_sets.fileout[g_k[i]].quality = FILEOUTQUALITYAACDEF;
				g_sets.fileout[g_k[i]].mode |= kACCADTS;
			break;
			case kMAMP4:
				g_sets.fileout[g_k[i]].quality = FILEOUTQUALITYMP4DEF;
			break;
			default:
			break;
		}		
	}

	g_sets.fileout[kOUTSOUND].mode = MACLOSESOUNDDEF;
	
} // LoadFactoryFileout

	
int LoadFactorySettings(MASETS *psets)
{
	int i, j;
	int mx, my;
	
	for(i = -1; i < NUMWINCONF; i++) // -1 for windows with single conf (readme, etc.)
		WindowSetFactoryValues(i);

	// if any window is offscreen (pretty much arbitrarily...) bring all windows on screen
	DisplayGetSize(&mx, &my);

	if(mx > 20 && my > 20) // if screen is not too small...
	{	
		for(i = 0; i < NUMWINCONF; i++)
		{
			if(psets->xpos[i]		> mx - 20 || psets->ypos[i]		> my - 20 ||
				psets->eqxpos[i]	> mx - 20 || psets->eqypos[i]   > my - 20 ||
				psets->eq2xpos[i]	> mx - 20 || psets->eq2ypos[i]  > my - 20 ||
				psets->listxpos[i]	> mx - 20 || psets->listypos[i]	> my - 20)
			{
				psets->xpos[i]		= 0;
				psets->ypos[i]		= 0;
				psets->eqxpos[i]	= psets->xpos[i] + 20;
				psets->eqypos[i]	= psets->ypos[i] + 20;
				psets->eq2xpos[i]	= psets->xpos[i] + 20 + EQ2POSDELTA;
				psets->eq2ypos[i]	= psets->ypos[i] + 20 + EQ2POSDELTA;
				psets->listxpos[i]	= psets->eqxpos[i] + 20;
				psets->listypos[i]	= psets->eqypos[i] + 20;
			}

			if(psets->xsize[i]		>= abs(mx - psets->xpos[i]))		psets->xsize[i]		= mx / 4;
			if(psets->ysize[i]		>= abs(my - psets->ypos[i]))		psets->ysize[i]		= my / 4;
			if(psets->eqxsize[i]    >= abs(mx - psets->eqxpos[i]))		psets->eqxsize[i]   = mx / 2;
			if(psets->eqysize[i]    >= abs(my - psets->eqypos[i]))		psets->eqysize[i]   = psets->ysize[i];
			if(psets->eq2xsize[i]   >= abs(mx - psets->eq2xpos[i]))		psets->eq2xsize[i]  = mx / 2;
			if(psets->eq2ysize[i]   >= abs(my - psets->eq2ypos[i]))		psets->eq2ysize[i]  = psets->ysize[i];
			if(psets->listxsize[i]	>  abs(mx - psets->listxpos[i]))	psets->listxsize[i]	= mx / 4;
			if(psets->listysize[i]	>= abs(my - psets->listypos[i]))	psets->listysize[i]	= psets->ysize[i];

			if(psets->eqxpos[i]	> mx - 20 || psets->eqypos[i]   > my - 20 ||
				psets->listxpos[i]	> mx - 20 || psets->listypos[i]	> my - 20)
			{
				psets->xpos[i]		= 0;
				psets->ypos[i]		= 0;
				psets->eqxpos[i]	= psets->xpos[i] + 20;
				psets->eqypos[i]	= psets->ypos[i] + 20;
				psets->eq2xpos[i]	= psets->xpos[i] + 20 + EQ2POSDELTA;
				psets->eq2ypos[i]	= psets->ypos[i] + 20 + EQ2POSDELTA;
				psets->listxpos[i]	= psets->eqxpos[i] + 20;
				psets->listypos[i]	= psets->eqypos[i] + 20;
			}
		} // for i
	}
	
	psets->volumesliderleft		= VOLUMEMAX;
	psets->volumesliderright	= VOLUMEMAX;
	psets->separationslider		= SEPARATIONMAX;
	psets->panslider			= SEPARATIONMIN + (SEPARATIONMAX - SEPARATIONMIN) / 2;
	psets->preampslider			= PREAMPMIN + 5 * (PREAMPMAX - PREAMPMIN) / 8; // +4 db
	psets->pitchslider			= EFFECTSPITCHMIN + (EFFECTSPITCHMAX - EFFECTSPITCHMIN) / 2;
	psets->pitchslider2			= EFFECTSMIN + (EFFECTSMAX - EFFECTSMIN) / 2;
	psets->temposlider			= 26; // 1.0, 0.48 to 2.48 // EFFECTSMIN + (EFFECTSMAX - EFFECTSMIN) / 2;
	psets->speedslider			= 26; // 1.0, 0.48 to 2.48 // EFFECTSMIN + (EFFECTSMAX - EFFECTSMIN) / 2;
	psets->autostart = 0;
	psets->singleplay = 0;
	psets->spindown = 0;
	psets->random = 0;
	psets->reptplayed = 0;
	psets->looploaded = 0;
	psets->looplist = 1;
	psets->highpriority = 0;
	psets->effects = EFFECTSPITCH | EFFECTSTEMPO | EFFECTSSPEED;
	psets->limiter = PREAMP | CLIPPING; 
	psets->balance = 0;
	psets->monodiff = 0;
	psets->invertright = 0;
	psets->eqflags = MAEQACTIVE | MAEQWINDOW | MAEQ31BANDS | MAEQEXTRA | MAEQZOOM | MAEQLINK;
	psets->effectswindow = 0;
	psets->listwindow = LBOXON | LBOXLOADED | LBOXINDEX | LBOXPATH;
	psets->fileoutwindow = 0;
	psets->eqpreset = 0;
	psets->vdiskname[0] = 0;
	psets->buffertype = BUFUSEMEM;
	psets->output = kOUTSOUND;
	psets->buffersize = SONGMEMDEF;
	psets->titleinfo = 0;
	psets->winconf = 0;
	psets->logflags = kMALOGMAX | kMALOGRATE;
	psets->logmax = MALOGMAXDEF;
	psets->logevents = MALOGEVENTSDEF;
	psets->logperiod = MALOGPERIODDEF;
#ifdef MAINSTANCES
	psets->multipleinst = 1;
#else
	psets->multipleinst = 0;
#endif
	
	// equalizer offsets and names
	for(i = 0; i < EQNUMPRESETS; i++)
 	{
 		psets->eqpresets[i].eqoffset = 0;
		sprintf(psets->eqpresets[i].eqname, "Preset%d", i + 1);
		for(j = 0; j < EQBANDS31; j++)
			psets->eqpresets[i].eqslider[j] = DBSLIDERMIN + (DBSLIDERMAX - DBSLIDERMIN) / 2;
	}
	
	// equalizer settings (for DBSLIDERMIN == 0 and DBSLIDERMAX == 63 and range -DBRANGE...DBRANGE dB)
	strcpy(psets->eqpresets[0].eqname, "Rock");
	psets->eqpresets[0].eqoffset = 4; // 4db
	psets->eqpresets[0].eqslider[0] = //SliderPosFromDB(8);
	psets->eqpresets[0].eqslider[1] = //SliderPosFromDB(8);
	psets->eqpresets[0].eqslider[2] = SliderPosFromDB(8);
	psets->eqpresets[0].eqslider[3] = //SliderPosFromDB(7);
	psets->eqpresets[0].eqslider[4] = SliderPosFromDB(7);
	psets->eqpresets[0].eqslider[5] = SliderPosFromDB(6);
	psets->eqpresets[0].eqslider[6] = SliderPosFromDB(4.5);
	psets->eqpresets[0].eqslider[7] = SliderPosFromDB(2.5);
	psets->eqpresets[0].eqslider[8] = SliderPosFromDB(0.5);
	psets->eqpresets[0].eqslider[9] = SliderPosFromDB(-1.5);
	psets->eqpresets[0].eqslider[10] = SliderPosFromDB(-3.5);
	psets->eqpresets[0].eqslider[11] = SliderPosFromDB(-4.5);
	psets->eqpresets[0].eqslider[12] = SliderPosFromDB(-6.5);
	psets->eqpresets[0].eqslider[13] = SliderPosFromDB(-8.5);
	psets->eqpresets[0].eqslider[14] = SliderPosFromDB(-10);
	psets->eqpresets[0].eqslider[15] = SliderPosFromDB(-9.5);
	psets->eqpresets[0].eqslider[16] = SliderPosFromDB(-8);
	psets->eqpresets[0].eqslider[17] = SliderPosFromDB(-7);
	psets->eqpresets[0].eqslider[18] = SliderPosFromDB(-4);
	psets->eqpresets[0].eqslider[19] = SliderPosFromDB(-2.5);
	psets->eqpresets[0].eqslider[20] = SliderPosFromDB(1);
	psets->eqpresets[0].eqslider[21] = SliderPosFromDB(2.5);
	psets->eqpresets[0].eqslider[22] = SliderPosFromDB(4);
	psets->eqpresets[0].eqslider[23] = //SliderPosFromDB(6);
	psets->eqpresets[0].eqslider[24] = //SliderPosFromDB(6);
	psets->eqpresets[0].eqslider[25] = SliderPosFromDB(6);
	psets->eqpresets[0].eqslider[26] = //SliderPosFromDB(5.5);
	psets->eqpresets[0].eqslider[27] = //SliderPosFromDB(5.5);
	psets->eqpresets[0].eqslider[28] = //SliderPosFromDB(5.5);
	psets->eqpresets[0].eqslider[29] = //SliderPosFromDB(5.5);
	psets->eqpresets[0].eqslider[30] = SliderPosFromDB(5.5);
	
	strcpy(psets->eqpresets[1].eqname, "Up");
	strcpy(psets->eqpresets[2].eqname, "Down");

	for(j = 0; j < EQBANDS31; j++)
	{
		psets->eqpresets[1].eqslider[j] =
				DBSLIDERMIN + ((j * (DBSLIDERMAX - DBSLIDERMIN)) / (EQBANDS31 - 1)); // up
	
		psets->eqpresets[2].eqslider[EQBANDS31 - 1 - j] =
				DBSLIDERMIN + ((j * (DBSLIDERMAX - DBSLIDERMIN)) / (EQBANDS31 - 1)); // down
	}
	memcpy(psets->eq2presets, psets->eqpresets, sizeof(psets->eq2presets));
	
	// different settings for cda	
	if(SongListIsCda())
	{
		psets->titleinfo = 1;
		// psets->random = 0; already set above
		psets->reptplayed = 1;
		psets->limiter = 0; 
		psets->buffertype = BUFUSEDIRECT; // cda does not use memory but builds a 'loaded' list
		psets->listwindow = LBOXON | LBOXLIST | LBOXINDEX | LBOXPATH;
	}
	
	LoadFactoryFileout(-1);

 #if defined(MAINSTANCES)
	if(g_thisinstance == 1)
		SetMultipleInstances();
 #endif

 #if defined(MAKEYBOARD) || defined(MAGUINONE)
	psets->keyboard = 1;
 #else
	psets->keyboard = 0;
 #endif

 #if defined(MAMOUSEWHEEL)
	psets->mousewheel = 1;
 #else
	psets->mousewheel = 0;
 #endif

	psets->usem3u = USEM3UDEF;
	psets->addmode = ADDMODEDEF;
	
	strcpy(psets->settingsversion, MOREAMP_SETTINGS_VERSION_STRING);
	 
	// conservative settings for slow OS9 machines
 #if defined(__MAOS9__)
	psets->eqflags &= ~(MAEQACTIVE | MAEQWINDOW | MAEQ31BANDS | MAEQEXTRA);
	psets->listwindow &= ~LBOXON;
	psets->buffertype = BUFUSEDIRECT;
  #ifdef MAINSTANCES
		psets->multipleinst = 0;
  #endif
  #if defined(MAKEYBOARD) && !defined(MAGUINONE)
		psets->keyboard = 0;
  #endif
 #endif 
	 
	return 0;
	
} // LoadFactorySettings


#if !defined(NODECODEAUDIOIN)
int LoadFactorySettingsAudioIn(MASETS *psets)
{
	int err;
	
	if((err = LoadFactorySettings(psets)))
	{
		maLog("LoadFactorySettingsAudioIn LoadFactorySettings failed");
		return err;
	}
	
	psets->listwindow = LBOXINFO | LBOXINDEX | LBOXPATH;			// List window off
	psets->eqflags = MAEQZOOM | MAEQLINK;							// Eq off
	psets->limiter = 0;												// Preamp off
	psets->preampslider = PREAMPMIN + (PREAMPMAX - PREAMPMIN) / 2;	// midpoint = 0 dB
	psets->volumesliderleft = psets->volumesliderright =
						VOLUMEMIN +  0.7 * (VOLUMEMAX - VOLUMEMIN);	// Volume -30 dB
	psets->buffertype = BUFUSEDIRECT;								// Buffer Direct
	
	return 0;
	
} // LoadFactorySettingsAudioIn
#endif


#if !defined(MANOCDA) && defined(__MAUNIX__)
int LoadFactorySettingsCDRom()
{
	int i;
	
	g_setscdrom.cdparanoia = 0;	
	for(i = 0; i <= CDAMAXNUMDRIVES; i++)
		sprintf(g_setscdrom.cdaname[i], "(drive%02d)", i);
	strcpy(g_setscdrom.cdaname[1], MACDANAMEDEF);
	g_setscdrom.item = 0;
	g_setscdrom.winx[0] = AUDIOCDXPOSDEF;
	g_setscdrom.winy[0] = AUDIOCDYPOSDEF;
	g_setscdrom.winw[0] = AUDIOCDXSIZEDEF;
	g_setscdrom.winh[0] = AUDIOCDYSIZEDEF;
	g_setscdrom.winx[1] = AUDIOCDXPOSDEF2;
	g_setscdrom.winy[1] = AUDIOCDYPOSDEF2;
	g_setscdrom.winw[1] = AUDIOCDXSIZEDEF2;
	g_setscdrom.winh[1] = AUDIOCDYSIZEDEF2;

	return 0;
	
} // LoadFactorySettingsCDRom
#endif


int LoadDefaultSettings(MASETS *psets)
{
	int ret;
	int version;
	FILE *fp;
	char settingsversion[VERSIONSTRINGSIZE];

	if((fp = mafopen(g_settingsdef, "rb")) == NULL)
		return LoadFactorySettings(psets);

	// check version
	fseek(fp, 0L - sizeof(psets->settingsversion), SEEK_END);
	fread(settingsversion, 1, sizeof(psets->settingsversion), fp);
	if((version = maCheckSettingsVersion(settingsversion)) == MAVERSIONISCURRENT)
	{
		rewind(fp);
		ret = fread(psets, 1, sizeof(MASETS), fp);
		fclose(fp);
		fp = NULL;
		if(ret != sizeof(MASETS))
		{
			maLog("LoadDefaultSettings error reading default settings, saving factory settings as defaults");
			if(LoadFactorySettings(psets))
				return 1;
			SaveSettingsAsDefault(psets);
			return 0;
		}
	}
#ifdef MOREAMP_PREVIOUS_VERSION_STRING
	else if(version == MAVERSIONISPREVIOUS)
	{
		char cstr[256];
		FixSettingsPrevious(fp, psets);
		fclose(fp);
		fp = NULL;
		SaveSettingsAsDefault(psets);
		sprintf(cstr, "LoadDefaultSettings updated default settings from settings version %s to %s",
													settingsversion, MOREAMP_SETTINGS_VERSION_STRING);
		maLog(cstr);
	}
#endif
	else // MAVERSIONISUNKNOWN
	{
		maLog("LoadDefaultSettings unknown settings version, saving factory settings as defaults");
		fclose(fp);
		if(LoadFactorySettings(psets))
			return 1;
		SaveSettingsAsDefault(psets);
		return 0;
	}
		
	if(fp)
		fclose(fp);		
		
	CheckSettings(psets);
	
#if defined(MAINSTANCES)
	if(g_thisinstance == 1)
		SetMultipleInstances();
#endif
			
	return 0;
	
} // LoadDefaultSettings


int SaveSettingsAsDefault(MASETS *psets)
{
	int ret;
	FILE *fp;
	char cstr[256];
	
	if((fp = mafopen(g_settingsdef, "wb")) == NULL)
	{		
		sprintf(cstr, "SaveSettingsAsDefault error mafopen '%s'", g_settingsdef);
		maLog(cstr);
		return 1;
	}

	/* // ss why was this here?
	if(g_w[WINDOWEQ])
			WindowGetCoordinates(WINDOWEQ, &psets->eqxpos[psets->winconf], &psets->eqypos[psets->winconf],
											&psets->eqxsize[psets->winconf], &psets->eqysize[psets->winconf]);
	
	if(g_w[WINDOWLIST])
			WindowGetCoordinates(WINDOWLIST, &psets->listxpos[psets->winconf], &psets->listypos[psets->winconf],
											 &psets->listxsize[psets->winconf], &psets->listysize[psets->winconf]);

	int i;
	for(i = kOUTFIRST; i <= kOUTLAST; i++)
	{
		if(g_w[WINDOWOUTFIRST + i - 1]) // WINDOWOUTCUR
		{
			WindowGetCoordinates(WINDOWOUTFIRST + i - 1,
				&psets->fileoutxpos[psets->winconf], &psets->fileoutypos[psets->winconf],
				&psets->fileoutxsize[psets->winconf], &psets->fileoutysize[psets->winconf]);
			break;
		}
	}
	*/
	
#if defined(__MAOSX__) || defined(__MAUNIX__)
	machmod(g_settingsdef, ACCESSPERMS); // 0777
#endif

	// save current settings into default settings file	
	ret = fwrite(psets, 1, sizeof(MASETS), fp);
	
	fclose(fp);		
		
	if(ret != sizeof(MASETS))
	{		
		sprintf(cstr, "%d SaveSettingsAsDefault error fwrite '%s'", ret, g_settingsdef);
		maLog(cstr);
		return 2;
	}
				
	return 0;
	
} // SaveSettingsAsDefault


void SetPause(int pause)
{
	if(pause && !l_ispaused && g_running)
	{
		PushButtonPaintDown(WINDOWAPP, ID_BTN_PAUSE);
		l_ispaused = 1;
		
		l_sliderposmsbu = l_sliderposms;
		
		MenuSetChecked(MENUMACONTROLS, ID_MENU_PAUSE, 1);

#if !defined(MAAUDIONONE)
		maPause(1);
#endif
	}
	else if(l_ispaused)
	{
		PushButtonPaintUp(WINDOWAPP, ID_BTN_PAUSE);
		l_ispaused = 0;

		if(l_sliderposmsbu != l_sliderposms && g_running) // user moved slider while paused
		{
			l_sliderposmsbu = l_sliderposms;
			PlayerSeek(l_sliderposms);
			l_playerseeking = 0;
		}
		
		MenuSetChecked(MENUMACONTROLS, ID_MENU_PAUSE, 0);
		
#if !defined(MAAUDIONONE)
		maPause(0);		
#endif
	}
	else
		MenuSetChecked(MENUMACONTROLS, ID_MENU_PAUSE, 0);
	
} // SetPause


void OnCheckInvertRight(int checked)
{
	g_sets.invertright = checked;
	MenuSetChecked(MENUMASETS, ID_MENU_INVERTRIGHT, g_sets.invertright ? 1 : 0);

} // OnCheckInvertRight


void OnCheckEq31Bands(int checked)
{
	if(checked)
	{
		g_sets.eqflags |= MAEQ31BANDS;
		if(g_w[WINDOWEQ])
			CheckBoxSetValue(WINDOWEQ, ID_CHK_EQ31BANDSB, 1);
		MenuSetChecked(MENUMASETS, ID_MENU_EQ31BANDS, 1);
		maEqualizerReset(EQBANDS31, 1, g_sets.eqflags & MAEQEXTRA); // bands, 16bit, extra	
	}
	else
	{
		g_sets.eqflags &= ~MAEQ31BANDS;
		if(g_w[WINDOWEQ])
			CheckBoxSetValue(WINDOWEQ, ID_CHK_EQ31BANDSB, 0);
		MenuSetChecked(MENUMASETS, ID_MENU_EQ31BANDS, 0);
		maEqualizerReset(EQBANDS10, 1, g_sets.eqflags & MAEQEXTRA); // bands, 16bit, extra	
	}

	EqDoParamChange();

} // OnCheckEq31Bands


void OnCheckEq31BandsB(int checked)
{
	if(checked)
	{
		g_sets.eqflags |= MAEQ31BANDS;
		CheckBoxSetValue(WINDOWAPP, ID_CHK_EQ31BANDS, 1);
		MenuSetChecked(MENUMASETS, ID_MENU_EQ31BANDS, 1);
		maEqualizerReset(EQBANDS31, 1, g_sets.eqflags & MAEQEXTRA); // bands, 16bit, extra	
	}
	else
	{
		g_sets.eqflags &= ~MAEQ31BANDS;
		CheckBoxSetValue(WINDOWAPP, ID_CHK_EQ31BANDS, 0);
		MenuSetChecked(MENUMASETS, ID_MENU_EQ31BANDS, 0);
		maEqualizerReset(EQBANDS10, 1, g_sets.eqflags & MAEQEXTRA); // bands, 16bit, extra	
	}

	EqDoParamChange();

} // OnCheckEq31BandsB


void SetSliders(int slider)
{
	int i, first, last;
	
	if((slider == EQSLIDERS || (slider >= EQSLIDERFIRST && slider <= EQSLIDERLAST)) && !g_w[WINDOWEQ])
		return;
	
	if((slider == EQ2SLIDERS || (slider >= EQ2SLIDERFIRST && slider <= EQ2SLIDERLAST)) && !g_w[WINDOWEQ2])
		return;
	
	// setup loop first to last consecutive sliders to set	
	if(slider == ALLSLIDERS)
	{
		first = EFFECTSSLIDERS;
		
		if(g_w[WINDOWEQ2])
			last = EQ2SLIDERLAST;
		else if(g_w[WINDOWEQ])
			last = EQSLIDERLAST;
		else
			last = PREAMPSLIDER;
	}
	else if(slider == EQSLIDERS)
	{
		first = EQSLIDERFIRST;
		last = EQSLIDERLAST;
	}
	else if(slider == EQ2SLIDERS)
	{
		first = EQ2SLIDERFIRST;
		last = EQ2SLIDERLAST;
	}
	else if(slider == EFFECTSSLIDERS) // all effects sliders
	{
		first = EFFECTSSLIDERSPEED;
		last = EFFECTSSLIDERPITCH;
	}
	else
		first = last = slider;
	
	// set sliders	
	for(i = first; i <= last; i++)
	{
		switch (i)
		{
			case VOLUMESLIDERLEFT :
				SliderSetValue(WINDOWAPP, ID_SLIDER_VOLUMEL, volminmaxL());
				SetSliderCaption(VOLUMESLIDERLEFT);
				continue;
			break;
			
			case VOLUMESLIDERRIGHT :
				SliderSetValue(WINDOWAPP, ID_SLIDER_VOLUMER, volminmaxR());
				SetSliderCaption(VOLUMESLIDERRIGHT);
				continue;
			break;
			
			case SEPARATIONSLIDER :
				if(l_outputfilter == OUTPUT_DIFF || l_outputfilter == OUTPUT_MONO) // pan
					SliderSetValue(WINDOWAPP, ID_SLIDER_VOLUMESEP, g_sets.panslider);
				else
					SliderSetValue(WINDOWAPP, ID_SLIDER_VOLUMESEP, g_sets.separationslider);
				SetSliderCaption(SEPARATIONSLIDER);
				continue;
			break;
			
			case PREAMPSLIDER :
				SliderSetValue(WINDOWAPP, ID_SLIDER_PREAMP, g_sets.preampslider);
				SetSliderCaption(PREAMPSLIDER);
				continue;
			break;
			
			case VOLUMESLIDERS: // both left and right volume
				SliderSetValue(WINDOWAPP, ID_SLIDER_VOLUMEL, volminmaxL());
				SliderSetValue(WINDOWAPP, ID_SLIDER_VOLUMER, volminmaxR());
				SetSliderCaption(VOLUMESLIDERS);
				continue;
			break;
			
			case EFFECTSSLIDERPITCH:
				if(g_w[WINDOWEFFECTS])
				{
					SliderSetValue(WINDOWEFFECTS, ID_SLIDER_PITCH, g_sets.pitchslider);
					SetSliderCaption(EFFECTSSLIDERPITCH);
				}
				continue;
			break;

			case EFFECTSSLIDERPITCH2:
				if(g_w[WINDOWEFFECTS])
				{
					SliderSetValue(WINDOWEFFECTS, ID_SLIDER_PITCH2, g_sets.pitchslider2);
					SetSliderCaption(EFFECTSSLIDERPITCH2);
				}
				continue;
			break;

			case EFFECTSSLIDERTEMPO:
				if(g_w[WINDOWEFFECTS])
				{
					SliderSetValue(WINDOWEFFECTS, ID_SLIDER_TEMPO, g_sets.temposlider);
					SetSliderCaption(EFFECTSSLIDERTEMPO);
				}
				continue;
			break;

			case EFFECTSSLIDERSPEED:
				if(g_w[WINDOWEFFECTS])
				{
					SliderSetValue(WINDOWEFFECTS, ID_SLIDER_SPEED, g_sets.speedslider);
					SetSliderCaption(EFFECTSSLIDERSPEED);
				}
				continue;
			break;

			case EFFECTSSLIDERS: // all effect sliders
				if(g_w[WINDOWEFFECTS])
				{
					SliderSetValue(WINDOWEFFECTS, ID_SLIDER_PITCH, g_sets.pitchslider);
					SetSliderCaption(EFFECTSSLIDERPITCH);
					SliderSetValue(WINDOWEFFECTS, ID_SLIDER_PITCH2, g_sets.pitchslider2);
					SetSliderCaption(EFFECTSSLIDERPITCH2);
					SliderSetValue(WINDOWEFFECTS, ID_SLIDER_TEMPO, g_sets.temposlider);
					SetSliderCaption(EFFECTSSLIDERTEMPO);
					SliderSetValue(WINDOWEFFECTS, ID_SLIDER_SPEED, g_sets.speedslider);
					SetSliderCaption(EFFECTSSLIDERSPEED);
				}
				continue;
			break;

			default:
				if(i >= EQSLIDERFIRST && i <= EQSLIDERLAST)
				{
					SliderSetValue(WINDOWEQ, ID_SLIDER_EQ0 + i,  g_sets.eqpresets[g_sets.eqpreset].eqslider[i]);	
					SetSliderCaption(i);
				}
				else if(i >= EQ2SLIDERFIRST && i <= EQ2SLIDERLAST)
				{
					SliderSetValue(WINDOWEQ2, ID_SLIDER_EQ0 + i, g_sets.eq2presets[g_sets.eqpreset].eqslider[i - EQ2SLIDERFIRST]);	
					SetSliderCaption(i);
				}
				continue;
			break;
		} // switch
			
	} // for

} // SetSliders


void SetSliderCaption(int slider)
{
	char cstr[64];
	
	if(!g_w[WINDOWEQ] && slider >= EQSLIDERFIRST && slider <= EQSLIDERLAST)
		return;
	if(!g_w[WINDOWEQ2] && slider >= EQ2SLIDERFIRST && slider <= EQ2SLIDERLAST)
		return;
		
	switch (slider)
	{
		case VOLUMESLIDERLEFT:
			volumevalstrL(cstr);
			StaticTextSetText(WINDOWAPP, ID_TEXT_100VOLL, cstr);
		break;
			
		case VOLUMESLIDERRIGHT:
			volumevalstrR(cstr);
			StaticTextSetText(WINDOWAPP, ID_TEXT_100VOLR, cstr);
		break;
			
		case VOLUMESLIDERS: // both Left and Right
			volumevalstrL(cstr);
			StaticTextSetText(WINDOWAPP, ID_TEXT_100VOLL, cstr);
			volumevalstrR(cstr);
			StaticTextSetText(WINDOWAPP, ID_TEXT_100VOLR, cstr);
		break;
			
		case SEPARATIONSLIDER:
			if(l_outputfilter == OUTPUT_DIFF || l_outputfilter == OUTPUT_MONO) // pan
				sprintf(cstr, "%d", (g_sets.panslider * SEPARATIONCALE) / SEPARATIONMAX);
			else
				sprintf(cstr, "%d", (g_sets.separationslider * SEPARATIONCALE) / SEPARATIONMAX);
			StaticTextSetText(WINDOWAPP, ID_TEXT_100SEP, cstr);
		break;
			
		case PREAMPSLIDER:
			sprintf(cstr, (char *)"%0.1f", SliderDBFromPos(g_sets.preampslider));
			StaticTextSetText(WINDOWAPP, ID_TEXT_0, cstr);
		break;

		case EFFECTSSLIDERPITCH:
			if(g_w[WINDOWEFFECTS])
			{
				sprintf(cstr, "%d", g_sets.pitchslider - EFFECTSPITCHMAX / 2);
				StaticTextSetText(WINDOWEFFECTS, ID_TEXT_PITCH0, cstr);
			}
		break;

		case EFFECTSSLIDERPITCH2:
			if(g_w[WINDOWEFFECTS])
			{
				sprintf(cstr, "%.2f", ((g_sets.pitchslider2 * 2.0f) / EFFECTSMAX - 1.0f));
				StaticTextSetText(WINDOWEFFECTS, ID_TEXT_PITCH02, cstr);
			}
		break;

		case EFFECTSSLIDERTEMPO:
			if(g_w[WINDOWEFFECTS])
			{
				sprintf(cstr, "%.2f", (g_sets.temposlider * 2.0f) / EFFECTSMAX + 0.48f); // 0.48 to 2.48
				StaticTextSetText(WINDOWEFFECTS, ID_TEXT_TEMPO0, cstr);
			}
		break;

		case EFFECTSSLIDERSPEED:
			if(g_w[WINDOWEFFECTS])
			{
				sprintf(cstr, "%.2f", (g_sets.speedslider * 2.0f) / EFFECTSMAX + 0.48f); // 0.48 to 2.48
				StaticTextSetText(WINDOWEFFECTS, ID_TEXT_SPEED0, cstr);
			}
		break;

		default:
			if(slider >= EQSLIDERFIRST && slider <= EQSLIDERLAST)
			{
				sprintf(cstr, (char *)"%0.1f",
							SliderDBFromPos(g_sets.eqpresets[g_sets.eqpreset].eqslider[slider]));
				StaticTextSetText(WINDOWEQ, ID_TEXT_EQ0 + slider, cstr);	
			}
			else if(slider >= EQ2SLIDERFIRST && slider <= EQ2SLIDERLAST)
			{
				if(g_w[WINDOWEQ2])
				{		
					sprintf(cstr, (char *)"%0.1f",
							SliderDBFromPos(g_sets.eq2presets[g_sets.eqpreset].eqslider[slider - EQ2SLIDERFIRST]));
					StaticTextSetText(WINDOWEQ2, ID_TEXT_EQ0 + slider, cstr);	
				}
			}
		break;
		
	} // switch

} // SetSliderCaption


void UpdateMenus()
{
	MenuSetChecked(MENUSHOW,	ID_MENU_ALTSIZE,	(g_sets.winconf) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_SPINDOWN,	(g_sets.spindown) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_RANDOM,		(g_sets.random) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_REPTPLAYED,	(g_sets.reptplayed) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_LOOPLOADED,	(g_sets.looploaded) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_LOOPLIST,	(g_sets.looplist) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_AUTOSTART,	(g_sets.autostart) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_ERRLOGGING,	(g_sets.logflags & kMALOGOFF) ? 0 : 1);
	MenuSetChecked(MENUMASETS,	ID_MENU_SINGLEPLAY,	(g_sets.singleplay) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_HIPRIORITY,	(g_sets.highpriority) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_EQUALIZER,	(g_sets.eqflags & MAEQACTIVE) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_EFFECTS,	(g_sets.effects & EFFECTSON) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_EQEXTRA,	(g_sets.eqflags & MAEQEXTRA) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_INVERTRIGHT,(g_sets.invertright) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_MONO,		(g_sets.monodiff & (MONOCHECKBOXL + MONOCHECKBOXR)) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_EQ31BANDS,	(g_sets.eqflags & MAEQ31BANDS) ? 1 : 0);
#if defined(MAINSTANCES)
	if(g_thisinstance == 1)
		MenuSetChecked(MENUMASETS, ID_MENU_MULTIPLEINST, (g_sets.multipleinst) ? 1 : 0);
 #if defined(MAGUIOSX) || defined(MAGUIGTK)
 	else
		MenuDisable(MENUMASETS, ID_MENU_MULTIPLEINST);
 #endif
#endif
#if defined(MAKEYBOARD)
	MenuSetChecked(MENUMASETS, ID_MENU_KEYBOARD,	(g_sets.keyboard) ? 1 : 0);
#endif
#if defined(MAMOUSEWHEEL)
	MenuSetChecked(MENUMASETS, ID_MENU_MOUSEWHEEL,	(g_sets.mousewheel) ? 1 : 0);
#endif
	MenuSetChecked(MENUSHOW,	ID_MENU_SHOWEQ,			(g_sets.eqflags & MAEQWINDOW) ? 1 : 0);
	MenuSetChecked(MENUSHOW,	ID_MENU_SHOWEFFECTS,	(g_sets.effectswindow) ? 1 : 0);
	MenuSetChecked(MENUSHOW,	ID_MENU_SHOWLIST,		(g_sets.listwindow & LBOXON) ? 1 : 0);
	MenuSetChecked(MENUSHOW,	ID_MENU_SHOWFILEOUT,	(g_sets.fileoutwindow) ? 1 : 0);
}


void UpdateDisplay()
{
	char cstr[64];
	
	WindowMoveAndSetSize(WINDOWAPP, g_sets.xpos[g_sets.winconf], g_sets.ypos[g_sets.winconf],
								g_sets.xsize[g_sets.winconf], g_sets.ysize[g_sets.winconf]);
	UpdateMenus();
	
	CheckBoxSetValue(WINDOWAPP, ID_CHK_ALTSIZE,		(g_sets.winconf) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_BALANCE,		(g_sets.balance) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_SPINDOWN,	(g_sets.spindown) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_RANDOM,		(g_sets.random) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_REPTPLAYED,	(g_sets.reptplayed) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_LOOPLOADED,	(g_sets.looploaded) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_LOOPLIST,	(g_sets.looplist) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_EQWINDOW,	(g_sets.eqflags & MAEQWINDOW) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_EFFECTS,		(g_sets.effects & EFFECTSON) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_LISTWINDOW,	(g_sets.listwindow & LBOXON) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_SINGLEPLAY,	(g_sets.singleplay) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_USEM3U,		(g_sets.usem3u) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_EQUALIZER,	(g_sets.eqflags & MAEQACTIVE) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP,	ID_CHK_MONOLEFT,	(g_sets.monodiff & MONOCHECKBOXL) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_MONORIGHT,	(g_sets.monodiff & MONOCHECKBOXR) ? 1 : 0);
	RefreshMonoDiff(); //		ID_CHK_MONODIFF
	CheckBoxSetValue(WINDOWAPP,	ID_CHK_LIMITER,		(g_sets.limiter & CLIPPING) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP,	ID_CHK_EQEXTRA,		(g_sets.eqflags & MAEQEXTRA) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP,	ID_CHK_INVERTRIGHT,	(g_sets.invertright) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_EQ31BANDS,   (g_sets.eqflags & MAEQ31BANDS) ? 1 : 0);

	sprintf(cstr, "Buffer %d", g_sets.buffersize);
	StaticTextSetText(WINDOWAPP, ID_TEXT_BUFFER, cstr);

	SetGUIPopupButtons();
	SetGUIUseMem();
	SetGUIMonoStereo();
	SetGUITitleInfo();
	
	if(g_sets.limiter & CLIPPING)
		CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_LIMITER, (char *)"0.0");
	else
		CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_LIMITER, (char *)"Limit");
	
	// create or destroy eq, effects, list, and fileout windows, as in settings
	DoWindowList(g_sets.listwindow & LBOXON);
	DoWindowEq(g_sets.eqflags & MAEQWINDOW);	
	DoWindowEq2((g_sets.eqflags & MAEQWINDOW && (g_sets.eqflags & MASEPARATELR)) ? 1 : 0);
	DoWindowEffects(g_sets.effectswindow);
	DoWindowFileOut(g_sets.fileoutwindow);
	if(g_w[WINDOWADVANCED])
		DoWindowAdvanced(1);
	
	SetSliders(ALLSLIDERS);

#if defined(MAKEYBOARD)
	UpdateMenuKbLabels();
	UpdateEffectsKbLabels();
	UpdateButtonLabels();
#endif

	SetLogMenus();
	
} // UpdateDisplay


int SetGUIUseMem()
{
	switch (g_sets.buffertype)
	{
		case BUFUSEMEM :
		case BUFUSEVDISK :
			if(g_w[WINDOWLIST])
			{
				PushButtonSetDescriptor(WINDOWLIST, ID_BTN_LOADED, (char *)"Load");
				PushButtonEnable(WINDOWLIST, ID_BTN_LOADED);
			}
		break;

		case BUFUSEDIRECT :
			if(g_w[WINDOWLIST])
			{
				PushButtonSetDescriptor(WINDOWLIST, ID_BTN_LOADED, (char *)"(Direct)");
				PushButtonDisable(WINDOWLIST, ID_BTN_LOADED);
			}
		break;

		default :
		break;
	}

	return 0;
		
} // SetGUIUseMem


void SetGUIPopupButtons()
{
	if(g_sets.output >= kOUTSOUND && g_sets.output <= kOUTLAST)
		PopupButtonSetValue(WINDOWAPP, ID_POPUP_OUTPUT, g_sets.output); 
	else
		PopupButtonSetValue(WINDOWAPP, ID_POPUP_OUTPUT, kOUTSOUND); 

	switch (g_sets.buffertype)
	{
		case BUFUSEMEM :	PopupButtonSetValue(WINDOWAPP, ID_POPUP_BUFFER, 0); break;
		case BUFUSEVDISK :	PopupButtonSetValue(WINDOWAPP, ID_POPUP_BUFFER, 1); break;
		case BUFUSEDIRECT :
		default :			PopupButtonSetValue(WINDOWAPP, ID_POPUP_BUFFER, 2); break;
	}

	if(g_w[WINDOWEQ])
		PopupButtonSetValue(WINDOWEQ, ID_POPUP_EQPRESET, g_sets.eqpreset);
	
} // SetGUIPopupButtons


int ShowLoaded()
{
	int j;
	
	if(!g_w[WINDOWLIST])
		return 1;
		
	CheckBoxHide(WINDOWLIST, ID_CHK_SHOWINDEX);
	CheckBoxHide(WINDOWLIST, ID_CHK_SHOWDIR);
	CheckBoxHide(WINDOWLIST, ID_CHK_SHOWPATH);
	CheckBoxHide(WINDOWLIST, ID_CHK_ADDMODE);

	SetListWindowTitle((char *)"Songs Loaded");
	ListWindowClear();
	ListWindowSetHeader((char *)"Songs loaded");
	
	if(g_loadednum <= 0)
	{
		ListWindowAddString(NOSONGSLOADED);	
		ListWindowShowRow(-1); // scroll to top, no hilite
		return 1;
	}
	
	if(g_sets.buffertype == BUFUSEDIRECT)
		return 1;
		
	for(j = 0; j < g_loadednum; j++)
		ListWindowAddString(g_loadedname[j]);
		
	ListWindowShowRow(g_loadedcurrent);
	
	return 0;
	
} // ShowLoaded


void DoBeginEndEdit(int panelid, int param)
{
#ifdef MAUNUSED
 (void)param;
#endif

	char strhms[32];
	long outlen;
	int timeval;
	char cstr[256];
		
	EditTextGetText(WINDOWAPP, panelid, strhms, strlen(ZEROMSSTR), &outlen);
	strhms[outlen] = 0;
		
	if((timeval = StrHMSToMilliSeconds(strhms)) < 0)
	{
		sprintf(cstr, (char *)"Bad time string '%s'", strhms);
		maLog(cstr);
		return;
	}
	
	if(panelid == ID_TEXT_TIMEBEG)
		g_posbeg = timeval;
	else
		g_posend = timeval;

} // DoBeginEndEdit


void SetGUIMonoStereo()
{
	if(CheckBoxGetValue(WINDOWAPP, ID_CHK_MONOLEFT))
	{	
		StaticTextSetText(WINDOWAPP, ID_TEXT_OUTPUT, (char *)"Output Mono");
		if(CheckBoxGetValue(WINDOWAPP, ID_CHK_MONORIGHT)) // L and R both checked
		{
			CheckBoxShow(WINDOWAPP, ID_CHK_MONODIFF);			
			StaticTextSetText(WINDOWAPP, ID_TEXT_SEP, (char *)"Pan");
			StaticTextSetText(WINDOWAPP, ID_TEXT_M, (char *)"L");
			StaticTextSetText(WINDOWAPP, ID_TEXT_S, (char *)"R");
			
			if(CheckBoxGetValue(WINDOWAPP, ID_CHK_MONODIFF))
			{
				l_outputfilter = OUTPUT_DIFF;
				CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_MONODIFF, (char *)"L-R");
			}
			else
			{
				l_outputfilter = OUTPUT_MONO;
				CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_MONODIFF, (char *)"L+R");
			}

			SliderEnable(WINDOWAPP, ID_SLIDER_VOLUMESEP);
			SetSliders(SEPARATIONSLIDER);
			MenuSetChecked(MENUMASETS, ID_MENU_MONO, 1);
		}
		else // L only
		{
			CheckBoxHide(WINDOWAPP, ID_CHK_MONODIFF);
			CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_MONODIFF, (char *)"");
			l_outputfilter = OUTPUT_LEFT;
			StaticTextSetText(WINDOWAPP, ID_TEXT_SEP, (char *)"");
			StaticTextSetText(WINDOWAPP, ID_TEXT_M, (char *)"");
			StaticTextSetText(WINDOWAPP, ID_TEXT_S, (char *)"");
			SliderDisable(WINDOWAPP, ID_SLIDER_VOLUMESEP);
			MenuSetChecked(MENUMASETS, ID_MENU_MONO, 1);
		}
	}
	else if(CheckBoxGetValue(WINDOWAPP, ID_CHK_MONORIGHT)) // R only
	{	
		CheckBoxHide(WINDOWAPP, ID_CHK_MONODIFF);
		CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_MONODIFF, (char *)"");
		l_outputfilter = OUTPUT_RIGHT;
		StaticTextSetText(WINDOWAPP, ID_TEXT_OUTPUT, (char *)"Output Mono");
		StaticTextSetText(WINDOWAPP, ID_TEXT_SEP, (char *)"");
		StaticTextSetText(WINDOWAPP, ID_TEXT_M, (char *)"");
		StaticTextSetText(WINDOWAPP, ID_TEXT_S, (char *)"");
		SliderDisable(WINDOWAPP, ID_SLIDER_VOLUMESEP);
		MenuSetChecked(MENUMASETS, ID_MENU_MONO, 1);
	}
	else
	{
		CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_MONODIFF, (char *)"");
		CheckBoxShow(WINDOWAPP, ID_CHK_MONODIFF);
		CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_MONODIFF, (char *)"rev");
		StaticTextSetText(WINDOWAPP, ID_TEXT_OUTPUT, (char *)"Output Stereo");
		StaticTextSetText(WINDOWAPP, ID_TEXT_SEP, (char *)"Sep");
		StaticTextSetText(WINDOWAPP, ID_TEXT_M, (char *)"m");
		StaticTextSetText(WINDOWAPP, ID_TEXT_S, (char *)"s");
		
		l_outputfilter = OUTPUT_STEREO;
		
		if(CheckBoxGetValue(WINDOWAPP, ID_CHK_MONODIFF))
			l_outputfilter |= OUTPUT_REVERSE;

		SliderEnable(WINDOWAPP, ID_SLIDER_VOLUMESEP);
		SetSliders(SEPARATIONSLIDER);
		MenuSetChecked(MENUMASETS, ID_MENU_MONO, 0);
	}
	
} // SetGUIMonoStereo


void SetGUITitleInfo()
{
	if(g_sets.titleinfo == 1)
	{
		CheckBoxSetValue(WINDOWAPP, ID_CHK_TITLEINFO, 1);
		MenuSetChecked(MENUMASETS, ID_SUBMENU_TITLEINFO1, 0);
		MenuSetChecked(MENUMASETS, ID_SUBMENU_TITLEINFO2, 1);
		MenuSetChecked(MENUMASETS, ID_SUBMENU_TITLEINFO3, 0);
	}
	else if(g_sets.titleinfo == 2)
	{
		CheckBoxSetValue(WINDOWAPP, ID_CHK_TITLEINFO, 1);
		MenuSetChecked(MENUMASETS, ID_SUBMENU_TITLEINFO1, 0);
		MenuSetChecked(MENUMASETS, ID_SUBMENU_TITLEINFO2, 0);
		MenuSetChecked(MENUMASETS, ID_SUBMENU_TITLEINFO3, 1);
	}
	else
	{
		CheckBoxSetValue(WINDOWAPP, ID_CHK_TITLEINFO, 0);
		MenuSetChecked(MENUMASETS, ID_SUBMENU_TITLEINFO1, 1);
		MenuSetChecked(MENUMASETS, ID_SUBMENU_TITLEINFO2, 0);
		MenuSetChecked(MENUMASETS, ID_SUBMENU_TITLEINFO3, 0);
	}
	
} // SetGUITitleInfo


void SetPosSlider(int value)
{
	int lvalue = value;
	
	if(value >= 0)
	{	
		lvalue = value;
		l_posslidersetvalue = 1;
	}
	else
	{	
		lvalue = -value;
		l_posslidersetvalue = -1;
	}
	
	HSliderSetValue(WINDOWAPP, ID_SLIDER_POSITION, lvalue);
	
	OnSliderPosition(lvalue);
	
} // SetPosSlider


// this gets called after moving the position slider manually and from SetPosSlider())
void OnSliderPosition(int value)
{	
	char cstr[32];
	
	switch (l_posslidersetvalue)
	{
		case 1 : //  SetValue was called and value is a fraction of g_state.length
			l_poscurrent = LROUND(((g_state.duration * (double)value) / POSMAX));
			MillisecondsToStrHMS(cstr, l_poscurrent);
			StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEPOS, cstr);
		break;
		
		case 0 : // the slider was moved manually
			if(g_running) 
			{	
				l_poscurrent = l_sliderposms = LROUND((g_state.duration * (double)value) / POSMAX);
				MillisecondsToStrHMS(cstr, l_sliderposms);
				StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEPOS, cstr);
				l_posslidervalue = value;
				l_possliderchanged = 1;
			}
			else
				StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEPOS, NOTIMESTR);
		break;
		
		default :
			StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEPOS, NOTIMESTR);
		break;
		
	} // switch
	
	l_posslidersetvalue = 0;

} // OnSliderPosition


void OnSliderVolumeR(int value) // Right
{	
	g_sets.volumesliderright = value;
	
	if(!g_sets.balance)
	{
		g_sets.volumesliderleft = g_sets.volumesliderright = volminmaxR();
		SliderSetValue(WINDOWAPP, ID_SLIDER_VOLUMEL, g_sets.volumesliderleft);		
		SetSliderCaption(VOLUMESLIDERLEFT);
	}

#if !defined(MAAUDIONONE)
	maOutputSetVolume((kVOLMAX * volumevalL())  / VOLUMEMAX, (kVOLMAX * volumevalR()) / VOLUMEMAX);
#endif
		
	SetSliderCaption(VOLUMESLIDERRIGHT);
		
} // OnSliderVolumeR


void OnSliderVolumeL(int value) // Left
{	
	g_sets.volumesliderleft = value;
	
	if(!g_sets.balance)
	{
		g_sets.volumesliderright = g_sets.volumesliderleft = volminmaxL();
		SliderSetValue(WINDOWAPP, ID_SLIDER_VOLUMER, g_sets.volumesliderright);
		SetSliderCaption(VOLUMESLIDERRIGHT);		
	}
	
#if !defined(MAAUDIONONE)
	maOutputSetVolume((kVOLMAX * volumevalL())  / VOLUMEMAX, (kVOLMAX * volumevalR()) / VOLUMEMAX);
#endif
	
	SetSliderCaption(VOLUMESLIDERLEFT);
		
} // OnSliderVolumeL


void OnSliderSeparation(int value) // Separation
{	
	if(l_outputfilter == OUTPUT_DIFF || l_outputfilter == OUTPUT_MONO) // pan
	{
		g_sets.panslider = value;
		l_pan = value;
		l_panright =                  l_pan  / (double)SEPARATIONMAX;
		l_panleft  = (SEPARATIONMAX - l_pan) / (double)SEPARATIONMAX;
	}
	else // separation
	{
		g_sets.separationslider = value;
		l_separation = value;
		l_separationch  = 0.5 + ((0.5 * l_separation) / SEPARATIONMAX);
		l_separationmix = 1. - l_separationch;
	}
	
	SetSliderCaption(SEPARATIONSLIDER);
	
} // OnSliderSeparation


void OnSliderPreamp(int value)
{	
	g_sets.preampslider = value;	
	SetSliderCaption(PREAMPSLIDER);	
	MadDoParamChange();
	
} // OnSliderPreamp


void OnSliderEq(int slider, int value, int setpos)
{	
	int i;
	int first, last;
	int delta;
	
	g_eqzoomval = -1.0;
	if(slider >= EQSLIDERFIRST && slider <= EQSLIDERLAST)
		g_sets.eqpresets[g_sets.eqpreset].eqslider[slider] = value;
	else if(slider >= EQ2SLIDERFIRST && slider <= EQ2SLIDERLAST)
		g_sets.eq2presets[g_sets.eqpreset].eqslider[slider - EQ2SLIDERFIRST] = value;
	SetSliderCaption(slider);
	if(setpos)
		SetSliders(slider);

	if(g_sets.eqflags & MAEQLINK)
	{
		switch (slider)
		{
			case 2 :
				first = 0;
				last = 4;
				
				g_sets.eqpresets[g_sets.eqpreset].eqslider[0] =
				g_sets.eqpresets[g_sets.eqpreset].eqslider[1] = value;
					
				delta = (value - g_sets.eqpresets[g_sets.eqpreset].eqslider[5]) / 3;
				
				g_sets.eqpresets[g_sets.eqpreset].eqslider[3] = 
					g_sets.eqpresets[g_sets.eqpreset].eqslider[5] + delta + delta;
							
				g_sets.eqpresets[g_sets.eqpreset].eqslider[4] = 
					g_sets.eqpresets[g_sets.eqpreset].eqslider[5] + delta;		
			break;
			
			case 5 : case 8 : case 11 : case 14 : case 17 : case 20 : case 23 : case 26 :
				first = slider - 2;
				last = slider + 2;
				
				delta = (g_sets.eqpresets[g_sets.eqpreset].eqslider[slider] - 
						g_sets.eqpresets[g_sets.eqpreset].eqslider[slider - 3]) / 3;

				g_sets.eqpresets[g_sets.eqpreset].eqslider[slider - 2] = 
					g_sets.eqpresets[g_sets.eqpreset].eqslider[slider - 3] + delta;		
				
				g_sets.eqpresets[g_sets.eqpreset].eqslider[slider - 1] = 
					g_sets.eqpresets[g_sets.eqpreset].eqslider[slider - 3] + delta + delta;
							
				delta = (g_sets.eqpresets[g_sets.eqpreset].eqslider[slider] - 
							g_sets.eqpresets[g_sets.eqpreset].eqslider[slider + 3]) / 3;
				
				g_sets.eqpresets[g_sets.eqpreset].eqslider[slider + 1] = 
					g_sets.eqpresets[g_sets.eqpreset].eqslider[slider + 3] + delta + delta;
							
				g_sets.eqpresets[g_sets.eqpreset].eqslider[slider + 2] = 
					g_sets.eqpresets[g_sets.eqpreset].eqslider[slider + 3] + delta;		
			break;
			
			case 29 :
				first = 27;
				last = 30;
				
				delta = (g_sets.eqpresets[g_sets.eqpreset].eqslider[29] - 
							g_sets.eqpresets[g_sets.eqpreset].eqslider[26]) / 3;
				
				g_sets.eqpresets[g_sets.eqpreset].eqslider[27] = 
					g_sets.eqpresets[g_sets.eqpreset].eqslider[26] + delta;
							
				g_sets.eqpresets[g_sets.eqpreset].eqslider[28] = 
					g_sets.eqpresets[g_sets.eqpreset].eqslider[26] + delta + delta;
					
				g_sets.eqpresets[g_sets.eqpreset].eqslider[30] = value;		
			break;
			
			default : // not one of 10 bands, skip the 'for i' below
				first = 1;
				last = 0;
			break;
		}
		
		for(i = first; i <= last; i++)
		{
			if(i == slider)
				continue;
			g_eqzoomval = -1.0;
			SetSliders(i);
		}
	}
		
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();
	
} // OnSliderEq


void OnSliderEq2(int slider, int value, int setpos)
{	
	int i;
	int lslider;
	int first, last;
	int delta;
	
	lslider = slider - EQ2SLIDERFIRST;
	g_eqzoomval = -1.0;
	g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider] = value;
	SetSliderCaption(slider);
	if(setpos)
		SetSliders(slider);

	if(g_sets.eqflags & MAEQLINK)
	{
		switch (lslider)
		{
			case 2 :
				first = 0 + EQ2SLIDERFIRST;
				last = 4 + EQ2SLIDERFIRST;
				
				g_sets.eq2presets[g_sets.eqpreset].eqslider[0] =
				g_sets.eq2presets[g_sets.eqpreset].eqslider[1] = value;
					
				delta = (value - g_sets.eq2presets[g_sets.eqpreset].eqslider[5]) / 3;
				
				g_sets.eq2presets[g_sets.eqpreset].eqslider[3] = 
					g_sets.eq2presets[g_sets.eqpreset].eqslider[5] + delta + delta;
							
				g_sets.eq2presets[g_sets.eqpreset].eqslider[4] = 
					g_sets.eq2presets[g_sets.eqpreset].eqslider[5] + delta;		
			break;
			
			case 5 : case 8 : case 11 : case 14 : case 17 : case 20 : case 23 : case 26 :
				first = lslider - 2 + EQ2SLIDERFIRST;
				last = lslider + 2 + EQ2SLIDERFIRST;
				
				delta = (g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider] - 
						g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider - 3]) / 3;

				g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider - 2] = 
					g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider - 3] + delta;		
				
				g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider - 1] = 
					g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider - 3] + delta + delta;
							
				delta = (g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider] - 
							g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider + 3]) / 3;
				
				g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider + 1] = 
					g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider + 3] + delta + delta;
							
				g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider + 2] = 
					g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider + 3] + delta;		
			break;
			
			case 29 :
				first = 27 + EQ2SLIDERFIRST;
				last = 30 + EQ2SLIDERFIRST;
				
				delta = (g_sets.eq2presets[g_sets.eqpreset].eqslider[29] - 
							g_sets.eq2presets[g_sets.eqpreset].eqslider[26]) / 3;
				
				g_sets.eq2presets[g_sets.eqpreset].eqslider[27] = 
					g_sets.eq2presets[g_sets.eqpreset].eqslider[26] + delta;
							
				g_sets.eq2presets[g_sets.eqpreset].eqslider[28] = 
					g_sets.eq2presets[g_sets.eqpreset].eqslider[26] + delta + delta;
					
				g_sets.eq2presets[g_sets.eqpreset].eqslider[30] = value;		
			break;
			
			default : // not one of 10 bands, skip the 'for i' below
				first = 1;
				last = 0;
			break;
		}
		
		for(i = first; i <= last; i++)
		{
			if(i == slider)
				continue;
			g_eqzoomval = -1.0;
			SetSliders(i);
		}
	}
		
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();
	
} // OnSliderEq2


void OnEditTextEq()
{
	int i;
	long outlen;
	char cstr[256];

	EditTextGetText(WINDOWEQ, ID_EDITTEXT_EQ, cstr, EQNAMELEN, &outlen);
	if(strlen(cstr) == 0)
		sprintf(cstr, "Preset%d", g_sets.eqpreset + 1);
	if(strcmp(cstr, g_sets.eqpresets[g_sets.eqpreset].eqname))
	{
		strcpy(g_sets.eqpresets[g_sets.eqpreset].eqname, cstr);
#if defined(MAGUIOSX) || defined(MAGUIGTK)
		PopupButtonSetText(WINDOWEQ, ID_POPUP_EQPRESET, g_sets.eqpreset, g_sets.eqpresets[g_sets.eqpreset].eqname);
		if(g_sets.eqpreset == 0) // force a redraw
			i = 1;
		else
			i = 0;
		PopupButtonSetValue(WINDOWEQ, ID_POPUP_EQPRESET, i);
#else
		PopupButtonClear(WINDOWEQ, ID_POPUP_EQPRESET); // PopupButtonSetText fails...
		for(i = 0; i < EQNUMPRESETS; i++)
			PopupButtonAppend(WINDOWEQ, ID_POPUP_EQPRESET, g_sets.eqpresets[i].eqname);
#endif
		PopupButtonSetValue(WINDOWEQ, ID_POPUP_EQPRESET, g_sets.eqpreset);
	}
}


void OnSliderPitch(int value)
{	
	g_sets.pitchslider = value;	
	SetSliderCaption(EFFECTSSLIDERPITCH);	
	if(g_sets.effects & EFFECTSON && g_sets.effects & EFFECTSPITCH)
		g_effectschange = 1;
		
} // OnSliderPitch


void OnSliderPitch2(int value)
{	
	g_sets.pitchslider2 = value;	
	SetSliderCaption(EFFECTSSLIDERPITCH2);	
	if(g_sets.effects & EFFECTSON && g_sets.effects & EFFECTSPITCH)
		g_effectschange = 1;
		
} // OnSliderPitch2


void OnSliderTempo(int value)
{	
	g_sets.temposlider = value;	
	SetSliderCaption(EFFECTSSLIDERTEMPO);	
	if(g_sets.effects & EFFECTSON && g_sets.effects & EFFECTSTEMPO)
		g_effectschange = 1;
		
} // OnSliderTempo


void OnSliderSpeed(int value)
{	
	g_sets.speedslider = value;	
	SetSliderCaption(EFFECTSSLIDERSPEED);	
	if(g_sets.effects & EFFECTSON && g_sets.effects & EFFECTSSPEED)
		g_effectschange = 1;
		
} // OnSliderSpeed


#define SPIN_BUFFER_INCREMENT 16
void OnLittleArrowBuffer(int value)
{
	int incr;
	char cstr[64];

	if(g_sets.buffertype == BUFUSEVDISK)
		incr = 1;
	else
		incr = SPIN_BUFFER_INCREMENT;
		
	if(value != -1 && value != 1)
		return;

	if(value > 0) // up
		g_sets.buffersize += incr;
	else
		g_sets.buffersize -= incr;
	
	if(g_sets.buffersize > SONGMEMMAX)
		g_sets.buffersize = SONGMEMMAX;

	if(g_sets.buffersize < SONGMEMMIN)
		g_sets.buffersize = SONGMEMMIN;
		
	sprintf(cstr, "Buffer %d", g_sets.buffersize);
	StaticTextSetText(WINDOWAPP, ID_TEXT_BUFFER, cstr);
	
} // OnLittleArrowBuffer


void OnLittleArrowVolumeL(int value)
{
	int val;
	
	if(value != -1 && value != 1)
		return;

	val = g_sets.volumesliderleft;
		
	if(value > 0)	// up
		val += 1;
	else			// down
		val -= 1;
			
	OnSliderVolumeL(val);

	SetSliders(VOLUMESLIDERLEFT);
			
} // OnLittleArrowVolumeL


void OnLittleArrowVolumeR(int value)
{
	int val;
	
	if(value != -1 && value != 1)
		return;

	val = g_sets.volumesliderright;
		
	if(value > 0)	// up
			val += 1;
	else			// down
			val -= 1;
			
	OnSliderVolumeR(val);
	
	SetSliders(VOLUMESLIDERRIGHT);
	
} // OnLittleArrowVolumeR


void ChangeVolume(int delta)
{
	int val;
	char cstr[256];
	
	if(!delta)
		return;

	if(g_sets.balance)
	{
		if(delta > 0 && g_sets.volumesliderleft >= VOLUMEMAX && g_sets.volumesliderright >= VOLUMEMAX)
			return;
		if(delta < 0 && g_sets.volumesliderleft <= VOLUMEMIN && g_sets.volumesliderright <= VOLUMEMIN)
			return;
	}

	val = g_sets.volumesliderright + delta;
	
	OnSliderVolumeR(val);
	SetSliders(VOLUMESLIDERRIGHT);
	
	if(g_sets.balance)
	{
		val = g_sets.volumesliderleft + delta;
		OnSliderVolumeL(val);
		SetSliders(VOLUMESLIDERLEFT);
	}
	
	sprintf(cstr, "Volume %d%%", (int)((100. * volminmaxL()) / VOLUMEMAX));
	WindowSetTitle(WINDOWAPP, cstr);
#if !defined(MAGUINONE)
	l_temptitle = 1;
#endif
	
} // ChangeVolume


void OnLittleArrowSeparation(int value)
{
	int val;

	if(value != -1 && value != 1)
		return;

	if(l_outputfilter == OUTPUT_DIFF || l_outputfilter == OUTPUT_MONO) // pan
		val = g_sets.panslider;
	else
		val = g_sets.separationslider;
	
	if(value > 0)	// up
		val += 1;
	else			// down
		val -= 1;
			
	if(val > SEPARATIONMAX)
		val = SEPARATIONMAX;
	else if(val < SEPARATIONMIN)
		val = SEPARATIONMIN;

	if(l_outputfilter == OUTPUT_DIFF || l_outputfilter == OUTPUT_MONO) // pan
	{
		g_sets.panslider = val;
		l_pan = val;
		l_panright =                  l_pan  / (double)SEPARATIONMAX;
		l_panleft  = (SEPARATIONMAX - l_pan) / (double)SEPARATIONMAX;
	}
	else
	{
		g_sets.separationslider = val;
		l_separation = val;
		l_separationch  = 0.5 + ((0.5 * l_separation) / SEPARATIONMAX);
		l_separationmix = 1. - l_separationch;
	}

	SetSliders(SEPARATIONSLIDER);
			
} // OnLittleArrowSeparation


void OnLittleArrowPreamp(int value)
{
	if(value != -1 && value != 1)
		return;

	if(value > 0)	// up
		g_sets.preampslider += 1;
	else			// down
		g_sets.preampslider -= 1;
			
	if(g_sets.preampslider > PREAMPMAX)
		g_sets.preampslider = PREAMPMAX;
	else if(g_sets.preampslider < PREAMPMIN)
		g_sets.preampslider = PREAMPMIN;

	SetSliders(PREAMPSLIDER);
			
	MadDoParamChange();

} // OnLittleArrowPreamp


void OnLittleArrowBegin(int value)
{
	char cstr[32];
	int ms;
	
	if((value != -1 && value != 1) || !g_posbegon)
		return;
		
	ms = g_posbeg;	

	if(value > 0)	// up
	{
		if((ms += POSINCR) > g_posend)
			ms = g_posend;
	}
	else			// down
		if((ms -= POSDECR) < 0)
			ms = 0;

	MillisecondsToStrHMS(cstr, ms);
	StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEBEG, cstr);	

	g_posbeg = ms;
					
} // OnLittleArrowBegin


void OnLittleArrowEnd(int value)
{
	char cstr[32];
	int ms;	

	if((value != -1 && value != 1) || !g_posendon)
		return;

	ms = g_posend;	

	if(value > 0)	// up
	{
		if((ms += POSINCR) > g_state.duration)
			ms = g_state.duration;
	}
	else			// down
		if((ms -= POSDECR) < g_posbeg)
			ms = g_posbeg;

	MillisecondsToStrHMS(cstr, ms);
	StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEEND, cstr);

	g_posend = ms;
	
} // OnLittleArrowEnd


void OnLittleArrowEqOffset(int value)
{
	if(value != -1 && value != 1)
		return;

	if(value > 0)	// up
	{
		if(++g_sets.eqpresets[g_sets.eqpreset].eqoffset > DBRANGE)
			g_sets.eqpresets[g_sets.eqpreset].eqoffset = DBRANGE;
	}
	else			// down
	{
		if(--g_sets.eqpresets[g_sets.eqpreset].eqoffset < -DBRANGE)
			g_sets.eqpresets[g_sets.eqpreset].eqoffset = -DBRANGE;
	}

	if(g_w[WINDOWEQ])
		StaticTextSetValue(WINDOWEQ, ID_TEXT_EQOFFSETVAL, g_sets.eqpresets[g_sets.eqpreset].eqoffset);

	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();
	
} // OnLittleArrowEqOffset


void OnLittleArrowEq(int band, int value)
{
	int val;
	
	if(value == 1)
	{
		if((val = g_sets.eqpresets[g_sets.eqpreset].eqslider[band]) >= DBSLIDERMAX)
			return;
		
		OnSliderEq(band, val + 1, 1);
	}
	else if(value == -1)
	{
		if((val = g_sets.eqpresets[g_sets.eqpreset].eqslider[band]) <= DBSLIDERMIN)
			return;
	
		OnSliderEq(band, val - 1, 1);
	}
	
} // OnLittleArrowEq


void OnLittleArrowEq2Offset(int value)
{
	if(value != -1 && value != 1)
		return;

	if(value > 0)	// up
	{
		if(++g_sets.eq2presets[g_sets.eqpreset].eqoffset > DBRANGE)
			g_sets.eq2presets[g_sets.eqpreset].eqoffset = DBRANGE;
	}
	else			// down
	{
		if(--g_sets.eq2presets[g_sets.eqpreset].eqoffset < -DBRANGE)
			g_sets.eq2presets[g_sets.eqpreset].eqoffset = -DBRANGE;
	}

	if(g_w[WINDOWEQ2])
		StaticTextSetValue(WINDOWEQ2, ID_TEXT_EQOFFSETVAL, g_sets.eq2presets[g_sets.eqpreset].eqoffset);

	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();
	
} // OnLittleArrowEq2Offset


void OnLittleArrowEq2(int band, int value)
{
	int val;
	
	if(value == 1)
	{
		if((val = g_sets.eq2presets[g_sets.eqpreset].eqslider[band - EQ2SLIDERFIRST]) >= DBSLIDERMAX)
			return;
		
		OnSliderEq(band, val + 1, 1);
	}
	else if(value == -1)
	{
		if((val = g_sets.eq2presets[g_sets.eqpreset].eqslider[band - EQ2SLIDERFIRST]) <= DBSLIDERMIN)
			return;
	
		OnSliderEq(band, val - 1, 1);
	}
	
} // OnLittleArrowEq2


void OnLittleArrowPitch(int value)
{
	int val;
	
	if(value != -1 && value != 1)
		return;

	val = g_sets.pitchslider;
		
	if(value > 0)	// up
		val += 1;
	else			// down
		val -= 1;
			
	if(val > EFFECTSPITCHMAX)
		val = EFFECTSPITCHMAX;
	else if(val < EFFECTSPITCHMIN)
		val = EFFECTSPITCHMIN;

	g_sets.pitchslider = val;
	
	SetSliders(EFFECTSSLIDERPITCH);
	
	if(g_sets.effects & EFFECTSON && g_sets.effects & EFFECTSPITCH)
		g_effectschange = 1;
	
} // OnLittleArrowPitch


void OnLittleArrowPitch2(int value)
{
	int val;
	
	if(value != -1 && value != 1)
		return;

	val = g_sets.pitchslider2;
		
	if(value > 0)	// up
		val += 1;
	else			// down
		val -= 1;
			
	if(val > EFFECTSMAX)
		val = EFFECTSMAX;
	else if(val < EFFECTSMIN)
		val = EFFECTSMIN;

	g_sets.pitchslider2 = val;
	
	SetSliders(EFFECTSSLIDERPITCH2);
	
	if(g_sets.effects & EFFECTSON && g_sets.effects & EFFECTSPITCH)
		g_effectschange = 1;
	
} // OnLittleArrowPitch2


void OnLittleArrowTempo(int value)
{
	int val;
	
	if(value != -1 && value != 1)
		return;

	val = g_sets.temposlider;
		
	if(value > 0)	// up
		val += 1;
	else			// down
		val -= 1;
			
	if(val > EFFECTSMAX)
		val = EFFECTSMAX;
	else if(val < EFFECTSMIN)
		val = EFFECTSMIN;

	g_sets.temposlider = val;
	
	SetSliders(EFFECTSSLIDERTEMPO);
	
	if(g_sets.effects & EFFECTSON && g_sets.effects & EFFECTSTEMPO)
		g_effectschange = 1;
	
} // OnLittleArrowTempo


void OnLittleArrowSpeed(int value)
{
	int val;
	
	if(value != -1 && value != 1)
		return;

	val = g_sets.speedslider;
		
	if(value > 0)	// up
		val += 1;
	else			// down
		val -= 1;
			
	if(val > EFFECTSMAX)
		val = EFFECTSMAX;
	else if(val < EFFECTSMIN)
		val = EFFECTSMIN;

	g_sets.speedslider = val;
	
	SetSliders(EFFECTSSLIDERSPEED);
	
	if(g_sets.effects & EFFECTSON && g_sets.effects & EFFECTSSPEED)
		g_effectschange = 1;
	
} // OnLittleArrowSpeed


void OnWindowClosedEq()
{
	if(g_w[WINDOWEQ])
	{
		WindowGetCoordinates(WINDOWEQ, &g_sets.eqxpos[g_sets.winconf], &g_sets.eqypos[g_sets.winconf],
									&g_sets.eqxsize[g_sets.winconf], &g_sets.eqysize[g_sets.winconf]);
		g_w[WINDOWEQ] = NULL;
	}
	g_sets.eqflags &= ~MAEQWINDOW;
	CheckBoxSetValue(WINDOWAPP, ID_CHK_EQWINDOW, 0);
	MenuSetChecked(MENUSHOW, ID_MENU_SHOWEQ, 0);
}


void OnWindowClosedEq2()
{
	if(g_w[WINDOWEQ2])
	{
		WindowGetCoordinates(WINDOWEQ2, &g_sets.eq2xpos[g_sets.winconf], &g_sets.eq2ypos[g_sets.winconf],
									&g_sets.eq2xsize[g_sets.winconf], &g_sets.eq2ysize[g_sets.winconf]);
		g_w[WINDOWEQ2] = NULL;
	}
}


void OnWindowClosedEffects()
{
	if(g_w[WINDOWEFFECTS])
	{
		WindowGetCoordinates(WINDOWEFFECTS, &g_sets.effectsxpos[g_sets.winconf], &g_sets.effectsypos[g_sets.winconf],
										&g_sets.effectsxsize[g_sets.winconf], &g_sets.effectsysize[g_sets.winconf]);
		g_w[WINDOWEFFECTS] = NULL;
	}
	g_sets.effectswindow = 0;
	MenuSetChecked(MENUSHOW, ID_MENU_SHOWEFFECTS, 0);
}


#if !defined(MANOCDA) && defined(__MAUNIX__)
void OnWindowClosedAudioCD()
{
	int item;	

	PopupButtonGetSelection(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, &item);
	if(item > -1)
		g_setscdrom.item = item;	

	if(g_w[WINDOWAUDIOCD])
	{
		WindowGetCoordinates(WINDOWAUDIOCD, &g_setscdrom.winx[g_sets.winconf], &g_setscdrom.winy[g_sets.winconf],
										&g_setscdrom.winw[g_sets.winconf], &g_setscdrom.winh[g_sets.winconf]);
		g_w[WINDOWAUDIOCD] = NULL;
	}
}
#endif


void OnWindowClosedList()
{
	if(g_w[WINDOWLIST])
	{
		WindowGetCoordinates(WINDOWLIST, &g_sets.listxpos[g_sets.winconf], &g_sets.listypos[g_sets.winconf],
									 &g_sets.listxsize[g_sets.winconf], &g_sets.listysize[g_sets.winconf]);
		g_w[WINDOWLIST] = NULL;
	}
	g_sets.listwindow &= ~LBOXON;
	CheckBoxSetValue(WINDOWAPP, ID_CHK_LISTWINDOW, 0);
	MenuSetChecked(MENUSHOW, ID_MENU_SHOWLIST, 0);
}


void OnWindowClosedFileOut() // called when a fileout window is closed
{
	int i;
	for(i = kOUTFIRST; i <= kOUTLAST; i++)
	{
		if(g_w[WINDOWOUTFIRST + i - 1]) // WINDOWOUTCUR
		{
			WindowGetCoordinates(WINDOWOUTFIRST + i - 1,
							&g_sets.fileoutxpos[g_sets.winconf], &g_sets.fileoutypos[g_sets.winconf],
							&g_sets.fileoutxsize[g_sets.winconf], &g_sets.fileoutysize[g_sets.winconf]);
			g_w[WINDOWOUTFIRST + i - 1] = NULL;
			break;
		}
	}

	g_sets.fileoutwindow = 0;
	MenuSetChecked(MENUSHOW, ID_MENU_SHOWFILEOUT, 0);
}


void OnWindowClosedPromptNum()
{
	g_w[WINDOWPROMPTNUM] = NULL;
	
#if defined(MAKEYBOARD)
		g_maOnKeySkip = 0;
#endif

	l_keyindex = 0;

} // OnWindowClosedPromptNum


void OnWindowClosedAdvanced()
{
	if(g_w[WINDOWADVANCED])
	{
		WindowGetCoordinates(WINDOWADVANCED, &g_sets.advancedxpos, &g_sets.advancedypos,
											&g_sets.advancedxsize, &g_sets.advancedysize);
		g_w[WINDOWADVANCED] = NULL;
	}
}


void OnMenuEqWindow()
{
	DoWindowEq(MenuIsChecked(MENUSHOW, ID_MENU_SHOWEQ) ? 1 : 0);
	DoWindowEq2((MenuIsChecked(MENUSHOW, ID_MENU_SHOWEQ) && (g_sets.eqflags & MASEPARATELR)) ? 1 : 0);
}


void OnMenuEffectsWindow()
{
	DoWindowEffects((MenuIsChecked(MENUSHOW, ID_MENU_SHOWEFFECTS)) ? 1 : 0);
}


void OnMenuList()
{
	DoWindowList((MenuIsChecked(MENUSHOW, ID_MENU_SHOWLIST)) ? 1 : 0);
}


void OnMenuFileOut()
{
	DoWindowFileOut((MenuIsChecked(MENUSHOW, ID_MENU_SHOWFILEOUT)) ? 1 : 0);
}


void OnMenuSpindown(int checked)
{
	g_sets.spindown = checked;
	CheckBoxSetValue(WINDOWAPP, ID_CHK_SPINDOWN, checked);
}


void OnMenuRandom(int checked)
{
	g_sets.random = checked;
	CheckBoxSetValue(WINDOWAPP, ID_CHK_RANDOM, checked);
}


void OnMenuReptPlayed(int checked)
{
	g_sets.reptplayed = checked;
	CheckBoxSetValue(WINDOWAPP, ID_CHK_REPTPLAYED, checked);
}


void OnMenuLoopLoaded(int checked)
{
	g_sets.looploaded = checked;
	CheckBoxSetValue(WINDOWAPP, ID_CHK_LOOPLOADED, checked);
}


void OnMenuLoopList(int checked)
{
	g_sets.looplist = checked;
	CheckBoxSetValue(WINDOWAPP, ID_CHK_LOOPLIST, checked);
}


void OnMenuAutoStart(int checked)
{
	g_sets.autostart = checked;
}


void OnMenuErrLogging(int checked)
{
	if(checked)
	{
		g_sets.logflags &= ~kMALOGOFF;
		l_logoff = 0;
	}
	else
	{
		g_sets.logflags |= kMALOGOFF;
		l_logoff = 1;
	}
}


void OnMenuSinglePlay(int checked)
{
	g_sets.singleplay = checked;
	CheckBoxSetValue(WINDOWAPP, ID_CHK_SINGLEPLAY, checked);
}


void OnMenuHiPriority(int checked)
{
	g_sets.highpriority = checked;
	CheckBoxSetValue(WINDOWADVANCED, ID_CHK_HIGHPRIORITY, checked);
}


void OnMenuEqualizer(int checked)
{
	if(checked)
	{
		g_sets.eqflags |= MAEQACTIVE;
		CheckBoxSetValue(WINDOWAPP, ID_CHK_EQUALIZER, 1);
		if(g_w[WINDOWEQ])
			CheckBoxSetValue(WINDOWEQ, ID_CHK_EQUALIZERB, 1);
	}
	else
	{
		g_sets.eqflags &= ~MAEQACTIVE;
		CheckBoxSetValue(WINDOWAPP, ID_CHK_EQUALIZER, 0);
		if(g_w[WINDOWEQ])
			CheckBoxSetValue(WINDOWEQ, ID_CHK_EQUALIZERB, 0);
	}
}


void OnMenuEqExtra(int checked)
{
	if(checked)
	{
		CheckBoxSetValue(WINDOWAPP, ID_CHK_EQEXTRA, 1);
		if(g_w[WINDOWEQ])
			CheckBoxSetValue(WINDOWEQ, ID_CHK_EQEXTRAB, 1);
		g_sets.eqflags |= MAEQEXTRA;
		eqcfg.extra_filtering = 1;
	}
	else
	{
		CheckBoxSetValue(WINDOWAPP, ID_CHK_EQEXTRA, 0);
		if(g_w[WINDOWEQ])
			CheckBoxSetValue(WINDOWEQ, ID_CHK_EQEXTRAB, 0);
		g_sets.eqflags &= ~MAEQEXTRA;
		eqcfg.extra_filtering = 0;
	}
}


void OnMenuEffects(int checked)
{
	if(checked)
	{
		g_sets.effects |= EFFECTSON;
		DoWindowEffects(1);
		if(g_sets.effects & (EFFECTSTEMPO | EFFECTSPITCH | EFFECTSSPEED))
			g_effectschange = 1;
	}
	else
	{
		g_sets.effects &= ~EFFECTSON;
		if(g_w[WINDOWEFFECTS])
			CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_EFFECTSB, 0);
	}
		
	CheckBoxSetValue(WINDOWAPP, ID_CHK_EFFECTS, checked);
}


void OnMenuMono(int checked)
{
	if(checked)
	{
		g_sets.monodiff |= MONOCHECKBOXL;
		g_sets.monodiff |= MONOCHECKBOXR;
		CheckBoxSetValue(WINDOWAPP, ID_CHK_MONOLEFT, 1);
		CheckBoxSetValue(WINDOWAPP, ID_CHK_MONORIGHT, 1);
	}
	else
	{
		g_sets.monodiff &= ~MONOCHECKBOXL;
		g_sets.monodiff &= ~MONOCHECKBOXR;
		CheckBoxSetValue(WINDOWAPP, ID_CHK_MONOLEFT, 0);
		CheckBoxSetValue(WINDOWAPP, ID_CHK_MONORIGHT, 0);
	}
	
	RefreshMonoDiff();
	SetGUIMonoStereo();
}


void OnMenuInvertRight(int checked)
{
	g_sets.invertright = checked;
	CheckBoxSetValue(WINDOWAPP, ID_CHK_INVERTRIGHT, checked);
}


void OnMenuEq31Bands(int checked)
{
	if(checked)
	{
		g_sets.eqflags |= MAEQ31BANDS;
		CheckBoxSetValue(WINDOWAPP, ID_CHK_EQ31BANDS, 1);
		if(g_w[WINDOWEQ])
			CheckBoxSetValue(WINDOWEQ, ID_CHK_EQ31BANDSB, 1);
		maEqualizerReset(EQBANDS31, 1, (g_sets.eqflags & MAEQEXTRA) ? 1 : 0); // bands, 16bit, extra	
	}
	else
	{
		g_sets.eqflags &= ~MAEQ31BANDS;
		CheckBoxSetValue(WINDOWAPP, ID_CHK_EQ31BANDS, 0);
		if(g_w[WINDOWEQ])
			CheckBoxSetValue(WINDOWEQ, ID_CHK_EQ31BANDSB, 0);
		maEqualizerReset(EQBANDS10, 1, (g_sets.eqflags & MAEQEXTRA) ? 1 : 0); // bands, 16bit, extra	
	}
		
	EqDoParamChange();
}


void OnMenuViewCatalog()
{
	char catalogpath[MAX_PATH];
	char cstr[MAX_PATH + 64];
	char *p;
	
	if(g_w[WINDOWCATALOG])
		return;
		
	if(FilenameFromPathname(g_songlist, cstr))
	{
		sprintf(cstr, "OnMenuViewCatalog FilenameFromPathname failed on '%s'", g_songlist);
		maLog(cstr);
		return;
	}
	if((p = strstr(cstr, g_extstr[kMAMAF])))
		*p = 0;

	sprintf(catalogpath, "%s%c%s.txt", g_madir, FILESEP, cstr);
	if(!maFileExists(catalogpath))
	{
		sprintf(cstr, "No catalog file '%s'\n\nMake one with the Make Catalog menu item.", catalogpath);
		ShowErr(cstr);
		return;
	}
	
	maViewCatalog();
	
} // OnMenuViewCatalog


void OnMenuViewLog() // get here from View Log menu if it is enabled, and always from keyboard command
{
	if(g_w[WINDOWLOG])
		return;
	
	maViewLog();

} // OnMenuViewLog


#if defined(MAKEYBOARD)
void OnMenuEditKeyBindings()
{
	char cstr[MAX_PATH + 64];

	if(g_w[WINDOWKEYBINDINGS])
		return;
	
	if(!maFileExists(g_keybindingspath)) // in case the key bindings file was deleted while this app was open
	{
		SetFactoryKeyBindings();
		SaveKeyBindings(0);
		if(!maFileExists(g_keybindingspath))
		{
			sprintf(cstr, "OnMenuEditKeyBindings No key bindings file '%s'", g_keybindingspath);
			maLog(cstr);
			return;
		}
	}

	maEditKeyBindings();

} // OnMenuEditKeyBindings


void OnMenuBlankKeyBindings()
{
	char cstr[MAX_PATH + 64];

	if(maFileExists(g_keybindingspath))
	{
		sprintf(cstr, "The current key bindings will be overwritten\n\n'%s'", g_keybindingspath);
		if(maMessageBox(cstr, (char *)"CONTINUE ?", maYES_NO | maICON_QUESTION) != maYES)
			return;
	}

	if(g_w[WINDOWKEYBINDINGS])
	{
		WindowDoClose(WINDOWKEYBINDINGS);
		g_w[WINDOWKEYBINDINGS] = NULL;
	}
	SaveKeyBindings(1);
	LoadKeyBindings(0);
	maEditKeyBindings();

} // OnMenuBlankKeyBindings
#endif


void OnMenuReadme()
{
	if(g_w[WINDOWREADME])
		return;
		
	sprintf(g_readmepath, "%s%c%s", MAREADMEDIR, FILESEP, MAREADME); // installed, default
	if(!maFileExists(g_readmepath))
	{
		sprintf(g_readmepath, "%s%c%s", g_madir, FILESEP, MAREADME); // home dir
		if(!maFileExists(g_readmepath))
		{
			sprintf(g_readmepath, "%s%c%s", l_cwd, FILESEP, MAREADME); // cwd
			if(!maFileExists(g_readmepath))
			{
				if(!l_bindir[0] && !magetbindir(l_bindir, MAX_PATH))
					maLog("OnMenuReadme no l_bindir");
				else
					sprintf(g_readmepath, "%s%c%s", l_bindir, FILESEP, MAREADME); // with ma binary
				if(!maFileExists(g_readmepath))
				{
					sprintf(g_readmepath, "README file not found '%s%c%s'", MAREADMEDIR, FILESEP, MAREADME); // give up
					maLog(g_readmepath);
					sprintf(g_readmepath, "%s%c%s", MAREADMEDIR, FILESEP, MAREADME); // give up
					
				}
			}
		}
	}

	maViewReadme();

} // OnMenuReadme


void OnMenuClearLog()
{
	maLog(&l_logcmdclear);

} // OnMenuClearLog


void OnMenuAltSize(int checked, int kbcommand)
{
	g_sets.winconf = checked;

	CheckBoxSetValue(WINDOWAPP, ID_CHK_ALTSIZE, checked);

	if(!kbcommand && maModKeyDown())
		WindowsCopyPositions();
	WindowsMoveAndSetSize();
		
} // OnMenuAltSize


void OnMenuSavedSize(int kbcommand)
{
	int i;
	FILE *fp;
	long s;
	MASETS settings;
	char *settingspath;
	char cstr[256];
	
	if(g_settingspath[0])
		settingspath = g_settingspath;
	else
	{
		settingspath = g_settingsdef;
		if(!maFileExists(settingspath))
		{
			OnMenuFactorySize(0);
			return;
		}
	}
	
	if((fp = mafopen(settingspath, "rb")) == NULL)
	{
		sprintf(cstr, "OnMenuSavedSize mafopen failed on '%s'", settingspath);
		maLog(cstr);
		return;
	}

	s = sizeof(MASETS);
	fseek(fp, -s, SEEK_END);		
	if(fread(&settings, 1, sizeof(MASETS), fp) != sizeof(MASETS))
	{
		fclose(fp);		
		sprintf(cstr, "OnMenuSavedSize fread failed on '%s'", settingspath);
		maLog(cstr);
		return;
	}
	
	fclose(fp);		

	g_sets.winconf = settings.winconf;
	
	for(i = 0; i < NUMWINCONF; i++)
	{
		g_sets.xpos[i]			= settings.xpos[i];
		g_sets.ypos[i]			= settings.ypos[i];
		g_sets.eqxpos[i]		= settings.eqxpos[i];
		g_sets.eqypos[i]		= settings.eqypos[i];
		g_sets.eq2xpos[i]		= settings.eq2xpos[i];
		g_sets.eq2ypos[i]		= settings.eq2ypos[i];
		g_sets.effectsxpos[i]	= settings.effectsxpos[i];
		g_sets.effectsypos[i]	= settings.effectsypos[i];
		g_sets.listxpos[i]		= settings.listxpos[i];
		g_sets.listypos[i]		= settings.listypos[i];
		g_sets.fileoutxpos[i]	= settings.fileoutxpos[i];
		g_sets.fileoutypos[i]	= settings.fileoutypos[i];
	
		g_sets.xsize[i]			= settings.xsize[i];
		g_sets.ysize[i]			= settings.ysize[i];
		g_sets.eqxsize[i]		= settings.eqxsize[i];
		g_sets.eqysize[i]		= settings.eqysize[i];
		g_sets.eq2xsize[i]		= settings.eq2xsize[i];
		g_sets.eq2ysize[i]		= settings.eq2ysize[i];
		g_sets.effectsxsize[i]  = settings.effectsxsize[i];
		g_sets.effectsysize[i]  = settings.effectsysize[i];
		g_sets.listxsize[i]		= settings.listxsize[i];
		g_sets.listysize[i]		= settings.listysize[i];
		g_sets.fileoutxsize[i]	= settings.fileoutxsize[i];
		g_sets.fileoutysize[i]	= settings.fileoutysize[i];
	}
	
	g_sets.aboutxpos		= settings.aboutxpos;
	g_sets.aboutypos		= settings.aboutypos;
	g_sets.aboutxsize		= settings.aboutxsize;
	g_sets.aboutysize		= settings.aboutysize;

	g_sets.viewerxpos		= settings.viewerxpos;
	g_sets.viewerypos		= settings.viewerypos;
	g_sets.viewerxsize		= settings.viewerxsize;
	g_sets.viewerysize		= settings.viewerysize;

	g_sets.keybindingsxpos  = settings.keybindingsxpos;
	g_sets.keybindingsypos  = settings.keybindingsypos;
	g_sets.keybindingsxsize = settings.keybindingsxsize;
	g_sets.keybindingsysize = settings.keybindingsysize;

	g_sets.advancedxpos = settings.advancedxpos;
	g_sets.advancedypos = settings.advancedypos;
	g_sets.advancedxsize = settings.advancedxsize;
	g_sets.advancedysize = settings.advancedysize;

	MenuSetChecked(MENUSHOW, ID_MENU_ALTSIZE, g_sets.winconf ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_ALTSIZE, g_sets.winconf ? 1 : 0);
		
	if(!kbcommand && maModKeyDown())
		WindowsCopyPositions();
	WindowsMoveAndSetSize();
							
} // OnMenuSavedSize


void OnMenuDefaultSize(int kbcommand)
{
	int i;
	MASETS settings;
	
	LoadDefaultSettings(&settings);
	
	g_sets.winconf = settings.winconf;
	
	for(i = 0; i < NUMWINCONF; i++)
	{
		g_sets.xpos[i]			= settings.xpos[i];
		g_sets.ypos[i]			= settings.ypos[i];
		g_sets.eqxpos[i]		= settings.eqxpos[i];
		g_sets.eqypos[i]		= settings.eqypos[i];
		g_sets.eq2xpos[i]		= settings.eq2xpos[i];
		g_sets.eq2ypos[i]		= settings.eq2ypos[i];
		g_sets.effectsxpos[i]	= settings.effectsxpos[i];
		g_sets.effectsypos[i]	= settings.effectsypos[i];
		g_sets.listxpos[i]		= settings.listxpos[i];
		g_sets.listypos[i]		= settings.listypos[i];
		g_sets.fileoutxpos[i]	= settings.fileoutxpos[i];
		g_sets.fileoutypos[i]	= settings.fileoutypos[i];
	
		g_sets.xsize[i]			= settings.xsize[i];
		g_sets.ysize[i]			= settings.ysize[i];
		g_sets.eqxsize[i]		= settings.eqxsize[i];
		g_sets.eqysize[i]		= settings.eqysize[i];
		g_sets.eq2xsize[i]		= settings.eq2xsize[i];
		g_sets.eq2ysize[i]		= settings.eq2ysize[i];
		g_sets.effectsxsize[i]  = settings.effectsxsize[i];
		g_sets.effectsysize[i]  = settings.effectsysize[i];
		g_sets.listxsize[i]		= settings.listxsize[i];
		g_sets.listysize[i]		= settings.listysize[i];
		g_sets.fileoutxsize[i]	= settings.fileoutxsize[i];
		g_sets.fileoutysize[i]	= settings.fileoutysize[i];
	}
	
	g_sets.aboutxpos		= settings.aboutxpos;
	g_sets.aboutypos		= settings.aboutypos;
	g_sets.aboutxsize		= settings.aboutxsize;
	g_sets.aboutysize		= settings.aboutysize;

	g_sets.viewerxpos		= settings.viewerxpos;
	g_sets.viewerypos		= settings.viewerypos;
	g_sets.viewerxsize		= settings.viewerxsize;
	g_sets.viewerysize		= settings.viewerysize;

	g_sets.keybindingsxpos  = settings.keybindingsxpos;
	g_sets.keybindingsypos  = settings.keybindingsypos;
	g_sets.keybindingsxsize = settings.keybindingsxsize;
	g_sets.keybindingsysize = settings.keybindingsysize;

	g_sets.advancedxpos = settings.advancedxpos;
	g_sets.advancedypos = settings.advancedypos;
	g_sets.advancedxsize = settings.advancedxsize;
	g_sets.advancedysize = settings.advancedysize;

	MenuSetChecked(MENUSHOW, ID_MENU_ALTSIZE, g_sets.winconf ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_ALTSIZE, g_sets.winconf ? 1 : 0);
		
	if(!kbcommand && maModKeyDown())
		WindowsCopyPositions();
	WindowsMoveAndSetSize();
							
} // OnMenuDefaultSize


void OnMenuFactorySize(int kbcommand)
{
	WindowSetFactoryValues(g_sets.winconf);
	WindowSetFactoryValues(-1);
	if(!kbcommand && maModKeyDown())
		WindowsCopyPositions();
	WindowsMoveAndSetSize();
							
} // OnMenuFactorySize


void OnCheckEqWindow(int checked) 
{
	DoWindowEq(checked);
	DoWindowEq2(checked && g_sets.eqflags & MASEPARATELR);
	
} // OnCheckEqWindow


void OnCheckListWindow(int checked) 
{
	DoWindowList(checked);
	
} // OnCheckListWindow


void OnCheckPitch(int checked)
{
	if(checked)
		g_sets.effects |= EFFECTSPITCH;
	else
		g_sets.effects &= ~EFFECTSPITCH;
		
	if(g_sets.effects & EFFECTSON)
		g_effectschange = 1;
}


void OnCheckTempo(int checked)
{
	if(checked)
		g_sets.effects |= EFFECTSTEMPO;
	else
		g_sets.effects &= ~EFFECTSTEMPO;
		
	if(g_sets.effects & EFFECTSON)
		g_effectschange = 1;
}


void OnCheckSpeed(int checked)
{
	if(checked)
		g_sets.effects |= EFFECTSSPEED;
	else
		g_sets.effects &= ~EFFECTSSPEED;
		
	if(g_sets.effects & EFFECTSON)
		g_effectschange = 1;
}


void OnCheckQualSpeed(int checked)
{
	if(checked)
		g_sets.effects |= EFFECTSQUALSPEED;
	else
		g_sets.effects &= ~EFFECTSQUALSPEED;
		
	if(g_sets.effects & EFFECTSON)
		g_effectschange = 1;
}


void OnCheckQualFilter(int checked)
{
	if(checked)
		g_sets.effects |= EFFECTSQUALFILTER;
	else
		g_sets.effects &= ~EFFECTSQUALFILTER;
		
	if(g_sets.effects & EFFECTSON)
		g_effectschange = 1;
}


#if !defined(MANOCDA) && defined(__MAUNIX__)
void OnCheckCdParanoia(int checked)
{
	g_setscdrom.cdparanoia = checked;		
}
#endif


// comes here from eq menu or eq checkbox or OnInit
void DoWindowEq(int show)
{
	if(show)
	{
		// on MacOS X something moves the window if we create it too close to the top
		// or left edges, so we save position settings and restore them and reposition
		// the window after the window is created
		int xpos = g_sets.eqxpos[g_sets.winconf];
		int ypos = g_sets.eqypos[g_sets.winconf];
		
		if(!g_w[WINDOWEQ])
			EqualizerWindowMake();
		
		if(g_w[WINDOWEQ])
		{
			g_sets.eqflags |= MAEQWINDOW;
			MenuSetChecked(MENUSHOW, ID_MENU_SHOWEQ, 1);
			CheckBoxSetValue(WINDOWAPP, ID_CHK_EQWINDOW, 1);
			
			g_sets.eqxpos[g_sets.winconf] = xpos;
			g_sets.eqypos[g_sets.winconf] = ypos;
			
			WindowMoveAndSetSize(WINDOWEQ, g_sets.eqxpos[g_sets.winconf], g_sets.eqypos[g_sets.winconf],
										g_sets.eqxsize[g_sets.winconf], g_sets.eqysize[g_sets.winconf]);
			SetSliders(EQSLIDERS);
			PopupButtonSetValue(WINDOWEQ, ID_POPUP_EQPRESET, g_sets.eqpreset);
			EditTextSetText(WINDOWEQ, ID_EDITTEXT_EQ, g_sets.eqpresets[g_sets.eqpreset].eqname);
			StaticTextSetValue(WINDOWEQ, ID_TEXT_EQOFFSETVAL, g_sets.eqpresets[g_sets.eqpreset].eqoffset);
			CheckBoxSetValue(WINDOWEQ, ID_CHK_EQZOOM, (g_sets.eqflags & MAEQZOOM) ? 1 : 0);
			CheckBoxSetValue(WINDOWEQ, ID_CHK_EQLINK, (g_sets.eqflags & MAEQLINK) ? 1 : 0);
			CheckBoxSetValue(WINDOWEQ, ID_CHK_SEPARATELR, (g_sets.eqflags & MASEPARATELR) ? 1 : 0);
			
			if(WindowIsShown(WINDOWAPP))
				WindowShow(WINDOWEQ);			
		}
		else
			maLog("DoWindowEq EqualizerWindowMake failed");
	}
	else
	{
		if(g_w[WINDOWEQ])
		{
			WindowDoClose(WINDOWEQ);
			g_w[WINDOWEQ] = NULL;
		}
	}
	
} // DoWindowEq


void DoWindowEq2(int show)
{
	if(show)
	{
		// on MacOS X something moves the window if we create it too close to the top
		// or left edges, so we save position settings and restore them and reposition
		// the window after the window is created
		int xpos = g_sets.eq2xpos[g_sets.winconf];
		int ypos = g_sets.eq2ypos[g_sets.winconf];
		
		if(!g_w[WINDOWEQ2])
			Equalizer2WindowMake();
		
		if(g_w[WINDOWEQ2])
		{
			g_sets.eq2xpos[g_sets.winconf] = xpos;
			g_sets.eq2ypos[g_sets.winconf] = ypos;
			
			WindowMoveAndSetSize(WINDOWEQ2, g_sets.eq2xpos[g_sets.winconf], g_sets.eq2ypos[g_sets.winconf],
										g_sets.eq2xsize[g_sets.winconf], g_sets.eq2ysize[g_sets.winconf]);
			SetSliders(EQ2SLIDERS);
			
			if(WindowIsShown(WINDOWAPP))
				WindowShow(WINDOWEQ2);			
		}
		else
			maLog("DoWindowEq2 Equalizer2WindowMake failed");
	}
	else
	{
		if(g_w[WINDOWEQ2])
		{
			WindowDoClose(WINDOWEQ2);
			g_w[WINDOWEQ2] = NULL;
		}
	}
	
} // DoWindowEq2


// comes here from effects menu or effects checkbox
void DoWindowEffects(int show)
{
	if(show) // && g_w[WINDOWEFFECTS])
	{
		// on MacOS X something moves the window if we create it too close to the top
		// or left edges, so we save position settings and restore them and reposition
		// the window after the window is created
		int xpos = g_sets.effectsxpos[g_sets.winconf];
		int ypos = g_sets.effectsypos[g_sets.winconf];
		
		if(!g_w[WINDOWEFFECTS])
			EffectsWindowMake();
			
		if(g_w[WINDOWEFFECTS])
		{
			g_sets.effectswindow = 1;
			MenuSetChecked(MENUSHOW, ID_MENU_SHOWEFFECTS, 1);

			CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_PITCH, (g_sets.effects & EFFECTSPITCH) ? 1 : 0);
			CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_TEMPO, (g_sets.effects & EFFECTSTEMPO) ? 1 : 0);
			CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_SPEED, (g_sets.effects & EFFECTSSPEED) ? 1 : 0);
			CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_QUALFILTER, (g_sets.effects & EFFECTSQUALFILTER) ? 1 : 0);
			CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_QUALSPEED, (g_sets.effects & EFFECTSQUALSPEED) ? 1 : 0);
			CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_EFFECTSB, (g_sets.effects & EFFECTSON) ? 1 : 0);

			SetSliders(EFFECTSSLIDERS);
			
#if defined(MAKEYBOARD)
			UpdateEffectsKbLabels();
#endif
			g_sets.effectsxpos[g_sets.winconf] = xpos;
			g_sets.effectsypos[g_sets.winconf] = ypos;
			
			WindowMoveAndSetSize(WINDOWEFFECTS,
				g_sets.effectsxpos[g_sets.winconf], g_sets.effectsypos[g_sets.winconf],	
				g_sets.effectsxsize[g_sets.winconf], g_sets.effectsysize[g_sets.winconf]);
			
			if(WindowIsShown(WINDOWAPP))
				WindowShow(WINDOWEFFECTS);
		}
		else
			maLog("DoWindowEffects EffectsWindowMake failed");
	}
	else
		if(g_w[WINDOWEFFECTS])
		{
			WindowDoClose(WINDOWEFFECTS);
			g_w[WINDOWEFFECTS] = NULL;
		}

} // DoWindowEffects


#if !defined(MANOCDA) && defined(__MAUNIX__)
void DoWindowAudioCD(int show)
{
	int i;
	
	if(show)
	{
		// on MacOS X something moves the window if we create it too close to the top
		// or left edges, so we save position settings and restore them and reposition
		// the window after the window is created
		int xpos = g_setscdrom.winx[g_sets.winconf];
		int ypos = g_setscdrom.winy[g_sets.winconf];
		
		if(g_w[WINDOWAUDIOCD])
			return; // already up

		AudioCDWindowMake();
			
		if(g_w[WINDOWAUDIOCD])
		{
			CheckBoxSetValue(WINDOWAUDIOCD, ID_CHK_CDPARANOIA, (g_setscdrom.cdparanoia) ? 1 : 0);			
			PopupButtonSetValue(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, 1);
			
			g_setscdrom.winx[g_sets.winconf] = xpos;
			g_setscdrom.winy[g_sets.winconf] = ypos;
			
			WindowMoveAndSetSize(WINDOWAUDIOCD,
				g_setscdrom.winx[g_sets.winconf], g_setscdrom.winy[g_sets.winconf],
				g_setscdrom.winw[g_sets.winconf], g_setscdrom.winh[g_sets.winconf]);
			
			for(i = 1; i <= CDAMAXNUMDRIVES; i++)
				 PopupButtonAppend(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, g_setscdrom.cdaname[i]);
				
			PopupButtonSetSelection(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, g_setscdrom.item);
			
			if(WindowIsShown(WINDOWAPP))
				WindowShow(WINDOWAUDIOCD);
		}
		else
			maLog("DoWindowAudioCD AudioCDWindowMake failed");
	}
	else
		if(g_w[WINDOWAUDIOCD])
		{
			WindowDoClose(WINDOWAUDIOCD);
			g_w[WINDOWAUDIOCD] = NULL;
		}

} // DoWindowAudioCD
#endif


void DoWindowAdvanced(int show)
{
	if(show)
	{
		// on MacOS X something moves the window if we create it too close to the top
		// or left edges, so we save position settings and restore them and reposition
		// the window after the window is created
		int xpos = g_sets.advancedxpos;
		int ypos = g_sets.advancedypos;
		
		if(!g_w[WINDOWADVANCED])
			AdvancedWindowMake();
			
		if(g_w[WINDOWADVANCED])
		{
			g_sets.advancedxpos = xpos;
			g_sets.advancedypos = ypos;
			
			WindowMoveAndSetSize(WINDOWADVANCED, g_sets.advancedxpos, g_sets.advancedypos,
												 g_sets.advancedxsize, g_sets.advancedysize);

			CheckBoxSetValue(WINDOWADVANCED, ID_CHK_CLOSEREOPEN, g_sets.fileout[kOUTSOUND].mode ? 1 : 0);
			CheckBoxSetValue(WINDOWADVANCED, ID_CHK_HIGHPRIORITY, g_sets.highpriority ? 1 : 0);			
			if(WindowIsShown(WINDOWAPP))
				WindowShow(WINDOWADVANCED);
		}
		else
			maLog("DoWindowAdvanced AdvancedWindowMake failed");
	}
	else
		if(g_w[WINDOWADVANCED])
		{
			WindowDoClose(WINDOWADVANCED);
			g_w[WINDOWADVANCED] = NULL;
		}

} // DoWindowAdvanced


// comes here from list menu or list checkbox
void DoWindowList(int show)
{
	if(show) // && g_w[WINDOWLIST])
	{
		// on MacOS X something moves the window if we create it too close to the top
		// or left edges, so we save position settings and restore them and reposition
		// the window after the window is created
		int xpos = g_sets.listxpos[g_sets.winconf];
		int ypos = g_sets.listypos[g_sets.winconf];
		
		if(!g_w[WINDOWLIST])
			ListWindowMake();
			
		if(g_w[WINDOWLIST])
		{
			g_sets.listwindow |= LBOXON;
			MenuSetChecked(MENUSHOW, ID_MENU_SHOWLIST, 1);
			CheckBoxSetValue(WINDOWAPP, ID_CHK_LISTWINDOW, 1);
			CheckBoxSetValue(WINDOWLIST, ID_CHK_SHOWINDEX, (g_sets.listwindow & LBOXINDEX) ? 1 : 0);
			CheckBoxSetValue(WINDOWLIST, ID_CHK_SHOWDIR, (g_sets.listwindow & LBOXDIR) ? 1 : 0);
			CheckBoxSetValue(WINDOWLIST, ID_CHK_SHOWPATH, (g_sets.listwindow & LBOXPATH) ? 1 : 0);
			CheckBoxSetValue(WINDOWLIST, ID_CHK_ADDMODE, g_sets.addmode ? 1 : 0);
			
			g_sets.listxpos[g_sets.winconf] = xpos;
			g_sets.listypos[g_sets.winconf] = ypos;
			
			WindowMoveAndSetSize(WINDOWLIST,
				g_sets.listxpos[g_sets.winconf], g_sets.listypos[g_sets.winconf],	
				g_sets.listxsize[g_sets.winconf], g_sets.listysize[g_sets.winconf]);
			
			if(WindowIsShown(WINDOWAPP))
				WindowShow(WINDOWLIST);
			ShowListWindow();
		}
		else
			maLog("DoWindowList ListWindowMake failed");
	}
	else if(g_w[WINDOWLIST])
	{
		WindowDoClose(WINDOWLIST);
		g_w[WINDOWLIST] = NULL;
	}

} // DoWindowList


void DumpString(FILE *fp, char *p)
{
	if(fp)
 		fprintf(fp, "%s\n", p);
 	else
 		ListWindowAddString(p);
 	
} // DumpString


void DoWindowFileOut(int show)
{
	int i;
	
	for(i = kOUTFIRST; i <= kOUTLAST; i++) // close all fileout windows
	{
		if(g_w[WINDOWOUTFIRST + i - 1]) // WINDOWOUTCUR
		{
			WindowGetCoordinates(WINDOWOUTFIRST + i - 1,
						&g_sets.fileoutxpos[g_sets.winconf], &g_sets.fileoutypos[g_sets.winconf],
						&g_sets.fileoutxsize[g_sets.winconf], &g_sets.fileoutysize[g_sets.winconf]);
			WindowDoClose(WINDOWOUTFIRST + i - 1);
			g_w[WINDOWOUTFIRST + i - 1] = NULL;
		}
	}
			
	if(!show)
		return;

	if(WINDOWOUTCUR < WINDOWOUTFIRST || WINDOWOUTCUR > WINDOWOUTLAST)
	{
		g_sets.fileoutwindow = 0;
		MenuSetChecked(MENUSHOW, ID_MENU_SHOWFILEOUT, 0);
		return;
	}
	
	if(!g_w[WINDOWOUTCUR])
		FileOutWindowMake();
		
	if(g_w[WINDOWOUTCUR])
	{
		g_sets.fileoutwindow = 1;
		MenuSetChecked(MENUSHOW, ID_MENU_SHOWFILEOUT, 1);
		WindowMoveAndSetSize(WINDOWOUTCUR, g_sets.fileoutxpos[g_sets.winconf],
										g_sets.fileoutypos[g_sets.winconf],
										g_sets.fileoutxsize[g_sets.winconf],
										g_sets.fileoutysize[g_sets.winconf]);

#if defined(ENCODEMP3) || defined(ENCODEOGG) || defined(ENCODEAAC) || defined(ENCODEMP4) 
		switch (g_sets.output)
		{
		
 #ifdef ENCODEMP3
			case kOUTMP3 :
				PopupButtonSetValue(WINDOWOUTCUR, ID_POPUP_QUALITY,		g_sets.fileout[g_k[g_sets.output]].quality);
				PopupButtonSetValue(WINDOWOUTCUR, ID_POPUP_BITRATE,		g_sets.fileout[g_k[g_sets.output]].bitrate);
				PopupButtonSetValue(WINDOWOUTCUR, ID_POPUP_MODE,		g_sets.fileout[g_k[g_sets.output]].mode);
				PopupButtonSetValue(WINDOWOUTCUR, ID_POPUP_SAMPLERATE,  g_sets.fileout[g_k[g_sets.output]].samplerate);
			break;
 #endif
 
 #ifdef ENCODEOGG
			case kOUTOGG :
				//PopupButtonSetValue(WINDOWOUTCUR, ID_POPUP_QUALITY, g_sets.fileout[g_k[g_sets.output]].quality); // not used
				PopupButtonSetValue(WINDOWOUTCUR, ID_POPUP_BITRATE, g_sets.fileout[g_k[g_sets.output]].bitrate);
				PopupButtonSetValue(WINDOWOUTCUR, ID_POPUP_MODE,    g_sets.fileout[g_k[g_sets.output]].mode);
			break;
 #endif
 
 #ifdef ENCODEAAC
			case kOUTAAC :
				PopupButtonSetValue(WINDOWOUTCUR, ID_POPUP_QUALITY, g_sets.fileout[g_k[g_sets.output]].quality);
				CheckBoxSetValue(WINDOWOUTCUR, ID_CHK_ADTS, g_sets.fileout[g_k[g_sets.output]].mode & kACCADTS ? 1 : 0);
			break;
 #endif
 
 #ifdef ENCODEMP4
			case kOUTMP4 :
				PopupButtonSetValue(WINDOWOUTCUR, ID_POPUP_QUALITY, g_sets.fileout[g_k[g_sets.output]].quality);
				CheckBoxSetValue(WINDOWOUTCUR, ID_CHK_M4A,   g_sets.fileout[g_k[g_sets.output]].mode & kACCM4A ? 1 : 0);
			break;
 #endif
 
			default :
			break;
		} // switch
#endif
		
		StaticTextSetText(WINDOWOUTCUR, ID_TEXT_FILEOUTDIR, g_sets.fileout[g_k[g_sets.output]].outdir);

		if(g_running)
			FileOutWindowDisable();
		
		if(WindowIsShown(WINDOWAPP))
			WindowShow(WINDOWOUTCUR);
	}
	else
		maLog("DoWindowFileOut FileOutWindowMake failed");

	if(!maDirExists(g_sets.fileout[g_k[g_sets.output]].outdir))
	{
		char cstr[MAX_PATH];
		sprintf(cstr, "File output folder does not exist:\n\n '%s'", g_sets.fileout[g_k[g_sets.output]].outdir);
		ShowErr(cstr);
	}
	
} // DoWindowFileOut


// comes here from eq menu or eq checkbox or OnInit
void DoWindowPromptNum(int show)
{
	if(show)
	{
		if(!g_w[WINDOWPROMPTNUM])
			PromptNumWindowMake();
		
		if(g_w[WINDOWPROMPTNUM])
		{
		}
		else
			maLog("DoWindowPromptNum PromptNumWindowMake failed");
	}
	else
		if(g_w[WINDOWPROMPTNUM])
		{
			WindowDoClose(WINDOWPROMPTNUM);
		}
	
} // DoWindowPromptNum


#ifdef __MAWIN__
 int GetTrackLength(const int nTrack);
#endif
int DumpSongInfo(char *songfpath, FILE *dumpfp, int nomem)
{
	MASTATE state;
	char cstr[256];
#if defined(__MAWIN__) && !defined(MANOCDA)
	int drive;
	int track;
	char tstr[32];
#endif

	char cstrx[512];
	
	memset((char *)&state, 0, sizeof(MASTATE));
	state.nomem = nomem;
	strcpy(state.inpath, songfpath);
	
	if(FileNameHasExtensionPlayable(songfpath))
	{
		if(nomem)
		{
			if((state.fp = mafopen(songfpath, (char *)"rb")) == NULL)
			{
				sprintf(cstr, "DumpSongInfo %d mafopen '%s'", errno, songfpath);
				maLog(cstr);
				return 0;
			}
		}
		else
		{
			if((state.fp = memfopen(songfpath, (char *)"rb")) == NULL)
			{
				sprintf(cstr, "DumpSongInfo %d memfopen", errno);
				maLog(cstr);
				return 0;
			}
		}
		
		state.size = maGetFileSize(state.fp, 0, nomem);

		if(state.size >= 1000000)
		{
			int millions = state.size / 1000000;
			int thousands = (state.size - millions * 1000000) / 1000;
			sprintf(cstrx, "Size : %d,%03d,%03d bytes", millions, thousands, (state.size - millions * 1000000) - thousands * 1000);
		}
		else
		if(state.size >= 1000)
		{
			int thousands = state.size / 1000;
			sprintf(cstrx, "Size : %d,%03d bytes", thousands, state.size - thousands * 1000);
		}
		else
			sprintf(cstrx, "Size : %d bytes", state.size);
		DumpString(dumpfp, cstrx);

#ifndef NODECODEMP3
		if(FileNameHasExtension(songfpath, g_extstr[kMAMP3]) ||
			FileNameHasExtension(songfpath, g_extstr[kMAMP2]) ||
			FileNameHasExtension(songfpath, g_extstr[kMAMP1]))
			DumpInfoMp3(&state, dumpfp);
		else
#endif

#ifndef NODECODEOGG
		if(FileNameHasExtension(songfpath, g_extstr[kMAOGG]))
			DumpInfoOgg(&state, dumpfp);
		else 
#endif

#ifndef NODECODEFLAC
		if(FileNameHasExtension(songfpath, g_extstr[kMAFLAC]) ||
			FileNameHasExtension(songfpath, g_extstr[kMAFLA]))
			DumpInfoFlac(&state, dumpfp);
		else 
#endif
			
#ifndef NODECODEWMA
		if(FileNameHasExtension(songfpath, g_extstr[kMAWMA]))
			DumpInfoWma(&state, dumpfp);
		else
#endif

#ifndef NODECODEAAC
		if(FileNameHasExtension(songfpath, g_extstr[kMAAAC]) ||
			FileNameHasExtension(songfpath, g_extstr[kMAM2A]) ||
			FileNameHasExtension(songfpath, g_extstr[kMAMP2]))
			DumpInfoAac(&state, dumpfp);
		else
#endif

#ifndef NODECODEMP4
		if(FileNameHasExtension(songfpath, g_extstr[kMAMP4]) ||
			FileNameHasExtension(songfpath, g_extstr[kMAM4A]))
			DumpInfoMp4(&state, dumpfp);
		else
#endif

#ifndef NODECODEAIF
		if(FileNameHasExtension(songfpath, g_extstr[kMAAIF]) ||
			FileNameHasExtension(songfpath, g_extstr[kMAAIFF]))
			DumpInfoAif(&state, dumpfp);
		else
#endif

#ifndef NODECODEWAV
		if(FileNameHasExtension(songfpath, g_extstr[kMAWAV]))
			DumpInfoWav(&state, dumpfp);
		else
#endif
			; // valuable semicolon here
		
		if(nomem)
			fclose(state.fp);
		else
			memfclose(state.fp);
	}
	
#if defined(__MAWIN__) && !defined(MANOCDA)
	else if((drive = IsAudioCd(songfpath)))
	{
		if(drive == g_cdaactive)
		{
			if(FileNameHasExtension(songfpath, g_extstr[kMACDA]))
				track = atoi(&songfpath[strlen(songfpath) - 6 ]);
			else
				track = atoi(&songfpath[strlen(songfpath) - 2 ]);
			sprintf(cstrx, "Duration : %s", SecondsToStrHMS(tstr, GetTrackLength(track)));
			DumpString(dumpfp, cstrx);
		}
	}
#endif
	
	return state.size;

} // DumpSongInfo


int ShowPlayerInfo()
{
#ifndef MANOCDA
	int i;
#endif
	char name[256];
	char cstr[256];
	
	if(!g_w[WINDOWLIST])
		return 1;
		
	CheckBoxHide(WINDOWLIST, ID_CHK_SHOWINDEX);
	CheckBoxHide(WINDOWLIST, ID_CHK_SHOWDIR);
	CheckBoxHide(WINDOWLIST, ID_CHK_SHOWPATH);
	CheckBoxHide(WINDOWLIST, ID_CHK_ADDMODE);
	
	name[0] = 0;
	
	SetListWindowTitle((char *)"Player Info");
	ListWindowClear();
	ListWindowSetHeader((char *)"Info");

	if(g_songlist == NULL || !g_songlist[0])
		sprintf(cstr, "No songlist");
	else if(FilenameFromPathname(g_songlist, name))
		sprintf(cstr, "Missing songlist '%s'", name);
	else if(!l_songrec || g_songsnum <= 0)
		sprintf(cstr, "Bad or empty '%s' : %d songs", name, g_songsnum);
	else
		sprintf(cstr, "'%s'", g_songlist);
	ListWindowAddString(cstr);
	ListWindowAddString((char *)"");

	// sprintf(cstr, "Settings version %s", g_sets.settingsversion);
	// ListWindowAddString(cstr);
	// ListWindowAddString((char *)"");


#if !defined(NODECODEAUDIOIN)
	if(!strcmp(g_songlist, g_songlistaudioin))
		sprintf(cstr, (char *)"Using audio input");
	else
#endif
	{
		sprintf(cstr, "%d songs, %d plays", g_songsnum, l_songsnumplayed);
		ListWindowAddString(cstr);
		ListWindowAddString((char *)"");
		
		if(g_sets.buffertype == BUFUSEMEM)
			sprintf(cstr, "Current buffer = %.2f Mb", g_loadedmemsize / (double)MEGABYTE);
		else if(g_sets.buffertype == BUFUSEVDISK)
			sprintf(cstr, "Virtual Disk : '%s'", g_sets.vdiskname);
		else
			sprintf(cstr, (char *)"Direct from disk (no buffer)");
	}
	ListWindowAddString(cstr);
	ListWindowAddString((char *)"");
	
#ifndef MANOCDA
	for(i = 1; i <= CDAMAXNUMDRIVES; i++)
	{
		if(g_cdadrive[i].exists)
		{
#ifdef __MAWIN__
			sprintf(cstr, "CDRom '%c:' : ready=%d active=%d tracks=%d",
				g_cdadrive[i].fsvolrefnum, g_cdadrive[i].isready, g_cdadrive[i].isactive,
				g_cdadrive[i].tracklast);
#endif

#ifdef __MAOSX__
	sprintf(cstr, "CDRom '%s:' : ready=%d tracks=%d",
				g_cdadrive[i].name, g_cdadrive[i].isready, g_cdadrive[i].tracklast);
#endif

#ifdef __MAOS9__
			sprintf(cstr, "CDRom '%s' : ready=%d", g_cdadrive[i].name, g_cdadrive[i].isready);
#endif

#ifdef __MAUNIX__
			if(g_cdadrive[i].isready)
				sprintf(cstr, "CDRom '%s' : ready=%d tracks=%d",
						g_cdadrive[i].name, g_cdadrive[i].isready, g_cdadrive[i].tracklast);
			else
				sprintf(cstr, "CDRom '%s' : ready=%d", g_cdadrive[i].name, g_cdadrive[i].isready);
#endif

			ListWindowAddString(cstr);
		}	
	}
#endif
	
	ListWindowAddString((char *)"");	
	ListWindowShowRow(-1); // scroll to top, no hilite
	
	return 0;
	
} // ShowPlayerInfo


int ShowSongList()
{
	int i, l;
	char header[32];
	char songpath[MAX_PATH];
	char name[MAX_PATH];
	char cstr[256 + MAX_PATH];
	FILE *songsfp;
	
#if !defined(MAGUINONE)
	if(!g_w[WINDOWLIST])
		return 1;
#endif
		
	CheckBoxShow(WINDOWLIST, ID_CHK_SHOWINDEX);
	CheckBoxShow(WINDOWLIST, ID_CHK_SHOWDIR);
	CheckBoxShow(WINDOWLIST, ID_CHK_SHOWPATH);
	if(!strcmp(g_songlist, g_songlistdrop))
		CheckBoxShow(WINDOWLIST, ID_CHK_ADDMODE);
	else
		CheckBoxHide(WINDOWLIST, ID_CHK_ADDMODE);
	
	header[0] = 0;
	if(g_sets.listwindow & LBOXINDEX)
		strcpy(header, "Index  ");
	if(g_sets.listwindow & LBOXDIR)
		strcat(header, "Dir / File name");
	else if(g_sets.listwindow & LBOXPATH)
		strcat(header, "File path");
	else
		strcat(header, "File name");
	SetListWindowTitle((char *)"Song List");
	ListWindowClear();
	ListWindowSetHeader(header);

	songpath[0] = 0;
	if(g_songlist == NULL || !g_songlist[0])
	{
		ListWindowAddString((char *)"(No songlist)");
		return 1;
	}
	else if(FilenameFromPathname(g_songlist, name))
	{
		sprintf(cstr, "(Missing songlist '%s')", g_songlist);
		ListWindowAddString(cstr);
		return 1;
	}
	else if(!l_songrec || g_songsnum <= 0)
	{
		sprintf(cstr, "Bad or empty songlist '%s' : %d songs", g_songlist, g_songsnum);
		ListWindowAddString(cstr);
		return 1;
	}
	
	g_cancel = 0;

	if((songsfp = mafopen(g_songlist, (char *)"rb")) == NULL)
	{
		sprintf(cstr, "ShowSongList mafopen failed on '%s'", g_songlist);
		maLog(cstr);
		return 2;
	}

	i = 0;
	
	if(fseek(songsfp, 0L, SEEK_SET))
	{
		sprintf(cstr, "ShowSongList fseek failed errno=%d on songlist '%s'", errno, g_songlist);
		maLog(cstr);
		return 2;
	} 
	
	while(1)
	{
		songpath[0] = 0;
		
		if(mafgets(songpath, MAX_PATH, songsfp) == NULL)
			break;
			
		// skip lines with '#' or files that don't exist
		if(songpath[0] == '#' || !SongFileExists(songpath))
			continue;

		cstr[0] = 0;
		
 #if !defined(MAGUINONE)
 		if(g_sets.listwindow & LBOXINDEX)
 #endif
 			sprintf(cstr, "%05d ", 1 + i);

		if(g_sets.listwindow & LBOXDIR)
		{
			l = strlen(songpath) - 1;						// .../dir/name.mp3
			while (l >= 0 && songpath[l--] != FILESEP){};	// .../dir
			while (l >= 0 && songpath[l--] != FILESEP){};	// ...
			strcat(cstr, &songpath[l + 2]);					// dir/name.mp2
		}
		else if(g_sets.listwindow & LBOXPATH)
			strcat(cstr, songpath);
		else
		{
			FilenameFromPathname(songpath, name);
			strcat(cstr, name);
		}

 #if defined(__MAOSX__)
		StrBSDtoStrOSX(cstr);
 #endif
		ListWindowAddString(cstr);

		if(g_cancel)
			break;
					
		i++;
						
	} // while
				
	fclose(songsfp);

	if(g_running)
		ListWindowShowRowPlaying();
	else
		ListWindowShowRow(-1); // scroll to top, no hilite

	return 0;
	
} // ShowSongList


void ListWindowShowRowPlaying()
{
	l_noshowrow = 0;
	if(g_sets.listwindow & LBOXON && g_sets.listwindow & LBOXLOADED)
		ListWindowShowRow(g_loadedcurrent);
	else if(g_sets.listwindow & LBOXON && g_sets.listwindow & LBOXLIST)
		ListWindowShowRow(l_songsindex[g_loadedcurrent]);
	
} // ListWindowShowRowPlaying


int ListWindowGetValue()
{
	if(!g_w[WINDOWLIST])
		return -1;
		
	return ListGetRowSelected(WINDOWLIST, ID_LIST_CTRL);
		
} // ListWindowGetValue


void ListWindowAddString(char *str)
{
#if !defined(MAGUINONE)
	if(!g_w[WINDOWLIST])
		return;
#endif
		
	ListAppend(WINDOWLIST, ID_LIST_CTRL, str);
	
} // ListWindowAddString


void ListWindowShowRow(int inrow) // 0-based
{
	int row;
	
	if(!g_w[WINDOWLIST])
		return;
	
	if((row = inrow) < 0) // no hilite if row < 0
	{
		row = -inrow;
#if defined(MAGUIOSX)
		if(inrow == -1)
			row = 0;
#endif
	}
	
	// scroll to make row visible
	ListEnsureVisible(WINDOWLIST, ID_LIST_CTRL, row);
	
	// hilite highlight row
	if(inrow >= 0)
		ListSelect(WINDOWLIST, ID_LIST_CTRL, row);
	
} // ListWindowShowRow


void ListWindowClear()
{
	if(!g_w[WINDOWLIST])
		return;
		
	ListClear(WINDOWLIST, ID_LIST_CTRL);
	
} // ListWindowClear


void ListWindowSetHeader(char *header)
{
	if(!g_w[WINDOWLIST])
		return;
		
	ListSetHeader(WINDOWLIST, ID_LIST_CTRL, 0, header);
	
} // ListWindowSetHeader


void ListWindowCopyRow(char *str, int row)
{
	ListGetText(WINDOWLIST, ID_LIST_CTRL, row, str);

} // ListWindowCopyRow


int ShowSongInfo()
{
	int clicked;
	int nomem;	
	char fpath[256];
 #if defined(__MAOSX__)
	char osxpath[256];
 #endif
	char cstr[256];

	if(!g_w[WINDOWLIST])
		return 1;
		
	CheckBoxHide(WINDOWLIST, ID_CHK_SHOWINDEX);
	CheckBoxHide(WINDOWLIST, ID_CHK_SHOWDIR);
	CheckBoxHide(WINDOWLIST, ID_CHK_SHOWPATH);
	CheckBoxHide(WINDOWLIST, ID_CHK_ADDMODE);

	if(l_strclicked[0]) // show song info for song clicked in song list
	{
		clicked = 1;
		strcpy(fpath, l_strclicked);
		l_strclicked[0] = 0;
	}
	else // show song info for g_loadedcurrent
	{
		clicked = 0;
		if(g_loadednum <= 0)
			strcpy(fpath, (char *)"(No songs loaded)");
		else
			strcpy(fpath, g_hdsongpath[g_loadedcurrent]);
	}

	g_sets.listwindow &= ~(LBOXON | LBOXINFO | LBOXLIST | LBOXLOADED | LBOXID3);
	g_sets.listwindow |= (LBOXON | LBOXID3);

	SetListWindowTitle((char *)"Song Info");
	ListWindowClear();
	ListWindowSetHeader((char *)"Song info");
	
 #if defined(__MAOSX__)
	strcpy(osxpath, fpath);
	StrBSDtoStrOSX(osxpath);			
	FilenameFromPathname(osxpath, cstr);
 #else
	FilenameFromPathname(fpath, cstr);
 #endif
 	ListWindowAddString(cstr);

	if(g_loadednum <= 0 && !clicked)
	{
		ListWindowShowRow(-1); // scroll to top, no hilite
		return 0;
	}
		
#ifndef MANOCDA
	if(IsAudioCd(fpath))
	{
		nomem = 1;	
	 	ListWindowAddString((char *)"(on CD)");
	}
	else
#endif
	if(SongIsInMemory(fpath))
	{
		nomem = 0;	
	 	ListWindowAddString((char *)"(loaded)");
	}
	else
	{
		nomem = 1;	
	 	ListWindowAddString((char *)"(on disk)");
	}

	DumpSongInfo(fpath, NULL, nomem);

	ListWindowShowRow(-1); // scroll to top, no hilite

	return 0;

} // ShowSongInfo


int GetRecentSonglist() // called from OnInit when app is starting
{
	char cstr[256];

	g_songlist[0] = 0;

	// get last songlist used
	if(maFileExists(l_recentlist))
		GetRecentSongListName();
		
	if(g_songlist[0] && !maFileExists(g_songlist))
	{
		if(g_songlist[0])
			sprintf(cstr,   "Songlist not found :\n\n"
							"'%s'\n\n"
							"Open a song list with the 'Open Song List' menu\n"
							"or\n"
							"Make one with the 'Make Song List' menu\n"
							"or\n"
							"Drop files on the MoreAmp icon or window.", g_songlist); 
		else
			sprintf(cstr,   "No song list.\n\n"
							"Make one with the 'Make Song List' menu\n"
							"or\n"
							"Drop files on the MoreAmp icon or window.");

		g_songlist[0] = 0;
		ShowErr(cstr);
		return 1;
	}
	
	if(OpenSongList())
		return 1;

	return 0;
	
} // GetRecentSonglist


// output
FILE *g_outputfilefp;
char g_outputfile[MAX_PATH];


int MakeOutputFileName()
{
	char *p;
	char songname[ MAX_PATH];
	char cstr[256 + MAX_PATH];
	
	// output file
	if(!maDirExists(g_sets.fileout[g_k[g_sets.output]].outdir))
	{
		sprintf(cstr, "File output folder does not exist:\n\n '%s'", g_sets.fileout[g_k[g_sets.output]].outdir);
		ShowErr(cstr);
		return 1;
	}
	
	strcpy(songname, g_loadedname[g_loadedcurrent]);
	
#ifdef __MAWIN__
	while((p = strchr(songname, VOLSEP)))
		*p = '_'; // mainly for cda files, e.g. "D:\Track01.cda"
#endif

	while((p = strchr(songname, FILESEP)))
		*p = '_';
	
	sprintf(g_outputfile, "%s%c%s%s", g_sets.fileout[g_k[g_sets.output]].outdir, FILESEP, songname, g_extstrout[g_sets.output]);	
#if defined(__MAOSX__)
	StrOSXtoStrBSD(g_outputfile);
#endif

	if(strlen(g_outputfile) > strlen(g_sets.fileout[g_k[g_sets.output]].outdir) + 1 + NAMELENMAX)
		strcpy(&g_outputfile[strlen(g_sets.fileout[g_k[g_sets.output]].outdir) + NAMELENMAX - 4], g_extstrout[g_sets.output]);

	// set extension for .m4a
#ifdef ENCODEMP4
	if(g_sets.output == kOUTMP4 && g_sets.fileout[kOUTMP4].mode & kACCM4A)
		strcpy(&g_outputfile[strlen(g_outputfile) - 4], EXTM4A);
#endif
	
	if(maFileExists(g_outputfile))
	{
		sprintf(cstr, "Output file already exists and will be overwritten\n\n'%s'", g_outputfile);
		if(maMessageBox(cstr, (char *)"CONTINUE ?", maYES_NO | maICON_QUESTION) != maYES)
			return 1;
		
		if(maunlink(g_outputfile))
		{
			sprintf(cstr, "MakeOutputFileName failed to delete '%s'", g_outputfile);
			maLog(cstr);
			return 1;
		}		
	}
	
	return 0;
	
} // MakeOutputFileName


int maCloseOutputFile()
{
	char cstr[256];
	
	if(!g_outputfilefp)
	{
		sprintf(cstr, "maCloseOutputFile file not open '%s'", g_outputfile);
		maLog(cstr);
		return 1;
	}
	else if(g_outputfilefp == DUMMYOUTFILEP) // dummy close when outputfile is handled by lib
	{
		g_outputfilefp = NULL;
		return 0;
	}
		
	if(fclose(g_outputfilefp))
	{
		sprintf(cstr, "maCloseOutputFile fclose errno=%d on '%s'", errno, g_outputfile);
		maLog(cstr);
		g_outputfilefp = NULL;
		return 1;
	}

	g_outputfilefp = NULL;
	
	return 0;
	
} // maCloseOutputFile


int maOpenOutputFile()
{
	char cstr[256 + MAX_PATH];
	
#ifdef ENCODEMP4
	if(g_sets.output == kOUTMP4) // this is opened by the lib
	{
		g_outputfilefp = NULL; // this is used by the lib
	}
	else
#endif

#ifdef ENCODEFLAC
	if(g_sets.output == kOUTFLAC) // this is opened by the lib
	{
		g_outputfilefp = DUMMYOUTFILEP; // this is not used by the lib, dummy value for close, etc
	}
	else
#endif

	{
		if((g_outputfilefp = mafopen(g_outputfile, (char *)"w+b")) == NULL)
		{
			sprintf(cstr, "maOpenOutputFile mafopen '%s'", g_outputfile);
			maLog(cstr);
			return 1;
		}

#if defined(__MAOSX__) || defined(__MAUNIX__)
		machmod(g_outputfile, ACCESSPERMS); // 0777
#endif
	}
	
	// format specific 'open'
	switch (g_sets.output)
	{
#ifdef ENCODEMP3
		case kOUTMP3 :
			if(!OpenWriteMp3())
				return 0;
		break;
#endif
		
#ifdef ENCODEOGG
		case kOUTOGG :
			if(!OpenWriteOgg())
				return 0;
		break;
#endif
		
#ifdef ENCODEAIF
		case kOUTAIF :
			if(!OpenWriteAif())
				return 0;
		break;
#endif
		
#ifdef ENCODEWAV
		case kOUTWAV :
			if(!OpenWriteWav())
				return 0;
		break;
#endif
		
#ifdef ENCODEAAC
		case kOUTAAC :
			if(!OpenWriteAac())
				return 0;
		break;
#endif
		
#ifdef ENCODEMP4
		case kOUTMP4 :
			if(!OpenWriteMp4())
				return 0;
		break;
#endif
		
#ifdef ENCODEWMA
		case kOUTWMA :
			if(!OpenWriteWma())
				return 0;
		break;
#endif
		
#ifdef ENCODEFLAC
		case kOUTFLAC :
			if(!OpenWriteFlac())
				return 0;
		break;
#endif
		
		default:
		break;
	}
	
	// error:
	if(g_outputfile != NULL)
		maCloseOutputFile();
		
	return 1;
	
} // maOpenOutputFile


int maOutputToFile(void* pcmblock, unsigned long frames, unsigned long len, int whichbuf)
{
#ifdef MAUNUSED
 (void)whichbuf;
#endif

	int err = 0;
	
	if(!g_outputfilefp)
	{
		maLog("maOutputToFile error null g_outputfilefp");
		return 1;
	}
	
	switch (g_sets.output)
	{
#ifdef ENCODEMP3
		case kOUTMP3 :
			err = WriteMp3(pcmblock, frames, len);
		break;
#endif
		
#ifdef ENCODEOGG
		case kOUTOGG :
			err = WriteOgg(pcmblock, frames, len);
		break;
#endif
		
#ifdef ENCODEAIF
		case kOUTAIF :
			err = WriteAif(pcmblock, frames, len);
		break;
#endif
		
#ifdef ENCODEWAV
		case kOUTWAV :
			err = WriteWav(pcmblock, frames, len);
		break;
#endif
		
#ifdef ENCODEAAC
		case kOUTAAC :
			err = WriteAac(pcmblock, frames, len);
		break;
#endif
		
#ifdef ENCODEMP4
		case kOUTMP4 :
			err = WriteMp4(pcmblock, frames, len);
		break;
#endif
		
#ifdef ENCODEWMA
		case kOUTWMA :
			err = WriteWma(pcmblock, frames, len);
		break;
#endif
		
#ifdef ENCODEFLAC
		case kOUTFLAC :
			err = WriteFlac(pcmblock, frames, len);
		break;
#endif
		
		default:
			err = 1;
		break;
	}
	
	return err;
	
} // maOutputToFile


// always stereo in, Hiword is right channel, Loword is left channel
// a frame is MAOUTBYTES wide (two shorts)
void ApplyChannelSelection(char *soundbuf, unsigned long numFrames)
{
#define LEFT *(short *)p
#define RIGHT *((short *)p + 1)
	unsigned long n;
	int *p;
	short s;

	n = numFrames;	
	p = (int *)soundbuf;
	
	switch (l_outputfilter & 0xff)
	{	
		case OUTPUT_MONO :
			if(g_sets.invertright) // invert right channel
			{
				while(n-- > 0)
				{
					LEFT  = (short)(LEFT * l_panleft);
					RIGHT = (short)(RIGHT * l_panright);
					LEFT = (short)(LEFT + RIGHT);
					RIGHT = -LEFT; p++;
				}
			}
			else
			{
				while(n-- > 0)
				{
					LEFT  = (short)(LEFT * l_panleft);
					RIGHT = (short)(RIGHT * l_panright);
					LEFT = (short)(LEFT + RIGHT);
					RIGHT = LEFT; p++;
				}
			}
		break;

		case OUTPUT_LEFT :
			if(g_sets.invertright)
			{
				while(n-- > 0)
				{
					RIGHT = -LEFT; p++;
				}
			}
			else
			{
				while(n-- > 0)
				{
					RIGHT = LEFT; p++;
				}
			}
		break;
		
		case OUTPUT_RIGHT :
			if(g_sets.invertright)
			{
				while(n-- > 0)
				{
					LEFT = RIGHT;
					RIGHT = -RIGHT; p++;
				}
			}
			else
			{
				while(n-- > 0)
				{
					LEFT = RIGHT; p++;
				}
			}
		break;
		
		case OUTPUT_DIFF :
			if(g_sets.invertright)
			{
				while(n-- > 0)
				{
					LEFT  = (short)(LEFT * l_panleft);
					RIGHT = (short)(RIGHT * l_panright);
					LEFT = (short)((LEFT - RIGHT));
					RIGHT = -LEFT; p++;
				}
			}
			else
			{
				while(n-- > 0)
				{
					LEFT  = (short)(LEFT * l_panleft);
					RIGHT = (short)(RIGHT * l_panright);
					LEFT = (short)((LEFT - RIGHT));
					RIGHT = LEFT; p++;
				}
			}
		break;
		
		case OUTPUT_STEREO :
		default :
			if(g_sets.invertright)
				while(n-- > 0)
				{
					RIGHT = -RIGHT; p++;
				}
		break;
	}

	if(l_outputfilter & OUTPUT_STEREO)
	{
		if(l_separation < SEPARATIONMAX)
		{
			n = numFrames;
			p = (int *)soundbuf;
			
			while(n-- > 0)
			{
				s     = (short)(((LEFT)  * l_separationch) + ((RIGHT) * l_separationmix));
				RIGHT = (short)(((RIGHT) * l_separationch) + ((LEFT)  * l_separationmix));
				LEFT = s; p++;
			}
		}
	
		if(l_outputfilter & OUTPUT_REVERSE)
		{
			n = numFrames;
			p = (int *)soundbuf;
			
			while(n-- > 0)
			{
				s = LEFT;
				LEFT = RIGHT;
				RIGHT = s; p++;
			}
		}
	}

} // ApplyChannelSelection


void OnListWindowSingleClick(int row) // single (or first of two) click in listbox or ListSelect()
{
	int lrow;
	int songsnum;
	FILE *fpsonglist;
	char songsource[MAX_PATH];
	char cstr[LISTLINELEN + 2];
	
	if(g_listselect > 0)
	{
		if(--g_listselect < 0)
			g_listselect = 0;
		return;
	}
		
	switch (g_sets.listwindow & (LBOXINFO | LBOXLIST | LBOXLOADED | LBOXID3))
	{
		case LBOXLIST :
			
			if(row > 0)
				lrow = row - 1;
			else if((lrow = ListWindowGetValue()) < 0)
				break;
			
			if(g_sets.listwindow & LBOXPATH)
			{
				if(g_sets.listwindow & LBOXINDEX)
				{
					ListWindowCopyRow(cstr, lrow);
					strcpy(l_strclicked, &cstr[INDEXLEN]);
				}
				else
					ListWindowCopyRow(l_strclicked, lrow);
			}
			else // get path from songlist
			{
				l_strclicked[0] = 0;
				// open the songlist file	
				if((fpsonglist = mafopen(g_songlist, "rb")) == NULL)
				{
					sprintf(cstr, "OnListWindowSingleClick mafopen failed on '%s'", g_songlist);
					maLog(cstr);
					return;
				}
				// get row'th entry
				songsnum = 0;
				while(mafgets(songsource, MAX_PATH, fpsonglist) != NULL)
				{
					// skip lines with '#' or files that are not playable or don't exist
					if(songsource[0] != '#')
					{
						if(SongFileExists(songsource))
							if(songsnum++ == lrow)
								strcpy(l_strclicked, songsource);		
					}
				}
				fclose(fpsonglist);
			}
		break;

		case LBOXLOADED :	

			if(g_loadednum <= 0)
				break;
				
			if(row > 0)
				lrow = row - 1;
			else if((lrow = ListWindowGetValue()) < 0)
				break;
				
			strcpy(l_strclicked, g_hdsongpath[lrow]);
					
		break;
		
		case LBOXINFO :
		case LBOXID3 :
		default :
		break;

	} // case

} // OnListWindowSingleClick


void OnListWindowDoubleClick() // doubleclick in listbox
{
	int row;

	l_strclicked[0] = 0;
	
	if((row = ListWindowGetValue()) < 0)
		return;
		
	switch (g_sets.listwindow & (LBOXINFO | LBOXLIST | LBOXLOADED | LBOXID3))
	{	
		case LBOXLIST :			
			l_droplaunch = row + 2; // 0-based row to play + 2

			if(g_running) // exit song loop...
			{
				g_loadedcurrent = g_loadednum = 0;
				l_loadedquit = 1;
			}
			else
				DoTimerPlay(100);
		break;

		case LBOXLOADED :	
			if(g_loadednum <= 0)
				break;	

			l_loadednew = row + 1;
			
			if(g_running)
				l_loadedquit = 1;
			else
			{
				g_noreload = 1;
				DoTimerPlay(100);
			}			
		break;

		case LBOXINFO :
		case LBOXID3 :
		default :
		break;
		
	} // case

} // OnListWindowDoubleClick


void OnCheckShowIndex(int checked)
{
	if(checked)
		g_sets.listwindow |= LBOXINDEX;
	else
		g_sets.listwindow &= ~LBOXINDEX;
	OnButtonList();
}


void OnCheckShowDir(int checked)
{
	if(checked)
	{
		g_sets.listwindow |= LBOXDIR;
		g_sets.listwindow &= ~LBOXPATH;
		CheckBoxSetValue(WINDOWLIST, ID_CHK_SHOWPATH, 0);
	}
	else
		g_sets.listwindow &= ~LBOXDIR;
	OnButtonList();
}


void OnCheckShowPath(int checked)
{
	if(checked)
	{
		g_sets.listwindow |= LBOXPATH;
		g_sets.listwindow &= ~LBOXDIR;
		CheckBoxSetValue(WINDOWLIST, ID_CHK_SHOWDIR, 0);
	}
	else
		g_sets.listwindow &= ~LBOXPATH;
	OnButtonList();
}


void OnCheckAddMode(int checked)
{
	if(checked)
		g_sets.addmode = 1;
	else
		g_sets.addmode = 0;
}


void OnPopupOutput(int output)
{
	if(output >= kOUTFIRST && output <= kOUTLAST)
	{
		if(g_sets.output == kOUTSOUND)
			if(!g_sets.singleplay)
			{
				MenuSetChecked(MENUMASETS, ID_MENU_SINGLEPLAY, 1);
				CheckBoxSetValue(WINDOWAPP, ID_CHK_SINGLEPLAY,	1);
				g_sets.singleplay = 1;
			}
			
		g_sets.output = output;
		DoWindowFileOut(1);
	}
	else
	{
		DoWindowFileOut(0);
		g_sets.output = kOUTSOUND;
		// PopupButtonSetValue(WINDOWAPP, ID_POPUP_OUTPUT, kOUTSOUND);
	}
	
	return;

} // OnPopupOutput


void OnPopupBuffer(int buffer)
{
	switch(buffer)
	{
		case 0 :
			OnPopupMem();
		break;
	
		case 1 :
			OnPopupVDisk();
		break;
	
		case 2 :
		default :
			OnPopupDirect();
		break;
	}
	
	return;

} // OnPopupBuffer


void OnPopupEqPreset(int preset)
{
	int i;
	long outlen;
	char cstr[256];
	
	if(preset == g_sets.eqpreset)
		return;
	
	if(preset >= EQNUMPRESETS)
	{
		sprintf(cstr, "OnPopupEqPreset preset %d >= max %d", preset, EQNUMPRESETS);
		maLog(cstr);
		return;	
	}
	
	if(g_w[WINDOWEQ])
	{
		EditTextGetText(WINDOWEQ, ID_EDITTEXT_EQ, cstr, EQNAMELEN, &outlen);
		if(strlen(cstr) == 0)
			sprintf(cstr, "Preset%d", g_sets.eqpreset + 1);
		if(strcmp(cstr, g_sets.eqpresets[g_sets.eqpreset].eqname))
		{
#if defined(MAGUIOSX) || defined(MAGUIGTK)
			PopupButtonSetText(WINDOWEQ, ID_POPUP_EQPRESET, g_sets.eqpreset, g_sets.eqpresets[g_sets.eqpreset].eqname);
			if(g_sets.eqpreset == 0) // force a redraw
				i = 1;
			else
				i = 0;
			PopupButtonSetValue(WINDOWEQ, ID_POPUP_EQPRESET, i);
#else
			strcpy(g_sets.eqpresets[g_sets.eqpreset].eqname, cstr); // PopupButtonSetText fails...
			PopupButtonClear(WINDOWEQ, ID_POPUP_EQPRESET);
			for(i = 0; i < EQNUMPRESETS; i++)
				PopupButtonAppend(WINDOWEQ, ID_POPUP_EQPRESET, g_sets.eqpresets[i].eqname);
#endif
		}
		PopupButtonSetValue(WINDOWEQ, ID_POPUP_EQPRESET, preset);
	}
		
	g_eqzoomval = -1.0;
	g_sets.eqpreset = preset;
	
	if(g_w[WINDOWEQ])
	{
		EditTextSetText(WINDOWEQ, ID_EDITTEXT_EQ, g_sets.eqpresets[g_sets.eqpreset].eqname);
		StaticTextSetValue(WINDOWEQ, ID_TEXT_EQOFFSETVAL, g_sets.eqpresets[g_sets.eqpreset].eqoffset);
	}
	
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();
	
	if(g_w[WINDOWEQ])
		SetSliders(EQSLIDERS);
	
	if(g_w[WINDOWEQ2])
		SetSliders(EQ2SLIDERS);
	
	return;

} // OnPopupEqPreset


void OnPopupBitRate(int bitrate)
{
	g_sets.fileout[g_k[g_sets.output]].bitrate = bitrate;
}


void OnPopupQuality(int quality)
{
	g_sets.fileout[g_k[g_sets.output]].quality = quality;
}


void OnPopupMode(int mode)
{
	g_sets.fileout[g_k[g_sets.output]].mode = mode;
}


void OnPopupSampleRate(int samplerate)
{
	g_sets.fileout[g_k[g_sets.output]].samplerate = samplerate;
}


#if !defined(MANOCDA) && defined(__MAUNIX__)
void OnPopupAudioCD(int preset)
{
	char cstr[256];
	
	if(preset >= CDAMAXNUMDRIVES)
	{
		sprintf(cstr, "OnPopupAudioCD preset %d >= max %d", preset, CDAMAXNUMDRIVES);
		maLog(cstr);
		return;	
	}
	
	// zilch...
	
	return;

} // OnPopupAudioCD
#endif


int maAllocBuffers()
{
	int i;
	
#ifndef NODECODEMP3
	// mp3 decoder input buffer
	if(g_mp3inpbuf == NULL)
	{
		if((g_mp3inpbuf = (unsigned char *)MAMALLOC(MP3INPBUFSIZE)) == NULL)
		{
			maLog("maAllocBuffers MAMALLOC g_mp3inpbuf failed");
			return 1;
		}
	
#if defined(__MAOS9__) && !TARGET_API_MAC_CARBON
		HoldMemory(g_mp3inpbuf, MP3INPBUFSIZE);
#endif
	}

	// mp3 decoder output buffer
	if(g_mp3decbuf == NULL)
	{
		if((g_mp3decbuf = (unsigned char *)MAMALLOC(MP3DECBUFSIZE)) == NULL)
		{
			maLog("maAllocBuffers MAMALLOC g_mp3decbuf failed");
			return 1;
		}
	
#if defined(__MAOS9__) && !TARGET_API_MAC_CARBON
		HoldMemory(g_mp3decbuf, MP3DECBUFSIZE);
#endif
	}
#endif

	// allocate memory for the pcm buffers
	for(i = 0; i < NSNDBUFS; i++)
	{
		if(!g_sndbuf[i])
		{
			if((g_sndbuf[i] = (char *)MAMALLOC(SOUNDBUFSIZE)) == NULL)
			{
				maFreeBuffers();
				maLog("maAllocBuffers MAMALLOC g_sndbuf failed");
				return 1;
			}
			
#if defined(__MAOS9__) && !TARGET_API_MAC_CARBON
			HoldMemory(g_sndbuf[i], SOUNDBUFSIZE);
#endif
		}
		memset(g_sndbuf[i], 0, SOUNDBUFSIZE);
	}

	// allocate the pcm fifo
	if(!l_pmafifo)
	{
		if((l_pmafifo = maFifoCreate(&l_mafifo, g_sndbuf[1], (unsigned long)FIFOSIZE)) == NULL)
		{
			maFreeBuffers();
			maLog("maFifoCreate failed");
			return 1;
		}
	}

#if defined(AUDIO_MAC_CORE) || defined(AUDIO_PORTAUDIO)
	if(!g_paufifo)
	{
		if((g_paufifo = maFifoCreate(&g_aufifo, g_aubuf, (unsigned long)AUBUFSIZE)) == NULL)
		{
			maFreeBuffers();
			maLog("maFifoCreate g_aufifo failed");
			return 1;
		}
	}
#endif

	return 0;
	
} // maAllocBuffers


int maFreeBuffers()
{
	int i;

	for(i = 0; i < NSNDBUFS; i++)
		if(g_sndbuf[i])
		{
			MAFREE(g_sndbuf[i]);
			g_sndbuf[i] = 0;
		}
		
	if(l_pmafifo)
	{
		maFifoDelete(l_pmafifo);
		l_pmafifo = NULL;
	}
	
#if defined(AUDIO_MAC_CORE) || defined(AUDIO_PORTAUDIO)
	if(g_paufifo)
	{
		maFifoDelete(g_paufifo);
		g_paufifo = NULL;
	}
#endif

	if(l_monobuf)
	{
		MAFREE(l_monobuf);
		l_monobuf = NULL;
	}

#ifndef NODECODEMP3
	if(g_mp3inpbuf)
	{
		MAFREE(g_mp3inpbuf);
		g_mp3inpbuf = NULL;
	}

	if(g_mp3decbuf)
	{
		MAFREE(g_mp3decbuf);
		g_mp3decbuf = NULL;
	}
#endif

	return 0;
	
} // maFreeBuffers


int maEraseBuffers()
{
	int i;

	for(i = 0; i < NSNDBUFS; i++)
		if(g_sndbuf[i])
			memset(g_sndbuf[i], 0, SOUNDBUFSIZE);
		
	if(l_pmafifo)
		maFifoErase(l_pmafifo);
	
#if defined(AUDIO_MAC_CORE) || defined(AUDIO_PORTAUDIO)
	if(g_paufifo)
		maFifoErase(g_paufifo);
#endif

	if(l_monobuf && l_monobytes)
		memset(l_monobuf, 0, l_monobytes);

#ifndef NODECODEMP3
	if(g_mp3inpbuf)
		memset(g_mp3inpbuf, 0, MP3INPBUFSIZE);

	if(g_mp3decbuf)
		memset(g_mp3decbuf, 0, MP3DECBUFSIZE);
#endif

	return 0;
	
} // maEraseBuffers


int maOutputOpen()
{
	int i;

	maFifoSetStorage(l_pmafifo, g_sndbuf[0]);
	g_fifoready = 0;

	maEraseBuffers();
	
	for(i = 0; i < NSNDBUFS; i++)
	{
		g_nframes[i] = 0;
		g_bufready[i] = 0;
	}
	
#ifdef AUDIO_MAC_SOUNDMGR
	g_buftoplay = 0;
#endif

	l_buftofill = 0;
	
	maEqualizerReset((g_sets.eqflags & MAEQ31BANDS) ? EQBANDS31 : EQBANDS10, 1,
									g_sets.eqflags & MAEQEXTRA); // bands, 16bit, extra
	EqDoParamChange();
	
#if !defined(MAAUDIONONE)
	maOutputSetVolume((kVOLMAX * volumevalL())  / VOLUMEMAX, (kVOLMAX * volumevalR()) / VOLUMEMAX);
#endif
	if(g_sets.output == kOUTSOUND)
	{
#if !defined(MAAUDIONONE)
		if(maOpenSndChannel())	
			return -1;
#endif
	}
	else
	{
		if(g_outputfilefp)
		{
			maLog("maOutputOpen warning output file was already open");
			g_outputfilefp = NULL;
		}

		if(maOpenOutputFile())
		{
			maLog("maOutputOpen maOpenOutputFile failed");
			return -1;
		}
#if !defined(NODECODEAUDIOIN)
 #if !defined(MAAUDIONONE)
		if(maOpenSndChannel())
		{
			maLog("maOutputOpen maOpenSndChannel (record playthrough) failed");
		}
 #endif
#endif
	}
				
	return 1; // ok
		
} // maOutputOpen


void maOutputDelay(int delay);
void maOutputDelay(int delay)
{
	// wait delay ms max or until fifo has been emptied to output
	if(delay> 0)
	{		
		unsigned long t = maGetLocalTimeMillis() + delay;
		while(l_pmafifo && maFifoGetFilledSpace(l_pmafifo))
		{
			ThreadYield();
			if(maGetLocalTimeMillis() > t)
				break;
		}
	}

} // maOutputDelay


void maOutputClose(int wait)
{
	if(g_sets.output == kOUTSOUND)
	{
		if(wait)
			maOutputDelay(3000);
#if !defined(MAAUDIONONE)
		maCloseSndChannel(wait);
#endif
	}
	else if(g_outputfilefp)
	{
		switch (g_sets.output)
		{
#ifdef ENCODEMP3
			case kOUTMP3 :
				CloseWriteMp3();
			break;
#endif

#ifdef ENCODEOGG
			case kOUTOGG :
				CloseWriteOgg();
			break;
#endif

#ifdef ENCODEFLAC
			case kOUTFLAC :
				CloseWriteFlac();
			break;
#endif
		
#ifdef ENCODEAAC
			case kOUTAAC :
				CloseWriteAac();
			break;
#endif

#ifdef ENCODEMP4
			case kOUTMP4 :
				CloseWriteMp4();
			break;
#endif

#ifdef ENCODEAIF
			case kOUTAIF :
				CloseWriteAif();
			break;
#endif
		
#ifdef ENCODEWAV
			case kOUTWAV :
				CloseWriteWav();
			break;
#endif
		
#ifdef ENCODEWMA
			case kOUTWMA :
				CloseWriteWma();
			break;
#endif
		
			default:
			break;
		}
		
		maCloseOutputFile();

#if !defined(NODECODEAUDIOIN)
 #if !defined(MAAUDIONONE)
		maCloseSndChannel(wait);
 #endif
#endif
	}
		
} // maOutputClose
	
	
// decoders set pstate-> inchannels, inwordsbigendian and ininterleaved
// and dump pcm data here. Our output is 2-channel interleaved machine-endian.
int maOutputWrite(char *buf, int size, MASTATE *pstate)
{
	int i;
	int bytes = (pstate->inchannels == 1) ? (FIFOSIZE / 2) : FIFOSIZE;
	int bytesleft = size;
	int monobytes;
	short *p;
	short *q;
	unsigned char *pbuf = (unsigned char *)buf;
	
#ifdef WORDS_BIGENDIAN
	if(!pstate->inwordsbigendian)
#else
	if(pstate->inwordsbigendian)
#endif
		for(i = 0, p = (short *)buf; i < size >> 1; i++, p++)
			*p = ((((*p)>>8)&0xFF)+(((*p)&0xFF)<<8));
	
	while(!pstate->stop && bytesleft > 0)
	{	
		if(bytes > bytesleft)
			bytes = bytesleft;

		// Write to fifo
		if(pstate->inchannels != 1)
		{
			if(!pstate->ininterleaved)
				maLog("maOutputWrite !pstate->ininterleaved"); // never expect this
			//else	
				WriteFiFo(l_pmafifo, (unsigned char *)pbuf, bytes);
		}
		else // one channel -> two channels : abcd -> aabbccdd
		{			
			monobytes = bytes << 1;

			// if bytes increased, realloc a mono buffer
			if(l_monobuf && monobytes > l_monobytes)
			{
				MAFREE(l_monobuf);
				l_monobuf = NULL;
			}

			if(l_monobuf == NULL)
			{
				if((l_monobuf = (unsigned char *)MAMALLOC(monobytes)) == NULL)
				{
					maLog("maOutputWrite MAMALLOC failed");
					WriteFiFo(l_pmafifo, (unsigned char *)pbuf, bytes); // sound funny
				}
				l_monobytes = monobytes;
			}
	
			p = (short *)l_monobuf;
			q = (short *)pbuf;
			for(i = 0; i < (bytes >> 1); i++)
			{
				*p++ = *q;
				*p++ = *q++;
			}
			
			WriteFiFo(l_pmafifo, l_monobuf, monobytes);
		}
			
		pbuf += bytes;
		bytesleft -= bytes;
			
	} // while bytesleft
	
	return 0;
	
} // maOutputWrite


int maOutputCanWrite()
{
	return maFifoGetFreeSpace(l_pmafifo);
				
} // maOutputCanWrite


// maFifo functions:
maFifo *maFifoCreate(maFifo* pfifo, char *storage, unsigned long fisize)
{
	if((pfifo->storage = storage) == NULL)
		return NULL;

	pfifo->internal = 0;		
	pfifo->size = fisize;
	maFifoClear(pfifo);
	pfifo->semaphore = NULL;
# if(defined(__MAOSX__) && !defined(MAUSEMPSERVICES)) || defined(__MACUNIX__)
	if(maNamedSemaphoreOpen(&(pfifo->semaphore), MAFIFOSEMNAME))
#else
	if(maSemaphoreNew(&(pfifo->semaphore)))
#endif
	{
		pfifo->semaphore = NULL;
		maLog("maFifoCreate maSemaphoreNew failed");
		return NULL;
	}

	return pfifo;
}


void maFifoDelete(maFifo* pfifo)
{
	if(pfifo->internal)
	{
		pfifo->internal = 0;
		if(pfifo->storage)
			MAFREE(pfifo->storage);
	}
	pfifo->size = 0;
	pfifo->contains = 0;
	pfifo->start = 0;
	pfifo->storage = NULL;

	if(pfifo->semaphore)
	{
# if(defined(__MAOSX__) && !defined(MAUSEMPSERVICES)) || defined(__MACUNIX__)
		if(maNamedSemaphoreUnlink(pfifo->semaphore, MAFIFOSEMNAME))
#else
		if(maSemaphoreDelete(pfifo->semaphore))
#endif
			maLog("maFifoDelete maSemaphoreDelete failed");
	}
}


unsigned long maFifoGetFreeSpace(maFifo* pfifo)
{
	return pfifo->size - pfifo->contains;
}


unsigned long maFifoGetFilledSpace(maFifo* pfifo)
{
	return pfifo->contains;
}


unsigned long maFifoStore(maFifo* pfifo, char *whereFrom, unsigned long requested)
{
	unsigned long fifoend;
	unsigned long amount;
	unsigned long amount1;
	unsigned long amount2;

	if(!pfifo->storage)
		return 0;
		
	if((amount = requested) > maFifoGetFreeSpace(pfifo))
		amount = maFifoGetFreeSpace(pfifo);
	
	fifoend = (pfifo->start + pfifo->contains) % pfifo->size;
	amount1 = pfifo->size - fifoend;

	if(amount <= amount1)
		memmove(pfifo->storage + fifoend, whereFrom, amount);
	else
	{
		amount2 = amount - amount1;
		memmove(pfifo->storage + fifoend, whereFrom, amount1);
		memmove(pfifo->storage, whereFrom + amount1, amount2);
	}

	pfifo->contains += amount;
	
	return amount;
	
} // maFifoStore


unsigned long maFifoDraw(maFifo* pfifo, char* whereTo, unsigned long requested)
{
	unsigned long amount;
	unsigned long amount1;
	unsigned long amount2;

	if(!pfifo->storage)
		return 0;

	if((amount = requested) > pfifo->contains)
		amount = pfifo->contains;
	
	amount1 = pfifo->size - pfifo->start;

	if(amount <= amount1)
		memmove(whereTo, pfifo->storage + pfifo->start, amount);
	else
	{
		amount2 = amount - amount1;

		memmove(whereTo, pfifo->storage + pfifo->start, amount1);

		memmove(whereTo+amount1, pfifo->storage, amount2);
	}
	
	pfifo->contains -= amount;
	pfifo->start += amount;
	
	if(pfifo->start >= pfifo->size)
		pfifo->start -= pfifo->size;
		
	return amount;
	
} // maFifoDraw


void maFifoClear(maFifo* pfifo)
{
	pfifo->contains = pfifo->start = 0;
}


void maFifoErase(maFifo* pfifo)
{
	pfifo->contains = pfifo->start = 0;
	if(!pfifo->storage)
		return;		
	memset(pfifo->storage, 0, pfifo->size);
}


void maFifoSetStorage(maFifo* pfifo, char *storage)
{
	if(pfifo->internal)
		return;
		
	pfifo->storage = storage;
	if(!pfifo->storage)
		return;

	maFifoClear(pfifo);
}


char *maFifoGetStorage(maFifo* pfifo)
{
	return pfifo->storage;
}


int WriteFiFoBlocking(maFifo* pfifo, unsigned char *data, unsigned int len)
{
	int roomleft;
	unsigned int length = len;
	unsigned char *p = data;
	
	// len == 0 means flush fifo
	if(len == 0)
	{
		if(maFifoGetFilledSpace(pfifo) > 0)
		{
			// xfer fifo to output
			g_fifoready = 1;
			while(g_fifoready && maFifoToOutput())	
			{
				while(g_pause)
				{
					ThreadYield();
#ifndef __MAOS9__
					AppYield();
					if(g_cancel)
						break;
					maSleep(100);
#endif
				}
			}
		}
	
		return 0;
	}
	 
	// if the fifo is full wait here until it has been emptied into an output
	if((roomleft = maOutputCanWrite()) < (int)len)
	{
		if(roomleft > 0)
		{
			maFifoStore(pfifo, (char *)data, roomleft);
			length = len - roomleft;
			p += roomleft;
		}
		
		g_fifoready = 1; // fifo is full
	
		// xfer fifo to output
		while(g_fifoready && maFifoToOutput())	
		{
			if(g_state.stop)
				return 1;
			while(g_pause)
			{
				ThreadYield();
#ifndef __MAOS9__
				AppYield();
				if(g_cancel)
					break;
				maSleep(100);
#endif
			}
		}

		if(g_state.stop)
			return 1;
						
	} // if the fifo is full
	
	if(length > 0)
		maFifoStore(pfifo, (char *)p, length);
	
	return 0;

} // WriteFiFoBlocking


 void FlushFifo()
 {
	if(g_cancel)
		return;
		
	if(g_sets.effects & EFFECTSON && (g_sets.effects & (EFFECTSTEMPO | EFFECTSPITCH | EFFECTSSPEED)))
	{
		if(g_fxbuffer)
		{
		
			l_fxoutlen = FXBUFFERSIZE;
			l_fxoutdata = g_fxbuffer;
	
			// call with inlen = 0 flushes out the last of effects data
			ApplyEffects(NULL, 0, (char *)l_fxoutdata, &l_fxoutlen);

			if(l_fxoutlen > 0)
				WriteFiFoBlocking(l_pmafifo, l_fxoutdata, (unsigned int)l_fxoutlen);
		}
	}
	
	// last call with len == 0 to flush
	WriteFiFoBlocking(l_pmafifo, l_fxoutdata, 0);

 } // FlushFifo


int WriteFiFo(maFifo* pfifo, unsigned char *data, unsigned int len)
{
	if(g_sets.effects & EFFECTSON && (g_sets.effects & (EFFECTSTEMPO | EFFECTSPITCH | EFFECTSSPEED)))
	{
		if(g_fxbuffer == NULL)
		{
			if((g_fxbuffer = (unsigned char *)MAMALLOC(FXBUFFERSIZE)) == NULL)
				maLog("WriteFiFo MAMALLOC g_fxbuffer failed");
			WriteFiFoBlocking(pfifo, data, len);
			return 0;
		}
	
		l_fxoutlen = FXBUFFERSIZE;
		l_fxoutdata = g_fxbuffer;
		
		ApplyEffects((char *)data, len, (char *)l_fxoutdata, &l_fxoutlen);
			
		if(l_fxoutlen > 0)
			WriteFiFoBlocking(pfifo, l_fxoutdata, (unsigned int)l_fxoutlen);
	}
	else
		WriteFiFoBlocking(pfifo, data, len);
							
	return 0;

} // WriteFiFo


// comes here when the fifo is full
// this is where the player process blocks
int maFifoToOutput()
{
	int next;
	int err = 0;
	unsigned long available;

	if((available = maFifoGetFilledSpace(l_pmafifo)) <= 0)
	{		
		maLog("maFifoToOutput available == 0");
		return 0; // no data in fifo
	}
	
	if(g_pause)
		return 1;

	if((next = l_buftofill + 1) >= NSNDBUFS)
		next = 0; // recirculate
		
	if(g_bufready[next] == 0)
	{
		g_nframes[l_buftofill] = available / MAOUTBYTES;
		
		ApplyChannelSelection(g_sndbuf[l_buftofill], g_nframes[l_buftofill]);
		
		if(g_sets.eqflags & MAEQACTIVE)
			maEqualizeBlock(g_sndbuf[l_buftofill], available);

		if(g_sets.output == kOUTSOUND)
		{
			if(available < SOUNDBUFSIZE)
				memset(g_sndbuf[l_buftofill] + available, 0, SOUNDBUFSIZE - available);

#ifdef AUDIO_PORTAUDIO
			err = maOutputToSound(g_sndbuf[l_buftofill], g_nframes[l_buftofill], available, l_buftofill);
#else
 #if defined(AUDIO_MAC_SOUNDMGR) || defined(AUDIO_WIN_WAVE)
			g_bufready[l_buftofill] = 1;
 #endif
 #if !defined(AUDIO_MAC_SOUNDMGR)
  #if !defined(MAAUDIONONE)
			err = maOutputToSound(g_sndbuf[l_buftofill], g_nframes[l_buftofill], available, l_buftofill);
  #else
			err = 0;
  #endif
 #endif
#endif
		}
		else
		{

#if !defined(NODECODEAUDIOIN)
			if(g_state.infiletype == kMAMAA)
				maOutputToSound(g_sndbuf[l_buftofill], g_nframes[l_buftofill], available, l_buftofill);
#endif

			err = maOutputToFile(g_sndbuf[l_buftofill], g_nframes[l_buftofill], available, l_buftofill);
		}
		
		if(err)
		{
			g_state.stop |= STOP_SNDCHANNEL;
			return 1;
		}
		
		l_buftofill = next;
		maFifoSetStorage(l_pmafifo, g_sndbuf[next]);
		g_fifoready = 0;
	
		return 0;
	}
	
#ifdef AUDIO_MAC_SOUNDMGR
	// gets here if 'next' sound buffer is still 'ready' i.e it hasn't played yet
	ThreadYield();
#endif

#ifdef AUDIO_WIN_WAVE
	// gets here if 'next' sound buffer is still 'ready' i.e it hasn't played yet
	// and waits for a semaphore to be signaled, or yields
	if(g_semaudiowin)
	{
 		g_semwaiting = next;
		maSemaphoreWaitTimeout(g_semaudiowin, INFINITE);
		g_semwaiting = -1;
		return 1;
	}
	else // should never get here...
		ThreadYield();
#endif

	return 1;		

} // maFifoToOutput


void SpinDown()
{
#if defined(__MAOS9__)
	SpinDownHardDisk();
#endif

	// only the OS is allowed to powerdown a drive...
	// possibly, the "energy saving" functionality can be affected here,
	// e.g. setting a very short "no-activity" delay.

} // SpinDown


#if defined(__MAOSX__) || defined(__MAOS9__)
 // events handled outside the gui
 static pascal OSErr AEQuit(const AppleEvent *theAppleEvent, AppleEvent *theReply, long Refcon);
 pascal OSErr AEQuit(const AppleEvent *theAppleEvent, AppleEvent *theReply, long Refcon)
 {
	(void)theAppleEvent; (void)theReply; (void)Refcon;

	OnButtonQuit();

	return noErr;
 }


 static pascal OSErr AEOpenDoc(const AppleEvent *theAppleEvent, AppleEvent *theReply, long Refcon);
 // make a songlist from the FSSpecs of files, dirs or links passed in a mac AEOpenDoc event
 pascal OSErr AEOpenDoc(const AppleEvent *theAppleEvent, AppleEvent *theReply, long Refcon)
 {
	(void)Refcon; (void)theReply;

	DoAEOpenDoc((void *)theAppleEvent);
	
	return noErr;
 }


 #if defined(MAGUINONE) && defined(MAKEYBOARD) && !defined(__MAOSX__) // __MAOS9__  noGUI only
  pascal OSStatus OnRawKeyDown(EventHandlerCallRef handler, EventRef event, void *data);
  pascal OSStatus OnRawKeyDown(EventHandlerCallRef handler, EventRef event, void *data)
  {
	(void)handler; (void)data;

	int clas;
	int kind;
	unsigned char key = 0;
	OSErr err;
	
	if(g_maOnKeySkip)
		return eventNotHandledErr;

	if(!g_sets.keyboard)
		return eventNotHandledErr;
		 
	clas = GetEventClass(event);
	kind = GetEventKind(event);
	if(clas == kEventClassKeyboard && kind == kEventRawKeyDown)
	{
		if((err = GetEventParameter(event, kEventParamKeyMacCharCodes, typeChar, NULL, 1, NULL, &key)) != noErr)
		{
			char cstr[256];
			sprintf(cstr, "OnRawKeyDown GetEventParameter err=%d", err);
			maLog(cstr);
			return eventNotHandledErr;
 		}						
	}
	
	maOnKey(WINDOWAPP, key);
	
	return noErr;
  }
 #endif
#endif


int OnInit_()
{
	int err;
	char instancestr[16];
	char cstr[256];
#if !defined(NODECODEAUDIOIN)
	FILE *fp;
#endif
	
	// init fileout indices
	Init_g_k();
	
	// song index before startup in direct no-random repeat mode
	l_songsindex[0] = -1;
		
	// seed random generator for random mode
	srand(maGetLocalTimeMillis());

#if defined(__MAUNIX__)
	// unix install data dir
	if(!strcmp("${prefix}/share", INSTALL_DATADIR)) // did not have configure --datadir...
		sprintf(l_mareadmedir, "%s%cshare%cmoreamp", INSTALL_PREFIX, FILESEP, FILESEP);
	else // had configure --datadir=/INSTALL_DATADIR...
		sprintf(l_mareadmedir, "%s%cmoreamp", INSTALL_DATADIR, FILESEP);
#endif

	// get moreampdir
	if(!magetmoreampdir(cstr, 256))
	{
		ShowErr("OnInit_ magetmoreampdir failed, cannot continue");
		goto oninitretnok;
	}
	strcpy(g_madir, cstr);
	
	// create MoreAmp dir if missing
	if(!maDirExists(g_madir))
	{
		if(maCreateDir(g_madir))
		{
			sprintf(cstr, "OnInit_ failed to create MoreAmp dir\n\n'%s'\n\ncannot continue", g_madir);
			ShowErr(cstr);
			goto oninitretnok;
		}
#if defined(__MAOSX__) || defined(__MAUNIX__)
		machmod(g_madir, ACCESSPERMS); // 0777
#endif
	}	

	// set log path
	sprintf(g_logpath, "%s%c%s%s", g_madir, FILESEP, MALOG, EXTTXT);
		
	// get cwd
	if(!magetcwd(l_cwd, MAX_PATH))
	{
		ShowErr("OnInit_ magetcwd failed, cannot continue");
		goto oninitretnok;
	}

	// create 'MASETTINGSDIR' dir if missing
	sprintf(g_settingsdir, "%s%c%s", g_madir, FILESEP, MASETTINGSDIR); // initial
	if(!maDirExists(g_settingsdir))
	{
		if(maCreateDir(g_settingsdir))
		{
			sprintf(cstr, "OnInit_ failed to create Settings dir\n\n'%s'\n\ncannot continue", g_settingsdir);
			ShowErr(cstr);
			goto oninitretnok;
		}		
	}		

	instancestr[0] = 0; // instance number to cat to strings

#if defined(MAINSTANCES)
 #if defined(MAUSESHF) || (defined(MAUSESHM) && defined(__MAWIN__))
	sprintf(l_mainstancefile, "%s%c%s", g_settingsdir, FILESEP, MAINSTANCEFILE);
 #endif
	if(maNamedLock(MAINITLOCK))
	{
		ShowErr("OnInit_ maNamedLock failed");
		return 0;
	}
	
	if((g_thisinstance = InstanceInit()) < 0)
	{
		maLog("Multiple instances not allowed, cannot continue");
		goto oninitretnok;
	}
	if(g_thisinstance == 0)
	{
		ShowErr("OnInit_ InstanceInit failed, cannot continue");
		goto oninitretnok;
	}
	if(g_thisinstance > 1)
		sprintf(instancestr, "%d", g_thisinstance);
#endif

	sprintf(g_prognametitle, "%s%s", MOREAMP_NAME, instancestr);

	// set some globals
	
#if !defined(MANOCDA) && defined(__MAUNIX__)
	sprintf(l_setscdrompath, "%s%c%s%c%s%s", g_madir, FILESEP, MASETTINGSDIR, FILESEP, MASETSCDROMFILE, EXTMAS);
#endif

#if defined(MAKEYBOARD)
	sprintf(g_keybindingspath, "%s%c%s%c%s%s", g_madir, FILESEP, MASETTINGSDIR, FILESEP, MAKEYBINDINGSFILE, EXTTXT);
#endif

	// "~/moreamp/masettings/" "~/moreamp/masettings2/" ...
	sprintf(g_settingsdir,  "%s%c%s%s",	g_madir, FILESEP, MASETTINGSDIR, instancestr);
	sprintf(g_songlistsdir, "%s%c%s",	g_madir, FILESEP, MASONGLISTSDIR);
	sprintf(g_fileoutdir,   "%s%c%s",	g_madir, FILESEP, MAFILEOUTDIR);

	// "~/moreamp/masettings/madrop.maf" "~/moreamp/masettings2/madrop.maf" ...
	sprintf(g_songlistdrop, "%s%c%s%s", g_settingsdir, FILESEP, SONGLISTDROP, EXTMAF);
	// "~/moreamp/masettings/macda.maf" "~/moreamp/masettings2/macda.maf" ...
	sprintf(g_songlistcda,  "%s%c%s%s", g_settingsdir, FILESEP, SONGLISTCDA, EXTMAF);
#if !defined(NODECODEAUDIOIN)
	// "~/moreamp/masettings/maaudioin.maf"
	sprintf(g_songlistaudioin, "%s%c%s%s", g_settingsdir, FILESEP, SONGLISTAUDIOIN, EXTMAF);
	// "~/moreamp/masettings/maaudioin.maa"
	sprintf(g_songfileaudioin, "%s%c%s%s", g_settingsdir, FILESEP, SONGLISTAUDIOIN, EXTMAA);
#endif
	// "~/moreamp/masettings/masonglisttmp.maf" "~/moreamp/masettings2/masonglisttmp.maf" ...
	sprintf(g_songlisttemp, "%s%c%s%s", g_settingsdir, FILESEP, SONGLISTTMP, EXTMAF);
		
	// "~/moreamp/masettings/madrop.mas" "~/moreamp/masettings2/madrop.mas" ...
	sprintf(l_settingsdrop, "%s%c%s%s", g_settingsdir, FILESEP, SONGLISTDROP, EXTMAS);

	// "~/moreamp/masettings/madefault.mas" "~/moreamp/masettings2/madefault.mas" ...
	sprintf(g_settingsdef,  "%s%c%s%s", g_settingsdir, FILESEP, MASETSDEFAULT, EXTMAS);
		
	// "~/moreamp/masettings/macda.mas" "~/moreamp/masettings2/macda.mas" ...
	sprintf(l_settingscda,	"%s%c%s%s",	g_settingsdir, FILESEP, SONGLISTCDA, EXTMAS);

	// "~/moreamp/masettings/maaudioin.mas" "~/moreamp/masettings2/maaudioin.mas" ...
	sprintf(l_settingsaudioin,	"%s%c%s%s",	g_settingsdir, FILESEP, SONGLISTAUDIOIN, EXTMAS);

	sprintf(l_recentlist,	"%s%c%s",	g_settingsdir, FILESEP, RECENTSONGLIST);

	// viewer default find text
	strcpy(g_readmefind, MOREAMP_HOME_URL);
	strcpy(g_catalogfind, MOREAMP_HOME_URL);
#if defined(MAKEYBOARD)
	strcpy(g_keybindingsfind, MOREAMP_HOME_URL);
#endif
	strcpy(g_logfind, MOREAMP_HOME_URL);

	l_settingspathlast[0] = 0;
	
	// create 'MASONGLISTSDIR' dir if missing
	if(!maDirExists(g_songlistsdir))
	{
		if(maCreateDir(g_songlistsdir))
		{
			sprintf(cstr, "OnInit_ failed to create SongLists dir\n\n'%s'\n\ncannot continue", g_songlistsdir);
			ShowErr(cstr);
			goto oninitretnok;
		}		
	}		

	// create 'MASETTINGSDIR' dir if missing
	if(!maDirExists(g_settingsdir))
	{
		if(maCreateDir(g_settingsdir))
		{
			sprintf(cstr, "OnInit_ failed to create Settings dir\n\n'%s'\n\ncannot continue", g_settingsdir);
			ShowErr(cstr);
			goto oninitretnok;
		}		
	}		
	
	// create 'MAFILEOUTDIR' dir if missing
	if(!maDirExists(g_fileoutdir))
	{
		if(maCreateDir(g_fileoutdir))
		{
			sprintf(cstr, "OnInit_ failed to create FileOut dir\n\n'%s'\n\ncannot continue", g_fileoutdir);
			ShowErr(cstr);
			goto oninitretnok;
		}		
	}

#if !defined(NODECODEAUDIOIN)
	// create g_songlistaudioin and g_songfileaudioin if missing
	if(!maFileExists(g_songlistaudioin))
	{
		if((fp = mafopen(g_songlistaudioin, "w")) == NULL)
		{
			sprintf(cstr, "OnInit_ mafopen failed on '%s'", g_songlistaudioin);
			maLog(cstr);
		}
		else
		{
			fprintf(fp, "%s\n", g_songfileaudioin);
			fclose(fp);
 #if defined(__MAOSX__) || defined(__MAUNIX__)
			machmod(g_songlistaudioin, ACCESSPERMS); // 0777
 #endif
		}
	}
	if(!maFileExists(g_songfileaudioin))
	{
		if((fp = mafopen(g_songfileaudioin, "w")) == NULL)
		{
			sprintf(cstr, "OnInit_ mafopen failed on '%s'", g_songfileaudioin);
			maLog(cstr);
		}
		else
		{
				fclose(fp);
 #if defined(__MAOSX__) || defined(__MAUNIX__)
			machmod(g_songfileaudioin, ACCESSPERMS); // 0777
 #endif
		}
	}
#endif

	// load keyboard bindings
#if defined(MAKEYBOARD)
	SetFactoryKeyBindings();
	if(LoadKeyBindings(0) == 2) // just created g_keybindingspath
		LoadKeyBindings(0);
#endif
 
	// get CDRom drives
#if !defined(MANOCDA)
 #if defined(__MAUNIX__)
	LoadSettingsCDRom();
 #endif
	CdaInitCDRom(); // OnInit_
#endif

	// allocate buffers
	if(maAllocBuffers())
	{
		ShowErr("OnInit_ maAllocBuffers failed, cannot continue");
		goto oninitretnok;
	}		
	
	if(DoPassedArgs())
		goto oninitretok; // had passed args, settings got loaded and window created

	if((err = GetRecentSonglist()))
		LoadDefaultSettings(&g_sets);
		
	AppWindowMakeA();
	
	// autostart
#if !defined(MAGUINONE) || !defined(__MAOS9__) // problem autostarting noguiOS9
	if(!err && g_sets.autostart && g_sets.output == kOUTSOUND)
		DoTimerPlay(100);
	else
#endif
		SetAppWindowTitle((char *)"Stopped");

oninitretok:
#if defined(MAINSTANCES)
	if(maNamedUnlock(MAINITLOCK))
	{
		ShowErr("OnInit_ maNamedUnlock failed");
		return 0;
	}
#endif
	return 1;
	
oninitretnok:
#if defined(MAINSTANCES)
	if(maNamedUnlock(MAINITLOCK))
		ShowErr("OnInit_ maNamedUnlock failed");
#endif
	
	return 0; // failed
	
} // OnInit_


#if defined(MAINSTANCES)
int ClearInitLock()
{
	if(maNamedUnlock(MAINITLOCK))
	{
		maLog("ClearInitLock maNamedUnlock failed");
		return 1;
	}
	
	return 0;
}
#endif


#if defined(MAINSTANCES)
// set or reset multiple instances flag in the shared instance table
int SetMultipleInstances()
{
	MAINSTANCE instance;
	int multiple;
	
	// read the current instance table
	if(InstanceRead(&instance))
	{
		ShowErr("SetMultipleInstances InstanceRead failed");
		return 1;
	}
	
	if(g_sets.multipleinst)
	{
		if(instance.multipleinst) // already set to multiple instances
			return 0;
		multiple = 1;
	}
	else
	{
		if(!instance.multipleinst) // already set to no multiple instances
			return 0;
		multiple = 0;
	}
	
	if(InstanceSet(MAINSTANCEMAX, multiple))
	{
		ShowErr("SetMultipleInstances InstanceSet failed");
		return 1;
	}
	
	return 0;
	
} // SetMultipleInstances
#endif


#if !defined(MANOCDA) && (defined(__MAOSX__) || defined(__MAOS9__))
 #if TARGET_API_MAC_CARBON
  pascal OSStatus OnDiskEvent(EventHandlerCallRef handler, EventRef event, void *data);
  pascal OSStatus OnDiskEvent(EventHandlerCallRef handler, EventRef event, void *data)
  {
	(void)handler; (void)data;

	int clas = GetEventClass(event);
	int kind = GetEventKind(event);
	EventParamType outActualType = 0;
	UInt32 outActualSize = 0;
	short volnum = 0;
	OSErr err;
	if(clas == kEventClassVolume && (kind == kEventVolumeMounted || kind == kEventVolumeUnmounted))
	{
		if((err = GetEventParameter(	event,					// EventRef inEvent,
										kEventParamDirectObject,// EventParamName inName,
										typeFSVolumeRefNum,		// EventParamType inDesiredType,
 										&outActualType,			// can be NULL
 										2,						// UInt32 inBufferSize,
 										&outActualSize, 		// UInt32 *outActualSize, // can be NULL
 										&volnum)) != noErr)
			return eventNotHandledErr;
 								
		if(g_running)
			l_diskeventpending = 1;
		else
		{
			CdaInitCDRom(); // OnDiskEvent
			if(g_sets.listwindow & LBOXON && g_sets.listwindow & LBOXINFO)
				ShowPlayerInfo();
		}
	}
	
	return eventNotHandledErr; //noErr;
	
  } // OnDiskEvent
 #endif // TARGET_API_MAC_CARBON
#endif


#if defined(__MAOSX__) || defined(__MAOS9__)
void InitAppleEvents(void *app)
{
 #if !TARGET_API_MAC_CARBON
  (void)app;
 #endif

	// check for MultiProcessing services
	if(!HaveMPServices())
		ShowErr("No Mac MultiProcessing Services, MoreAmp cannot continue");

	// on mac need this to quit program when mac is shut off
	AEInstallEventHandler(kCoreEventClass, kAEQuitApplication, NewAEEventHandlerUPP(AEQuit), 0, 0);
	
	// on mac need this for dropped files
	AEInstallEventHandler(kCoreEventClass, kAEOpenDocuments, NewAEEventHandlerUPP(AEOpenDoc), 0, 0);
	
 #ifndef MANOCDA
  #if !TARGET_API_MAC_CARBON
	//AEInstallEventHandler(kAEDiskEvent, diskEvt, NewAEEventHandlerUPP(OnDiskEvent), 0, 0);
  #else
	static const EventTypeSpec eventList[] = {{kEventClassVolume, kEventVolumeMounted},
	   					 						{kEventClassVolume, kEventVolumeUnmounted},};
	InstallApplicationEventHandler(NewEventHandlerUPP(OnDiskEvent), sizeof(eventList)/sizeof(eventList[0]), eventList, app, NULL);
  #endif // !TARGET_API_MAC_CARBON
 #endif
 
 #if defined(MAGUINONE) && defined(MAKEYBOARD) && !defined(__MAOSX__) // __MAOS9__  noGUI only
	static const EventTypeSpec eventListK[] = {{kEventClassKeyboard, kEventRawKeyDown},};
	InstallApplicationEventHandler(NewEventHandlerUPP(OnRawKeyDown), sizeof(eventListK)/sizeof(eventListK[0]), eventListK, app, NULL);
 #endif
 
}
#endif
	

void AppQuit() // app exit
{
	int i;

	if(!g_sets.fileout[kOUTSOUND].mode)
	{
		g_sets.fileout[kOUTSOUND].mode = 1;
#if !defined(MAAUDIONONE)
		maCloseSndChannel(0);
#endif
	}

#if defined(AUDIO_PORTAUDIO)
	maPaTerminate();
#endif
	
	CleanupEffects();

	if(g_memhandle)
	{
#if defined(__MAOSX__) || defined(__MAOS9__)
		DisposeHandle(g_memhandle);
#else
		MAFREE(g_memhandle);
#endif
		g_memhandle = NULL; 
		l_songmem = NULL; 
	}

#if defined(__MAWIN__) && !defined(MANOCDA)
	CdaDeInitCDRom();
#endif

	// close all windows
	for(i = 1; i < WINDOWLAST; i++)
		if(g_w[i])
		{
			WindowDoClose(i);
			g_w[i] = NULL;
		}

	WindowDoDestroy(WINDOWAPP);
	
	if(l_fplogpath)
	{
		fflush(l_fplogpath);
		fclose(l_fplogpath);
	}
	
	maFreeBuffers();
	
#if defined(MAINSTANCES)
	InstanceClearThis();
 #if defined(MAUSESHM)
	if(g_pmashm)
	{
		MAFREE(g_pmashm);
		g_pmashm = NULL;
	}
 #endif
#endif
	
	if(l_songrec)
	{
		MAFREE(l_songrec);
		l_songrec = NULL;
	}
	
	maGUIfree();

#if defined(MAMEMTRACE)
{
char cstr[256];
sprintf(cstr, "g_nmamallocs=%d g_nmacallocs=%d g_ndupallocs=%d (%d) g_nmafrees=%d g_ndupfrees=%d (%d)",
	g_nmamallocs, g_nmacallocs, g_ndupallocs, g_nmamallocs + g_nmacallocs + g_ndupallocs,
	g_nmafrees, g_ndupfrees, g_nmafrees + g_ndupfrees);
maLog(cstr);
maTrace(cstr);
}
#endif

	maGUIQuit();

} // AppQuit


int SetVDisk()
{
	char dirpath[MAX_PATH];
	
	if(g_sets.vdiskname && g_sets.vdiskname[0])
		strcpy(dirpath, g_sets.vdiskname); // default dir for dirSelector
	else
		dirpath[0] = 0;
	
	if(maDirSelector(dirpath, (char *)"Select a folder on a virtual disk"))	
		return 1; // cancelled

	strcpy(g_sets.vdiskname, dirpath);
	
	return 0;

} // SetVDisk


#if !defined(MANOCDA) && defined(__MAUNIX__)
int SetAudioCD()
{
	int i;
	int item;
	
	PopupButtonGetSelection(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, &item);
	if(item == -1)
		return 1;

	item++;
	
	if(maFileDialog(g_setscdrom.cdaname[item], WINDOWAPP, "Select or enter the name of an audio cd drive",
					 "/dev", g_setscdrom.cdaname[item], "(*)|*|Any file (*)|*", maOPEN)) 
		return 1; // cancelled

	
#if defined(MAGUIOSX) || defined(MAGUIGTK)
	PopupButtonSetText(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, item, g_setscdrom.cdaname[item + 1]);
	if(g_sets.eqpreset == 0) // force a redraw
		i = 1;
	else
		i = 0;
	PopupButtonSetValue(WINDOWEQ, ID_POPUP_AUDIOCD, i);
#else
	// PopupButtonSetText fails...
	PopupButtonClear(WINDOWAUDIOCD, ID_POPUP_AUDIOCD);
	for(i = 1; i <= CDAMAXNUMDRIVES; i++)
		PopupButtonAppend(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, g_setscdrom.cdaname[i]);
#endif
	PopupButtonSetValue(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, item - 1);

	return 0;

} // SetAudioCD


int ClearAudioCD()
{
	int i;
	int item;
	
	PopupButtonGetSelection(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, &item);
	if(item == -1)
		return 1;

	sprintf(g_setscdrom.cdaname[item + 1], "(drive%02d)", item + 1);
	
#if defined(MAGUIOSX) || defined(MAGUIGTK)
	PopupButtonSetText(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, item, g_setscdrom.cdaname[item + 1]);
	if(g_sets.eqpreset == 0) // force a redraw
		i = 1;
	else
		i = 0;
	PopupButtonSetValue(WINDOWEQ, ID_POPUP_AUDIOCD, i);
#else
	// PopupButtonSetText fails...
	PopupButtonClear(WINDOWAUDIOCD, ID_POPUP_AUDIOCD);
	for(i = 1; i <= CDAMAXNUMDRIVES; i++)
		PopupButtonAppend(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, g_setscdrom.cdaname[i]);
#endif
	PopupButtonSetSelection(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, item);

	return 0;

} // ClearAudioCD
#endif


#if defined(MAINSTANCES)
int InstanceWrite(MAINSTANCE *pinstance)
{
	if(!g_pmashm)
	{
		ShowErr("InstanceWrite null g_pmashm");
		return 1;
	}

	if(mashmwrite((char *)pinstance,  1, sizeof(MAINSTANCE), g_pmashm) != sizeof(MAINSTANCE))
	{
		ShowErr("InstanceWrite mashmwrite failed");
		return 1;
	}
	
	return 0;
	
} // InstanceWrite


int InstanceRead(MAINSTANCE *pinstance)
{
	// attach to our shared instance table, create one if needed
	if(!g_pmashm)
	{
		if((g_pmashm = mashmopen(sizeof(MAINSTANCE), MAKEYFILE)) == NULL)
		{
			ShowErr("InstanceRead mashmopen failed");
			return 1;
		}
	}

	if(mashmread((char *)pinstance,  1, sizeof(MAINSTANCE), g_pmashm) != sizeof(MAINSTANCE))
	{
		ShowErr("mashmread mashmread failed");
		return 1;
	}
	
	return 0;
		
} // InstanceRead


// get a number n for masettings1, masettings2, ... masettingsn
int InstanceInit()
{
	int i;
	int ninstances;
	int pid;
	MAINSTANCE instance;
#if defined(__MAWIN__)
	int l;
	MAINSTANCE linstance;
#endif
	char cstr[MAX_PATH + 64];
	
	// wait here until no other instance has this lock
	if(maNamedLock(MAINSTANCELOCK))
	{
		ShowErr("InstanceInit maNamedLock failed");
		return 0;
	}

	// read the shared instance table
	if(InstanceRead(&instance))
	{
		ShowErr("InstanceInit InstanceRead failed");
		maNamedUnlock(MAINSTANCELOCK);
		return 0;
	}

	// do some integrity checks on our pid table, e.g. in case a previous
	// instance failed to clear its pid table entry when it quit (killed or crashed)

	// clear pid table entries for instances not found to be running
#if defined(__MAWIN__)
	// on win (but not win9x) we know how to get an array of MoreAmp pids
	memset((void *)&linstance, 0, sizeof(MAINSTANCE));
	if(maNumProcesses(linstance.pid))
	{
		for(i = 0; i < MAINSTANCEMAX; i++)
			if(instance.pid[i])
			{
				for(l = 0; l < MAINSTANCEMAX; l++)
					if(instance.pid[i] == linstance.pid[l])
						break;
				if(l >= MAINSTANCEMAX)
					instance.pid[i] = 0; // no longer running, clear it
			}
	}
	else // some non-NT win
#endif
	{
		// otherwise, we only know how to check for a given MoreAmp pid running
		for(i = 0; i < MAINSTANCEMAX; i++)
			if(instance.pid[i] && maNumProcesses(&instance.pid[i]) != 1)
				instance.pid[i] = 0; // no longer running, clear it
	}

	// count instances running according to our pid table
	ninstances = 0;
	for(i = 0; i < MAINSTANCEMAX; i++)
		if(instance.pid[i] != 0)
			ninstances++;
			
	// quit if multiple instances are not allowed
	if(ninstances && !instance.multipleinst && instance.pid[0] != 0)
	{
		maNamedUnlock(MAINSTANCELOCK);
		return -1;
	}

	// put our pid in the first free slot in the pid table
	pid = magetpid();
	for(i = 0; i < MAINSTANCEMAX; i++)
		if(instance.pid[i] == pid)
			instance.pid[i] = 0;
	for(i = 0; i < MAINSTANCEMAX; i++)
		if(instance.pid[i] == 0)
			break;
	if(i >= MAINSTANCEMAX)
	{
		sprintf(cstr, "InstanceInit cannot run more than %d instances", MAINSTANCEMAX);
		ShowErr(cstr);
		maNamedUnlock(MAINSTANCELOCK);
		return 0;
	}
	instance.pid[i] = pid;
	if(InstanceWrite(&instance))
	{
		ShowErr("InstanceInit InstanceWrite failed");
		maNamedUnlock(MAINSTANCELOCK);
		return 0;
	}
	
	maNamedUnlock(MAINSTANCELOCK);
		
	// we are 1-based instance i + 1
	return i + 1;
	
} // InstanceInit


// clear the pid field of this instance (called on app exit)
int InstanceClearThis()
{
	int i;
	int pid;
	MAINSTANCE instance;
	char cstr[MAX_PATH + 64];
	
	if(maNamedLock(MAINSTANCELOCK))
	{
		maLog("InstanceClearThis maNamedLock failed");
		return 1;
	}

	if(InstanceRead(&instance))
	{
		ShowErr("InstanceClearThis InstanceRead failed");
		maNamedUnlock(MAINSTANCELOCK);
		return 1;
	}
	
	pid = magetpid();
	for(i = 0; i < MAINSTANCEMAX; i++)
		if(instance.pid[i] == pid)
			break;
	
	if(i >= MAINSTANCEMAX)
	{
		sprintf(cstr, "InstanceClearThis pid %d not found in instance file", (int)pid);
		maLog(cstr);
		maNamedUnlock(MAINSTANCELOCK);
		return 1;
	}
		
	instance.pid[i] = 0;
		
	if(InstanceWrite(&instance))
	{
		ShowErr("InstanceClearThis InstanceWrite failed");
		maNamedUnlock(MAINSTANCELOCK);
		return 1;
	}

	maNamedUnlock(MAINSTANCELOCK);

	return 0;
	
} // InstanceClearThis


int InstanceClearAll()
{
	int i;
	MAINSTANCE instance;
	MALOCK *lock;
	
	lock = MALOGLOCK;
	if(maNamedSemaphoreOpen(&(lock->psem), lock->name))
		ShowErr("InstanceClearAll maNamedSemaphoreOpen MALOGLOCK failed");
	maNamedUnlock(lock);
	
	lock = MAINITLOCK;
	if(maNamedSemaphoreOpen(&(lock->psem), lock->name))
		ShowErr("InstanceClearAll maNamedSemaphoreOpen MAINITLOCK failed");
	maNamedUnlock(lock);
	
	lock = MAINSTANCELOCK;
	if(maNamedSemaphoreOpen(&(lock->psem), lock->name))
		ShowErr("InstanceClearAll maNamedSemaphoreOpen MAINSTANCELOCK failed");
	maNamedUnlock(lock);

	if(InstanceRead(&instance))
		ShowErr("InstanceClearAll InstanceRead failed");

	for(i = 0; i < MAINSTANCEMAX; i++)
		instance.pid[i] = 0;
		
	if(InstanceWrite(&instance))
		ShowErr("InstanceClearAll InstanceWrite failed");

	return 0;
	
} // InstanceClearAll


int InstanceListAll()
{
	int i, j;
	MAINSTANCE instance;

	if(InstanceRead(&instance))
	{
		printf("InstanceListAll InstanceRead failed\n");
		maNamedUnlock(MAINSTANCELOCK);
		return 1;
	}
	
	j = 0;
	for(i = 0; i < MAINSTANCEMAX; i++)
	{
		if(instance.pid[i])
		{
			j++;
			printf("InstanceListAll %2d pid=%d\n", i + 1, instance.pid[i]);
		}
	}
	
	printf("InstanceListAll this pid=%d multipleinst=%d locks=%d\n", magetpid(), instance.multipleinst, j);

	return 0;
	
} // InstanceListAll


// set some field of an instance table
int InstanceSet(int which, int value)
{
	MAINSTANCE instance;
	char cstr[MAX_PATH + 64];
	
	if(maNamedLock(MAINSTANCELOCK))
	{
		maLog("InstanceSet maNamedLock failed");
		return 1;
	}

	if(InstanceRead(&instance))
	{
		ShowErr("InstanceSet InstanceRead failed");
		maNamedUnlock(MAINSTANCELOCK);
		return 1;
	}
	
	if(which < 0 || which > MAINSTANCEMAX)
	{
		sprintf(cstr, "InstanceSet index=%d out of range", which);
		maLog(cstr);
		maNamedUnlock(MAINSTANCELOCK);
		return 1;
	}
		
	if(which == MAINSTANCEMAX)
		instance.multipleinst = value;
	else
		instance.pid[which] = value;
		
	if(InstanceWrite(&instance))
	{
		ShowErr("InstanceSet InstanceWrite failed");
		maNamedUnlock(MAINSTANCELOCK);
		return 1;
	}

	maNamedUnlock(MAINSTANCELOCK);

	return 0;
	
} // InstanceSet
#endif


void maLog(const char *str)
{
	time_t stTime;
	struct tm *tmTime;
	char dateline[64];
	char line[MAXLOGLINELEN];
	char cstr[MAXLOGLINELEN + MAXLOGLINELEN];
		
	if((l_logoff || g_sets.logflags & kMALOGOFF) && str != &l_logcmdclear && str != &l_logcmdexist)
		return;

#if defined(MAINSTANCES)
	if(maNamedLock(MALOGLOCK))
	{
		ShowErr("maLog maNamedLock failed");
		return;
	}
#endif

	if(str == &l_logcmdexist)
	{
		if(maFileExists(g_logpath))
			l_havelog = 1;
		else
			l_havelog = 0;
		SetLogMenus();
		goto maLogret;
	}
	
	if(!l_havelog)
	{
		l_havelog = 1;
		SetLogMenus();
	}
	
	if(!l_fplogpath)
	{
		if((l_fplogpath = mafopen(g_logpath, "a")) == NULL)
		{
			sprintf(cstr, "maLog failed to open logfile '%s'", g_logpath);
			ShowErr(cstr);
			goto maLogret;
		}
	}

	if(str == &l_logcmdclear)
	{
		fclose(l_fplogpath);
		l_fplogpath = NULL;
		if(maunlink(g_logpath))
		{
			sprintf(cstr, "maLog failed to delete '%s'", g_logpath);
			ShowErr(cstr);
		}
		l_logcount = l_logtotal = 0;
		l_havelog = 0;
		SetLogMenus();
		goto maLogret;
	}
	
	if(l_logoff || g_sets.logflags & kMALOGOFF)
		goto maLogret;
		
	stTime = time(NULL);
	tmTime = localtime((time_t *)&stTime);
	sprintf(dateline, "%2.2d:%2.2d:%2.2d", tmTime->tm_hour, tmTime->tm_min, tmTime->tm_sec);
	dateline[24] = 0;	
#if defined(MAINSTANCES)
	sprintf(line, "%d %s %s\n", g_thisinstance, dateline, str);
#else
	sprintf(line, "%s %s\n", dateline, str);
#endif
	fwrite(line, 1, strlen(line), l_fplogpath);
	fflush(l_fplogpath);
	if(g_sets.logflags & kMALOGMAX)
	{
		if(++l_logtotal >= g_sets.logmax)
		{			
			g_cancel = 1;
			l_logoff = 1;
			
			sprintf(line, "%s Too many total errors : %d. Logging is now turned off.\n",
																	dateline, l_logtotal);
			fwrite(line, 1, strlen(line), l_fplogpath);
			fflush(l_fplogpath);
			fclose(l_fplogpath);
			l_fplogpath = NULL;			
			sprintf(line, "Too many total errors : %d.\n\nLogging is now turned off.\n\n"
							"You should quit MoreAmp and see file\n\n'%s'.", l_logtotal, g_logpath);
			ShowErr(line);
			goto maLogret;
		}
	}
	
	if(g_sets.logflags & kMALOGRATE)
	{
		if(!l_logcount)
			l_logdelay = maGetLocalTimeMillis() + g_sets.logperiod * 1000;
	
		if(++l_logcount >= g_sets.logevents)
		{
			if(maGetLocalTimeMillis() < l_logdelay)
			{
				g_cancel = 1;
				sprintf(line, "%s Too many errors : %d in less than %d seconds\n",
													dateline, l_logcount, g_sets.logperiod);
				fwrite(line, 1, strlen(line), l_fplogpath);
				fflush(l_fplogpath);
				
				sprintf(line, "Too many errors : %d in less than %d seconds.\n\nSee file '%s'.",
												l_logcount, g_sets.logperiod, g_logpath);
				ShowErr(line);
			}
			l_logcount = 0;
		}
	}
	
maLogret:
#if defined(MAINSTANCES)
	if(l_fplogpath)
	{
		fclose(l_fplogpath); // because os9 cannot share, and for the ClearLog function
		l_fplogpath = NULL;
	}
	if(maNamedUnlock(MALOGLOCK))
		ShowErr("maLog maNamedUnlock failed");
#endif
	
#if defined(MAGUINONE)
	printf("%s\n", str);
#else
	if(g_w[WINDOWLOG]) // update the log window
	{
		WindowDoClose(WINDOWLOG);
		g_w[WINDOWLOG] = NULL;
		OnMenuViewLog();
		// scroll to last line:
	}
#endif

	return; // for maLogret label
	
} // maLog


int AppWindowMakeA()
{
	int ret;
	
	ret = AppWindowMake();
	SetAppWindowTitle((char *)"");
	maLog(&l_logcmdexist);
	
	return ret;
	
} // AppWindowMakeA


// gets vdisk volume name from settings and checks existence, and builds songdir path
int GetVDisk()
{
	if(!maDirExists(g_sets.vdiskname))
		return 1; // error

	// maybe do some more checks
	//...
	
#if defined(MAINSTANCES)
	sprintf(l_loadeddir, "%s%c%s%d", g_sets.vdiskname, FILESEP, MAVDISKTMP, g_thisinstance);
#else
	sprintf(l_loadeddir, "%s%c%s", g_sets.vdiskname, FILESEP, MAVDISKTMP);
#endif
		
	return 0;
	
} // GetVDisk


void OnWindowMoved(int window, int xdelta, int ydelta)
{
	if(window == WINDOWAPP && !maModKeyDown())
		WindowsMoveDelta(WINDOWAPP, xdelta, ydelta);
	
} // OnWindowMoved


// this is called when the main window is moved
// (xpos, ypos for skipwindow have already been updated in g_sets)
void WindowsMoveDelta(int skipwindow, int xdelta, int ydelta)
{
	if(skipwindow != WINDOWAPP)
	{
		g_sets.xpos[g_sets.winconf] += xdelta;
		g_sets.ypos[g_sets.winconf] += ydelta;
		if(g_w[WINDOWAPP])
			WindowMove(WINDOWAPP, g_sets.xpos[g_sets.winconf], g_sets.ypos[g_sets.winconf]);
	}
	
	if(skipwindow != WINDOWLIST)
	{
		g_sets.listxpos[g_sets.winconf] += xdelta;
		g_sets.listypos[g_sets.winconf] += ydelta;
		if(g_w[WINDOWLIST])
			WindowMove(WINDOWLIST, g_sets.listxpos[g_sets.winconf], g_sets.listypos[g_sets.winconf]);
	}
		
	if(skipwindow != WINDOWEQ)
	{
		g_sets.eqxpos[g_sets.winconf] += xdelta;
		g_sets.eqypos[g_sets.winconf] += ydelta;
		if(g_w[WINDOWEQ])
			WindowMove(WINDOWEQ, g_sets.eqxpos[g_sets.winconf], g_sets.eqypos[g_sets.winconf]);
	}
	
	if(skipwindow != WINDOWEQ2)
	{
		g_sets.eq2xpos[g_sets.winconf] += xdelta;
		g_sets.eq2ypos[g_sets.winconf] += ydelta;
		if(g_w[WINDOWEQ2])
			WindowMove(WINDOWEQ2, g_sets.eq2xpos[g_sets.winconf], g_sets.eq2ypos[g_sets.winconf]);
	}
	
#if !defined(MANOCDA) && defined(__MAUNIX__)
	if(skipwindow != WINDOWAUDIOCD)
	{
		g_setscdrom.winx[g_sets.winconf] += xdelta;
		g_setscdrom.winy[g_sets.winconf] += ydelta;
		if(g_w[WINDOWAUDIOCD])
			WindowMove(WINDOWAUDIOCD, g_setscdrom.winx[g_sets.winconf], g_setscdrom.winy[g_sets.winconf]);
	}
#endif
	
	if(skipwindow != WINDOWEFFECTS)
	{
		g_sets.effectsxpos[g_sets.winconf] += xdelta;
		g_sets.effectsypos[g_sets.winconf] += ydelta;
		if(g_w[WINDOWEFFECTS])
			WindowMove(WINDOWEFFECTS, g_sets.effectsxpos[g_sets.winconf], g_sets.effectsypos[g_sets.winconf]);
	}

	if(skipwindow != WINDOWOUTCUR)
	{
		g_sets.fileoutxpos[g_sets.winconf] += xdelta;
		g_sets.fileoutypos[g_sets.winconf] += ydelta;
		if(g_w[WINDOWOUTCUR])
			WindowMove(WINDOWOUTCUR, g_sets.fileoutxpos[g_sets.winconf],
									 g_sets.fileoutypos[g_sets.winconf]);
	}
	
	/* don't move text viewers
	if(skipwindow != WINDOWREADME && skipwindow != WINDOWCATALOG && skipwindow != WINDOWLOG)
	{
		g_sets.viewerxpos += xdelta;
		g_sets.viewerypos += ydelta;
		if(g_w[WINDOWREADME])
			WindowMove(WINDOWREADME, g_sets.viewerxpos, g_sets.viewerypos);
		if(g_w[WINDOWCATALOG])
			WindowMove(WINDOWCATALOG, g_sets.viewerxpos, g_sets.viewerypos);
		if(g_w[WINDOWLOG])
			WindowMove(WINDOWLOG, g_sets.viewerxpos, g_sets.viewerypos);
	}

#if defined(MAKEYBOARD)
	if(skipwindow != WINDOWKEYBINDINGS)
	{
		g_sets.keybindingsxpos += xdelta;
		g_sets.keybindingsypos += ydelta;
		if(g_w[WINDOWKEYBINDINGS])
			WindowMove(WINDOWKEYBINDINGS, g_sets.keybindingsxpos, g_sets.keybindingsypos);
	}
#endif

	if(skipwindow != WINDOWABOUT)
	{
		g_sets.aboutxpos += xdelta;
		g_sets.aboutypos += ydelta;
		if(g_w[WINDOWABOUT])
			WindowMove(WINDOWABOUT, g_sets.aboutxpos, g_sets.aboutypos);
	}
	*/

} // WindowsMoveDelta


void WindowMoveAndSetSize(int window, int xpos, int ypos, int xsize, int ysize)
{
	if(!g_w[window])
		return;
	WindowMove(window, xpos, ypos);
	WindowSetSize(window, xsize, ysize);
}


void WindowsMoveAndSetSize()
{
	if(g_w[WINDOWAPP])
	{
		WindowMoveAndSetSize(WINDOWAPP,
			g_sets.xpos[g_sets.winconf], g_sets.ypos[g_sets.winconf],
			g_sets.xsize[g_sets.winconf], g_sets.ysize[g_sets.winconf]);
	}
	
	if(g_w[WINDOWLIST])
	{
		WindowMoveAndSetSize(WINDOWLIST,
			g_sets.listxpos[g_sets.winconf], g_sets.listypos[g_sets.winconf],
			g_sets.listxsize[g_sets.winconf], g_sets.listysize[g_sets.winconf]);
	}
		
	if(g_w[WINDOWEQ])
	{
		WindowMoveAndSetSize(WINDOWEQ,
			g_sets.eqxpos[g_sets.winconf], g_sets.eqypos[g_sets.winconf],
			g_sets.eqxsize[g_sets.winconf], g_sets.eqysize[g_sets.winconf]);
	}
	
	if(g_w[WINDOWEQ2])
	{
		WindowMoveAndSetSize(WINDOWEQ2,
			g_sets.eq2xpos[g_sets.winconf], g_sets.eq2ypos[g_sets.winconf],
			g_sets.eq2xsize[g_sets.winconf], g_sets.eq2ysize[g_sets.winconf]);
	}
	
#if !defined(MANOCDA) && defined(__MAUNIX__)
	if(g_w[WINDOWAUDIOCD])
	{
		WindowMoveAndSetSize(WINDOWAUDIOCD,
			g_setscdrom.winx[g_sets.winconf], g_setscdrom.winy[g_sets.winconf],
			g_setscdrom.winw[g_sets.winconf], g_setscdrom.winh[g_sets.winconf]);
	}
#endif
	
	if(g_w[WINDOWEFFECTS])
	{
		WindowMoveAndSetSize(WINDOWEFFECTS,
			g_sets.effectsxpos[g_sets.winconf], g_sets.effectsypos[g_sets.winconf],
			g_sets.effectsxsize[g_sets.winconf], g_sets.effectsysize[g_sets.winconf]);
	}

	if(WINDOWOUTCUR >= WINDOWOUTFIRST && WINDOWOUTCUR <= WINDOWOUTLAST && g_w[WINDOWOUTCUR])
	{
		WindowMoveAndSetSize(WINDOWOUTCUR,
		g_sets.fileoutxpos[g_sets.winconf], g_sets.fileoutypos[g_sets.winconf],
		g_sets.fileoutxsize[g_sets.winconf], g_sets.fileoutysize[g_sets.winconf]);
	}
	
	if(g_w[WINDOWREADME])
	{
		WindowMoveAndSetSize(WINDOWREADME,
			g_sets.viewerxpos, g_sets.viewerypos, g_sets.viewerxsize, g_sets.viewerysize);
	}

	if(g_w[WINDOWCATALOG])
	{
		WindowMoveAndSetSize(WINDOWCATALOG,
			g_sets.viewerxpos, g_sets.viewerypos, g_sets.viewerxsize, g_sets.viewerysize);
	}

	if(g_w[WINDOWLOG])
	{
		WindowMoveAndSetSize(WINDOWLOG,
			g_sets.viewerxpos, g_sets.viewerypos, g_sets.viewerxsize, g_sets.viewerysize);
	}

#if defined(MAKEYBOARD)
	if(g_w[WINDOWKEYBINDINGS])
	{
		WindowMoveAndSetSize(WINDOWKEYBINDINGS,
		g_sets.keybindingsxpos, g_sets.keybindingsypos, g_sets.keybindingsxsize, g_sets.keybindingsysize);
	}
#endif

	if(g_w[WINDOWABOUT])
	{
		WindowMoveAndSetSize(WINDOWABOUT,
			g_sets.aboutxpos, g_sets.aboutypos, g_sets.aboutxsize, g_sets.aboutysize);
	}

	if(g_w[WINDOWADVANCED])
	{
		WindowMoveAndSetSize(WINDOWADVANCED,
			g_sets.advancedxpos, g_sets.advancedypos, g_sets.advancedxsize, g_sets.advancedysize);
	}

} // WindowsMoveAndSetSize


void WindowsCopyPositions()
{
	int w = g_sets.winconf ? 0 : 1;
	
	g_sets.xpos[g_sets.winconf]			= g_sets.xpos[w];
	g_sets.ypos[g_sets.winconf]			= g_sets.ypos[w];
	g_sets.eqxpos[g_sets.winconf]		= g_sets.eqxpos[w];
	g_sets.eqypos[g_sets.winconf]		= g_sets.eqypos[w];
	g_sets.eq2xpos[g_sets.winconf]		= g_sets.eq2xpos[w];
	g_sets.eq2ypos[g_sets.winconf]		= g_sets.eq2ypos[w];
	g_sets.effectsxpos[g_sets.winconf]	= g_sets.effectsxpos[w];
	g_sets.effectsypos[g_sets.winconf]	= g_sets.effectsypos[w];
	g_sets.listxpos[g_sets.winconf]		= g_sets.listxpos[w];
	g_sets.listypos[g_sets.winconf]		= g_sets.listypos[w];
	g_sets.fileoutxpos[g_sets.winconf]	= g_sets.fileoutxpos[w];
	g_sets.fileoutypos[g_sets.winconf]	= g_sets.fileoutypos[w];
	
#if !defined(MANOCDA) && defined(__MAUNIX__)
	g_setscdrom.winx[g_sets.winconf] = g_setscdrom.winx[w];
	g_setscdrom.winy[g_sets.winconf] = g_setscdrom.winy[w];
#endif

} // WindowsCopyPositions


char g_AboutText[] =
	" MoreAmp "MOREAMP_VERSION_STRING " Beta (c) 2004-2008 moreforge\n\n"

	" Play aac, aif, fla, flac, m2a, m4a, mp1, mp2, mp3, mp4 sound, ogg, wav, wma files, Audio CD,\n"
	" and live audio input.\n\n"

	" Create aac, aif, flac, mp3, mp4, ogg, wav files.\n\n"

	" On Mac OS 9, Mac OS X, Win32, Linux/Unix/X11/GTK.\n\n"

	" This program is free software which anyone can redistribute and/or modify under the terms of the\n"
	" GNU General Public License as published by the Free Software Foundation; either version 2 of the\n"
	" License, or (optionally) any later version.\n\n"
	
	" This program is distributed WITHOUT ANY WARRANTY, without even the implied warranty of\n"
	" MERCHANTABILITYor FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License\n"
	" for more details.\n\n"
	
	" This program should come with a copy of the GNU General Public License (the file named 'COPYING'\n"
	" if not, write to :\n"
	" The Free Software Foundation, Inc.\n"
	" 59 Temple Place, Suite 330\n"
	" Boston, MA  02111-1307  USA\n\n"
	
	" This program should also come with a file named 'COPYINGOggVorbis', a copy of the file named\n"
	" 'COPYING' in the public releases of libogg and libvorbis, Copyright (c) 2002, Xiph.org Foundation.\n\n"

	" This program uses all or parts of the following GNU GPL licensed or otherwise free software :\n"

#if defined(MAGUIWX)
	" audacity (extracts) gui buttons and sliders\n"
#endif

#if !defined(MANOCDA)
 #if defined(__MAWIN__)
	" cdex/cdrip (extracts) cda reader\n"
 #endif
 #if defined(__MAUNIX__)
	" cdparanoia (as released) cda reader\n"
 #endif
#endif

	" eq-xmms (extracts) equalizer\n"

#if !defined(NODECODEFLAC) || defined(ENCODEFLAC)
	" flac (modified) flac encoder/decoder\n"
#endif

#if !defined(NODECODEWMA)
	" ffmpeg (extracts) wma decoder\n"
#endif

#if defined(ENCODEMP3)
	" lame (as released) mp3 encoder\n"
#endif

#if defined(ENCODEAAC) || defined(ENCODEMP4) 
	" libfaac (as released) aac mp4 encoder\n"
#endif

#if !defined(NODECODEAAC) || !defined(NODECODEMP4) 
	" libfaad (as released) aac, m2a, m4a, mp4 decoder\n"
#endif

#if !defined(NODECODEMP3)
	" libid3tag (modified) id3 tag reader\n"
#endif

#if defined(USE_LIBJACK)
 	" libjack (as released) Jack Audio Connection Kit\n"
#endif

#if !defined(NODECODEMP3)
 #if defined(__MAOS9__)
 	" libmad (modified for CW6) MPEG decoder\n"
 #else
 	" libmad (as released) MPEG decoder\n"
 #endif
#endif

#if !defined(NODECODEOGG) || defined(ENCODEOGG) 
	" libogg (as released) ogg encoder/decoder\n"
#endif

#if !defined(NODECODEAIF) || !defined(NODECODEWAV) || defined(ENCODEAIF) || defined(ENCODEWAV) 
	" libsndfile (extracts) aif wav reader\n"
#endif

#if !defined(NODECODEOGG) || defined(ENCODEOGG) 
	" libvorbis (as released) ogg encoder/decoder\n"
#endif

	" libz/zlib (as released) gz decompression\n"

#if defined(AUDIO_PORTAUDIO) 
 #if !defined(NODECODEAUDIOIN)
  	" portaudio (as released) audio input/output\n"
 #else
 	" portaudio (as released) audio output\n"
 #endif
#endif

	" SoundTouch (extracts) pitch/tempo/speed effects\n"

	"                                                                " // 64 spaces for gui info

	; // valuable semi-colon here


#ifdef __cplusplus
}
#endif

// eof
